#!/bin/bash
# $Id: CESecurity.sh,v 1.7 2012/05/28 19:15:15 sgupta2 Exp $
# $Copyright: Copyright (c) 2012 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $

# To check whether certificate for VCS-VM is created or not.
# Create it if not already created. In that case password for VCS-VM's certificate
# is expected as argument.
create_vcsvm_certificate() 
{
	local RC=0

# showcred command will output "Found: 0" or "Found: 1". So we grep 1 to find whether
# credential exists or not.
	EAT_DATA_DIR=${VCSAUTH_DATA_DIR}/CE; export EAT_DATA_DIR
	${VCSAT} showcred -d vx:VCS_SERVICES -p CE | grep 1 >> $LOGFILE
	if [ $? -eq 0 ]
	then
		unset EAT_DATA_DIR
		echo "Credentials for VCS cluster node already exist. Will not create again." >> $LOGFILE
	else
		unset EAT_DATA_DIR
		echo "Credentials for VCS cluster node do not exist. Will create now." >> $LOGFILE
		local vcs_vm_password=$1
		local fips_mode=$2

		${VSSAT} createpd -d VCS_SERVICES -t ab
		RC=$?
		if [ ${RC} -ne 0 ]
		then
			echo "Error while creating private domain [${RC}]." >> $LOGFILE
			exit 1;
		fi

		${VSSAT} addprpl -t ab -d VCS_SERVICES -q service -p CE -s "${vcs_vm_password}"
		RC=$?
		if [ ${RC} -ne 0 ]
		then
			echo "Error while creating VCS cluster node credentials [${RC}]." >> $LOGFILE
			exit 1
		fi

		EAT_DATA_DIR=${VCSAUTH_DATA_DIR}/CE; export EAT_DATA_DIR

		# Remove the old credential data before pushing the new credential data in that directory
		rm -rf ${EAT_DATA_DIR}

		${VCSAT} setuptrust -b localhost:14149 -s low
		RC=$?
		if [ ${RC} -ne 0 ]
		then
			echo "Error while trusting the broker certificate [${RC}]." >> $LOGFILE
			exit 1;
		fi

		# if FIPS mode is enabled, set fips mode
		if [ ${fips_mode} -eq 1 ]
		then
			${VCSAT} setfipsmode -e
			RC=$?
			if [ ${RC} -ne 0 ]
			then
				echo "Error while enabling fips mode [${RC}]." >> $LOGFILE
				exit 1;
			fi
		fi

		${VCSAT} authenticate -p CE -d vx:VCS_SERVICES -s "${vcs_vm_password}" -b localhost:14149 >> $LOGFILE
		RC=$?
		if [ ${RC} -ne 0 ]
		then
			echo "Error while authenticating VCS cluster node credentials [${RC}]." >> $LOGFILE
			exit 1
		fi

		unset EAT_DATA_DIR
	fi
}

# To create credential for AppVM and export it into a file.
# Needs the name of AppVM and password for AppVM's certificate as argument.
create_appvm_credential() 
{
	local app_vm_name=$1
	local app_vm_password=$2
	local RC=0

	EAT_DATA_DIR=${export_dir}; export EAT_DATA_DIR
	${VCSAT} showcred -d vx:VCS_SERVICES -p ${app_vm_name} | grep 1 >> $LOGFILE
	if [ $? -ne 0 ]
	then
		unset EAT_DATA_DIR
		echo "Creating credentials for ApplicationHA node." >> $LOGFILE
		${VSSAT} addprpl -t ab -d VCS_SERVICES -p "${app_vm_name}" -s "${app_vm_password}" -q service
		RC=$?
		if [ ${RC} -ne 0 ]
		then
			echo "Error while creating ApplicationHA node credentials [${RC}]." >> $LOGFILE
			exit 1
		fi
	
	fi
}

export_appvm_credential()
{
	local app_vm_name=$1
	local app_vm_password=$2
	local app_vm_export_password=$3
	local fips_mode=$4
	local RC=0

	EAT_DATA_DIR=${export_dir}; export EAT_DATA_DIR

	# Remove the old credential data before pushing the new credential data in that directory
	rm -rf ${EAT_DATA_DIR}

	${VCSAT} setuptrust -b localhost:14149  -s low
	RC=$?
	if [ ${RC} -ne 0 ]
	then
		echo "Error while trusting the broker certificate [${RC}]." >> $LOGFILE
		exit 1;
	fi

	# if FIPS mode is enabled, set fips mode
	if [ ${fips_mode} -eq 1 ]
	then
		${VCSAT} setfipsmode -e
		RC=$?
		if [ ${RC} -ne 0 ]
		then
			echo "Error while enabling fips mode [${RC}]." >> $LOGFILE
			exit 1;
		fi
	fi

	${VCSAT} authenticate -d vx:VCS_SERVICES -p "${app_vm_name}" -s "${app_vm_password}" -b localhost:14149
	RC=$?
	if [ ${RC} -ne 0 ]
	then
		echo "Error while authenticating ApplicationHA node credentials [${RC}]." >> $LOGFILE
		exit 1
	fi

	${ATUTIL} export -c -z ${export_dir} -f ${export_file} -p "${app_vm_export_password}"
	RC=$?
	if [ ${RC} -ne 0 ]
	then
		echo "Error while exporting ApplicationHA node credentials [${RC}]." >> $LOGFILE
		exit 1
	fi

	# Will set this tag only if FIPS is enabled because we want to support old version in AppVM
	# when running in non-FIPS mode
	if [ ${fips_mode} -eq 1 ]
	then
		local fips_comment_tag="<!-- Added by ApplicationHA. DO NOT REMOVE OR CHANGE THIS TAG! -->"
		local fips_tag="<fipsmode>${fips_mode}</fipsmode>"
		echo ${fips_comment_tag} >> ${export_file}
		echo ${fips_tag} >> ${export_file}
		echo "Added FIPS info in export file ${export_file}." >> $LOGFILE
	fi

	unset EAT_DATA_DIR
}

# To copy credentials of AppVM from VCS-VM to AppVM
# Needs the name AppVM as argument
copy_cred() 
{
	local app_vm_name=$1
	local RC=0

	# Check whether passwordless ssh is configured or not
	ssh -o NumberOfPasswordPrompts=0 -o StrictHostKeyChecking=no -C -l root "${app_vm_name}" x=x
	RC=$?
	if [ ${RC} -ne 0 ]
	then
		echo "password-free ssh is NOT configured" >> $LOGFILE
		exit 1
	fi

	echo "Copying credentials for ApplicationHA node." >> $LOGFILE
	scp ${export_file} root@"${app_vm_name}":${import_file}
#	${XPRTLC} -l https://${app_vm}:5634/admin/cgi-bin/push_file.pl -u ${user} -p ${password} -f ${export_file} -d destfile=${import_file} -d force=1
	RC=$?
	if [ ${RC} -ne 0 ]
	then
		echo "Error while copying ApplicationHA node credentials [${RC}]." >> $LOGFILE
		rm ${export_file}
		exit 1
	else
		rm ${export_file}
	fi
}

# To delete credential of $app_vm_name from VCS-VM
# Needs the name of AppMV as argument
del_credential() 
{
	local app_vm_name=$1
	local RC=0

	echo "Deleting credentials for ApplicationHA node from VCS cluster node." >> $LOGFILE
	unset EAT_DATA_DIR
	${VSSAT} deleteprpl -t ab -p "${app_vm_name}" -d VCS_SERVICES -s
	RC=$?
	if [ ${RC} -ne 0 ]
	then
		echo "Error while deleting ApplicationHA node credentials from VCS cluster node [${RC}]." >> $LOGFILE
		exit 1
	fi

	rm -rf ${export_dir}
}

# To import credential on AppVM
# Needs the name of AppMV as argument
import_appvm_credential() 
{
	local app_vm_name=$1
	local app_vm_export_password=$2
	local RC=0

	mkdir -p ${VCSAUTH_DATA_DIR}

	if [ -d "${VCSAUTH_DATA_DIR}/${app_vm_name}" ]
	then
		rm -rf ${VCSAUTH_DATA_DIR}/"${app_vm_name}"
	fi

	echo "Extracting fips info from credential file." >> $LOGFILE
	local FIPSMode=`grep fipsmode ${import_file} | sed -e 's#<fipsmode>\(.\)</fipsmode>#\1#'`
	sed -e '/<fipsmode>.*<\/fipsmode>/d' -e '/<!--.*ApplicationHA.*-->/d' ${import_file} > ${tmp_import_file}

	sed -e '/\(VCS_CE_FIPS=\).*/d' ${VCS_CE_CONF_FILE} > ${VCS_CE_CONF_FILE}.1
	mv ${VCS_CE_CONF_FILE}.1 ${VCS_CE_CONF_FILE}
	echo "VCS_CE_FIPS=${FIPSMode}" >> ${VCS_CE_CONF_FILE}

	echo "Importing credentials for ApplicationHA node." >> $LOGFILE
	${ATUTIL} import -f ${tmp_import_file} -z ${VCSAUTH_DATA_DIR}/"${app_vm_name}" -p "${app_vm_export_password}"
	RC=$?
	if [ ${RC} -ne 0 ]
	then
		echo "Error while importing ApplicationHA node credentials [${RC}]." >> $LOGFILE
		exit 1
	fi
}

