#!/opt/VRTSsfmh/bin/perl
#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Id: discoverNodeDetails.pl,v 1.17 2012/07/23 14:27:41 sgupta2 Exp $
#

#------------------------
# Common Header
#------------------------
use strict;
use warnings;

use Data::Dumper;
use CGI;
use Config;
use POSIX;

# Push include paths!
my $LibPath1 = "";

use constant HAD_HELPER_SERVICE => "HadHelper";

BEGIN
{
	if ( $^O =~ m/win/i ) {
		require VRTS::Paths;

		my $xprtlInstallDir= VRTS::Paths::get_path("InstallDir");
		my $regquery = "\"$xprtlInstallDir"."\\bin\\regquery.exe\" query SOFTWARE\\VERITAS\\VCS\\Base HomeDirectory";
		my $vcshome = `$regquery`;
		$LibPath1 = "$vcshome\\portal\\admin\\lib\\";
		$ENV{'VCS_HOME'} = $vcshome;

	} else {
		my $APPDISCOVER_HOME = "/opt/VRTSvcs/portal/admin";
	
		$LibPath1 = "$APPDISCOVER_HOME/lib";
	}
}

use lib $LibPath1;

use VCS;
use Constants;
#------------------------
# End of Common Header
#------------------------



# ---------------------------
# Cluster Details
# ---------------------------
sub getClusterID
{
	my $xmlClusterID;
	my $clusID = VCS::getClusterID();
	$xmlClusterID = "\t<ClusterID>$clusID</ClusterID>\n" if $clusID;
	return $xmlClusterID;
}


sub IsSecureCluster
{	
	my $cmdRet = CONST_SUCCESS;
	my $retMsg = "";
	my $op = "";
	my $xmlSecureClus = "";
	
	# Create new object of VCS.
	my $vcsobj = new VCS();
	
	($cmdRet, $retMsg, $op) = $vcsobj->IsSecureCluster();
	goto END if($cmdRet ne CONST_SUCCESS);

	$xmlSecureClus = "\t<SecureClus>$op</SecureClus>\n";	
END:
	return ($cmdRet,$retMsg,$xmlSecureClus);
}


sub getClusterSystemsList
{
	my $cmdRet = CONST_SUCCESS;
	my $retMsg = "";
	my $op = "";
	my $xmlSystemList = "";
	
	($cmdRet, $retMsg, $op) = VCS::getClusterSystemsList();
	goto END if($cmdRet ne CONST_SUCCESS) ;
	
	$xmlSystemList .= "\t<SystemList>\n";
	my @fields = split(/\s+/, $op);
	
	foreach my $clusSys (  @fields ) {	
		$xmlSystemList .= "\t\t<System>\n";
		$xmlSystemList .= "\t\t\t<Name>$clusSys</Name>\n";
		$xmlSystemList .= "\t\t</System>\n";
	}
	$xmlSystemList .= "\t</SystemList>\n";
END:	
	return ($cmdRet,$retMsg,$xmlSystemList);
}


sub getClusterConfigurationStatus
{
	my $clusterConfigured =	VCS::getClusterConfigurationStatus();	
	my $xmlClusterConfigured = "\t<ClusterConfigured>$clusterConfigured</ClusterConfigured>\n";	
	return $xmlClusterConfigured,$clusterConfigured;
}


sub getClusterName
{	
	my ($CmdRet,$op);
	my $xmlClusterName;	
	($CmdRet, $op) = VCS::getClusterName();
	
	return "" if($CmdRet ne CONST_SUCCESS) ;

	$xmlClusterName .= "\t<ClusterName>$op</ClusterName>\n";
	return $xmlClusterName;
}


sub getClusterIDAndName
{	
	my $clusterConfigured = shift;
	my $xmlClusterIDAndName ;
	if($clusterConfigured =~ 0)
	{
		my ($clusterID,$clusterName) = VCS::getRandomClusterIDAndName() ;		
		$xmlClusterIDAndName  = "\t<ClusterID>$clusterID</ClusterID>\n" if $clusterID;
		$xmlClusterIDAndName .= "\t<ClusterName>$clusterName</ClusterName>\n" if $clusterName;;
		return $xmlClusterIDAndName;
	}

	$xmlClusterIDAndName  = getClusterID();
	$xmlClusterIDAndName .= getClusterName();
	return $xmlClusterIDAndName;
}


# ---------------------------
# LLT Link Details
# ---------------------------
sub getLLTLink
{
	my ($xmlLLTLink,$xmlLLTLinkID,$xmlLLTLinkMac,$xmlLLTLinkType,$xmlIPVersion,$xmlIPAddress,$xmlSubnetMask,$xmlSAPPort,$xmlLowPriority);
	my $lltLink = shift;
	
	$xmlLLTLinkID = "\t\t<LinkID>$lltLink->{LinkID}</LinkID>\n" if $lltLink->{LinkID}; 
	$xmlLowPriority = "\t\t<LowPri>$lltLink->{IsLowPriority}</LowPri>\n"  if defined $lltLink->{IsLowPriority};
	$xmlLLTLinkMac 	= "\t\t<MAC>$lltLink->{MAC}</MAC>\n" if $lltLink->{MAC};
	$xmlLLTLinkType = "\t\t<Type>$lltLink->{LinkType}</Type>\n" if $lltLink->{LinkType}; 		
	$xmlIPVersion 	= "\t\t<IPVersion>$lltLink->{IPVersion}</IPVersion>\n" if $lltLink->{IPVersion};
	$xmlSAPPort 	= "\t\t<Port>$lltLink->{SAPPort}</Port>\n" if $lltLink->{SAPPort};
	$xmlIPAddress 	= "\t\t<IP>$lltLink->{IPAddress}</IP>\n"  if $lltLink->{IPAddress};
	$xmlSubnetMask 	= "\t\t<NetMask>$lltLink->{SubnetMask}</NetMask>\n"  if $lltLink->{SubnetMask};
		
	$xmlLLTLink 		 = "\t<Link>\n";
	$xmlLLTLink 		.= "$xmlLLTLinkID" if $xmlLLTLinkID;	
	$xmlLLTLink 		.= "$xmlLLTLinkMac" if $xmlLLTLinkMac;
	$xmlLLTLink 		.= "$xmlLowPriority" if $xmlLowPriority;
	$xmlLLTLink 		.= "$xmlLLTLinkType" if $xmlLLTLinkType;
	
	$xmlLLTLink 		.= "$xmlIPVersion" if $xmlIPVersion;
	$xmlLLTLink 		.= "$xmlIPAddress" if $xmlIPAddress;
	$xmlLLTLink 		.= "$xmlSubnetMask" if $xmlSubnetMask;
	$xmlLLTLink 		.= "$xmlSAPPort" if $xmlSAPPort;
	
	$xmlLLTLink 		.= "\t</Link>\n";
	
	return $xmlLLTLink;
}


sub getLLTLinkDetails
{
	my ($xmlLLTDetails,$xmlLLTLinkDetails);
	my ($retCode,$errorMsg,$lltLinkDetails) = VCS::getLLTLinkDetails();
	if($retCode != CONST_SUCCESS)
	{
		goto EXIT;
	}
	my $linkID = 0;
	
	foreach my $lltLink (keys %$lltLinkDetails) {	
		$xmlLLTLinkDetails .= getLLTLink($lltLinkDetails->{$lltLink});		
	}
	
	$xmlLLTDetails = "<LLTLinks>\n";
	$xmlLLTDetails .= "$xmlLLTLinkDetails" if $xmlLLTLinkDetails;
	$xmlLLTDetails .= "</LLTLinks>\n";
	
EXIT:
	return ($retCode,$errorMsg,$xmlLLTDetails);
}


sub getServiceAccountName
{	
	my $ret 					= CONST_SUCCESS;
	my $msg 					= "Successful completion of operation.";
	my $serviceName 			= shift;
	my $serviceAccountName 		= "";
	my $serviceAccountNameXml 	= "";
	
	($ret,$msg,$serviceAccountName) = PlatUtils::getServiceAccountName($serviceName);
	goto END if ( $ret != CONST_SUCCESS );
			
	if ( !defined $serviceAccountName or $serviceAccountName eq "" ) 
	{
		Utils::vxlog ("failed to retrieve service account,$serviceAccountName.");
	}
	else
	{
		$serviceAccountNameXml 	= "\t<HadHelperUser>$serviceAccountName</HadHelperUser>\n" ;
	}
END:
	return ($ret, $msg,$serviceAccountNameXml);
}

# ---------------------------
# Current Node Details
# ---------------------------
sub getClusterDetails
{	
	my $retCode 					= CONST_SUCCESS;
	my $errorMsg 					= "";
	my $xmlClusterSystems 			= "";
	my $xmlSecureClus 				= "";
	my $xmlHadHelperServiceAccount;
	
	my ($xmlClusterDetails,$xmlClusterID,$xmlClusterConfigured,$clusterConfigured,$xmlLLTDetails);
	($xmlClusterConfigured,$clusterConfigured) = getClusterConfigurationStatus();
	#
	# Had helper service's account name is retrieved only in windows
	#
	if ( $^O =~ m/win/i )
	{
		($retCode,$errorMsg,$xmlHadHelperServiceAccount)  = getServiceAccountName(HAD_HELPER_SERVICE);
		goto END if ($retCode != CONST_SUCCESS);
	}
	
	if($clusterConfigured =~ 1)
	{
		($retCode,$errorMsg,$xmlClusterSystems)  = getClusterSystemsList();
		goto END if ($retCode != CONST_SUCCESS);

		($retCode,$errorMsg,$xmlSecureClus)  = IsSecureCluster();
		goto END if ($retCode != CONST_SUCCESS);
	}
	$xmlClusterDetails  = "<ClusterDetails>\n";
	$xmlClusterDetails .=  $xmlClusterConfigured;
	$xmlClusterDetails .=  getClusterIDAndName($clusterConfigured);
	$xmlClusterDetails .=  $xmlClusterSystems if defined $xmlClusterSystems;
	$xmlClusterDetails .=  $xmlSecureClus if defined $xmlSecureClus;
	$xmlClusterDetails .=  $xmlHadHelperServiceAccount if defined $xmlHadHelperServiceAccount;
	$xmlClusterDetails .= "</ClusterDetails>\n";
END:
	return ($retCode,$errorMsg,$xmlClusterDetails);
}


# ---------------------------
# Current OS Details
# ---------------------------
sub getOSInfo
{
	my $retCode = CONST_SUCCESS;
	my $osInfoHash = Utils::getOSInfo();
	
	my $osName  	 = $osInfoHash->{OS};
	my $domain 		 = $osInfoHash->{DOMAIN};
	my $computerName = $osInfoHash->{COMPUTERNAME};
	my $platformBits = $osInfoHash->{BITS};
	my $OSCaption	 = $osInfoHash->{OSCAPTION};
	my $padv	 = $osInfoHash->{PADV};

	#TODO:Remove special characters from computer name
	my $xmlOSInfo .= "<OSInfo>\n";
	$xmlOSInfo .= "\t<ComputerName>$computerName</ComputerName>\n";
	$xmlOSInfo .= "\t<Domain>$domain</Domain>\n";
	$xmlOSInfo .= "\t<OS>$osName</OS>\n";
	$xmlOSInfo .= "\t<OSVersion>$OSCaption</OSVersion>\n";
	$xmlOSInfo .= "\t<PADV>$padv</PADV>\n";
	$xmlOSInfo .= "\t<Platform>$platformBits</Platform>\n";
	
	$xmlOSInfo .= "</OSInfo>\n";
	return ($retCode,"",$xmlOSInfo);
}


# ---------------------------
# Current OS's NIC Details
# ---------------------------
sub getNICList
{
	my $retCode = CONST_SUCCESS;
	my $nicListHash = PlatUtils::getNICList();
	
	my $xmlNICList;

	$xmlNICList = "<NICList>\n";

	foreach my $nicKey ( keys %$nicListHash ) {
	   my $nicItem = $nicListHash->{$nicKey};
	   $xmlNICList .= "\t<NIC>\n";
	   
	   $xmlNICList .= "\t\t<Name>$nicItem->{Caption}</Name>\n";
	   $xmlNICList .= "\t\t<MAC>$nicItem->{MACAddress}</MAC>\n";
	   
	   my $dhcpEnabled = 0;
	   $dhcpEnabled = $nicItem->{DHCPEnabled} if defined $nicItem->{DHCPEnabled};
	   $xmlNICList .= "\t\t<DHCPEnabled>$dhcpEnabled</DHCPEnabled>\n";	   
	   $xmlNICList .= "\t\t<IPAddresses>\n";
	   foreach my $IPAddressElement (keys %{$nicItem->{IPAddresses}})
	   {
			$xmlNICList .= "\t\t\t<IP>\n";
			$xmlNICList .= "\t\t\t\t<IPVersion>" .
				$nicItem->{IPAddresses}{$IPAddressElement}{IPVersion} .
				"</IPVersion>\n";
			$xmlNICList .= "\t\t\t\t<Address>" .
				$nicItem->{IPAddresses}{$IPAddressElement}{IPAddress} .
				"</Address>\n";
			$xmlNICList .= "\t\t\t\t<NetMask>" .
				$nicItem->{IPAddresses}{$IPAddressElement}{NetMask} .
				"</NetMask>\n";
			$xmlNICList .= "\t\t\t\t<PrefixLen>" .
				$nicItem->{IPAddresses}{$IPAddressElement}{PrefixLen} .
				"</PrefixLen>\n";
			$xmlNICList .= "\t\t\t</IP>\n";
	   }
	   $xmlNICList .= "\t\t</IPAddresses>\n";
	   $xmlNICList .= "\t\t<Prefixes>\n";
	   foreach my $prefixValue (@{$nicItem->{Prefixes}})
	   {
			$xmlNICList .= "\t\t\t<Prefix>$prefixValue</Prefix>\n";
	   }
	   $xmlNICList .= "\t\t</Prefixes>\n";
	   $xmlNICList .= "\t</NIC>\n";
	}
	$xmlNICList .= "</NICList>\n";
	return ($retCode,"",$xmlNICList);
}


# ---------------------------
# main() Subroutine
# ---------------------------
sub main
{
	my $retCode 		= CONST_SUCCESS;
	my $retMessage 		= "Successful completion of the operation";
	my ($errorCode,$errorMessage);
	my ($xmlClusterDetails, $xmlLLTLinkDetails, $xmlCurrentNodeDetails,$xmlCurrentNICList);
	
	my $query = new CGI;
	print $query->header("text/xml");
	
	my $cmdArgs = Utils::parseArguments($query);

	my @paramsArr = split(",",$cmdArgs->{'params'}) if $cmdArgs->{'params'};
	my $discoveryRequest;
	@{$discoveryRequest}{@paramsArr} = @paramsArr;
	my $xmlStr 	= "<DiscoveryData>\n";
	if($discoveryRequest->{'ClusterDetails'})
	{
		($errorCode,$errorMessage,$xmlClusterDetails) 		= getClusterDetails() ;
		if($errorCode != CONST_SUCCESS)
		{
			$retCode 		= $errorCode;
			$retMessage 	= $errorMessage;
		}
	}
	if($discoveryRequest->{'LLTLinks'})
	{
		($errorCode,$errorMessage,$xmlLLTLinkDetails)		=  getLLTLinkDetails() ;
		if($errorCode != CONST_SUCCESS)
		{
			$retCode 		= $errorCode;
			$retMessage 	= $errorMessage;
		}
	}
	
	if($discoveryRequest->{'OSInfo'})
	{
		($errorCode,$errorMessage,$xmlCurrentNodeDetails) 	= getOSInfo() ;
		if($errorCode != CONST_SUCCESS)
		{
			$retCode 		= $errorCode;
			$retMessage 	= $errorMessage;
		}
	}
	
	if($discoveryRequest->{'NICList'})
	{
		($errorCode,$errorMessage,$xmlCurrentNICList) 		= getNICList() ;
		if($errorCode != CONST_SUCCESS)
		{
			$retCode 		= $errorCode;
			$retMessage 	= $errorMessage;
		}
	}
	
	$xmlStr .= $xmlClusterDetails 		if $xmlClusterDetails;
	$xmlStr .= $xmlLLTLinkDetails 		if $xmlLLTLinkDetails;
	$xmlStr .= $xmlCurrentNodeDetails 	if $xmlCurrentNodeDetails;
	$xmlStr .= $xmlCurrentNICList 		if $xmlCurrentNICList;	
	$xmlStr .= "</DiscoveryData>\n";
	$xmlStr = Utils::getCmdResponseXML("discoverNodeDetails.pl", $retCode, $retMessage , $xmlStr);
	
	print $xmlStr;
	exit (CONST_SUCCESS);
}

main();


