#!/opt/VRTSsfmh/bin/perl

#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Id: getVCSAppInfo.pl,v 1.8 2012/08/23 17:09:17 sgupta2 Exp $
#

=pod
	This script returns the VCS Resource and Service Group information.
=cut

#------------------------
# Common Header
#------------------------
use strict;
use warnings;

use Data::Dumper;
use CGI;
use Config;
use POSIX;

# Push include paths!
my $LibPath1 = "";

BEGIN
{
	if ( $^O =~ m/win/i ) {
		require VRTS::Paths;

		my $xprtlInstallDir= VRTS::Paths::get_path("InstallDir");
		my $regquery = "\"$xprtlInstallDir"."\\bin\\regquery.exe\" query SOFTWARE\\VERITAS\\VCS\\Base HomeDirectory";
		my $vcshome = `$regquery`;
		$LibPath1 = "$vcshome\\portal\\admin\\lib\\";
		$ENV{'VCS_HOME'} = $vcshome;

	} else {
		my $APPDISCOVER_HOME = "/opt/VRTSvcs/portal/admin";
	
		$LibPath1 = "$APPDISCOVER_HOME/lib";
	}
}

use lib $LibPath1;
#------------------------
# End of Common Header
#------------------------

use Constants;
use Utils;
use VCS;
use MessageUtils;

use constant MSG_FILE => "clusterop.msg";

my $query = new CGI;
print $query->header("text/xml");

my $xmlstr = "<Result>\n";

#-------------------
# OS Information
#-------------------
my $osObj = Utils::getOSInfo();

$xmlstr .= "\t<OSInfo>\n";
$xmlstr .= "\t\t<ComputerName>"   . $osObj -> { COMPUTERNAME }   . "</ComputerName>\n";
$xmlstr .= "\t\t<OS>"   . $osObj -> { OS }   . "</OS>\n";
$xmlstr .= "\t\t<Arch>" . $osObj -> { ARCH } . "</Arch>\n";
$xmlstr .= "\t\t<Name>" . $osObj -> { OSCAPTION } . "</Name>\n";
$xmlstr .= "\t\t<Date>" . $osObj -> { DATE } . "</Date>\n";
$xmlstr .= "\t\t<VirtualizationPlatform>"    .
		 $osObj -> { VIRTUALIZATION_PLATFORM } . 
		"</VirtualizationPlatform>\n";
$xmlstr .= "\t</OSInfo>\n";

#------------------------
# VCS Cluster Information
#------------------------
my $vcsobj = VCS -> new();
my ($ret, $msg, $clusRef ) = $vcsobj -> getClusterDetails();

if ( $ret != CONST_SUCCESS ) {
	($ret, $msg) = getMessage( MSG_FILE, "VCS_INFO_GET_FAILED", $msg);
	$xmlstr = "";
	goto END;
}

my $VCSclusRef = $clusRef -> { ClusRef };
my $resRef = $clusRef -> { ResRef };
my $grpRef = $clusRef -> { GroupRef };
my $sysRef = $clusRef -> { SysRef };
my $resDepRef = $clusRef -> { ResDepRef };
my $grpDepRef = $clusRef -> { GrpDepRef };


$xmlstr .= "\t<Cluster>\n";
foreach my $attr ( sort keys %$VCSclusRef ) {
	$xmlstr .= "\t\t<$attr>" .
		$VCSclusRef -> { $attr } .  
		"</$attr>\n";
}
$xmlstr .= "\t</Cluster>\n";

$xmlstr .= "\t<Systems>\n";
foreach my $sys ( sort keys %$sysRef ) {

	$xmlstr .= "\t\t<System Name=\"$sys\">\n";
	my $hSys = $sysRef -> { $sys };

	foreach my $attr ( sort keys %{ $hSys } ) {
		$xmlstr .= "\t\t\t<$attr>" .
			$hSys -> { $attr } .
			"</$attr>\n";
	}
	$xmlstr .= "\t\t</System>\n";
}
$xmlstr .= "\t</Systems>\n";

$xmlstr .= "\t<ServiceGroups>\n";
foreach my $grp ( sort keys %$grpRef ) {
	
	$xmlstr .= "\t\t<ServiceGroup Name=\"$grp\">\n";
	my $hSG = $grpRef -> { $grp };

	foreach my $attr ( sort keys %{ $hSG } ) {
		if ( $attr =~ /resources/ ) {
			next;
		}

		$xmlstr .= "\t\t\t<$attr>\n";
		my $sysRef = $hSG -> { $attr };
		foreach my $sys (  keys %{ $hSG -> { $attr } } ) {
			if ( $sys =~ /global/ ) {
				$xmlstr .= "\t\t\t\t<$sys>" . 
					$hSG -> { $attr } { $sys } . 
					"</$sys>\n";
			}
			else {
				$xmlstr .= "\t\t\t\t<local system=\"$sys\">" .
					$hSG -> { $attr } { $sys } .
					"</local>\n";
			}
		}
		$xmlstr .= "\t\t\t</$attr>\n";
	}
	$xmlstr .= "\t\t</ServiceGroup>\n";
}
$xmlstr .= "\t</ServiceGroups>\n";

$xmlstr .= "\t<Resources>\n";
foreach my $res ( sort keys %$resRef ) {

	my $hRes = $resRef -> { $res };
	my $grp = $hRes -> { Group } { global };
	my $type = $hRes -> { Type } { global };
	$xmlstr .= "\t\t<Resource Name=\"$res\" Type=\"$type\" SG=\"$grp\">\n";

	# add ResLabel as a global attribute for resource.
	my $resLabel = VCS::getResourceLabel( $hRes );
	$hRes -> { ResLabel } = {};
	$hRes -> { ResLabel } { global } = $resLabel;

	foreach my $attr ( sort keys %{ $hRes } ) {
		if ( $attr =~ /resourcename/i ) {
			next;
		}

		$xmlstr .= "\t\t\t<$attr>\n";
		my $sysRef = $hRes -> { $attr };
		foreach my $sys ( keys %{ $hRes -> { $attr} } ) {
			if ( $sys =~ /global/ ) {
				$xmlstr .= "\t\t\t\t<$sys>" .
					$hRes -> { $attr } { $sys } .
					"</$sys>\n";
			}
			else {
				$xmlstr .= "\t\t\t\t<local system=\"$sys\">" .
					$hRes -> { $attr } { $sys } .
					"</local>\n";
			}
		}
		$xmlstr .= "\t\t\t</$attr>\n";
	}

	$xmlstr .= "\t\t</Resource>\n";
}
$xmlstr .= "\t</Resources>\n";

$xmlstr .= "\t<ResourceDependencies>\n";
foreach my $resDep ( @$resDepRef ) {

	$xmlstr .= "\t\t<Dependency>\n";
	$xmlstr .= "\t\t\t<Group>" .
		$resDep -> { Group } .
		"</Group>\n";
	$xmlstr .= "\t\t\t<Parent>" .
		$resDep -> { Parent } .
		"</Parent>\n";
	$xmlstr .= "\t\t\t<Child>" .
		$resDep -> { Child } .
		"</Child>\n";
	$xmlstr .= "\t\t</Dependency>\n";
}
$xmlstr .= "\t</ResourceDependencies>\n";

$xmlstr .= "\t<ServiceGroupDependencies>\n";
foreach my $grpDep ( @$grpDepRef ) {
	$xmlstr .= "\t\t<Dependency>\n";
	$xmlstr .= "\t\t\t<Relationship>" .
		$grpDep -> { Type } .
		"</Relationship>\n";
	$xmlstr .= "\t\t\t<Parent>" .
		$grpDep -> { Parent } .
		"</Parent>\n";
	$xmlstr .= "\t\t\t<Child>" .
		$grpDep -> { Child } .
		"</Child>\n";
	$xmlstr .= "\t\t</Dependency>\n";
}
$xmlstr .= "\t</ServiceGroupDependencies>\n";
$xmlstr .= "</Result>\n";

END:

my $sCmdResponse = Utils::getCmdResponseXML(
	"getVCSAppInfo.pl", 
	$ret, 
	$msg,
	$xmlstr);

print $sCmdResponse;
exit (CONST_SUCCESS);

