
#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
# 
# $Id: AppOperations.pm,v 1.18 2012/08/30 13:26:21 sgupta2 Exp $
#

package AppOperations;

use Exporter;
@ISA = ('Exporter');

use strict;
use warnings;

use Data::Dumper;
use Constants;
use Utils;
use VCS;
use MessageUtils;
use SettingManager;

use constant MSG_FILE => "appmon.msg";


sub new
{
	my $class = shift;
	my $self = {}; # allocate new hash for object
	bless($self, $class);
	return $self;
}


#---------------------------
# Subroutine section
#---------------------------

sub waitForSG
{
	my $self = shift;
	my $tree = shift;
	my $state = shift;

	my $sMyName = ( caller 0 )[3];

	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);
	
	my ($ret, $msg) = ();
	$ret = CONST_SUCCESS;
	$msg = "";
	
	my @SGs = ();
	my $ref = $tree->{Group};
	if ("$ref" =~ /ARRAY/) {
		@SGs = @$ref;
	} else {
		push(@SGs, $ref);
	}

	my $settingMgr = new SettingManager();
	
	foreach my $SG ( @SGs ) 
	{
		my $SGName = $SG -> { Name };
		if ( $SGName eq "" ) {
			next;
		}

		my $timeout = $settingMgr -> getSettingValue ( $SGName, "App.StartStopTimeout" );
		Utils::vxlog("INFO", "App.StartStopTimeout=[$timeout]");

		my $anySystem = $SG -> { AnySystem };
		my $system = $SG -> { System };

	        # If case handles when system name is  passed from UI. Else case handle the case when any is selected from UI.		
                if ( $anySystem == 0 and $system and $system ne "" ) {
                        my $cmd = "hagrp -wait $SGName State $state -sys $system -time $timeout";
                        ($ret, $msg) = Utils::runCmd($cmd);
			if ($ret == CONST_FAILURE) {
				Utils::vxlog("DEBUG", "Operation timed out waiting for service group $SGName to come $state.");
				$ret =  CONST_TIMEOUT;
			}
                        goto END;
                }
                else {
			my ($line, @hagrp_state_var, $i);
                        my $href = PlatUtils::getVCSPaths();
                        my $hagrp  = $href -> { hagrp };

			if ($state eq "ONLINE") {
                		for ( $i = 0; $i < $timeout; $i++) {
					$ret = CONST_FAILURE;
                        		@hagrp_state_var = split(/\n/, `$hagrp -state $SGName`);
                        		foreach $line (@hagrp_state_var) {
                                		if (($line =~ m/\|$state\|$/) or ($line =~ m/\|FAULTED\|$/))  {
							Utils::vxlog("DEBUG", "Final state retrived ,returning success.");
                                        		$ret =  CONST_SUCCESS;
                                        		last;
                                		}
                        		}
                        		last if ( CONST_SUCCESS == $ret);
                        		sleep(1);

                		}
				if ($i == $timeout) {
					Utils::vxlog("DEBUG", "Operation timed out waiting for service group $SGName to come $state.");
					$ret =  CONST_TIMEOUT;
				}
			}
			else {
                                for ($i = 0; $i < $timeout; $i++) {
					$ret =  CONST_SUCCESS;
                                        @hagrp_state_var = split(/\n/, `$hagrp -state $SGName`);
                                        foreach $line (@hagrp_state_var) {
						if (($line !~ /^\s*#/)) {
                                               		if (($line =~ /\|ONLINE\|/) or ($line =~ /\|PARTIAL\|/))  {
									$ret = CONST_FAILURE;
                                                        		last;
                                                	}
						}
                                        }
                                        last if ( CONST_SUCCESS == $ret);
                                        sleep(1);
                                }
				if ($i == $timeout) {
					Utils::vxlog("DEBUG", "Operation timed out waiting for service group $SGName to come $state.");
					$ret = CONST_TIMEOUT;
					
				}	
				
			}		

                }
	}

END:
	return ($ret, $msg); 
}


sub checkSGParams
{
	my $self		= shift;
	my $tree		= shift;
	my $StartStopChecks 	= shift;
	my $CheckAppHASG	= shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	$logStr = Dumper($tree);
	Utils::vxlog("INFO", $logStr);

	my ($ret, $msg) = ();
	$ret =	CONST_SUCCESS;
	$msg = "";

	my @SGs = ();
	if (not defined $tree or $tree eq "" ) {
		($ret, $msg) = getMessage ( MSG_FILE, "INVALID_PARAMS_XML_SCHEMA");
		goto EXIT_END;
	}

	my $ref = $tree->{Group};
	if (not defined $ref) {
		($ret, $msg) = getMessage ( MSG_FILE, "PARAMS_XML_TAG_MISSING", "Group");
		goto EXIT_END;
	}

	if ("$ref" =~ /ARRAY/) {
		@SGs = @$ref;
	} else {
		push(@SGs, $ref);
	}

	# check if passed service group names are under VCS.
	my $vcsobj = new VCS();

	my $clusRef	= $vcsobj -> getClusterDetails();
	my $VCSclusRef	= $clusRef -> { ClusRef };
	my $resRef	= $clusRef -> { ResRef };
	my $grpRef	= $clusRef -> { GroupRef };
	my $resDepRef	= $clusRef -> { ResDepRef };
	my $grpDepRef	= $clusRef -> { GrpDepRef };

	my $SGDetails = ();
	
	foreach my $SG ( @SGs ) {
		my $SGName = $SG -> { Name };
		if ( $SGName eq "" ) {
			($ret, $msg) = getMessage (MSG_FILE, "PARAMS_TAG_VALUE_EMPTY", "Name");
			goto EXIT_END;
		}

		if ( $SGName =~ /\s+/ ) {
			($ret, $msg) = getMessage (MSG_FILE, "PARAMS_TAG_VALUE_HAS_SPACES", "Name");
			goto EXIT_END;
		}

		Utils::vxlog("INFO", "Setting up SGDetails");

		# check if group has any resources configured ..
		$ref = $grpRef -> { $SGName } { resources };
		my $Frozen	= $grpRef -> { $SGName } { Frozen } { global };

		Utils::vxlog("INFO", "SGName=[$SGName], Frozen=[$Frozen]");

		my $ResCount	= 0;
		my @resList		= ();

		my $EnabledRes	= 0;
		my @EnabledResList = ();

		my $UnknownRes	= 0;
		my @UnknownResList = ();

		$SGDetails -> { $SGName } = ();
		$SGDetails -> { $SGName } { Details } = ();
		$SGDetails -> { $SGName } { Details } { Frozen }  = $Frozen;

		#TODO: Review whether the following 3 lines require any change...commenting for now
		#TODO: And correspondingly whether AT, Tomcat and xprtld need special handling here
	#	$SGDetails -> { $SGName } { AppHA } { VxATService   } = 0;
	#	$SGDetails -> { $SGName } { AppHA } { TomcatService } = 0;
	#	$SGDetails -> { $SGName } { AppHA } { xprtldService } = 0;

		foreach my $res ( @$ref ) {

			my $Enabled = $resRef -> { $res } { Enabled } { global };
			my $hResState   = $resRef -> { $res } { State };

			$ResCount += 1;
			push( @resList, $res );

			if ( $Enabled == 1 ) {
				$EnabledRes += 1;
				push( @EnabledResList, $res );
			}

			foreach my $sys ( keys %$hResState ) {
				my $State = $resRef -> { $res } { State } { $sys };
				if ( $State =~ /UNKNOWN/i ) {
					$UnknownRes += 1;;
					push ( @UnknownResList, $res );
				}
	
				Utils::vxlog("INFO", "SGName=[$SGName], Res=[$res], Enabled=[$Enabled], State=[$State], System=[$sys]");
			}

		#	my $ServiceName = $resRef -> { $res } { ServiceName } || "";
		#	Utils::vxlog("INFO", "SGName=[$SGName], Res=[$res], ServiceName=[$ServiceName]");

		#	if ( $ServiceName eq "Symantec ApplicationHA Authentication Service" ) {
		#		$SGDetails -> { $SGName } { AppHA } { VxATService } = 1;
		#	} elsif ( $ServiceName eq "Symantec ApplicationHA Service" ) {
		#		$SGDetails -> { $SGName } { AppHA } { TomcatService } = 1;
		#	} elsif ( $ServiceName eq  "Veritas Storage Foundation Messaging Service" ) {
		#		$SGDetails -> { $SGName } { AppHA } { xprtldService } = 1;
		#	}
		}

		$SGDetails -> { $SGName } { Res } { List } = \@resList;
		$SGDetails -> { $SGName } { Res } { Cnt }  = $ResCount;

		$SGDetails -> { $SGName } { Enabled } { List } = \@EnabledResList;
		$SGDetails -> { $SGName } { Enabled } { Cnt }  = $EnabledRes;

		$SGDetails -> { $SGName } { Unknown } { List } = \@UnknownResList;
		$SGDetails -> { $SGName } { Unknown } { Cnt }  = $UnknownRes;

		Utils::vxlog("INFO", "SGName=[$SGName], Rescount=[$ResCount], ResList=[" . join(", ", @resList)	. "]");
		Utils::vxlog("INFO", "SGName=[$SGName], Rescount=[$ResCount], EnabledList=[" . join(", ", @EnabledResList) . "]");
		Utils::vxlog("INFO", "SGName=[$SGName], Rescount=[$ResCount], UnknownList=[" . join(", ", @UnknownResList) . "]");
#		Utils::vxlog("INFO", "SGName=[$SGName], AppHASecD=[" . $SGDetails -> { $SGName } { AppHA } { VxATService } . "], " . 
#		                             "TomcatService=[" . $SGDetails -> { $SGName } { AppHA } { TomcatService } . "], " .
#		                             "xprtld=["        . $SGDetails -> { $SGName } { AppHA } { xprtldService } . "]");
	}

	Utils::vxlog("INFO", Dumper( $SGDetails ));
	Utils::vxlog("INFO", "StartStopChecks=[" . ($StartStopChecks || '') . "]");

	# now check SGDetails ...
	if ( not defined $StartStopChecks or $StartStopChecks != 1 ) {
		goto EXIT_END;
	}
 
	Utils::vxlog("INFO", "Checking details ...");
	foreach my $SGName ( keys %$SGDetails ) {
		my $AppName  = $SGDetails -> { $SGName } { Details } { AppName };
		my $SGFrozen = $SGDetails -> { $SGName } { Details } { Frozen };

		my $ResCount = $SGDetails -> { $SGName } { Res } { Cnt };
		my $Unknown  = $SGDetails -> { $SGName } { Unknown } { Cnt };
		my $Enabled  = $SGDetails -> { $SGName } { Enabled } { Cnt };

	#	if ( defined $CheckAppHASG and $CheckAppHASG != 0 ) {
	#		Utils::vxlog("INFO", "Checking if SG is AppHA SG...");

	#		my $VxATService   = $SGDetails -> { $SGName } { AppHA } { VxATService };
	#		my $TomcatService = $SGDetails -> { $SGName } { AppHA } { TomcatService };
	#		my $xprtldService = $SGDetails -> { $SGName } { AppHA } { xprtldService };

	#		Utils::vxlog("INFO", "SGName=[$SGName], VxATService=[$VxATService], TomcatService=[$TomcatService], xprtldService=[$xprtldService]");

	#		if ( $VxATService == 1 and
	#		     $TomcatService == 1 and
	#		     $xprtldService == 1) {
	#			$ret = APPCONTROL_APPLICATION_STARTSTOP_APPHACONSOLSERVER;
	#			$msg = "This operation is prohibited.\nIf ApplicationHA Console services are stopped, the ApplicationHA tab does not display the status of the configured applications.";
	#			goto EXIT_END;
	#		}
	#	}

		Utils::vxlog("INFO", "SGName=[$SGName], Rescount=[$ResCount]");
		if ( $ResCount == 0 ) {
			($ret, $msg) = getMessage (MSG_FILE, "NO_RES_IN_GROUP");
			goto EXIT_END;
		}

		Utils::vxlog("INFO", "SGName=[$SGName], Frozen=[$SGFrozen]");
		if ( $SGFrozen ) {
			($ret, $msg) = getMessage ( MSG_FILE, "GROUP_FROZEN");
			goto EXIT_END;
		}

		Utils::vxlog("INFO", "SGName=[$SGName], Enabled=[$Enabled]");
		if ( $ResCount != $Enabled ) {
			($ret, $msg) = getMessage (MSG_FILE, "GROUP_HAS_DISABLED_RES");
			goto EXIT_END;
		}

		Utils::vxlog("INFO", "SGName=[$SGName], Unknown=[$Unknown]");
		if ( $Unknown != 0 ) {
			($ret, $msg) = getMessage (MSG_FILE, "GROUP_HAS_UNKNOWN_RES");
			goto EXIT_END;
		}
	}

EXIT_END:
	return ($ret, $msg);
}


sub operateSG
{
	my $self = shift;
	my $tree = shift;
	my $op 	 = shift;
	my $skipSysParam = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $vcsobj = new VCS();

	my ($ret, $msg) = ();
	$ret =	CONST_SUCCESS;
	$msg = "";

	my @SGs = ();
	my $ref = $tree->{Group};
	if ("$ref" =~ /ARRAY/) {
		@SGs = @$ref;
	} else {
		push(@SGs, $ref);
	}

	$vcsobj -> makeConfigWritable();

	# control is here, means passed names are valid
	foreach my $SG ( @SGs ) {
		my $sgName = $SG -> { Name };
		my $propagate = $SG -> { Propagate } || 0;
		my $persistent = $SG -> { Persistent } || 0;
		my $force = $SG -> { Force } || 0;
		my $anySystem = $SG -> { AnySystem };
		my $system = $SG -> { System };
		my $sOptions = "";
		$sOptions = " -propagate" if ( $propagate == 1);
		$sOptions = " -persistent" if ( $persistent == 1);
		$sOptions = " -force" if ( $force == 1);

		Utils::vxlog("INFO", "SGName = [" . $sgName . "], Any = [" . $anySystem . 
			"], SysName = [" . $system . "], options = [" . $sOptions . "]");

		($ret, $msg) = $vcsobj -> operateSG( $sgName, $op, $anySystem, $sOptions, $system );
		if ( $ret != CONST_SUCCESS ) {
			goto EXIT_END;
		}
	}

EXIT_END:
	$vcsobj -> dumpAndMakeConfigReadOnly();
	return ($ret, $msg);
}


sub probeAppSG
{
	my $self = shift;
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my ($ret, $msg, $dbg) = ();
	$ret =	CONST_SUCCESS;
	$msg = "";
	$dbg = "";

	my $vcsobj = new VCS();
	my $tree = Utils::convertXMLTextToHash($params);

#	Utils::vxlog("INFO", "Validating SG names ...");
#	($ret, $msg) = $self->checkSGParams($tree);
#	if ( $ret != CONST_SUCCESS ) {
#		goto END;
#	}

	my $ref = $tree->{Group};
	my @SGs = Utils::convertHashValuesToArray($ref);

	foreach my $sg ( @SGs ) {
		my $group = $sg -> { Name };
		my $sysRef = $sg -> { System };
		my @systemList = Utils::convertHashValuesToArray($sysRef);

		foreach my $sys ( @systemList ) {
			($ret, $msg, $dbg) = $vcsobj -> probeAllGroupResources ( $group, $sys );
			if ( $ret != CONST_SUCCESS ) {
				$msg .= "VCS failed to probe resources in group [$group] on $sys - $dbg";
			}
		}
	}

END:
	return ($ret, $msg); 
}


#------------------------
# Operations ...
#------------------------

sub onlineAppSG
{
	my $self = shift;
	my $params = shift;

	my $sMyName = ( caller 0 )[3];

	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);
	
	my ($ret, $msg) = ();
	$ret = CONST_SUCCESS;
	$msg = "";

	my $tree = Utils::convertXMLTextToHash($params);

	Utils::vxlog("INFO", "Validating SG names ...");
	($ret, $msg) = $self->checkSGParams($tree, 1);
	if ( $ret != CONST_SUCCESS ) {
		goto END;
	}

	Utils::vxlog("INFO", "Calling start on service groups");
	($ret, $msg) = $self->operateSG($tree, "online", 0);
	if ( $ret !=  CONST_SUCCESS ) {
		$msg = "VCS failed to start the configured application - [$msg]";
		goto END;
	}

	($ret, $msg) = $self->waitForSG($tree, "ONLINE");
	if ( $ret !=  CONST_SUCCESS ) {
		$msg = "The application has not started completely. One or more components may take more time.
VCS will continue to start the application components in the background.
Wait for some time, and then click Refresh to verify if the application has started.";
	}

END:
	return ($ret, $msg); 
}


sub offlineAppSG
{
	my $self = shift;
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my ($ret, $msg) = ();
	$ret =	CONST_SUCCESS;
	$msg = "";

	my $tree = Utils::convertXMLTextToHash($params);

	Utils::vxlog("INFO", "Validating SG names ...");
	($ret, $msg) = $self->checkSGParams($tree, 1);
	if ( $ret != CONST_SUCCESS ) {
		goto END;
	}

	Utils::vxlog("INFO", "Calling stop on service groups");
	($ret, $msg) = $self->operateSG($tree, "offline", 0);
	if ( $ret != CONST_SUCCESS ) {
		$msg = "VCS failed to stop the configured application - [$msg]";
		goto END;
	}

	($ret, $msg) = $self->waitForSG($tree, "OFFLINE");
	if ( $ret !=  CONST_SUCCESS ) {
		$msg = "The application has not stopped completely. One or more components may take more time.
VCS will continue to stop the application components in the background.
Wait for some time, and then click Refresh to verify if the application has stopped.";
	}

END:
	return ($ret, $msg); 
}


sub switchAppSG
{
	my $self = shift;
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my ($ret, $msg) = ();
	$ret =	CONST_SUCCESS;
	$msg = "";

	my $tree = Utils::convertXMLTextToHash($params);

	Utils::vxlog("INFO", "Validating SG names ...");
	($ret, $msg) = $self->checkSGParams($tree, 1);
	if ( $ret != CONST_SUCCESS ) {
		goto END;
	}

	Utils::vxlog("INFO", "Calling switch of service group");
	($ret, $msg) = $self->operateSG($tree, "switch", 0);
	if ( $ret != CONST_SUCCESS ) {
		$msg = "VCS failed to switch the configured application - [$msg]";
		goto END;
	}

	($ret, $msg) = $self->waitForSG($tree, "ONLINE");
	if ( $ret !=  CONST_SUCCESS ) {
		$msg = "The application has not switched completely. One or more components may take more time.
VCS will continue to switch the application components in the background.
Wait for some time, and then click Refresh to verify if the application has switched.";
	}

END:
	return ($ret, $msg); 
}


sub clearAppSG
{
	my $self = shift;
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my ($ret, $msg) = ();
	$ret =	CONST_SUCCESS;
	$msg = "";

	my $tree = Utils::convertXMLTextToHash($params);

	Utils::vxlog("INFO", "Validating SG names ...");
	($ret, $msg) = $self->checkSGParams($tree);
	if ( $ret != CONST_SUCCESS ) {
		goto END;
	}

	Utils::vxlog("INFO", "Calling clear of service group");
	($ret, $msg) = $self->operateSG($tree, "clear", 0);
	if ( $ret != CONST_SUCCESS ) {
		$msg = "VCS failed to clear the fault state of configured application - [$msg]";
		goto END;
	}

END:
	return ($ret, $msg); 
}


sub flushAppSG
{
	my $self = shift;
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my ($ret, $msg) = ();
	$ret =	CONST_SUCCESS;
	$msg = "";

	my $tree = Utils::convertXMLTextToHash($params);

	Utils::vxlog("INFO", "Validating SG names ...");
	($ret, $msg) = $self->checkSGParams($tree);
	if ( $ret != CONST_SUCCESS ) {
		goto END;
	}

	Utils::vxlog("INFO", "Calling flush of service group");
	($ret, $msg) = $self->operateSG($tree, "flush", 0);
	if ( $ret != CONST_SUCCESS ) {
		$msg = "VCS failed to resolve the held-up operation - [$msg]";
		goto END;
	}

END:
	return ($ret, $msg); 
}


sub freezeAppSG
{
	my $self = shift;
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my ($ret, $msg) = ();
	$ret =	CONST_SUCCESS;
	$msg = "";

	my $tree = Utils::convertXMLTextToHash($params);

	Utils::vxlog("INFO", "Validating SG names ...");
	($ret, $msg) = $self->checkSGParams($tree);
	if ( $ret != CONST_SUCCESS ) {
		goto END;
	}

	my $skipSysParam = 0;
	Utils::vxlog("INFO", "Calling freeze for service groups");
	($ret, $msg) = $self->operateSG($tree, "freeze", $skipSysParam);
	if ( $ret != CONST_SUCCESS ) {
		$msg = "VCS failed to move configured application to maintenance mode - [$msg]";
		Utils::vxlog("INFO", "$msg, returning from freeze operation.");
		goto END;
	}

	# sleep for 5 sec ...
	# otherwise, if user sends unfreeze cmd very soon, it may result into failure operatoon.
	sleep 5;

END:
	return ($ret, $msg);
}


sub unFreezeAppSG
{
	my $self = shift;
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my ($ret, $msg) = ();
	$ret =	CONST_SUCCESS;
	$msg = "";

	my $tree = Utils::convertXMLTextToHash($params);

	Utils::vxlog("INFO", "Validating SG names ...");
	($ret, $msg) = $self->checkSGParams($tree);
	if ( $ret != CONST_SUCCESS ) {
		goto END;
	}



#	Utils::vxlog("INFO", "Calling proberes for service groups");
#	($ret, $msg) = $self->probeSGRes($params);
#	if ( $ret != CONST_SUCCESS ) {
#		$msg = "ApplicationHA failed to move configured application to monitoring mode. Retry the operation.";
#		Utils::vxlog("INFO", "$msg, returning from proberes operation.");
#		goto END;
#	}

	my $skipSysParam = 0;
	Utils::vxlog("INFO", "Calling un-freeze for service groups");
	($ret, $msg) = $self->operateSG($tree, "unfreeze", $skipSysParam);
	if ( $ret != CONST_SUCCESS ) {
		$msg = "VCS failed to resume application monitoring - [$msg]";
		Utils::vxlog("INFO", "$msg, returning from unfreeze operation.");
	}

END:
	return ($ret, $msg);
}


sub deleteSystemAppSG
{
	my $self = shift;
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my ($ret, $msg, $dbg) = ();
	$ret =	CONST_SUCCESS;
	$msg = "";
	$dbg = "";

	my $vcsobj = new VCS();
	my $tree = Utils::convertXMLTextToHash($params);

	my $rmSysRef = {};
	$rmSysRef -> { sys } = "global";
	$rmSysRef -> { attr } = "SystemList";
	$vcsobj -> makeConfigWritable();

	my $ref = $tree->{Group};
	my @SGs = Utils::convertHashValuesToArray($ref);

	foreach my $sg ( @SGs ) {
		my $group = $sg -> { Name };
		my $sysRef = $sg -> { System };
		my @systemList = Utils::convertHashValuesToArray($sysRef);
		$rmSysRef -> { SG } = $group;

		foreach my $sys ( @systemList ) {
			my @delArgs = ( "-delete", $sys );
			$rmSysRef -> { value } = \@delArgs;

			($ret, $msg, $dbg) = $vcsobj -> modifyServiceGroup ( $rmSysRef );
			if ( $ret != CONST_SUCCESS ) {
				$msg = "VCS failed to delete system [$sys] from SystemList of group [$group] - $dbg";
				goto END;
			}
		}
	}

END:
	$vcsobj -> dumpAndMakeConfigReadOnly();
	return ($ret, $msg); 
}


sub unconfigureAppSG
{
	my $self = shift;
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my ($ret, $msg, $dbg) = ();
	$ret =	CONST_SUCCESS;
	$msg = "";
	$dbg = "";

	my $vcsobj = new VCS();
	my $tree = Utils::convertXMLTextToHash($params);

	if ( ( not defined $tree ) or ( $tree eq "" ) ) {
		Utils::vxlog("DEBUG", "No group sent for deletion");
		goto END;
	}

	my $ref = $tree->{Group};
	my @SGs = (Utils::convertHashValuesToArray($ref));
	my @sgNames = ();
	my $clusRef = {};

	foreach my $SG ( @SGs ) {
		my $sgName = $SG -> { Name };
		if ( ( not defined $sgName ) or ( $sgName eq "" ) ) {
			Utils::vxlog("ERROR", "Group name is not present for deletion");
			goto END;
		}

		push ( @sgNames, $SG -> { Name } );
	}

	my $clusDiscoveryInputRef = ();

	$clusDiscoveryInputRef -> { ServiceGroupDep }{ Groups } = \@sgNames;

	($ret, $msg, $clusRef) = $vcsobj -> getClusterDetails( $clusDiscoveryInputRef );
	if ( $ret != CONST_SUCCESS ) {
		$msg = "Error occurred while fetching group dependencies - [$msg]";
		Utils::vxlog("ERROR", "$msg, returning from unconfigure operation.");
		goto END;
	}

	my $grpDepRef = $clusRef -> { GrpDepRef };

	my $xmlOfGrpDeps = "<xml>\n";
	foreach my $grpDep ( @$grpDepRef ) {
		if ( $xmlOfGrpDeps !~ /<Parent>$grpDep->{Parent}<\/Parent>/ or 
				$xmlOfGrpDeps !~ /<Child>$grpDep->{Child}<\/Child>/ ) {
			$xmlOfGrpDeps .= "<GroupDependency>" .
				"<Parent>" . $grpDep -> { Parent } . "</Parent>" .
				"<Child>" . $grpDep -> { Child } . "</Child>" .
				"</GroupDependency>\n";
		}
	}
	$xmlOfGrpDeps .= "</xml>";

	Utils::vxlog("INFO", "$xmlOfGrpDeps");
	($ret, $msg) = $self -> deleteGrpDependencies ( $xmlOfGrpDeps );
	if ( $ret != CONST_SUCCESS ) {
		$msg = "Error occurred while deleting group dependencies - [$msg]";
		Utils::vxlog("INFO", "$msg, returning from unconfigure operation.");
		goto END;
	}

	$vcsobj -> makeConfigWritable();

	foreach my $SG ( @SGs ) {
		my $sgName = $SG -> { Name };

		Utils::vxlog("DEBUG", "Unconfiguring SG [$sgName]");

		($ret, $msg, $dbg) = $vcsobj -> deleteAllGroupResources( $sgName );
		if ( $ret != CONST_SUCCESS ) {
			$msg = "Failed to delete resources from service group [$sgName - $dbg].";
			Utils::vxlog("INFO", "$msg, returning from unconfigure operation.");
			goto END;
		}

		($ret, $msg, $dbg) = $vcsobj -> deleteServiceGroup ( $sgName );
		if ( $ret != CONST_SUCCESS ) {
			$msg = "Failed to delete service group [$sgName - $dbg].";
			Utils::vxlog("INFO", "$msg, returning from unconfigure operation.");
			goto END;
		}
	}

	VCS::sendFullSnap();

END:
	$vcsobj -> dumpAndMakeConfigReadOnly();
	return ($ret, $msg);
}


sub deleteGrpDependencies
{
	my $self = shift;
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my ($ret, $msg, $dbg) = ();
	$ret =	CONST_SUCCESS;
	$msg = "";
	$dbg = "";

	my $vcsobj = new VCS();
	my $tree = Utils::convertXMLTextToHash($params);

	if ( ( not defined $tree ) or ( $tree eq "" ) ) {
		Utils::vxlog("DEBUG", "No group dependency sent for deletion");
		goto END;
	}

	my $ref = $tree -> { GroupDependency };
	my @sgDeps = (Utils::convertHashValuesToArray($ref));

	$vcsobj -> makeConfigWritable();

	Utils::vxlog("INFO", "Deleting SG dependencies ...");
	foreach my $dependency ( @sgDeps ) {
		my $parent = $dependency -> { Parent };
		my $child  = $dependency -> { Child };

		Utils::vxlog("DEBUG", "[Parent:$parent, Child:$child]");

		($ret, $dbg) = $vcsobj -> deleteGrpDependency ( $parent, $child );
		if ( $ret != CONST_SUCCESS ) {
			$msg = "Failed to delete SG dependency [Parent:$parent, Child:$child] - [$dbg]";
			Utils::vxlog("INFO", "$msg");
			goto END;
		}
	}

END:
	$vcsobj -> dumpAndMakeConfigReadOnly();
	return ($ret, $msg);
}


sub unconfigureAllSG
{
	my $self = shift;
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my ($ret, $msg, $dbg) = ();
	$ret =	CONST_SUCCESS;
	$msg = "";
	$dbg = "";
	my $ref = {};
	my $clusDiscoveryInputRef = ();

	$clusDiscoveryInputRef -> { ServiceGroup } = { };

	my $vcsobj = new VCS();
	($ret, $msg, $ref) = $vcsobj -> getClusterDetails( $clusDiscoveryInputRef );
	if ( $ret != CONST_SUCCESS ) {
		Utils::vxlog("INFO", "$msg, returning from unconfigure operation.");
		goto END;
	}
	my $grpRef = $ref -> { GroupRef };

	my $xmlOfGroups = "<xml>\n";
	foreach my $grp ( keys %$grpRef ) {
		$xmlOfGroups .= "<Group><Name>$grp</Name></Group>\n";
	}
	$xmlOfGroups .= "</xml>";

	Utils::vxlog("INFO", "$xmlOfGroups");
	($ret, $msg) = $self -> unconfigureAppSG ($xmlOfGroups);
	if ( $ret != CONST_SUCCESS ) {
		Utils::vxlog("INFO", "$msg, returning from unconfigure operation.");
		goto END;
	}

END:
	return ($ret, $msg);
}

#
#sub freezeAllAppSG
#{
#	my $self = shift;
#	my $params = shift;
#
#	my $sMyName = ( caller 0 )[3];
#	#---------
#	# Announce
#	#---------
#	my $logStr = "Subroutine $sMyName called";
#	Utils::vxlog("INFO", $logStr);
#
#	my ($ret, $msg) = ();
#	$ret =	CONST_SUCCESS;
#	$msg = "";
#
#	my $iRet = ();
#	my $vcsobj = VCS->new();
#	my $sNamesStr = "";
#	my @Names = ();
#	
#	($iRet, $sNamesStr) = $vcsobj -> getResAttrValue( $sVCSAppMonHBRes, "ServiceGroupName" );
#	$sNamesStr = "" if (not defined $sNamesStr);
#	$sNamesStr =~ s/\s+$//;
#	$sNamesStr =~ s/^\s+//;
#	if ( $iRet != CONST_SUCCESS or $sNamesStr eq "" ) {
#		goto END;
#	}
#
#	@Names = split(/\s+/, $sNamesStr);
#	my $args = ();
#	my $xmlstr = "<ServiceGroup>";
#	foreach my $Name ( @Names ) {
#		$xmlstr .= "<Name>$Name</Name>";
#	}
#	$xmlstr .= "</ServiceGroup>";
#
#	$args -> { params } = $xmlstr;
#	($ret, $msg) = $self -> freezeAppSG( $args );
#
#END:
#	return ($ret, $msg);
#}
#
#
#sub unFreezeAllAppSG
#{
#	my $self = shift;
#	my $params = shift;
#
#	my $sMyName = ( caller 0 )[3];
#	#---------
#	# Announce
#	#---------
#	my $logStr = "Subroutine $sMyName called";
#	Utils::vxlog("INFO", $logStr);
#
#	my ($ret, $msg) = ();
#	$ret =	CONST_SUCCESS;
#	$msg = "";
#
#	my $iRet = ();
#	my $vcsobj = VCS->new();
#	my $sNamesStr = "";
#	my @Names = ();
#	
#	($iRet, $sNamesStr) = $vcsobj -> getResAttrValue( $sVCSAppMonHBRes, "ServiceGroupName" );
#	$sNamesStr = "" if (not defined $sNamesStr);
#	$sNamesStr =~ s/\s+$//;
#	$sNamesStr =~ s/^\s+//;
#	if ( $iRet != CONST_SUCCESS or $sNamesStr eq "" ) {
#		goto END;
#	}
#
#	@Names = split(/\s+/, $sNamesStr);
#	my $args = ();
#	my $xmlstr = "<ServiceGroup>";
#	foreach my $Name ( @Names ) {
#		$xmlstr .= "<Name>$Name</Name>";
#	}
#	$xmlstr .= "</ServiceGroup>";
#
#	$args -> { params } = $xmlstr;
#	($ret, $msg) = $self -> unFreezeAppSG( $args );
#
#END:
#	return ($ret, $msg);
#}
#

1; # so that perl doesn't complain!




