
#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Id: Application.pm,v 1.37 2012/09/13 14:05:53 sgupta2 Exp $
#

package Application;

use strict;
use warnings;

use Data::Dumper;

use Constants;
use Utils;
use PlatUtils;
use VCS;



my $lockFile = PlatUtils::getLockDir() . "HVLock.l"; 

my $uniqueSGList = {};
my $uniqueResourceList = {};


#-----------------
# Constructor
#-----------------

sub new
{
	my $class = shift;

	my $href = {};
	bless $href, $class;

	return $href;
}


#-------------------------------------------------------------------------
# Interfaces: Each derived application need to implemet these interfaces
# available interfaces for override purpose are:
#     discover, shallowDiscover, validate, findApplicationMounts, discoverExistingConfig
#     createCommitSGDetails, performPreConfigSteps, performPostConfigSteps
#     createUpdateAppDetails
#-------------------------------------------------------------------------
# Interfaces: Following interfaces need to be implemented for each platform:
#	createUpdateResDetails
#-------------------------------------------------------------------------


sub discover
{
	my $self = shift;
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $xml = "";

	return ($ret, $msg, $xml);
}


sub shallowDiscover
{
	my $self = shift;
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $xml = "";

	return ($ret, $msg, $xml);
}


sub validate
{
	my $self = shift;
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $xml = "";

	return ($ret, $msg, $xml);
}

sub findApplicationMounts
{
	my $self = shift;
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $xml = "";

	return ($ret, $msg, $xml);
}


sub discoverExistingConfig
{
	my $self = shift;
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $xml = "";

	return ($ret, $msg, $xml);
}


sub createCommitSGDetails
{
	my $self = shift;
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	#
	# Note: The return of this function is designed to be different
	# Reason: commit input details may change from application to application
	#  pre config and post config steps of commit may need all generated details
	#  Return hash need to have at-least 3 fields: ret, msg, xml
	#  Each application can add required key-value pairs in the hash
	#
	my $retRef = ();

	$retRef -> { ret } = CONST_SUCCESS;
	$retRef -> { msg } = "";
	$retRef -> { xml } = "";

	return $retRef;
}


sub performPreConfigSteps
{
	my $self = shift;
	my $params = shift;
	my $ref = shift;  

	#
	# Note: The second param ref is generated in createCommitSGDetails function.
	#

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $xml = "";

	return ($ret, $msg, $xml);
}


sub performPostConfigSteps
{
	my $self = shift;
	my $params = shift;
	my $ref = shift;  

	#
	# Note: The param '$ref' is the returned ref from createCommitSGDetails function.
	#

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $xml = "";

	return ($ret, $msg, $xml);
}


sub performPreUpdateSteps
{
	my $self = shift;
	my $params = shift;
	my $ref = shift;  

	#
	# Note: The second param ref is generated in createUpdateSGDetails function.
	#

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $xml = "";

	return ($ret, $msg, $xml);
}


sub performPostUpdateSteps
{
	my $self = shift;
	my $params = shift;
	my $ref = shift;  

	#
	# Note: The param '$ref' is the returned ref from createUpdateSGDetails function.
	#

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $xml = "";
	my $dbg = "";

	my $vcsobj = new VCS();

	# As of now VCSInfraSG is hard coded while creating also. So keeping it same here.
	my $sgName = "VCSInfraSG";

	my $arInfraSGSystemList = $ref -> { infraSysList };
	if ( scalar ( @$arInfraSGSystemList ) >= 1 ) {
		my $vcsobj = new VCS();
		my @aAutoStartList = ( "-add", @$arInfraSGSystemList );
		my $href = ();

		$href -> { SG    } = $sgName;
		$href -> { sys   } = CONST_VCS_GLOBAL;
		$href -> { attr  } = "AutoStartList";
		$href -> { value } = \@aAutoStartList;

		$vcsobj -> makeConfigWritable();
		($ret, $msg, $dbg) = $vcsobj -> modifyServiceGroup ( $href );
		$vcsobj -> dumpAndMakeConfigReadOnly();

		if ( $ret != CONST_SUCCESS ) {
			$msg = "Unable to add system to AutoStartList for group [$sgName] - [$dbg]";
			# goto END;
		}

		# Notify Sink resource should be probed before online is fired. Hence wait for some time.
		sleep 5;
	
		Utils::vxlog("INFO", "Onlining SG [$sgName]");
	
		($ret, $msg) = $vcsobj -> operateSG( $sgName, "online", 1);
		if ( $ret != CONST_SUCCESS ) {
			$msg = "Error occurred while onlining Infrastructure SG - [$msg]";
			goto END;
		}

	}

END:
	return ($ret, $msg, $xml);
}


sub createUpdateResDetails
{
	my $self = shift;
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $xml = "";

	return ($ret, $msg, $xml);
}


sub createSystemNICSGDetails
{
	my $self = shift;
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $xml = "";

	return ($ret, $msg, $xml);
}


sub createUpdateSGDetails
{
	my $self = shift;
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	#
	# Note: The return of this function is designed to be different
	# Reason: Update input details may change from application to application
	#  Return hash need to have at-least 3 fields: ret, msg, xml
	#  Each application can add required key-value pairs in the hash
	#
	my $retRef = ();
	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $xml = "";

	Utils::vxlog("DEBUG", "Input XML [$params]");

	my $tree = Utils::convertXMLTextToHash( $params );
	my $href = ();

	# Assumption: Only 1 SG would be updated through a single invokation
	my $currGroup = "";
	my $addSys = 0;
	my @lSysToBeAdded = ();

	if ( defined $tree -> { ServiceGroups } and
		defined $tree -> { ServiceGroups } { ServiceGroup } and 
		$tree -> { ServiceGroups } { ServiceGroup } ne "" ) {

		my $grpRef = $tree -> { ServiceGroups } { ServiceGroup };
		if ( ref( $grpRef ) =~ /ARRAY/ ) {
			foreach my $grp ( @$grpRef ) {
				$href -> { ServiceGroup } { $grp -> { Name } } = $grp;
			}
		}
		else {
			# TODO: Find related SGs based on Name of App
			# TODO: Also find related SGs based on group dependency

			$href -> { ServiceGroup } { $grpRef -> { Name } } = $grpRef;
			if ( $currGroup eq "" ) {
				$currGroup = $grpRef -> { Name };
			}
		}
	}

	if ( defined $tree -> { SystemList } and
		defined $tree -> { SystemList } { System } and
		$tree -> { SystemList } { System } ne "" ) {

		my @sysList = Utils::convertHashValuesToArray($tree -> { SystemList } { System });
		foreach my $sys ( @sysList ) {
			$href -> { SystemList } { $sys -> { Name } } = $sys;

			if ( $sys -> { SGConfigured } =~ /false/i ) {
				$addSys = 1;
				push (@lSysToBeAdded, $sys -> { Name });
			}
		}
	}
	Utils::vxlog("DEBUG", "Systems to be added [ @lSysToBeAdded ]");

	# TODO: In future when attribute localization would be supported, app-specific code needs to be called here.
	# $self -> createUpdateAppDetails ( $tree );

	# TBD: What to do if there are multiple NICs mentioned?
	my ($resRet, $resMsg, $resXml) = $self -> createUpdateResDetails ( $tree, $currGroup );
	Utils::vxlog("DEBUG", "XML received for updating res details [$resXml]");

	if ( $resRet == CONST_SUCCESS ) {
		$href -> { ServiceGroup } { $currGroup } { Resources } = $resXml;
	}
	else {
		Utils::vxlog("WARNING", "There was an error while processing VIP info - [$resMsg]");

		# TODO: Ignoring errors as of now. May have to handle that
		$href -> { ServiceGroup } { $currGroup } { Resources } = "";
	}

	($resRet, $resMsg, $resXml) = $self -> createUpdateVMwareDiskResDetails ( $tree );
	Utils::vxlog("DEBUG", "XML received for updating vmwaredisk res details [$resXml]");

	if ( $resRet == CONST_SUCCESS ) {
		$href -> { ServiceGroup } { $currGroup } { Resources } .= $resXml;
	}
	else {
		Utils::vxlog("WARNING", "There was an error while processing VMwareDisks resource info - [$resMsg]");

		# TODO: Ignoring errors as of now. May have to handle that
	}


	$xml = "<xml>";

	if ( scalar ( @lSysToBeAdded ) >= 1 ) {
		# Add system specific SG for NICs
		if ( $tree -> { VirtualIPInfo } and $tree -> { VirtualIPInfo }{ VirtualIP } ) {

			my $vipref = $tree -> { VirtualIPInfo }{ VirtualIP };
			my ($cmdRet, $sSystemNICSGxml);
			($cmdRet , $sSystemNICSGxml) = $self -> createSystemNICSGDetails($vipref, \@lSysToBeAdded);
			Utils::vxlog("DEBUG", "XML received for adding NIC SG details [$sSystemNICSGxml]");

			if ( $cmdRet == CONST_SUCCESS ) {
				$xml .= $sSystemNICSGxml;
			}
			else {
				Utils::vxlog("WARNING", "There was an error while adding NIC SG - [$resMsg]");

				# TODO: Ignoring errors as of now. May have to handle that
			}
		}
	}



	foreach my $SG ( values ( %{ $href -> { ServiceGroup } } ) ) {
		if ( $addSys == 1 ) {
			$xml .= "<ServiceGroup>" .
				"<Name>" . $SG -> { Name } . "</Name>" .
				"<Mode>edit</Mode>";

			$xml .= "<Attributes>" .
				"<SystemList>" .
				"<global>-update</global>";
			foreach my $sys ( values ( %{ $href -> { SystemList } } ) ) {
				if ( $sys -> {SGConfigured} =~ /true/i ) {
					$xml .= "<global>" . $sys -> { Name } . "</global>";
					$xml .= "<global>" . $sys -> { Priority } . "</global>";
				}
			}

			$xml .= "</SystemList>" .
				"</Attributes>";
			$xml .= "</ServiceGroup>";
		}
		$xml .= "<ServiceGroup>" .
			"<Name>" . $SG -> { Name } . "</Name>" .
			"<Mode>edit</Mode>";
						
		if ( $addSys == 1 ) {
			$xml .= "<Attributes>" .
				"<SystemList>" .
				"<global>-add</global>";

			foreach my $sys ( values ( %{ $href -> { SystemList } } ) ) {
					if ( $sys -> {SGConfigured} =~ /false/i ) {
						$xml .= "<global>" . $sys -> { Name } . "</global>";
						$xml .= "<global>" . $sys -> { Priority } . "</global>";
					}
			}

			$xml .= "</SystemList>" .
				"</Attributes>";
		}

		if ( defined $SG -> { Resources } and
			$SG -> { Resources } ne "" ) {

			$xml .= "<Resources>" . 
				$SG -> { Resources } .
				"</Resources>";
		}

		$xml .= "</ServiceGroup>";
	}

	# As of now VCSInfraSG is hard coded while creating also. So keeping it same here.
	my $sInfraSGName = "VCSInfraSG";
	my $vcsobj = VCS -> new();
	($ret, my $sOutPut) = $vcsobj -> getGrpAttrValue ($sInfraSGName, "SystemList");
	my %assocVal = split(/\t+/, $sOutPut);
	my @infraSysList = keys %assocVal;
	my @infraSysToAdd = ();
	foreach my $sys (@lSysToBeAdded) {
		if (grep ( /^$sys$/, @infraSysList)) {
			next;
		}
		push(@infraSysToAdd, $sys);
	}

	$retRef -> { infraSysList } = \@infraSysToAdd;

	if ( scalar (@infraSysToAdd) >= 1 ) {
		# Add systems to SystemList of VCSInfraSG
		$xml .= "<ServiceGroup>" .
			"<Name>$sInfraSGName</Name>" .
			"<Mode>edit</Mode>";

		if ( $addSys == 1 ) {
			$xml .= "<Attributes><SystemList>" .
				"<global>-add</global>";

			foreach my $sys ( values ( %{ $href -> { SystemList } } ) ) {
				if (grep {$sys -> { Name } eq $_} @infraSysList) {
					next;
				}
				if ( $sys -> { SGConfigured } =~ /false/i ) {
					$xml .= "<global>" . $sys -> { Name } . "</global>";
					$xml .= "<global>" . $sys -> { Priority } . "</global>";
				}
			}

			$xml .= "</SystemList></Attributes>";
		}

		$xml .= "</ServiceGroup>";
	}

	$xml .= "</xml>";

	$retRef -> { ret } = $ret;
	$retRef -> { msg } = $msg;
	$retRef -> { xml } = $xml;

	return $retRef;
}


#--------------------------------------------
# Function to update the VCSInfraSG detail
# when we configure an application and trying
# to include another system in the cluster
#--------------------------------------------

sub UpdateVCSInfraSG
{
	my $self = shift;

	my $sMyName = ( caller 0 )[3];
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $vcsobj = VCS -> new ();
	my $cmdRet = CONST_SUCCESS;
	my $retMsg = "";
	my $sClusterSystemList = "";
	my $infraxml = "";
	my $sInfraSGName = "VCSInfraSG";

	($cmdRet, $retMsg, $sClusterSystemList) = $vcsobj -> getClusterSystemsList();
	if ($cmdRet ne CONST_SUCCESS)  {
		$retMsg = "ERROR: Failed to obtain cluster system list\n";
		return ($cmdRet, $retMsg, "");
	}
	my @clustersyslist = split(/\s+/, $sClusterSystemList);

	($cmdRet, my $sSystemList) = $vcsobj -> getGrpAttrValue ($sInfraSGName, "SystemList");
	if ($cmdRet ne CONST_SUCCESS)  {
		$retMsg = "ERROR: Failed to get SystemList attribute for '$sInfraSGName' group\n";
		return ($cmdRet, $retMsg, "");
	}
	my %hSystemList = split(/\t+/, $sSystemList);
	my @infraSysList = keys %hSystemList;
	my @priorder = values %hSystemList;
	@priorder = sort @priorder;
	my @infraSysToAdd = ();
	foreach my $sys (@clustersyslist) {
		if (grep ( /^$sys$/, @infraSysList)) {
			next;
		}
		push(@infraSysToAdd, $sys);
	}

	if ( scalar (@infraSysToAdd) >= 1 ) {

		$infraxml = "<xml>".
			"<ServiceGroup><Name>$sInfraSGName</Name>".
			"<Mode>edit</Mode>";
		$infraxml .= "<Attributes>";
		$infraxml .= "<SystemList>";
		$infraxml .= "<global>-add</global>";
		foreach my $sys ( @infraSysToAdd ) {
			$infraxml .= "<global>$sys</global>";
			$infraxml .= "<global>$priorder[-1]</global>";
		}
		$infraxml .= "</SystemList>";
		$infraxml .= "</Attributes>";
		$infraxml .= "</ServiceGroup>";
		$infraxml .= "</xml>";
		my $infraRef = ();

		$infraRef -> { ret } = CONST_SUCCESS;
		$infraRef -> { msg } = "";
		$infraRef -> { xml } = $infraxml;

		($cmdRet, $retMsg, $infraxml) = $self -> performConfigSteps( $infraRef -> { xml } );
	}

	return ($cmdRet, $retMsg, $infraxml);


}
sub CreateInfraSG
{
	my $self = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $vcsobj = VCS -> new ();
	my $cmdRet = CONST_SUCCESS;
	my $retMsg = "";
	my $op = "";
	($cmdRet, $retMsg, $op) = $vcsobj -> getClusterSystemsList();
	return ($cmdRet,$retMsg,"") if($cmdRet ne CONST_SUCCESS) ;
	my @syslist = split(/\s+/, $op);
	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $xml = "";
	my $systems = "";
	my $id = 0;

	my $infraxml = "<xml><ServiceGroup><Name>VCSInfraSG</Name>";
	$infraxml .= "<Attributes>";
	$infraxml .= "<SystemList>";

	foreach my $sys ( @syslist ) {
		$infraxml .= "<global>$sys</global>";
		$infraxml .= "<global>$id</global>";
		$id++;
	}

	$infraxml .= "</SystemList>";
	$infraxml .= "<Parallel><global>1</global></Parallel>";
	$infraxml .= "<OnlineRetryLimit><global>5</global></OnlineRetryLimit>";
	$infraxml .= "<UserAssoc>";
	$infraxml .= "<global>Type</global>";
	$infraxml .= "<global>vcs internal</global>";
	$infraxml .= "<global>Name</global>";
	$infraxml .= "<global>VCS Infrastructure service group</global>";
	$infraxml .="</UserAssoc>";
	$infraxml .= "</Attributes>";
	$infraxml .= "<Resources><Resource>";
	$infraxml .= "<Name>VCSNotifySinkRes</Name><Type>Process</Type><SG>VCSInfraSG</SG>";
	if ( $^O !~ m /win/i ) {
		$infraxml .= "<Attributes><PathName><global>/opt/VRTSvcs/portal/admin/notify_sink</global></PathName></Attributes>";
	}
	else {
		require VRTS::Paths;
		my $InstallDir= VRTS::Paths::get_path("InstallDir");
		my $regquery = "\"$InstallDir"."\\bin\\regquery.exe\" query SOFTWARE\\VERITAS\\VCS\\Base HomeDirectory";
		my $vcshome = `$regquery`;
		my $WinNotifySinkDir = "$vcshome\\bin";
		my $WinNotifySinkBin = "$WinNotifySinkDir\\notify_sink.exe";

		$infraxml .= "<Attributes>";
		$infraxml .= "<StartProgram><global>$WinNotifySinkBin</global></StartProgram>";
		$infraxml .= "<StartupDirectory><global>$WinNotifySinkDir</global></StartupDirectory>";
		$infraxml .= "</Attributes>";
	}
	$infraxml .= "</Resource></Resources>";
	$infraxml .= "</ServiceGroup></xml>";


	my $infraRef = ();

	$infraRef -> { ret } = CONST_SUCCESS;
	$infraRef -> { msg } = "";
	$infraRef -> { xml } = $infraxml;

	($ret, $msg, $xml) = $self -> performConfigSteps( $infraRef -> { xml } );

	return ($ret, $msg, $xml);
}

sub SetAutoStartListAttr
{
        my $sMyName = ( caller 0 )[3];
	my $self = shift;
	my $SGName = shift;
        #---------
        # Announce
        #---------
        my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	Utils::vxlog("INFO SGName:", $SGName);
	my $xml = "";

	my ($ret, $msg, $op) = VCS::getClusterSystemsList();
	return ($ret,$msg,"") if ($ret ne CONST_SUCCESS);
	my @syslist = split(/\s+/, $op);
	Utils::vxlog("INFO SystemList:", @syslist);

	my $vcsobj = VCS -> new();
	my $ref = ();
	$ref -> { SG }    = $SGName;
	$ref -> { sys }   = CONST_VCS_GLOBAL;
	$ref -> { attr }  = "AutoStartList";
	$ref -> { value } = \@syslist;

	$vcsobj -> makeConfigWritable();
	($ret, $msg, $xml) = $vcsobj -> modifyServiceGroup($ref);
	$vcsobj -> dumpAndMakeConfigReadOnly();

	if ($ret == CONST_SUCCESS) {
		Utils::vxlog ("INFO", "Set AutoStartList attribute for VCSInfraSG");
	}
	else {
		Utils::vxlog ("ERROR", "Failed to set AutoStartList attribute for VCSInfraSG");
	}
}

sub registerNotifySinkScheduler
{
	my $sMyName = ( caller 0 )[3];
#---------
# Announce
#---------
	my $logStr = "Subroutine $sMyName called";
        Utils::vxlog("INFO", $logStr);

        my $href = Utils::readGuestConfig();
        my $scheduleTime = $href -> { AppHA_GUEST_APPHB_Interval } || 5;
        my $cmd = "";
	my $xprtlc ="";
        my $notify_sink = "";

	if ( $^O !~ m /win/i ) {
                $notify_sink = "/opt/VRTSvcs/portal/admin/notify_sink";
		$xprtlc = "/opt/VRTSsfmh/bin/xprtlc";
        	$cmd = "$xprtlc -l https://localhost:5634/admin/cgi-bin/schedule.pl/set/APPHA_GUEST_APP_HB  -d sched=$scheduleTime";
                $cmd .= " -d command=\"$notify_sink -hb\"";
        }
        else {
                require VRTS::Paths;
                my $InstallDir= VRTS::Paths::get_path("InstallDir");
                my $regquery = "\"$InstallDir"."\\bin\\regquery.exe\" query SOFTWARE\\VERITAS\\VCS\\Base HomeDirectory";
                my $vcshome = `$regquery`;
                $notify_sink= "$vcshome\\bin\\notify_sink.exe";
		$xprtlc = "\"$InstallDir\\bin\\xprtlc.exe\"";
        	$cmd = "$xprtlc -l https://localhost:5634/admin/cgi-bin/schedule.pl/set/APPHA_GUEST_APP_HB  -d sched=$scheduleTime";
		$cmd .= " -d command=\"\\\"$notify_sink\\\" -hb\"";
        }

        my $ret = CONST_SUCCESS;
        my $fret = CONST_SUCCESS;
        my $op = "";

        Utils::vxlog ("INFO", "Executing [$cmd]");
	($fret, $op, $ret) = Utils::runCmd($cmd);
        Utils::vxlog ("INFO", "Return code=[$ret], CmdRet=[$op]");

        if ($ret == CONST_SUCCESS) {
		Utils::vxlog ("INFO", "Registered notify sink scheduler");
	}
	else {
		Utils::vxlog ("ERROR", "Failed to register notify sink scheduler");
	}
		
        return $ret;
}

#--------------------------------------------------------
# common subroutines:
#   app modules can overload, but is it really required?
#--------------------------------------------------------
sub commit
{
	my $self = shift;
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	Utils::vxlog("INFO", "Acquiring lock ...");
	my $lockfd = Utils::acquire_lock($lockFile, 1);
	Utils::vxlog("INFO", "Got lock, proceeding ...");

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $xml = "";
	my $aslXML = ""; # XML for setting AutoStartList attribute

	my $tree = Utils::convertXMLTextToHash( $params );

	if ( (defined $tree -> { IsExistingConfig }) and 
		$tree -> { IsExistingConfig } == 1 ) {
		my $ref = $self -> createUpdateSGDetails($params);

		$ret = $ref -> { ret };
		$msg = $ref -> { msg };
		$xml = $ref -> { xml };

		Utils::vxlog("DEBUG", "XML generated for updation [$xml]");

		if ($ret != CONST_SUCCESS) {
			goto END;
		}

		($ret, $msg, $xml) = $self -> performPreUpdateSteps($params, $ref);
		if ($ret != CONST_SUCCESS) {
			Utils::vxlog("ERROR", "Failed to perform pre-update steps");
			goto END;
		}
	

		($ret, $msg, $xml) = $self -> performConfigSteps( $ref -> { xml } );
		if ($ret != CONST_SUCCESS) {
			Utils::vxlog("ERROR", "Failed to perform update steps");
			goto END;
		}

		($ret, $msg, $aslXML) = $self -> createAutoStartListXML( $ref -> { xml } );
		if ($ret != CONST_SUCCESS) {
			goto END;
		}

		($ret, $msg, $xml) = $self -> performConfigSteps( $aslXML );
		if ($ret != CONST_SUCCESS) {
			Utils::vxlog("ERROR", "Failed to update AutoStartList");
			goto END;
		}

		($ret, $msg, $xml) = $self -> performPostUpdateSteps($params, $ref);
		if ($ret != CONST_SUCCESS) {
			Utils::vxlog("ERROR", "Failed to perform post-update steps");
			goto END;
		}

		goto END;
	}

	# Discover existing SG & Resource names
	my $vcsobj = new VCS();
	$uniqueSGList = $vcsobj -> getAllGroups();
	$uniqueResourceList = $vcsobj -> getAllResources();

	my $ref = $self -> createCommitSGDetails($params);

	$ret = $ref -> { ret };
	$msg = $ref -> { msg };
	$xml = $ref -> { xml };

	if ($ret != CONST_SUCCESS) {
		goto END;
	}

	($ret, $msg, $xml) = $self -> performPreConfigSteps($params, $ref);
	if ($ret != CONST_SUCCESS) {
		goto END;
	}
	
	($ret, $msg, $xml) = $self -> performConfigSteps( $ref -> { xml } );
	if ($ret != CONST_SUCCESS) {
		goto END;
	}

	($ret, $msg, $aslXML) = $self -> createAutoStartListXML( $ref -> { xml } );
	if ($ret != CONST_SUCCESS) {
		goto END;
	}

	($ret, $msg, $xml) = $self -> performConfigSteps( $aslXML );
	if ($ret != CONST_SUCCESS) {
		goto END;
	}

	($ret, $msg, $xml) = $self -> performPostConfigSteps($params, $ref);
	if ($ret != CONST_SUCCESS) {
		goto END;
	}
END:
	Utils::vxlog("INFO", "Releasing lock ...");
	Utils::release_lock( $lockfd );

	return ($ret, $msg, $xml);
}


#-------------------------------------------------------------------------
# Purpose: this function receives VCS xml.
# From VCS xml, create ha commands and create SG, res, dependencies etc...
#-------------------------------------------------------------------------
sub performConfigSteps
{
	my $self = shift;
	my $inputXML = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $xml = "";

	my $tree = Utils::convertXMLTextToHash( $inputXML );
	my $vcsobj = VCS -> new ();

	# create Service groups and resources in it.
	my @SGList = ();
	my @SGNameList = ();
	my $reft = $tree -> { ServiceGroup };

	if ( "$reft" =~ /ARRAY/i ) {
		@SGList = @ { $tree -> { ServiceGroup } };
	} else {
		push(@SGList, $tree -> { ServiceGroup } );
	}

	# open VCS configuration!
	$vcsobj -> makeConfigWritable();

	foreach my $SG ( @SGList ) {
		if ( "$SG" eq "" ) {
			next;
		}

		# During add-node, NIC SG for the new node will have Mode set to add.
		# As per this change, if SystemList of any SG is updated during add-node, that will not be rolled back
		# if any error occurs.
		push ( @SGNameList, $SG -> { Name } ) if ( (not defined $SG -> { Mode }) or ($SG -> { Mode } =~ /^add/) );
		eval {
			($ret, $msg, $xml) = $self -> createServiceGroup( $SG );
			if ( $ret != CONST_SUCCESS ) {
				goto END;
			}
		}; 
		if ($@) {
			$ret = CONST_EXCEPTION;
			$xml = "<Debug><![CDATA[$@]]></Debug>";
			goto END;
		}
	}

	# create SG dependencies now!
	my @SGDepList = ();
	$reft = "";
	if (defined $tree -> { ServiceGroupDependency } and 
	    $tree -> { ServiceGroupDependency } ne "" and
		defined $tree -> { ServiceGroupDependency } { Dependency } ) {
		$reft = $tree -> { ServiceGroupDependency } { Dependency };
	}
	if ( "$reft" =~ /ARRAY/i ) {
		@SGDepList = @ { $tree -> { ServiceGroupDependency } { Dependency } };
	} elsif ( "$reft" ne "" ) {
		push (@SGDepList, $tree -> { ServiceGroupDependency } { Dependency });
	}
	($ret, $msg, $xml) = $self -> createServiceGroupDependencies( \@SGDepList );
	if ( $ret != CONST_SUCCESS ) {
		goto END;
	}

	# anything remaining??

END:
	if ( $ret != CONST_SUCCESS ) {
		$self -> rollback( \@SGNameList );
	}
	# close VCS Configuration
	$vcsobj -> dumpAndMakeConfigReadOnly();

	return ($ret, $msg, $xml);
}


sub createServiceGroup
{
	my $self = shift;
	my $SG = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $xml = "";

	my $vcsobj = VCS -> new();

	# add SG if Mode is not defined or if Mode is "add"
	my $SGName = $SG -> { Name };
	if ( (not defined $SG -> { Mode }) or ($SG -> { Mode } =~ /^add/) ) {
		($ret, $msg, $xml) = $vcsobj -> addServiceGroup ( $SGName );
		if ($ret != CONST_SUCCESS) {
			goto END;
		}
	}
	
	# modify SG attributes
	if (not defined $SG -> { Attributes }) {
		goto RES_ADD;
	}

	foreach my $attr ( keys %{$SG -> { Attributes }} ) {
		my @sysList = keys %{ $SG -> { Attributes } { $attr }};
		foreach my $sys ( @sysList ) {
			my $ref = ();
			$ref -> { SG }    = $SGName;
			$ref -> { sys }   = $sys;
			$ref -> { attr }  = $attr;
			$ref -> { value } = $SG -> { Attributes } { $attr } { $sys };

			($ret, $msg, $xml) = $vcsobj -> modifyServiceGroup($ref);
			if ( $ret != CONST_SUCCESS ) {
				goto END;
			}
		}
	}

RES_ADD:
	# add and modify resources
	if ( not defined $SG -> { Resources } or
	     $SG -> { Resources } eq "" or
	     not defined $SG -> { Resources } { Resource } ) {
		goto END;
	}

	my @ResList = ();
	my $reft = $SG -> { Resources } { Resource };
	if ( "$reft" =~ /ARRAY/i ) {
		@ResList = @ { $reft };
	} else {
		push (@ResList, $reft);
	}
	foreach my $res ( @ResList ) {
		my $resName = $res -> { Name };
		my $resType = $res -> { Type };
		my $resMode = $res -> { Mode };

		my $ref = ();
		$ref -> { Name } = $resName;
		$ref -> { Type } = $resType;
		$ref -> { SG }   = $SGName;

		if ( (not defined $res -> { Mode }) or ($res -> { Mode } =~ /^add/) ) {
			($ret, $msg, $xml) = $vcsobj -> addResource( $ref );
			if ($ret != CONST_SUCCESS) {
				goto END;
			}
		}

		# override attributes if any!
		my @OverrideAttrs = ();
		if (defined $res -> { OverrideAttributes } and
			$res -> { OverrideAttributes } ne "" and
			defined $res -> { OverrideAttributes } { Name }) {
			$reft = $res -> { OverrideAttributes } { Name };
			if ( "$reft" =~ /ARRAY/i ) {
				@OverrideAttrs = @ { $reft };
			} else {
				push(@OverrideAttrs, $reft);
			}
		}

		foreach my $attr ( @OverrideAttrs ) {
			my $ref = ();
			$ref -> { Name } = $resName;
			$ref -> { attr } = $attr;
			($ret, $msg, $xml) = $vcsobj -> overrideResAttribute( $ref );
			if ( $ret != CONST_SUCCESS ) {
				goto END;
			}
		}

		# modify resources
		my @Attributes = ();
		if (defined $res -> { Attributes }) {
			@Attributes = keys %{$res -> { Attributes }};
		}

		foreach my $attr ( @Attributes ) {
			my @sysList = keys %{ $res -> { Attributes } { $attr } };
			foreach my $sys ( @sysList ) {
				my $ref = ();
				$ref -> { Name } = $resName;
				$ref -> { attr } = $attr;
				$ref -> { sys }  = $sys;
				$ref -> { value } = $res -> { Attributes } { $attr } { $sys };

				($ret, $msg, $xml) = $vcsobj -> modifyResource( $ref );
				if ( $ret != CONST_SUCCESS ) {
					goto END;
				}
			}
		}
	}

	# create res dependencies
	my @DepList = ();
	if ( defined $SG -> { ResourceDependency } and
		defined $SG -> { ResourceDependency } { Dependency } ) {
		$reft = $SG -> { ResourceDependency } { Dependency };
		if ( "$reft" =~ /ARRAY/i ) {
			@DepList = @ { $SG -> { ResourceDependency } { Dependency } };
		} else {
			push ( @DepList, $SG -> { ResourceDependency } { Dependency } );
		}
	}

	foreach my $dep ( @DepList ) {
		my $parent = $dep -> { Parent };
		my $child  = $dep -> { Child };
		($ret, $msg, $xml) = $vcsobj -> createResourceDependency( $parent, $child);
		if ($ret != CONST_SUCCESS) {
			goto END;
		}
	}

	# enable resources in the SG
	$vcsobj -> enableAllResourcesInServiceGroup($SGName);

END:
	return ($ret, $msg, $xml);
}



sub createServiceGroupDependencies
{
	my $self = shift;
	my $SGDepList = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $xml = "";

	my $vcsobj = VCS -> new();
	foreach my $dep ( @$SGDepList ) {
		($ret, $msg, $xml) = $vcsobj -> createServiceGroupDependency($dep);
		if ($ret != CONST_SUCCESS) {
			goto END;
		}
	}

END:
	return ($ret, $msg, $xml);
}



sub rollback
{
	my $self = shift;
	my $SGList = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $xml = "";

	my $vcsobj = VCS -> new();

	# for each passed SG, delete all resources in the SG and then delete SG
	foreach my $SG ( @$SGList ) {
		($ret, $msg, $xml) = $vcsobj -> deleteAllGroupResources($SG);
		if ($ret != CONST_SUCCESS) {
			goto END;
		}

		($ret, $msg, $xml) = $vcsobj -> deleteServiceGroup( $SG );
		if ($ret != CONST_SUCCESS) {
			goto END;
		}
	}

END:
	return ($ret, $msg, $xml);
}

sub createUpdateVMwareDiskResDetails
{
	my $self = shift;
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);
	my $tree = $params;
	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $xml = "";

	if (! defined $tree->{VMwareStorageInfo} &&
		! defined $tree->{VMwareStorageInfo}{ESXDetails} &&
		! defined $tree->{VMwareStorageInfo}{ESXDetails}{ESX}) {
		# No VMwareDisks resource needs to be updated.
		return ($ret, $msg, $xml);
	}
	my @esxList = Utils::convertHashValuesToArray($tree->{VMwareStorageInfo}{ESXDetails}{ESX});
	my $attrESXDetails = "";
	my @resList = ();

	$attrESXDetails = "<ESXDetails>";
	foreach my $ESX (@esxList) {
		my $encPasswd = $ESX -> {Password};
		if (defined $ESX -> {Configured} &&
			$ESX -> {Configured} =~ /true/) {
			@resList = split (/:/, $ESX -> {Resources});
		} else{
			($ret,  $encPasswd) = Utils::encrypt_password ($ESX -> {Password});
			if ($ret == CONST_FAILURE) {
				$msg = "Could not encrypt passowrd of ESX host" . $ESX -> {Name};
				goto END;
			}
		}
		$attrESXDetails .= "<global>";
		$attrESXDetails .= $ESX -> {Name};
		$attrESXDetails .= "</global><global>";
		$attrESXDetails .= $ESX -> {Username} . "=" . $encPasswd;
		$attrESXDetails .= "</global>";
	}
	$attrESXDetails .= "</ESXDetails>";
	foreach my $res (@resList) {
		my $resName = $res;
		my $resType = "VMwareDisks";
		my $resMode = "Edit";
		$xml .= "<Resource>";
		$xml .= "<Name>$resName</Name>";
		$xml .= "<Type>$resType</Type>";
		$xml .= "<Mode>$resMode</Mode>";
		$xml .= "<Attributes>";
		$xml .= $attrESXDetails;
		$xml .= "</Attributes>";
		$xml .= "</Resource>";
	}
 END:
	return ($ret, $msg, $xml);
}

# Syntax:
# ($ret, $xml) = getSystemListXML ( $tree -> {'SystemList'} );
#
# XML tags for SystemList:
=comment
	<SystemList>
	  <System>
	    <Name>InGuestVM02</Name>
	    <Priority>0</Priority>
	  </System>
	  <System>
	    <Name>InguestVM01</Name>
	    <Priority>1</Priority>
	  </System>
	</SystemList>
=cut
sub getSystemListXML
{
	my $systemListTree = shift;
	my $systemListXML ="";
	my $ret = CONST_SUCCESS;

	if($systemListTree)
	{
		my @systemList = Utils::convertHashValuesToArray($systemListTree);
		
		foreach my $sl(@systemList) {
			if ( ref ($sl) !~ /^HASH/ || not defined $sl->{System} ) {
				next;
			}
			my @systems = Utils::convertHashValuesToArray($sl->{System});
			$systemListXML .= "<SystemList>";
			foreach my $sys ( @systems ) {
				$systemListXML .= "<global>$sys->{Name}</global>";
				$systemListXML .= "<global>$sys->{Priority}</global>";
			}
			$systemListXML .= "</SystemList>";
			last;
		}
	}
	else 
	{
		$ret = CONST_FAILURE;
	}
	return ($ret, $systemListXML);
}


sub configureVCSInfraSG
{
	my $self = shift;
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "Successully configured VCS infrastructure service group";
	my $xml = "";

	my $vcsobj = VCS -> new ();


	if ( CONST_YES != $vcsobj->isGroupPresent( "VCSInfraSG" ) ) {
		($ret, $msg, $xml) = $self -> CreateInfraSG();
		if ($ret != CONST_SUCCESS) {
			$msg = "ERROR: Failed to configure VCS infrastructure service group";
		}
	} else {
		($ret, $msg, $xml) = $self -> UpdateVCSInfraSG();
		if ($ret != CONST_SUCCESS) {
			$msg = "ERROR: Failed to update system list of VCS infrastructure service group";
		}
	}

	if ($ret == CONST_SUCCESS) {
		my $ret_code = ();
		$ret_code = $self -> SetAutoStartListAttr( "VCSInfraSG" );
		if ( $ret_code != CONST_SUCCESS ) {
			Utils::vxlog("WARNING", "Unable to set AutoStartList attribute for VCSInfraSG service group ");
			$msg = "WARNING : Unable to set AutoStartList attribute for VCSInfraSG service group ";
		}
		sleep 5;
		$ret_code = $vcsobj->operateSG( "VCSInfraSG" , "online", 1 );
		if ( $ret_code != CONST_SUCCESS ) {
			Utils::vxlog("WARNING", "Unable to online VCSInfraSG service group ");
			$msg = "WARNING : Unable to online VCSInfraSG service group";
		}
		$ret_code = $self -> registerNotifySinkScheduler();
	}

	return ($ret, $msg, $xml);
}

sub getUniqueSGName
{
    my $sgNameSeed = shift;
	$sgNameSeed =~ s/[^a-zA-Z0-9_-]+//g;
	my $sgName = $sgNameSeed;

	for(my $index = 0; $index < 1000; $index++) {	
		if($index != 0) {
			$sgName = $sgNameSeed . "-" . $index;
		}
		if(!defined $uniqueSGList->{$sgName}) {
			$uniqueSGList->{$sgName} = 1;
			return $sgName;
		}
	}
}

sub getUniqueResourceName
{
    my $resNameSeed = shift;
	$resNameSeed =~ s/[^a-zA-Z0-9_-]+//g;
	my $resName = $resNameSeed;

	for(my $index = 0; $index < 1000; $index++) {	
		if($index != 0) {
			$resName = $resNameSeed . "-" . $index;
		}
		if(!defined $uniqueResourceList->{$resName}) {
			$uniqueResourceList->{$resName} = 1;
			return $resName;
		}
	}
}


sub createAutoStartListXML
{
	my $self = shift;
	my $xml  = shift;

	my $sMyName = ( caller 0 )[3];
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $op  = "";
	my $commitXML = "<xml>";

	my $tree = Utils::convertXMLTextToHash( $xml );
	Utils::vxlog("INFO", Dumper( $tree ));

	my @SGList = Utils::convertHashValuesToArray( $tree -> { ServiceGroup } );
	my $vcsobj = new VCS();

	foreach my $SG ( @SGList ) {
		if ( "$SG" eq "" ) {
			next;
		}

		my $attr = "SystemList";
		my $sys  = "global";
		my $autoStartListXML = "";

		if ( $SG -> { Name } ) {

			($ret, $op) = $vcsobj -> getGrpAttrValue ( $SG -> { Name }, $attr );
			my %hSysList = split (/\s+/, $op);

			$autoStartListXML .= "<ServiceGroup>" .
				"<Name>" . $SG -> { Name } . "</Name>".
				"<Mode>edit</Mode>" .
				"<Attributes><AutoStartList>";

			foreach my $system ( keys %hSysList ) {
				$autoStartListXML .= "<global>$system</global>";
			}

			$autoStartListXML .= "</AutoStartList></Attributes>" .
				"</ServiceGroup>";

			$commitXML .= $autoStartListXML;
		}
		else {
			Utils::vxlog("INFO", "Attribute [$attr] is not present for service group [$SG]");
		}
	}

	$commitXML .= "</xml>";

	Utils::vxlog("DEBUG", "XML generated for AutoStartList updation [$commitXML]");

	return ( $ret, $msg, $commitXML );
}


1;

