
package ESXStorage;

use base qw(Application);
use Data::Dumper;
use CGI;
use Config;
use POSIX;
use strict;

my $DEBUG = 0;

#------------------------
# End of Common Header
#------------------------
use Constants;
use Utils;
use VCS;
use MessageUtils;
use PlatUtils;
use XML::Simple;

use constant MSG_FILE => "appmon.msg";
#---------------------
# Sample XMLs 
#---------------------

my $sampleParams='<Cmd><ESXDetails><ESX><Name>1.2.3.4</Name><Port>443</Port><Username>root</Username><Password>passwdstring1</Password></ESX><ESX><Name>1.2.3.5</Name><Port>443</Port><Username>root</Username><Password>passwdstring2</Password></ESX></ESXDetails><StorageList><Mount>3:0</Mount><Mount>0:3</Mount><Mount>0:5</Mount></StorageList></Cmd>';

my $VCS_HOME = "/opt/VRTSvcs";
#---------------------
# Subroutines
#---------------------
sub convertHashToXMLtag
{
	my $params = shift;
	my $xmlstr = "";
	my $key = "";
	my $value = "";
	while (($key, $value) = each (%$params)) {
		$xmlstr = $xmlstr . "<$key>$value</$key>";
	}
	return $xmlstr;
}

# convertAttrTreeToArray takes three parameters as input. 
# 	1. attributes list as given to the hadiscover script
# 	2. Array of discovered resoruces, each entry is an array of attribute
# 	   values for that resource
#	3. Attribute tag mapping as required by the output XML.
# It returns an array of hashes, each hash entry corresponding to a resource

sub convertAttrTreeToArray
{
	my $attrList = shift;
	my $disks = shift;
	my $attrTagList = shift;
	my @attrArr = split ('_', $attrList);
	my @retArr=();
	my $num = 0;
	foreach my $_disk ( @$disks ) {
		my $count = 0;
		my $rec = {};
		foreach my $_attr (@attrArr) {
			$rec->{ $attrTagList -> { $_attr } } =  $_disk->{ 'Scalar_value' }[$count]; 
			$rec->{ $attrTagList -> { $_attr } } =~ s/^\n//;
			$rec->{ $attrTagList -> { $_attr } } =~ s/\n$//;
			$rec->{ $attrTagList -> { $_attr } } =~ s/^\s+//;
			$rec->{ $attrTagList -> { $_attr } } =~ s/\s+$//;
			$count++;
		}
		push @retArr, $rec;
		$num++;
	}
	return \@retArr;
}

sub convertParams
{
	my $params = shift;
	my $sMyName = ( caller 0 )[3];

	my @diskIDList = ();
	my @esxHosts = ();
	my @esxAttr = "";
	my $strOut = "";
	my $encPasswd = "";
	my $ret = CONST_SUCCESS;
	my $tree = Utils::convertXMLTextToHash( $params );
	my $isExistingResource = 0;
	my $ref = ();
	if ((! defined ($tree-> { 'ESXDetails' })) ||

		(ref ($tree-> { 'ESXDetails' }) !~ /HASH/ ) ||

		(! defined ($tree-> { 'ESXDetails' }-> {'ESX'} ))) {   

		return ($isExistingResource, $strOut, \@diskIDList, \@esxHosts);

 	}


	$ref = $tree-> { 'ESXDetails' }->{ 'ESX' };
	@esxHosts = Utils::convertHashValuesToArray ($ref);

	foreach (@esxHosts) {
		if ($_->{Configured} =~ /true/) {
			$isExistingResource = 1;
			next;
		}
		$strOut = $strOut . "\$" . $_->{ 'Name' };
		$strOut = $strOut . ":" . $_->{ 'Username' };
		($ret,  $encPasswd) = Utils::encrypt_password ($_->{ 'Password' });
		$_->{ 'Password' } = $encPasswd;
		$strOut = $strOut . ":" . $encPasswd;
	}

	# We get the DiskID tag in case of new discovery.
	# In case of add node wizard, we get the DataStore tag.
	if ((!defined ($tree-> { 'StorageList' })) ||
		(ref ($tree-> { 'StorageList' }) !~ /HASH/ ) ||
		((! defined ($tree-> { 'StorageList' }-> {'DiskID'} )) &&
		(! defined ($tree-> { 'StorageList' }-> {'DataStore'} )))) {   
		return ($isExistingResource, $strOut, \@diskIDList, \@esxHosts);
	}
	if (defined ($tree-> { 'StorageList' }-> {'DiskID'})) {	
		$ref = $tree-> { 'StorageList' }->{ 'DiskID' };
	} elsif (defined ($tree-> { 'StorageList' }-> {'DataStore'} )) {
		$ref = $tree-> { 'StorageList' }->{ 'DataStore' };
	}
	if (ref ( $ref ) =~ /ARRAY/ ) {
		@diskIDList = @$ref;
	} else {
		push (@diskIDList, $ref);
	}
	return ($isExistingResource, $strOut, \@diskIDList, \@esxHosts);
}

sub getDiscoveryErrors
{
	my $esxHosts = shift;
	my $diskErrors = shift;
	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $count = 1;
	foreach my $_esx (@$esxHosts) {
		if ($_esx->{ 'ErrorCode' } != 0) {
			$ret = CONST_FAILURE;
			$msg = $msg . "\n" . $_esx->{ 'ErrorString' };
			$count++;
		}
	}
	if ($diskErrors->{ 'ErrorCode' } != 0) {
		$ret = CONST_FAILURE;
		$msg = $msg . "\n" . $diskErrors->{ 'ErrorString' }; 
		$count++;
	}
	if ( $ret != CONST_SUCCESS ) {
		$msg = "Following errors were observed during strorage discovery:" . $msg;
	}
	return ($ret, $msg);
}


#---------------------------------------------
# Overriding the disocver method
#---------------------------------------------

# Sample input for discover
#<Cmd><ESXDetails><ESX><Name>1.2.3.4</Name><Port>443</Port><Username>root</Username><Password>passwdstring1</Password></ESX><ESX><Name>1.2.3.5</Name><Port>443</Port><Username>root</Username><Password>passwdstring2</Password></ESX></ESXDetails><StorageList><Mount>3:0</Mount><Mount>0:3</Mount><Mount>0:5</Mount></StorageList></Cmd>';

# Sample output of discover
#<Cmd> <ID>ESXStorage</ID> <ReturnCode>0</ReturnCode> <Message></Message> <ESXDetails><ESX><ErrorString>Success</ErrorString><Password>xxxx</Password><ErrorCode>0</ErrorCode><Username>root</Username><Port>443</Port><Name>1.2.3.4</Name></ESX><ESX><ErrorString>Success</ErrorString><Password>xxxx</Password><ErrorCode>0</ErrorCode><Username>root</Username><Port>443</Port><Name>1.2.3.5</Name></ESX></ESXDetails><StorageDetails><Storage><DiskPath>[DS] diskpath.vmdk</DiskPath><CtrlPort>3:0</CtrlPort><DataStore>DS</DataStore><RDM>1</RDM></Storage></StorageDetails> </Cmd>

sub discover
{
	my $self = shift;
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	if ($DEBUG == 1) {
		# Sample XMLs for input from Wizard.
		$params = $sampleParams;
	}

	return performDiscovery ($params);
}

sub performDiscovery
{
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
	my $logStr = "Subroutine $sMyName called";
	my $ret = CONST_SUCCESS;
	my $msg = 0;
	my $xmlOut = "";	

	my $VMUUID = "";
	my $op = "";
	my $cmdstr = "";

	# Option to discFinder utility.
	# 1: perform disc and ESX datastore discovery
	# 2: perform only datastore discovery
	my $option = 1;

	my ($isExistingResource, $hadiscverIn, $diskIDList, $esxHosts) = convertParams ($params);
	if (scalar @$diskIDList < 1 or $hadiscverIn eq "") {	
		# Return blank xml
		my %blankError = ( 'ErrorCode', 0, 'ErrorString', "" );
		my @blankDiscs = ();
		($ret, $msg, $xmlOut) = createOutputXML(\@blankDiscs, $esxHosts, \%blankError);
		return ($ret, $msg, $xmlOut);
	}
	$hadiscverIn =~ s/\$/ /g;

	$cmdstr = '/opt/VRTSsfmh/bin/testVMware';
	($ret, $VMUUID) = Utils::runCmd( $cmdstr );
	chomp $VMUUID;
	$VMUUID =~ s/UUID://;
	if ($isExistingResource == 0) {
		$option = 1;
	} else {
		$option = 2;
	}
	$cmdstr = "export LD_LIBRARY_PATH=$VCS_HOME/lib::\\\$LD_LIBRARY_PATH; $VCS_HOME/bin/VMwareDisks/discFinder $option " .  $VMUUID . " $hadiscverIn";
	($ret, $op) = Utils::runCmd( $cmdstr );
	# Sample Output of discFinder utility
	#our @diskInfo = (
	#                 {
	#                       DiskPath => "[DS] diskpath.vmdk",
	#                       DataStore => "DS",
	#                       CtrlPort => "0:0",
	#                       RDM => 0,
	#						SHARED => 0,
	#						DiskUUID  => "6000C29d-6bea-0a9a-8437-c0d926200fe8",
	#                 }
	#         );
	# our @ctrlMapESX = (
        #         {
        #               CtrlNumber => 0,
        #               CtrlPCISlot=> 16,
        #				CtrlType => 3,
        #         },
        #         {
        #               CtrlNumber => 2,
        #               CtrlPCISlot=> 38,
        #				CtrlType => 3,
        #         },
        #         {
        #               CtrlNumber => 3,
        #               CtrlPCISlot=> 36,
        #				CtrlType => 3,
        #         },
        # );
	#our $discoveryErrors = {
	#                       ErrorCode => 0,
	#                       ErrorString => ""
	#       };
	#our @esx_dsinfo = (
	#                 {
	#                       Name => "1.2.3.4",
	#                       ErrorCode => 0,
	#                       ErrorString => "",
	#                       DSList => [
	#                                        "DataStore1",
	#                                        "DataStore2",
	#                                        "DS",
	#                                 ]
	#                 }
	#         );
	# 1;

	
	no strict;
	my $discdataFile=  Utils::dumpToTmpFile ($op);

	# The below 'do' statement adds the output of the discfinder utility in
	# the program.
	do $discdataFile or return (1, "Error", "");
	unlink($discdataFile);

	# Create an array of controllers attached to the virtual machine.
	$cmdstr = "/sbin/lspci | grep SCSI | grep -i controller | cut -d' ' -f1";
	($ret, $op) = Utils::runCmd( $cmdstr );
	my @arVMCtrlList = split (/\n/, $op);
	my $i = 0;
	my %hrHostToGuestDiskID=();
	@ctrlMapESX = sort {$a->{CtrlPCISlot} <=> $b->{CtrlPCISlot}}@ctrlMapESX;
	while ($i < scalar @ctrlMapESX)	{
		$ctrlMapESX[$i]->{VMCtrlID} = $arVMCtrlList[$i];
		foreach my $diskID (@$diskIDList) {
			if ($diskID =~ m/^$arVMCtrlList[$i]/) {
				my $guestDiskID = $diskID;
				$diskID =~ s/$arVMCtrlList[$i]/$ctrlMapESX[$i]->{CtrlNumber}/;
				$hrHostToGuestDiskID{$diskID} = "$guestDiskID";
			}
		}
		$i++;
	}


	my $appDisks = ();
	my %DSMap = ();
	my @datastores = ();
	if ( ! $isExistingResource ) {
		# New discovery 

		# remove unwanted disks 
		($ret, $msg) = validateControllers(\@ctrlMapESX);
		if ($ret != CONST_SUCCESS) {
			goto END;
		}
		$appDisks = match_disks($diskIDList, \@diskInfo, \%hrHostToGuestDiskID);
		%DSMap = map { $_->{'DataStore'} => 1 } @$appDisks;
		@datastores = keys %DSMap;
	} else {
		# In case of add node wizard, the diskIDList variable contains the
		# datastore list.
		@datastores = Utils::convertHashValuesToArray ($diskIDList);
	}
	 
	($ret) = validateESX ($esxHosts, \@datastores, \@esx_dsinfo);
	($ret, $msg, $xmlOut) = createOutputXML($appDisks, $esxHosts, $discoveryErrors);
	
	use strict;
END:
	return ($ret, $msg, $xmlOut);
}

sub match_disks
{
	my $diskIDList = shift;
	my $disks = shift;
	my $hrHostToGuestDiskID = shift;
	my $sMyName = ( caller 0 )[3];
	my @retArr = ();
	foreach my $disk (@$disks) {
		if ($disk-> { SHARED } == 1) {
		#Skip shared Disks
			next;
		}
		if (grep { $disk-> { 'CtrlPort' } eq $_ } @$diskIDList) {
			$disk -> {guestDiskID} = $hrHostToGuestDiskID -> { $disk-> { 'CtrlPort' } };
			push @retArr, $disk;
		}
	}
	return (\@retArr);
}

# Verify whether all the controllers are of same type.
# We can not map the disks to ESX view if the controllers are of different types.
sub validateControllers
{
	my $ctrls = shift;
	my $_ctrl = ();
	my $ctrltype = -1;
	my $ret = CONST_SUCCESS;
	my $msg = "";
	foreach my $_ctrl (@$ctrls) {
		if ($ctrltype == -1) {
			$ctrltype = $_ctrl-> {CtrlType};
		} elsif ($ctrltype != $_ctrl-> {CtrlType} ) {
			($ret, $msg) = getMessage(MSG_FILE, "ESX_MULTITYPE_CONTROLLER");
		}
	}
	return ($ret, $msg);
}

sub createOutputXML
{

	my $diskDetails = shift;
	my $esxDetails = shift;
	my $discoveryErrors = shift;
	##########################
	# Create XML
	##########################
	my $xmlDisks = "";
	my $xmlOut = "";
	my $ret = 0;
	my $msg = "";
	
	if ($discoveryErrors-> {'ErrorCode'} != 0) {
		$ret = $discoveryErrors-> {'ErrorCode'};
		$msg = $discoveryErrors-> {'ErrorString'}; 
	}
	# Create XML for ESX info
	$xmlOut  =  $xmlOut . "<ESXDetails>";
	foreach my $_esx ( @$esxDetails ) {
		$xmlOut  =  $xmlOut . "<ESX>";
		$xmlOut = $xmlOut . convertHashToXMLtag($_esx);
		if ($_esx -> {'ErrorCode'} != 0) {
			$ret = $_esx -> {'ErrorCode'};
			$msg = $msg . $_esx -> {'ErrorString'};
		}
		$xmlOut  =  $xmlOut . "</ESX>";
	}
	$xmlOut  =  $xmlOut . "</ESXDetails>";

	# Create XML for disk info
	$xmlOut = $xmlOut . "<StorageDetails>" ;

	if ($ret == 0) {
		foreach my $_disk ( @$diskDetails ) {
			# Remove irrelevant entries.
			if ( exists ( $_disk -> {'DiskPath'} ) ) {

			$xmlOut = $xmlOut . "<Storage>" ;
			$xmlOut = $xmlOut . convertHashToXMLtag($_disk);
			$xmlOut = $xmlOut . "</Storage>" ;
			}
		}
	}

	$xmlOut = $xmlOut . "</StorageDetails>" ;
	
	return ($ret, $msg, $xmlOut);
}

#---------------------------------------------
# Overriding the discoverExistingConfig method
#---------------------------------------------

sub discoverExistingConfig
{
    my $sMyName = ( caller 0 )[3];
    my $logStr = "Subroutine $sMyName called";
    Utils::vxlog("INFO", $logStr);

    my $self        = shift;
    my $params      = shift;
    my @paramsArr;

    if(defined $params)
    {
        @paramsArr = split ",",$params;
    }


    my $ret     = CONST_SUCCESS;
    my $msg     = "Successful completion of operation";
    my $op      = "";
    my $xmlstr  = "";

    ($ret, $msg, $op) = discoverExistingVMwareDisksConfig(\@paramsArr);
    goto END if($ret ne CONST_SUCCESS);
    $xmlstr .= $op;

END:
    return ($ret, $msg, $xmlstr);
}

sub discoverExistingVMwareDisksConfig
{
    my $ret         = CONST_SUCCESS;
    my $msg         = "Successful completion of operation";
    my $xmlstr      = "";
    my $sgNamesArr  = shift;

    my $clusDiscoveryInputRef   = ();
    my $esx_found = 0;
    my $esxAttr = ();
    my @datastores = ();
    my %hrEsxHosts = ();

    $clusDiscoveryInputRef->{Resources}{Attributes}     = [ "ESXDetails", "DiskPaths" ];
    $clusDiscoveryInputRef->{Resources}{Types}          = [ "VMwareDisks" ];
    $clusDiscoveryInputRef->{Resources}{Groups}         = \@$sgNamesArr;

    my $vcsobj = VCS -> new();
    ($ret, $msg, my $clusRef ) = $vcsobj -> getClusterDetails($clusDiscoveryInputRef);
    goto END if($ret ne CONST_SUCCESS);

    my $resRef = $clusRef -> { ResRef };

    # We create ESX list only once as it will not change across resources
    # We create the datastore list to be passed to discovery module
    # discovery module will understand that datastore tag means that this
    # is add node wizard, and therefore disk discovery is not required.
    foreach my $res ( sort keys %$resRef )
    {
	    if ($esx_found == 0) {
            $esxAttr = $resRef->{ $res }{ESXDetails}{global};
			$esx_found = 1;
            %hrEsxHosts = split(/\s+/, $esxAttr); 
        }
	    my $vmdk_paths = $resRef->{ $res }{DiskPaths}{global};
	    @datastores = ( $vmdk_paths =~ /\[([^\]]*)\]/g );
    }

END:
    my $xmlStr = "";
    $xmlStr .= "<ESXDetails>";
    foreach my $esx (sort keys %hrEsxHosts) {
        my $resList = "";
	$xmlStr .= "<ESX>"; 	
	$xmlStr .= "<Name>" . $esx . "</Name>";
	my ($user, $password) = ($hrEsxHosts {$esx} =~ /(.*)=(.*)/);
	$xmlStr .= "<Username>" . $user . "</Username>"; 	
	$xmlStr .= "<Password>" . $password . "</Password>"; 	
        foreach my $res ( sort keys %$resRef ) {
			$resList .= $res . ":";
		}
        if($resList ne "") {
	    $resList =~ s/:$//;
            $xmlStr .= "<Resources>$resList</Resources>";
        }
	$xmlStr .= "<Configured>true</Configured>"; 	
	$xmlStr .= "</ESX>"; 	
    }
    $xmlStr .= "</ESXDetails>";
    $xmlStr .= "<StorageList>";
    foreach (@datastores) {
	$xmlStr .= "<DataStore>" . $_ . "</DataStore>";
    }
    $xmlStr .= "</StorageList>";
    return  ($ret, $msg, $xmlStr);
}
#################################################
# validateESX will accept the disk details and 
# ESX data store details. 
# this function will update the $esxHosts with 
# erroCode and erroString after validating the 
# datastore information 
# Chcks whether the datastore of shared disk is
# visible form all the ESX hosts.
#################################################
sub validateESX
{
	my $esxHosts = shift;
	my $datastores = shift;
	my $esxDetails = shift;
	my $ret = CONST_SUCCESS;
	foreach my $esx (@$esxDetails) {
		foreach my $ds (@$datastores) {
			my $esxName = $esx -> { 'Name' };
			my $dsperESX = $esx -> { 'DSList' };
			my $index=0;
			foreach (@$esxHosts) {
				if ( $_ -> { 'Name' } eq $esxName ) {
					if ($esx-> { 'ErrorCode' } != 0) {
						$_ -> { 'ErrorCode' } = $esx-> { 'ErrorCode' };
						$_ -> { 'ErrorString' } = $esx-> { 'ErrorString' };
						last;
					}
					if ( ! grep { $ds eq $_ } @$dsperESX) {
						($_ -> { 'ErrorCode' }, $_ -> { 'ErrorString' }) = getMessage(MSG_FILE, "ESX_DS_NOT_ACCESSIBLE", "$ds", "$esxName");
					} else {
						$_ -> { 'ErrorCode' } = 0;
						$_ -> { 'ErrorString' } = "Success";
					}
					last;
				}
			}
		}	
	}
	return ($ret); 
}

1;

