
#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Id: PlatVCS.pm,v 1.24.8.1 2013/11/30 18:56:18 sgupta2 Exp $
#

package PlatVCS;

use base qw (Exporter);
our @EXPORT = qw(
	CreateClusterInputFile_plat
	LaunchClusterCreateTool_plat
	UnconfigureClusterViaProxy_plat
);

use strict;
use warnings;

use Utils;
use Constants;
use MessageUtils;

use constant MSG_FILE => "clusterop.msg";
use constant MAX_WAIT_TIME => 30;

my @arrSystemList = ();
my $hNodeInfo     = ();

#--------------------------------------------------------
# Create the response file for the native tool (e.g. CPI)
#--------------------------------------------------------
sub CreateClusterInputFile_plat
{
	my $subName = ( caller 0 )[3];
	Utils::vxlog("INFO", "Subroutine $subName called");

	my $tree = shift;
	my $ret = CONST_SUCCESS;
	my $msg = "";

	my $hNewNodeInfo = ();

	my $prodVersion = CPI_PROD;
	my $PROD = "VCS$prodVersion";
	my $respFile = "our \%CFG;\n\n" .
		"\$CFG{fencingenabled}=0;\n" .
		"\$CFG{prod}=qw($PROD);\n";

	my $discoverData = $tree -> { DiscoveryData };
	if ( $discoverData && $discoverData -> { Cluster } ) {
		my $cluster = $discoverData -> { Cluster };

		my $operation = $cluster -> { Operation };
		my $clusterName = $cluster -> { ClusterName };
		my $clusterID = $cluster -> { ClusterID };
		my $singleNode = $cluster -> { SingleNode };
		my $lltType = $cluster -> { LLTType };
		my $osFamily = $cluster -> { OSFamily };

		my $addNode = 0;

		if ( $operation =~ /new/i ) {
			$respFile .= "\$CFG{accepteula}=1;\n";
		}
		elsif ( $operation =~ /addnode/i ) {
			$respFile .= "\$CFG{opt}{addnode}=1;\n";
			$addNode = 1;
		}

		$respFile .= "\$CFG{opt}{configure}=1;\n" .
			"\$CFG{vcs_clusterid}=$clusterID;\n" .
			"\$CFG{vcs_clustername}=qw($clusterName);\n" .
			"\$CFG{vcs_allowcomms}=1;\n";

		if ( $lltType =~ /^udp/i ) {
			$respFile .= "\$CFG{lltoverudp}=1;\n";
		}

		if ( $cluster -> { NodeList } && $cluster -> { NodeList } -> { Node } ) {
			my $strNode = "$cluster->{NodeList}->{Node}";
			my $refNode = $cluster -> {NodeList} -> {Node};

			my @arrNode = ();

			if ( $strNode =~ /^ARRAY/ ) {
				@arrNode = @$refNode;
			}
			else {
				push (@arrNode, $refNode);
			}

			my (@existingSystems, @newSystems) = ();

			foreach my $node ( @arrNode ) {
				my $nodeName = $node -> { Name };

				push (@arrSystemList, $nodeName);

				$hNodeInfo -> { $nodeName }{ Username } = $node -> { Username };
				$hNodeInfo -> { $nodeName }{ Password } = $node -> { Password };

				if ( defined $node -> { ClusterConfigured } &&
					$addNode == 1 ) {
					if ( $node -> { ClusterConfigured } =~ /false/i ) {
						push ( @newSystems, $nodeName );
						$hNewNodeInfo -> { $nodeName }{ Username } = $node -> { Username };
						$hNewNodeInfo -> { $nodeName }{ Password } = $node -> { Password };
					}
					else {
						push ( @existingSystems, $nodeName );
					}
				}

				if ( $node -> { LLTLink } ) {
					my $strLinks = "$node->{LLTLink}";
					my $refLinks = $node -> {LLTLink};
					my @arrLinks = ();

					if ( $strLinks =~ /^ARRAY/ ) {
						@arrLinks = @$refLinks;
					}
					else {
						push (@arrLinks, $refLinks);
					}

					my $linkCntr = 1;
					my $lowPriLinkCntr = 1;

					foreach my $link ( @arrLinks ) {
						if ( $lltType =~ /^udp/i ) {
							if ( $link -> { LowPri } ) {
								$respFile .= "\$CFG{vcs_udplinklowpri". $lowPriLinkCntr .
									"_address}{\"$nodeName\"}=qw(" . $link->{IP} . ");\n" .
									"\$CFG{vcs_udplinklowpri" . $lowPriLinkCntr . 
									"_port}{\"$nodeName\"}=" . $link->{Port} . ";\n" .
									"\$CFG{vcs_udplinklowpri" . $lowPriLinkCntr .
									"_netmask}{\"$nodeName\"}=qw(" . $link->{NetMask} . ");\n";

							}
							else {
								$respFile .= "\$CFG{vcs_udplink". $linkCntr .
									"_address}{\"$nodeName\"}=qw(" . $link->{IP} . ");\n" .
									"\$CFG{vcs_udplink" . $linkCntr . 
									"_port}{\"$nodeName\"}=" . $link->{Port} . ";\n" .
									"\$CFG{vcs_udplink" . $linkCntr .
									"_netmask}{\"$nodeName\"}=qw(" . $link->{NetMask} . ");\n";

							}
						}
						if ( $link -> { LowPri } ) {
							$respFile .= "\$CFG{vcs_lltlinklowpri$lowPriLinkCntr}" .
								"{\"$nodeName\"}=qw(" . $link->{Name} . ");\n";
							$lowPriLinkCntr++;
						}
						else {
							$respFile .= "\$CFG{vcs_lltlink$linkCntr}" .
								"{\"$nodeName\"}=qw(" . $link->{Name} . ");\n";
							$linkCntr++;
						}
					}
				}
				else {
# TODO: Error if no LLT link specified
				}
			}
			$respFile .= "\$CFG{systems}=[ qw( @arrSystemList ) ];\n";

			if ( $addNode == 1 ) {
				$respFile .= "\$CFG{clustersystems}=[ qw( @existingSystems ) ];\n";
				$respFile .= "\$CFG{newnodes}=[ qw( @newSystems ) ];\n";

				# Backup llttab, llthosts, gabtab, main.cf
				# on the nodes specified
				($ret, $msg) = Utils::RemoteOperation ( $hNewNodeInfo, "unconfigureCluster.pl", "BackupFilesIfVCSStopped" );
				if ( $ret != CONST_SUCCESS ) {;
					Utils::vxlog("ERROR", "Could not backup configuration files - [$msg]. Will not proceed with add node.");
					goto END;
				}

			}
		}

		if( $cluster -> { Security } ) {
			my $security = $cluster -> { Security };
			my $secureClus = $security -> { SecureClus };

			if ( 1 == $secureClus ) {
				$respFile .= "\$CFG{vcs_eat_security}=1;\n";

				if ( 1 == $security->{ FipsMode } ) {
					$respFile .= "\$CFG{vcs_eat_security_fips}=1;\n";
				}
			}
			else {
				my $role = $security -> { User } -> { Role };
				my $user = $security -> { User } -> { Name };
				my ($ret, $encPasswd) = Utils::encrypt_password ( $security -> { User } -> { Password } );

				$respFile .= "\$CFG{vcs_username}=[ qw($user) ];\n" .
					"\$CFG{vcs_userenpw}=[ qw($encPasswd) ];\n" .
					"\$CFG{vcs_userpriv}=[ qw($role) ];\n";

# TODO: Make provision for multiple users
#				if ( $users && $users->{ User } ) {
#					foreach my $user () {
#					}
#				}

			}
		}
	}

	$respFile .= "\n1;";

END:
	Utils::vxlog("INFO", "Subroutine $subName end");
	return ($ret, $respFile);
}


#-----------------------------------------
# Launch the native cluster creation tool
#-----------------------------------------
sub LaunchClusterCreateTool_plat
{
	my $respFile = shift;

	my $subName = ( caller 0 )[3];
	Utils::vxlog("INFO", "Subroutine $subName called");

	my $ret = CONST_SUCCESS;
	my $fret = CONST_SUCCESS;
	my $msg = "";
	my $op = "";
	my $output = "";

	my @aSysRequiresCommSetup = ();
	my $currSysName = Utils::getOSInfo() -> { COMPUTERNAME };

	foreach my $sys ( @arrSystemList ) {
		if ( $currSysName ne $sys and CONST_YES != PlatUtils::hasPasswdFreeSSH ( $sys ) ) {
			push ( @aSysRequiresCommSetup, $sys );
		}
	}

	Utils::vxlog("INFO", "Systems for which communication would be setup [ @aSysRequiresCommSetup ].");

	if ( scalar (@aSysRequiresCommSetup) ) {

		($ret, $msg) = PlatUtils::createKeysForSSH();
		if ( $ret != CONST_SUCCESS ) {
			Utils::vxlog("ERROR", "Failed to create public and private key for $currSysName. Will not configure cluster.");
			goto END;
		}

		foreach my $sys ( @aSysRequiresCommSetup ) {

			($ret, $msg) = PlatUtils::setupPasswdFreeSSH ( "configureSSH", $sys, 
				$hNodeInfo -> { $sys }{ Username }, $hNodeInfo -> { $sys }{ Password } );

			if ( $ret != CONST_SUCCESS ) {
				Utils::vxlog("ERROR", "Failed to setup password-less SSH from $currSysName to $sys. Will not configure cluster.");
				goto END;
			}
		}
	}

	my $href = PlatUtils::getVCSPaths();
	my $installDir = $href -> { InstallDir };
	my $logDir = $href -> { LogDir };

	my $clusCreateTool = "";
	my $respFilePath = "$logDir/configure_cluster.response";

	($ret, $clusCreateTool) = PlatUtils::getBestClusCreateTool("$installDir/installvcs*");
	if ( $ret == CONST_SUCCESS ) {

		my $clusCreateCmd = "unset PERL5LIB; $clusCreateTool -responsefile $respFilePath";

		open RF, "> $respFilePath";
		print RF $respFile;
		close RF;

		($fret, $op, $ret) = Utils::runCmd($clusCreateCmd);

		$ret = CONST_FAILURE if ( $op =~ /CPI ERROR/ );
	}
	else {
		($ret, $msg) = getMessage( MSG_FILE, "NATIVETOOL_CREATECLUS_NOT_FOUND" );
		goto END;
	}

	if ($ret != CONST_SUCCESS) {

		$output = join (' ', split ( /\n/, $op ) );
	
		if ( $output =~ /CPI ERROR V-9-40-.*install(\S+)\s-addnode/ ) {
	
			($ret, $clusCreateTool) = PlatUtils::getBestClusCreateTool("$installDir/install$1*");
			if ( $ret != CONST_SUCCESS ) {
				($ret, $msg) = getMessage( MSG_FILE, "NATIVETOOL_CREATECLUS_NOT_FOUND" );
				goto END;
			}

			Utils::vxlog("INFO", "Found [$1] installed. Will invoke [$clusCreateTool] now.");
	
			$op =~ m/installed product on cluster.*\s\.*\s(\S+)\s.*/;
			my $newProd = $1;
			$respFile =~ s/(CFG{prod}=qw\()VCS/$1$newProd/;
			Utils::vxlog("DEBUG", "Modified prod in response file to [$newProd]");
	
			open RF, "> $respFilePath";
			print RF $respFile;
			close RF;
		
			my $clusCreateCmd = "unset PERL5LIB; $clusCreateTool -responsefile $respFilePath";
			($fret, $op, $ret) = Utils::runCmd($clusCreateCmd);
		
			$output = join (' ', split ( /\n/, $op ) );
			if ( $output =~ /(CPI ERROR.*)/ ) {
				$ret = CONST_FAILURE;
			}
			else {
				$ret = CONST_SUCCESS;
			}

			if ($ret != CONST_SUCCESS) {
				($ret, $msg) = getMessage( MSG_FILE, "NATIVETOOL_CREATECLUS_FAILED", $1);
				goto END;
			}
		}
		else {
			$output =~ /(CPI ERROR.*)/;
			($ret, $msg) = getMessage( MSG_FILE, "NATIVETOOL_CREATECLUS_FAILED", $1);
			goto END;
		}
	}

	my $vcsobj = new VCS();

	foreach my $sys ( @arrSystemList ) {
		$ret = $vcsobj -> waitForVCS(MAX_WAIT_TIME, $sys);

		if ( $ret != CONST_YES ) {
			$ret = CONST_FAILURE;
			$msg = "VCS has not started on $sys even when configuration is complete.";
			last;
		}
	}

	if ( CONST_YES == $ret ) {
		$ret = CONST_SUCCESS;
		$msg = "";
	}

END:
	foreach my $sys ( @aSysRequiresCommSetup ) {

		Utils::vxlog("INFO", "Removing password-less SSH to $sys.");
		($ret, $msg) = PlatUtils::setupPasswdFreeSSH ( "unconfigureSSH", $sys, 
			$hNodeInfo -> { $sys }{ Username }, $hNodeInfo -> { $sys }{ Password } );

		if ( $ret != CONST_SUCCESS ) {
			Utils::vxlog("WARNING", "Unable to remove password-less SSH to $sys.");
		}
	}

	return ($ret, $msg, $op);
}


#-----------------------------------------
# Unconfigure the cluster through proxy on
# each node of the cluster
#-----------------------------------------
sub UnconfigureClusterViaProxy_plat
{
	my $tree = shift;

	my $subName = ( caller 0 )[3];
	Utils::vxlog("INFO", "Subroutine $subName called");

	my $ret = CONST_SUCCESS;
	my $fret = CONST_SUCCESS;
	my $msg = "";
	my $dbg = "";

	my $href = PlatUtils::getVCSPaths();

	my $discoverData = $tree -> { DiscoveryData };
	if ( $discoverData && $discoverData -> { Cluster } ) {
		my $cluster = $discoverData -> { Cluster };

		my $operation = $cluster -> { Operation };
		my $clusterName = $cluster -> { ClusterName };
		my $clusterID = $cluster -> { ClusterID };
		my $singleNode = $cluster -> { SingleNode };

		my $hNodeDetails = ();

		if ( $operation =~ /delete/i ) {

			if ( $cluster -> { NodeList } && $cluster -> { NodeList } -> { Node } ) {
				my $refNode = $cluster -> {NodeList} -> {Node};
				my @arrNode = Utils::convertHashValuesToArray( $refNode );

				my $xprtlc = $href -> { xprtlc };

				foreach my $node ( @arrNode ) {
					$hNodeDetails -> { $node -> { Name } }{ Username } = $node -> { UserName };
					$hNodeDetails -> { $node -> { Name } }{ Password } = $node -> { Password };
				}

				($ret, $msg) = Utils::RemoteOperation ( $hNodeDetails, "unconfigureCluster.pl", "NoOp" );
				if ($ret != CONST_SUCCESS) {
					$msg = "Unable to unconfigure the cluster.\n\nPlease verify the following:\n- Correct username and password have been provided for all the virtual machines\n- Veritas Storage Foundation Messaging Service (xprtld) is running on all the virtual machines\n- The virtual machine(s) are powered on and accessible";
					goto END;
				}

				foreach my $node ( @arrNode ) {
					my $nodeName = $node -> { Name };
					my $userName = $node -> { UserName };
					my $passwd = $node -> { Password };

					$passwd =~ s/'/'"'"'/g;

					my $targetUrl = "https://$nodeName:5634/vcs/admin/unconfigureCluster.pl";
					my $args = "-d TargetURL=$targetUrl -d TargetUser=\'$userName\' -d TargetPassword=\'$passwd\'";

					my $proxyCmd = $xprtlc . " -l https://localhost:5634/vcs/admin/proxy.pl $args";

					($fret, $msg, $ret) = Utils::runCmd($proxyCmd, { doNotLogCmd => 1 });
					if ($fret != CONST_SUCCESS) {
						Utils::vxlog("ERROR", "Failed to stop VCS on $nodeName.");
						goto END;
					}
					else {
						($ret, $msg, $dbg) = Utils::parseCmdResponseXML ( $msg );
						if ( $ret != CONST_SUCCESS ) {
							Utils::vxlog("ERROR", "Failed to stop VCS on $nodeName - $msg [$dbg]" );
							goto END;
						}
					}
				}
			}
		}
	}

END:
	return ($ret, $msg, $dbg);
}


# Creating empty function as of now. Will add functionality to it when required.
sub StartLLT_plat
{
	my $subName = ( caller 0 )[3];
	Utils::vxlog("INFO", "Subroutine $subName called");

	my $ret = CONST_SUCCESS;
	my $msg = "";

	Utils::vxlog("INFO", "Subroutine $subName end");
	return ($ret, $msg);
}


# Creating empty function as of now. Will add functionality to it when required.
sub StopLLT_plat
{
	my $subName = ( caller 0 )[3];
	Utils::vxlog("INFO", "Subroutine $subName called");

	my $ret = CONST_SUCCESS;
	my $msg = "";

	Utils::vxlog("INFO", "Subroutine $subName end");
	return ($ret, $msg);
}


1; # perl won't complain now!










