
#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Id: Storage.pm,v 1.28.8.1 2012/12/17 12:18:04 lpatil Exp $
#

package Storage;

use base qw(Exporter);
our @EXPORT = qw(
	createSGs
	getDiskResources
	createStorageSGTables
	getVxVMDgDisks
	getLVMVgDisks
	getDiskId
	@gInstances
	$hrInsChildParents
	$hrInsParentChildren
	$hrSGs
	$hrMountInstances
	$hrInstanceMounts
	$hrDgMounts
	$hrMountDg
	$hrResXMLs
	$hrResNames
	$hrDepMountVol
	$hrDepVolDg
	$hrDepDgDisks
	$hrIPInstances
	$hrInstanceIP
	$hrIPNIC
	$hrNICIPs
	$hrSystemNICResName
	$hrIPProxy
	 );

use strict;
use warnings;

use Data::Dumper;
use POSIX;
use File::Copy;

use Constants;
use MessageUtils;
use PlatUtils;
use Utils;
use Logger;
use constant MSG_FILE => "appmon.msg";
our (@gInstances, $hrMountInstances, $hrInstanceMounts, $hrDgMounts, $hrMountDg, $hrInsChildParents, $hrInsParentChildren, $hrSGs) = ();
our ( $hrResXMLs, $hrResNames, $hrDepMountVol, $hrDepVolDg, $hrDepDgDisks ) = ();
our ( $hrIPInstances, $hrInstanceIP, $hrIPNIC, $hrNICIPs, $hrSystemNICResName, $hrIPProxy ) = ();

sub createSGs
{
	my $sMyName = ( caller 0 )[3];
	#-------------------
	# Announce ourselves
	#-------------------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my ( $self, $sSGNamePrefix ) = (@_);
	my $iSGCnt = 1;
	$sSGNamePrefix = "SG" if ( not defined $sSGNamePrefix );
	my $sSGName = $sSGNamePrefix . "_$iSGCnt";
	$sSGName =  $self->GetUniqueGroupName("$sSGName");
	Utils::vxlog("DEBUG", "createSGs: Group Name:[$sSGName]");

	Utils::vxlog("INFO", " [[hrDgMounts]]"); Utils::vxlog("INFO", Dumper( $hrDgMounts ));
	Utils::vxlog("INFO", " [[hrMountDg]]"); Utils::vxlog("INFO", Dumper( $hrMountDg ));
	Utils::vxlog("INFO", " [[hrMountInstances]]"); Utils::vxlog("INFO", Dumper( $hrMountInstances ));
	Utils::vxlog("INFO", " [[hrInstanceMounts]]"); Utils::vxlog("INFO", Dumper( $hrInstanceMounts ));
	Utils::vxlog("INFO", " [[hrIPInstances]]"); Utils::vxlog("INFO", Dumper( $hrIPInstances ));
	Utils::vxlog("INFO", " [[hrInstanceIP]]"); Utils::vxlog("INFO", Dumper( $hrInstanceIP ));
	Utils::vxlog("INFO", " [[hrIPNIC]]"); Utils::vxlog("INFO", Dumper( $hrIPNIC ));
	Utils::vxlog("INFO", " [[hrNICIPs]]"); Utils::vxlog("INFO", Dumper( $hrNICIPs ));
	Utils::vxlog("INFO", " [[hrInsChildParents]]"); Utils::vxlog("INFO", Dumper( $hrInsChildParents ));
	Utils::vxlog("INFO", " [[hrInsParentChildren]]"); Utils::vxlog("INFO", Dumper( $hrInsParentChildren ));

	my $hrVisited =();
	my ($arResList, $hrResList) = ();
	my $sResName ="";

	foreach my $sInstance ( @gInstances ) {
		if ( defined ($hrVisited -> {$sInstance}) ) {
			next;
		}
		$arResList =();
		$hrResList =();
		push ( @{$arResList}, $sInstance  );

		foreach my $sRes ( @{$arResList} ) {	
		  #### print "\n Processing [$sRes] : ";
		  if ( defined ($hrVisited -> {$sRes}) ) {
			next;
		  }
		  my @tmpResList = ();
		  if ( defined $hrDgMounts->{ $sRes } ) {
		  	@tmpResList = Utils::convertHashValuesToArray ($hrDgMounts->{$sRes});
		  	push ( @{$arResList}, @tmpResList ) ;
		  }
		  if ( defined $hrMountDg->{ $sRes } ) {
			@tmpResList = Utils::convertHashValuesToArray ($hrMountDg->{$sRes});
		  	push ( @{$arResList}, @tmpResList ) ;
		  }
		  if ( defined $hrMountInstances->{ $sRes } ) {
		  	@tmpResList = Utils::convertHashValuesToArray ($hrMountInstances->{$sRes});
		  	push ( @{$arResList}, @tmpResList ) ;
		  }
		  if ( defined $hrInstanceMounts->{ $sRes } ) {
		  	@tmpResList = Utils::convertHashValuesToArray ($hrInstanceMounts->{$sRes});
		  	push ( @{$arResList}, @tmpResList ) ;
		  }
		  if ( defined $hrInstanceIP->{ $sRes } ) {
		  	@tmpResList = Utils::convertHashValuesToArray ($hrInstanceIP->{$sRes});
		  	push ( @{$arResList}, @tmpResList ) ;
		  }
		  if ( defined $hrIPInstances->{ $sRes } ) {
		  	@tmpResList = Utils::convertHashValuesToArray ($hrIPInstances->{$sRes});
		  	push ( @{$arResList}, @tmpResList ) ;
		  }
		  if ( defined $hrIPNIC->{ $sRes } ) {
		  	@tmpResList = Utils::convertHashValuesToArray ($hrIPNIC->{$sRes});
		  	push ( @{$arResList}, @tmpResList ) ;
		  }
		  if ( defined $hrIPProxy->{ $sRes } ) {
		  	@tmpResList = Utils::convertHashValuesToArray ($hrIPProxy->{$sRes});
		  	push ( @{$arResList}, @tmpResList ) ;
		  }
		  if ( defined $hrNICIPs->{ $sRes } ) {
		  	@tmpResList = Utils::convertHashValuesToArray ($hrNICIPs->{$sRes});
		  	push ( @{$arResList}, @tmpResList ) ;
		  }
		  if ( defined $hrInsChildParents->{ $sRes } ) {
		  	@tmpResList = Utils::convertHashValuesToArray ($hrInsChildParents->{$sRes});
		  	push ( @{$arResList}, @tmpResList ) ;
		  }
		  if ( defined $hrInsParentChildren->{ $sRes } ) {
		  	@tmpResList = Utils::convertHashValuesToArray ($hrInsParentChildren->{$sRes});
		  	push ( @{$arResList}, @tmpResList ) ;
		  }
		  #### my $firststring = join(", ",@{$arResList});
		  #### print "\n current Queue: " . $firststring ;
		  #### print "\n Visited [$sRes] ";
		  $hrVisited -> {$sRes} = 1;
		  $hrResList->{$sRes} =1;	
		}

		$sSGName = $sSGNamePrefix . "_$iSGCnt";
		$sSGName =  $self->GetUniqueGroupName("$sSGName");
		Utils::vxlog("DEBUG", "createSGs: Group Name:[$sSGName]");
		push ( @{$hrSGs->{$sSGName}}, keys %{$hrResList});  
		$iSGCnt++;
	}
}

sub getDiskResources
{
	my ($ret, $msg) = (CONST_SUCCESS, "");
	my $vcsencrypt = "/opt/VRTSvcs/bin/vcsencrypt";
        my $self = shift;
        my $esxList = shift;
        my $diskList = shift;
 	my $hrDiskResInfo = ();
	my $sESXAttr = "";
        my @esxListArr = Utils::convertHashValuesToArray ($esxList);
        my @diskListArr = Utils::convertHashValuesToArray ($diskList);
	Utils::vxlog("INFO", "Input ESXList:");
	Utils::vxlog("INFO", Dumper( \@esxListArr ));
	Utils::vxlog("INFO", "Input DiskList:");
	Utils::vxlog("INFO", Dumper( \@diskListArr ));
	$sESXAttr .= "<ESXDetails>";
        foreach my $sESX (@esxListArr){

	#here	
		my $esxPasswd = $sESX->{Password};
		$esxPasswd =~ s/'/'"'"'/g;
		my $cmd = "$vcsencrypt -agent \'$esxPasswd\'";
		my ($CmdRet, $op ) = Utils::encrypt_password ($sESX->{Password});
		if ($CmdRet != CONST_SUCCESS ) {
			$ret = $CmdRet;
			$msg = "failed while executing of vcsencrypt";
			goto END;
		}

		$sESXAttr .= "<global>";
		$sESXAttr .= $sESX -> {Name};
		$sESXAttr .= "</global><global>";
		$sESXAttr .= $sESX -> {Username} . "=" . $op;
		$sESXAttr .= "</global>";
	}
	$sESXAttr .= "</ESXDetails>";
	my $iCnt = 1;
        foreach my $hrDisk (@diskListArr) {
        #print "\nhrDisk = " . Dumper ($hrDisk);
		my $sResxml = "";
		my $sDiskPathsAttr = "";
		my $resName_key = "VMwareDisk_$iCnt";
		my $resName =  $self->GetUniqueResName($resName_key);
		Utils::vxlog("DEBUG", "resource Name:$resName");
		if ( not defined $hrResNames->{$resName_key} ) {
			$hrResNames->{$resName_key} = $resName;
		}
		$sResxml .= "<Resource>";
		$sResxml .= "<Name>$resName</Name>"; 
		$sResxml .= "<Type>VMwareDisks</Type>"; 
		$sResxml .= "<Attributes>";
			$sResxml .= "<VMRegisterWait><global>5</global></VMRegisterWait>"; 
			$sResxml .= $sESXAttr;
			$sResxml .= "<IsVMHAEnabled><global>1</global></IsVMHAEnabled>"; 
			if ( $hrDisk ->{RDM} == 1 ) {
				$sDiskPathsAttr .= "RDM:";
			}	
			my $diskUUID = $hrDisk ->{DiskUUID};
			chomp($diskUUID);
			if ( $diskUUID ne "" ) {
				$sDiskPathsAttr .= $diskUUID . ":";
			}
			$sDiskPathsAttr .= $hrDisk ->{DiskPath};
			$sResxml .= "<DiskPaths>";
			$sResxml .= "<global>$sDiskPathsAttr</global>"; 
			$sResxml .= "<global>$hrDisk->{CtrlPort}</global>"; 
			$sResxml .= "</DiskPaths>";
		$sResxml .= "</Attributes>";
		$sResxml .= "</Resource>";
		$hrDiskResInfo -> { $hrDisk->{ guestDiskID } }{ XML } = $sResxml;
		$hrDiskResInfo -> { $hrDisk->{ guestDiskID } }{ ResName } = $resName;
		$iCnt++;
        }
END:
	return ($ret, $msg, $hrDiskResInfo); 
}


sub createStorageSGTables
{
	my ($xml,$mntXML, $dgXML) = ();
	my ($ret, $msg ) = (CONST_SUCCESS, "");
	my ($self, $hrStorageList, $hrVMwareStorageInfo ) = (@_);

	my $sMySubName = ( caller 0 )[3];
	my $announceStr = "Subroutine $sMySubName called";
	Utils::vxlog("INFO", $announceStr);

	my ($hrStorageDetails) = ();
	my ($sMountPoint, $sBlockDevice, $sFSType, $sMountOpt, $sDiskGroup, $sVolume, $sVolumeGroup, $sLogicalVolume) = ();
	my $hrDiskResInfo = ();
	if ( $hrVMwareStorageInfo ne "" and
	     defined $hrVMwareStorageInfo->{ESXDetails} and
	     defined  $hrVMwareStorageInfo->{StorageDetails} and
	     $hrVMwareStorageInfo->{ESXDetails} ne "" and
	     $hrVMwareStorageInfo->{StorageDetails} ne "") {
		($ret , $msg, $hrDiskResInfo ) = $self-> getDiskResources($hrVMwareStorageInfo->{ESXDetails}{ESX}, $hrVMwareStorageInfo->{StorageDetails}{Storage});
		if ($ret != CONST_SUCCESS ) {
			Utils::vxlog("ERROR", "Error occurred while finding information of disks attached to the virtual machine.");
                	$ret = CONST_FAILURE;
			goto END;
		}
		Utils::vxlog("INFO", Dumper( $hrDiskResInfo ));
       	}  else {
		Utils::vxlog("INFO", "Storage discovery skipped.");
		goto END; 
	}

        my @Mount_inst = ();
        if ( (defined $hrStorageList) and ($hrStorageList ne "" )
		and (defined ($hrStorageList -> { Storage }) ) ) {

        	my $Mount_ref = $hrStorageList -> { Storage };

		if ("$Mount_ref" =~ /ARRAY/) {
                	@Mount_inst = @$Mount_ref;
        	} else {
                	push(@Mount_inst, $Mount_ref);
        	} 

                foreach my $Mount_ins (@Mount_inst) {
			my @DiskIDList = ();
			my $Name = $Mount_ins -> { Name };
			my ($mntResName, $volResName, $dgResName, $diskResName) = ();
			$sMountPoint   	= $Mount_ins -> { MountPoint } { Value };
			if ( not defined $hrMountInstances -> { $sMountPoint } ) {
				Utils::vxlog("INFO", "Skipped [$sMountPoint]. Since it is not used by any application instance.");
				next;
			}
			$sBlockDevice 	= $Mount_ins -> { BlockDevice }     { Value };
			$sFSType 	= $Mount_ins -> { FSType }     { Value };
			$sMountOpt 	= $Mount_ins -> { MountOpt }     { Value };
			if ( defined ($Mount_ins -> { DiskGroup }     { Value })) {
				$sDiskGroup 	= $Mount_ins -> { DiskGroup }     { Value };
				$sVolume 	= $Mount_ins -> { Volume }     { Value };
				if ((defined ($Mount_ins ->{DiskIDList})) and
	     				$Mount_ins ->{DiskIDList} ne "" and
					(defined ($Mount_ins ->{DiskIDList}{DiskID}))) {
	        		@DiskIDList = Utils::convertHashValuesToArray ($Mount_ins ->{DiskIDList}{DiskID});
				}
			}
			if ( defined ($Mount_ins -> { VolumeGroup }     { Value })) {
                $sVolumeGroup 	= $Mount_ins -> { VolumeGroup }     { Value };
                $sLogicalVolume	= $Mount_ins -> { LogicalVolume }     { Value };
				if ((defined ($Mount_ins ->{DiskIDList})) &&
					(defined ($Mount_ins ->{DiskIDList}{DiskID}))) {
	        		@DiskIDList = Utils::convertHashValuesToArray ($Mount_ins ->{DiskIDList}{DiskID});
				}
			}
			# Mount resource
			$mntXML = "";
			$mntXML .= "<Resource>";
			$mntResName = "mnt_" . $Name;
			$mntResName =  $self->GetUniqueResName($mntResName);
			Utils::vxlog("DEBUG", "resource Name:$mntResName");
			$mntXML .= "<Name>$mntResName</Name>"; 
			$mntXML .= "<Type>Mount</Type>"; 
			$mntXML .= "<Attributes>";
				$mntXML .= "<MountPoint><global><![CDATA[" . $sMountPoint . "]]></global></MountPoint>"; 
				$mntXML .= "<BlockDevice><global>$sBlockDevice</global></BlockDevice>"; 
				$mntXML .= "<FSType><global>$sFSType</global></FSType>"; 
				$mntXML .= "<MountOpt><global>$sMountOpt</global></MountOpt>"; 
				$mntXML .= "<FsckOpt><global>%-y</global></FsckOpt>"; 
			$mntXML .= "</Attributes>";
			$mntXML .= "</Resource>";

			# Volume resource
			if ( defined ($Mount_ins -> { DiskGroup }     { Value })) {

				$mntXML .= "<Resource>";
				if ( not defined $hrResNames->{ $sVolume } ) {
					$volResName = "vol_" . $sVolume . "_" . $sDiskGroup;
					$volResName =  $self->GetUniqueResName($volResName);
					Utils::vxlog("DEBUG", "resource Name:$volResName");
					$hrResNames->{ $sVolume } = $volResName;
				} else {
					$volResName = $hrResNames->{ $sVolume };
				}

				$mntXML .= "<Name>$volResName</Name>"; 
				$mntXML .= "<Type>Volume</Type>"; 
				$mntXML .= "<Attributes>";
					$mntXML .= "<DiskGroup><global>$sDiskGroup</global></DiskGroup>"; 
					$mntXML .= "<Volume><global>$sVolume</global></Volume>"; 
				$mntXML .= "</Attributes>";
				$mntXML .= "</Resource>";
			
				$hrDepMountVol-> { $mntResName } = $volResName; 
			}
			# LVMLogicalVolume resource
			if ( defined ($Mount_ins -> { VolumeGroup }     { Value })) {
				$mntXML .= "<Resource>";
				if ( not defined $hrResNames->{ $sLogicalVolume } ) {
					$volResName = "lvol_" . $sLogicalVolume . "_" . $sVolumeGroup;
					$volResName =  $self->GetUniqueResName($volResName);
					Utils::vxlog("DEBUG", "resource Name:$volResName");
					$hrResNames->{ $sLogicalVolume } = $volResName;
				} else {
					$volResName = $hrResNames->{ $sLogicalVolume };
				}

				$mntXML .= "<Name>$volResName</Name>"; 
				$mntXML .= "<Type>LVMLogicalVolume</Type>"; 
				$mntXML .= "<Attributes>";
					$mntXML .= "<VolumeGroup><global>$sVolumeGroup</global></VolumeGroup>"; 
					$mntXML .= "<LogicalVolume><global>$sLogicalVolume</global></LogicalVolume>"; 
				$mntXML .= "</Attributes>";
				$mntXML .= "</Resource>";

				$hrDepMountVol-> { $mntResName } = $volResName; 
			}
			$hrResXMLs -> { $sMountPoint } = $mntXML;
			$hrResNames -> { $sMountPoint } = $mntResName;

			$dgXML = "";	
			if ( defined ($Mount_ins -> { DiskGroup }     { Value })) {
				if ( not defined $hrResNames->{ $sDiskGroup } ) {
					$dgResName = "dg_" . $sDiskGroup;
					$dgResName =  $self->GetUniqueResName($dgResName);
					Utils::vxlog("DEBUG", "resource Name:$dgResName");
					$hrResNames->{ $sDiskGroup } = $dgResName;
				} else {
					$dgResName = $hrResNames->{ $sDiskGroup };
				}

				# MountDg mapping
				$hrMountDg -> { $sMountPoint } = $sDiskGroup;
				$hrDepVolDg -> { $volResName } = $dgResName;
				# DgMounts mapping
				push ( @{$hrDgMounts->{ $sDiskGroup }} , $sMountPoint );

				if ( not defined $hrResXMLs -> { $sDiskGroup } ) {
# DiskGroup resource
					$dgXML .= "<Resource>";
					$dgXML .= "<Name>$dgResName</Name>"; 
					$dgXML .= "<Type>DiskGroup</Type>"; 
					$dgXML .= "<Attributes>";
					$dgXML .= "<DiskGroup><global>$sDiskGroup</global></DiskGroup>"; 
					$dgXML .= "</Attributes>";
					$dgXML .= "</Resource>";

# VMwareDisk resources
					foreach (@DiskIDList) {
						if ( 	( $hrDiskResInfo ne "" ) &&
								( defined $hrDiskResInfo->{$_ ->{Value}}) ){
							$dgXML .= $hrDiskResInfo->{$_ ->{Value}}{ XML };
							push ( @{$hrDepDgDisks -> {$dgResName }}, $hrDiskResInfo->{$_ ->{Value}}{ ResName });
							delete($hrDiskResInfo->{$_ ->{Value}});
						}
					}
					$hrResXMLs -> { $sDiskGroup } = $dgXML;
				}
        		}
			if ( defined ($Mount_ins -> { VolumeGroup }     { Value })) {

				if ( not defined $hrResNames->{ $sVolumeGroup } ) {
					$dgResName = "volg_" . $sVolumeGroup;
					$dgResName =  $self->GetUniqueResName($dgResName);
					Utils::vxlog("DEBUG", "resource Name:$dgResName");
					$hrResNames -> { $sVolumeGroup } = $dgResName;
				} else {
					$dgResName = $hrResNames->{ $sVolumeGroup };
				}


				$hrDepVolDg -> { $volResName } = $dgResName;
				# MountDg mapping
				$hrMountDg -> { $sMountPoint } = $sVolumeGroup;
				# DgMounts mapping
				push ( @{$hrDgMounts->{ $sVolumeGroup }} , $sMountPoint );

				if ( not defined $hrResXMLs -> { $sVolumeGroup } ) {
					$dgXML .= "<Resource>";
					$dgXML .= "<Name>$dgResName</Name>"; 
					$dgXML .= "<Type>LVMVolumeGroup</Type>"; 
					$dgXML .= "<Attributes>";
					$dgXML .= "<VolumeGroup><global>$sVolumeGroup</global></VolumeGroup>"; 
					$dgXML .= "</Attributes>";
					$dgXML .= "</Resource>";

					foreach (@DiskIDList) {
						if ( 	( $hrDiskResInfo ne "" ) &&
								( defined $hrDiskResInfo->{$_ ->{Value}}) ){
							$dgXML .= $hrDiskResInfo->{$_ ->{Value}}{ XML };
							push ( @{$hrDepDgDisks -> {$dgResName }}, $hrDiskResInfo->{$_ ->{Value}}{ ResName });
							delete($hrDiskResInfo->{$_ ->{Value}});
						}
					}
					$hrResXMLs -> { $sVolumeGroup } = $dgXML;
				}
        		}
		}
        } 
END:
	return ($ret);
}

# This function accepts diskgroup as input and returns the disks Id information
# in this case it is the contorller and port number
sub getVxVMDgDisks
{
        my $sDiskGroup  = shift;

	my $sMySubName = ( caller 0 )[3];
	my $announceStr = "Subroutine $sMySubName called";
	Utils::vxlog("INFO", $announceStr);

        my @disk;
        my $ret = CONST_SUCCESS;
        my $ctrl_port = "";
        my @diskIdArr = ();
	my $msg = "";
	scanControllers();	
	my $naming_scheme = "osn";
	my $cmd = "/usr/sbin/vxddladm get namingscheme | tail -1";
	my ($CmdRet, $op) = Utils::runCmd($cmd);

	if ($CmdRet != CONST_SUCCESS) {
		Utils::vxlog("ERROR", "Could not get naming scheme used for VxVM volumes.");
		goto END;
	}

	if ($op =~ /Enclosure/) {
		$cmd = "/usr/sbin/vxddladm set namingscheme=$naming_scheme";
		($CmdRet, $op) = Utils::runCmd($cmd);
		# We shall revert to Enclosure based naming scheme before returning.
		$naming_scheme="ebn";
		if ($CmdRet != CONST_SUCCESS) {
			Utils::vxlog("ERROR", "Error occurred while changing the naming scheme from Enclosure Based to OS Native.");
			goto END;
		}
	}

        my $vxdisk_op = `vxdisk list -g $sDiskGroup | grep $sDiskGroup | awk '{ print \$1" "\$4 }'`;

        my %disklist = split (/\n|\s/, $vxdisk_op );
        foreach (keys %disklist) {
                ($ret, $msg, $ctrl_port ) = getDiskId($_);
                if ($ret == CONST_SUCCESS) {
                	push (@diskIdArr, $ctrl_port);
                } else {
					goto END;
				}
        }
	# Revert to Enclosure based naming scheme before returning.
	if ($naming_scheme =~ /ebn/) {
		$cmd = "/usr/sbin/vxddladm set namingscheme=$naming_scheme";
		($CmdRet, $op) = Utils::runCmd($cmd);
		if ($CmdRet != CONST_SUCCESS) {
			# This message is just marked as info, because the discovery does not get affected by this error.
			Utils::vxlog("WARNING", "Error occurred while changing the naming scheme from OS Native to Enclosure Based.");
			goto END;
		}
	}
END:
        return ($ret, $msg, \@diskIdArr);
}


# This function accepts volume group as input and returns the disks Id information
# in this case it is the contorller and port number
sub getLVMVgDisks
{
    my $lvmvg = shift;
    my $sMySubName = ( caller 0 )[3];
    my $announceStr = "Subroutine $sMySubName called";
    Utils::vxlog("INFO", $announceStr);
    my @diskIdArr = ();
    my $diskname = "";
    my $ctrl_port = "";
    my $msg = "";
    my $ret = CONST_SUCCESS;
    scanControllers();	
    my $cmd = "pvs -o pv_name,vg_name,vg_attr --noheadings";
    my ($CmdRet, $op) = Utils::runCmd($cmd);
    my @vglist = split (/\n/, $op);
    # We expect only one entry to match
    my @vgtuple = grep { /\b$lvmvg\b/ } @vglist;
    my @disks = ();
	foreach (@vgtuple) {
		$_ =~ s/^\s+//;
		$_ =~ s/\s+$//;
		my @diskname = split (/\s+/, $_);
		$diskname[0] =~ s/\/dev\///g;
		push @disks, $diskname[0];
		($ret, $msg, $ctrl_port ) = getDiskId($diskname[0]);
		if ($ret == CONST_SUCCESS) {
			push (@diskIdArr, $ctrl_port);
		} else {
			goto END;
		}

		## This is a HACK. The below code creates a VCS status file for the 
		## volume group if the group is active. This will cause the first 
		## monitor of the volume group resource to return ONLINE. 
		## This should be removed once the appropriate changes are done
		## in the LVMVolumeGroup agent.
		if ($diskname[2] !~ /x/) {
			# Volume Group is active! Check for Logical volumes
    		$cmd = "lvs -o lv_attr $lvmvg --noheadings";
		    ($CmdRet, $op) = Utils::runCmd($cmd);
			my @lvattr = split (/\n/, $op);
			my @attr = grep { /a.\s*$/ } @lvattr;
			if ((scalar @attr )> 0) {
				my $lvgsf = "/var/VRTSvcs/lock/volatile/". ".volg_$lvmvg" . "_lvg";
				open(FH,"> $lvgsf");
				close(FH);
				Utils::vxlog("INFO", "Created status file [$lvgsf] for $lvmvg.");
			}
		}
	}
END:
    return ($ret, $msg, \@diskIdArr);
}

sub getDiskId
{
        # Returns controller and port
        my $disk = shift;
	$disk =~ s/\d+$//;
	my $sMySubName = ( caller 0 )[3];
	my $announceStr = "Subroutine $sMySubName called";
	Utils::vxlog("INFO", $announceStr);
        my $op = `ls -l /dev/disk/by-path/`;
        my @oplist = (split (/\n/, $op));
        my $ret = CONST_SUCCESS;
		my $msg = "";
        my @entry = ();
		@entry = grep { $_ =~ /$disk$/ } @oplist;
        if( scalar @entry == 0 ) {
			($ret, $msg) = getMessage(MSG_FILE, "STORAGE_DISKID_NOT_FOUND", "$disk");
			goto END;
        }
        $entry[0] =~ s/.*pci/pci/;
        my @pathFields = split (':', $entry[0]);
        my $port = -1;
		$pathFields[2] =~ s/-.*//;
        my $sCtrl =  $pathFields[1] . ":" . $pathFields[2];

		($ret, $msg, $port) = getport($sCtrl, $entry[0], $disk);
		if ($ret != CONST_SUCCESS) {
			goto END;
		}
        my $outstr = "$sCtrl:$port";
END:
        return ($ret, $msg, $outstr );
}
sub scanControllers()
{

    my $sMySubName = ( caller 0 )[3];
    my $announceStr = "Subroutine $sMySubName called";
    Utils::vxlog("INFO", $announceStr);
    my $cmd = "for i in /sys/class/scsi_host/host*; do echo \"- - -\" > \$i/scan; done";
    my ($CmdRet, $op) = Utils::runCmd($cmd);
    if ($CmdRet != CONST_SUCCESS) {
	my $logStr = "Failed to scan the scsi controllers.";
	Utils::vxlog("INFO", $logStr);
    }
    return;
}

sub getport
{
	my $sCtrl = shift;
	my $pciStr = shift;
	my $disk = shift;
    my $sMySubName = ( caller 0 )[3];
    my $announceStr = "Subroutine $sMySubName called";
    Utils::vxlog("INFO", $announceStr);
	my $OS = PlatUtils::getOSPADV();
	
	my $port = -1;
	my ($ret, $msg, $CtrlType) = getCtrlType($sCtrl);
	if ($ret != CONST_SUCCESS) {
		goto END;
	}
	if ($CtrlType =~ /SAS/i) {
		my $cmd = "ls -l /sys/block/$disk/device";
		my ($CmdRet, $op) = Utils::runCmd($cmd);
		if ($CmdRet != CONST_SUCCESS) {
			($ret, $msg) = getMessage(MSG_FILE, "STORAGE_DISKPORT_NOT_FOUND", $disk);
			goto END;
		}
		$op =~ s/.*\///;
		my @sasfields = split (':', $op);
		my $sasHostNum = $sasfields[0];
		my $sasPortNum = $sasfields[2];
		$cmd = "ls /sys/class/sas_host/host$sasHostNum/device/port-$sasHostNum:$sasPortNum/ | grep phy-";
		($CmdRet, $op) = Utils::runCmd($cmd);
		if ($CmdRet != CONST_SUCCESS) {
			($ret, $msg) = getMessage(MSG_FILE, "STORAGE_DISKPORT_NOT_FOUND", $disk);
			goto END;
		}
		$op =~ s/.*://;
		$port = $op;
	} else {
        my @pathFields = split (':', $pciStr);
		$port = $pathFields[4];
	}
	chomp $port;
END:
	return ($ret, $msg, $port);
}

sub getCtrlType
{
	my $inCtrl = shift;
    my $sMySubName = ( caller 0 )[3];
    my $announceStr = "Subroutine $sMySubName called";
    Utils::vxlog("INFO", $announceStr);
	my $cmd = "lspci | grep -i SCSI | grep -i controller"; #$ctrl | sed 's/^[0-9a-f :\.]*//' | sed 's/(.*//'";
	my $ctrlType = "";
	my $msg = "";
	my ($CmdRet, $op) = Utils::runCmd($cmd);
	if ($CmdRet != CONST_SUCCESS) {
		#$msg = "Could not discover the details of the controllers attached to the virtual machine.";
		($CmdRet, $msg) = getMessage(MSG_FILE, "STORAGE_CONTROLLER_NOT_FOUND");
		goto END;
	}
	my @ctrllist = split ('\n', $op);
	foreach my $ctrl (@ctrllist) {
		if ($ctrl =~ /^$inCtrl/) {
			$ctrlType = $ctrl;
			$ctrlType =~ s/^[0-9a-f :\.]*//;
			$ctrlType =~ s/\(.*//;
			last;
		}
	}
	# We only need to handle SAS controllers differently.
	if ($ctrlType =~ /SAS/) {
		$ctrlType = "SAS";
	}
END:
	return ($CmdRet, $msg, $ctrlType);
}

1;

