
#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
# 
# $Id: UnixNetwork.pm,v 1.19 2012/08/17 09:58:41 lpatil Exp $
#

package UnixNetwork;

use strict;
use warnings;

use Data::Dumper;
use Constants;
use Utils;
use PlatUtils;
use Storage;

use base qw(Exporter);
our @EXPORT = qw(
		getNetworkXML
		createSystemNICSGDetails
	 );

sub createSystemNICSGDetails 
{
	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $self   = shift;
	my $vipref = shift;
	my $sysRef = shift;
	
	my $ret = CONST_SUCCESS;
	my $sSystemNICSGxml = "";
	my $hrSystemXMLs = ();
        my @vip = Utils::convertHashValuesToArray($vipref);
	my @sysList;

	if ( $sysRef and ref($sysRef) =~ /ARRAY/ ) {
		@sysList = @$sysRef;
	}

	my $sysListPassed = 0;

	if ( @sysList and scalar (@sysList) >= 1 ) {
		$sysListPassed = 1;
	}

	foreach my $hrVirtualIP (@vip) {
		my @nics = ();
		my $nicsref = $hrVirtualIP->{NIC};

		if ( defined ($hrVirtualIP->{NIC}) ) {
			@nics = Utils::convertHashValuesToArray($nicsref);

			foreach my $hrnic ( @nics) {
				my $sSystemName = $hrnic->{System};

				if ($sysListPassed == 0 or (grep (/^$sSystemName$/, @sysList)) ) {

					my $sDeviceName = $hrnic->{Device};
					my $mii = "0";
					my $vcsobj = new VCS();
					my $clusDetailsInputRef = ();
					$clusDetailsInputRef->{Resources}{Types} = ["NIC"];
					$clusDetailsInputRef->{Resources}{Attributes} = [ "Device", "Mii" ];
					$clusDetailsInputRef->{Resources}{AttributesVal}{Device}{$sSystemName} = "$sDeviceName";
					$clusDetailsInputRef->{Resources}{AttributesVal}{Mii}{global} = "$mii";
					my ($retVal, $MatchedRes) = $vcsobj->isSimilarResourceExist($clusDetailsInputRef);
					if ($retVal == 0) {
						if ( ( not defined $hrSystemXMLs->{$sSystemName} ) or
								( $hrSystemXMLs->{$sSystemName} eq "" )){
							# Creating new NIC SG for $sSystemName
							# if NIC xml for that system is not defined

							my $sSysNICSG = $sSystemName. "_NIC_SG";
							$sSysNICSG =  $self->GetUniqueGroupName($sSysNICSG);
							Utils::vxlog("DEBUG", "createSGs: Group Name:[$sSysNICSG]");

							$hrSystemXMLs->{$sSystemName} = "<ServiceGroup><Name>" .
								$sSysNICSG .
								"</Name><Attributes>" .
								"<SystemList> <global>" . $sSystemName . "</global>" .
								"<global>0</global></SystemList>" .
								"<UserAssoc>" .
								"<global>Type</global>" .
								"<global>vcs internal</global>" .
								"<global>Name</global>" .
								"<global>NIC service group</global>" .
								"</UserAssoc>" .
								"</Attributes><Resources>";

							my $sPhantomResName = "Phantom_NIC_SG" . $sSystemName;
							$sPhantomResName =  $self->GetUniqueResName($sPhantomResName);
							Utils::vxlog("DEBUG", "resource Name:$sPhantomResName");
							$hrSystemXMLs->{$sSystemName} .= "<Resource><Name>" .
								$sPhantomResName .
								"</Name><Type>Phantom</Type></Resource>";
						}
						my $sNICResName = "NIC_". $sSystemName . "_" . $sDeviceName ;
						$sNICResName =  $self->GetUniqueResName($sNICResName);

						if ( not defined $hrSystemNICResName->{$sSystemName}{$sDeviceName} ) {
							$hrSystemNICResName->{$sSystemName}{$sDeviceName} = $sNICResName;

							$hrSystemXMLs->{$sSystemName} .= "<Resource><Name>" .
								$sNICResName .
								"</Name><Type>NIC</Type><Attributes><Device>" .
								"<$sSystemName>" . $sDeviceName . "</$sSystemName>".
								"</Device><Mii><global>$mii</global></Mii></Attributes></Resource>";
						}
					} else {
						if ( not defined $hrSystemNICResName->{$sSystemName}{$sDeviceName} ) {
							$hrSystemNICResName->{$sSystemName}{$sDeviceName} = $MatchedRes;
						}
					}
				}
			}
		}
	}

	foreach my $sys (keys %{$hrSystemXMLs} ) {
		$hrSystemXMLs->{$sys} .= "</Resources></ServiceGroup>";
		$sSystemNICSGxml .= $hrSystemXMLs->{$sys};
	}

        return ( $ret , $sSystemNICSGxml);
}

sub getNetworkXML
{
	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $self = shift;
	my $tree = shift;
	my $ret = CONST_SUCCESS;
	my $sSystemNICSGxml = "";
	my $net_res_xml = "";
	my $ipResXml = "";
	my $proxyResXml = "";
	my $net_deps_xml = "";
	my $sDeviceName = "";
	my $ipResName = "";
        if ((defined ($tree->{VirtualIPInfo})) and ($tree->{VirtualIPInfo} ne "")) {

		# <InstanceIPMapping>
		#	<Instance>AmitV</Instance>
		# 	<Address>7.7.7.7</Address>
		# </InstanceIPMapping>
		my @vipmapping = ();
		if ( defined $tree->{VirtualIPInfo}{InstanceIPMapping} ) {
			my $vipinstref = $tree->{VirtualIPInfo}{InstanceIPMapping};
			@vipmapping = Utils::convertHashValuesToArray($vipinstref);
		}
                my @vip = ();
		if ( defined $tree->{VirtualIPInfo}{VirtualIP} ) {
			my $vipref = $tree->{VirtualIPInfo}{VirtualIP};
			( $ret , $sSystemNICSGxml) = $self -> createSystemNICSGDetails($vipref);
			if ( $ret == CONST_FAILURE ) {
				Utils::vxlog("ERROR", "Error while creating NIC SG");
				goto END;
			}
			@vip = Utils::convertHashValuesToArray($vipref);
		}

                foreach (@vip) {
                        if ( defined ($_->{Type}) && defined ($_->{Address}) ){
                                $ipResXml = "<Resource>";
                                $proxyResXml = "<Resource>";
                                my $ip = $_->{Address};
                                $ip =~ s/(\.|:)/-/g;
				my $ipResName_key = "IP_" . $ip;
				$ipResName =  $self->GetUniqueResName($ipResName_key);
				Utils::vxlog("DEBUG", "IP resource: key [$ipResName_key] Name [$ipResName]");
				if ( not defined $hrResNames->{$ipResName_key} ) {
					$hrResNames->{$ipResName_key} = $ipResName;
				} else {
					Utils::vxlog("WARNING", "IP address [". $_->{Address} . "] specified again. Corresponding IP resource dependencies may not be created.");
					next;
				}


                                my $proxyResName_key = "NICProxy_" . $ip;
				my $proxyResName =  $self->GetUniqueResName($proxyResName_key);
				Utils::vxlog("DEBUG", "resource Name:$proxyResName");

                                $ipResXml .= "<Name>$ipResName</Name>";
                                $ipResXml .= "<Type>IP</Type>";
                                $ipResXml .= "<Attributes>";
                                $ipResXml .= "<Address>";
                                $ipResXml .= "<global>$_->{Address}</global>";
                                $ipResXml .= "</Address>";
                                if ( $_->{Type} =~ /ipv4/i && defined ($_->{NetMask}) ) {
                                        $ipResXml .="<NetMask>";
                                        $ipResXml .="<global>$_->{NetMask}</global>";
                                        $ipResXml .="</NetMask>";
                                }
				elsif ( $_->{Type} =~ /ipv6/i && defined ($_->{Prefix}) ) {
					my @prefixlen = split(/\//,$_->{Prefix});
					$ipResXml .="<PrefixLen>";
					$ipResXml .="<global>$prefixlen[1]</global>";
					$ipResXml .="</PrefixLen>";
				}

                                my @nics = ();
                                my $nicref = $_->{NIC};
                                if ( defined ($_->{NIC}) ) {
                                        @nics = Utils::convertHashValuesToArray($nicref);
                                        $ipResXml .= "<Device>";
                                        foreach ( @nics) {
                                                $ipResXml .="<$_->{System}>$_->{Device}</$_->{System}>";
					}
                                        $ipResXml .= "</Device>";

					$proxyResXml .= "<Name>$proxyResName</Name>";
					$proxyResXml .= "<Type>Proxy</Type>";
					$proxyResXml .= "<Attributes>";
                                        $proxyResXml .= "<TargetResName>";
					# Actually we should not form sTargetResName but query it
					my $sTargetResName = "";
                                        foreach ( @nics) {
						my $sSystemName = $_->{System};
						$sDeviceName = $_->{Device};
			
						if ( defined $hrSystemNICResName->{$sSystemName}{$sDeviceName} ){
							$sTargetResName = $hrSystemNICResName->{$sSystemName}{$sDeviceName};
						} else {
							Utils::vxlog("ERROR", "Could not find target resource name for System [$sSystemName] Device [$sDeviceName].");
						}
                                                $proxyResXml .="<$_->{System}>$sTargetResName</$_->{System}>";
					}
                                        $proxyResXml .= "</TargetResName>";
					$proxyResXml .= "</Attributes>";
					$proxyResXml .= "</Resource>";
                                }
                                $ipResXml .= "</Attributes>";
                                $ipResXml .= "</Resource>";

				$hrResXMLs -> { $ipResName_key } = $ipResXml;
				$hrResNames -> { $ipResName_key } = $ipResName;

				$hrResXMLs -> { $proxyResName_key } = $proxyResXml;
				$hrResNames -> { $proxyResName_key} = $proxyResName;
				$hrIPProxy -> { $ipResName_key } = $proxyResName_key;
                        }
                }

		foreach my $hrInstIPmapping (@vipmapping) {
			if ( defined ($hrInstIPmapping->{Instance}) && defined ($hrInstIPmapping->{Address}) ){ 
				my @aInstances = Utils::convertHashValuesToArray($hrInstIPmapping->{Instance});
				my $sInstName = "";
				my $sIPAddress = $hrInstIPmapping->{Address};

				# Call back to update Instance information in Network mapping
				$ret = $self->updateNetworkInstanceInfo(\@aInstances, $sIPAddress);
				if ( $ret == CONST_FAILURE ) {
					Utils::vxlog("ERROR", "Error while updating instance information in network mapping");
					goto END;
				}
                                $sIPAddress =~ s/(\.|:)/-/g;
				my $ipResName_key = "IP_" . $sIPAddress;
				if ( defined $hrResNames->{$ipResName_key} ) {
					$ipResName =  $hrResNames->{$ipResName_key};
				} else {
					Utils::vxlog("ERROR", "Could not find resource Name for [$ipResName_key]");
                			$ret = CONST_FAILURE;
					goto END;
				}


				foreach $sInstName (@aInstances) {
					$hrInstanceIP -> { $sInstName } = $ipResName_key;
					push (@{$hrIPInstances -> { $ipResName_key }}, $sInstName);
				}
			}
		}


        }
END:
        return ( $ret , $sSystemNICSGxml);
}

1;
