
#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $ 
#
# $Id: WebSphereMQ.pm,v 1.35.8.7 2013/06/17 16:25:55 lpatil Exp $
#

package WebSphereMQ;
@ISA = ("UnixApplication");  # required for Inheritance

use strict;
use warnings;
use Scalar::Util 'blessed';
use UnixWebSphereMQCodes;
use UnixApplication;
use Data::Dumper qw(Dumper);
use Constants;
use Utils;
use PlatUtils;
use Storage;
use UnixNetwork;

#-------------
# Gloabals
#-------------
my $WebSphereMQResType = "WebSphereMQ";

#-------------------
# Subroutine section
#-------------------
sub discover
{
	my $self = shift;
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "Discovery successful.";
	my $xml = "";
	my $instances = ();

	my $packageName = "" ;
	my $packageVersion = "5.1.14.0" ;
	
	($ret,$packageName) = PlatUtils::isAgentPackageInstalled("WebSphereMQ", $packageVersion);
	
	if ( $ret != PlatUtils::PACKAGE_INSTALLED ) {
		$msg = "Appropriate version of Package [$packageName] is not installed. Install version [$packageVersion] or higher, and then retry.";
		Utils::vxlog("INFO", $msg);
		return (APPCONTROL_WEBSPHEREMQ_DISCOVER_WEBSPHEREMQ_AGENTNOTINSTALLED, $msg, $xml);
	}	
	else
	{
		$logStr = "Package [$packageName] with version [$packageVersion] or higher, is installed";
		Utils::vxlog("INFO", $logStr);
	}

	$ret = PlatUtils::isAcclibPackageInstalled("WebSphereMQ");

	if ( $ret != PlatUtils::PACKAGE_INSTALLED ) {
		$msg = "Package [VRTSacclib] for the WebSphereMQ agent is not installed. Install the package and then retry";
		Utils::vxlog("INFO", $msg);
		return ( APPCONTROL_WEBSPHEREMQ_DISCOVER_ACCLIB_NOTINSTALLED , $msg, $xml);
	}
	else
	{
		$logStr = "Package [VRTSacclib] is installed.";
		Utils::vxlog("INFO", $logStr);
	} 

	($ret, $msg, $xml) = $self->doDiscovery();
        Utils::vxlog("INFO", "Discovered WebSphereMQ configuration information.");
        Utils::vxlog("INFO", $xml);
	return ($ret, $msg, $xml);
}


sub _discover_qmgr_names
{
	my $self = shift;
	my $sMyName = ( caller 0 )[3];
	my $resRef = shift;
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my @instances = ();
	my $cmd = "ls -1 /var/mqm/qmgrs";
	my $CmdRet = CONST_SUCCESS;
	my $op = "";
	($CmdRet, $op) = Utils::runCmd($cmd);
	if ( $CmdRet != CONST_SUCCESS ) {
			goto END;
	}

	my @lines = split(/\n/, $op);
	foreach my $mgr ( @lines ) {
		if ( $mgr !~ /SYSTEM$/i ) {

			$mgr =~ s/\!/\./g;
			$mgr =~ s/\&/\//g;

			if(CONST_SUCCESS == $self->isQueueManagerPresent($mgr, $resRef)) {
				$logStr = "Skipping the already configured instance of WebSphereMQ Queue Manager: [$mgr]";
				Utils::vxlog("INFO", $logStr);
				next;
			}
			push ( @instances, $mgr );
		}
	}

END:
	return \@instances;
}

sub getMounts
{
	my $sMyName = ( caller 0 )[3];
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

        my $data = `mount`;
        my @mount_data= split(/\n/, $data);
        my ($mount_point,$BlkDev,$fstype) = ();
        my @mounts = ();

        foreach my $mount( @mount_data ) {
                chomp $mount;
                if ( $mount =~ m/^(\S+)\s(\S+)\s(\S+)\stype\s(\w+).*/ ) {
                        $BlkDev = $1;
                        $mount_point= $3;
                        $fstype = $4;
                        next if ( $mount_point eq "/" );
                }
                if ( $BlkDev =~ m/^\/dev\/vx\/dsk\/([a-zA-Z0-9_-]+)\/([a-zA-Z0-9_-]+).*/  || $BlkDev =~ m/^\/dev\/mapper\/([a-zA-Z0-9_+\.-]+)-([a-zA-Z0-9_+\.-]+).*/ ) {
                        push(@mounts,$mount_point);
                }
        }
        return @mounts;
}

sub getAllMountPoints
{
	my $self = shift;
	my $sMyName = ( caller 0 )[3];
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);
	my $msg = "";

        my @Storageinsdata = ();
        my $SysDiskIDdata = ();

	my ($hrStorageDetails) = ();	
	my ($sDiskGroup, $sVolumeGroup) = ();
	my @mountpoints = ();
	@mountpoints = getMounts();
	my $Storage_ret = CONST_SUCCESS;
	my $arDiskIDs = ();
	my $iCnt =1;
	my (@StorageData) = ();
	my $resRef = $self->discoverExistingAppConfig();

	foreach my $mountpoint( @mountpoints) { 
		$arDiskIDs = ();
		@StorageData = ();
		($Storage_ret, $hrStorageDetails) = $self->GetStorageDetails ($mountpoint);
		if ( CONST_SUCCESS == $Storage_ret ) {
			$hrStorageDetails -> { Instance } = "mq" . "_" . "$iCnt";

			if ( defined $hrStorageDetails -> { DiskGroup } ) {
				$sDiskGroup = $hrStorageDetails -> { DiskGroup };
				($Storage_ret, $msg, $arDiskIDs) = getVxVMDgDisks($sDiskGroup);
				if ($Storage_ret != CONST_SUCCESS) {
					goto END;
				}
			}
			if ( defined $hrStorageDetails -> { VolumeGroup } ) {
				$sVolumeGroup = $hrStorageDetails -> { VolumeGroup };
				($Storage_ret, $msg, $arDiskIDs) = getLVMVgDisks($sVolumeGroup);
				if ($Storage_ret != CONST_SUCCESS) {
					goto END;
				}
			}

			$hrStorageDetails -> { DiskID }  = $arDiskIDs;
			# Generate uniq DiskIDs
			foreach my $DiskID ( @$arDiskIDs ) {
				$SysDiskIDdata -> { $DiskID } = 1;
			}
			push ( @StorageData, $hrStorageDetails);
			if (CONST_SUCCESS != $self->isStorageConfigured($resRef, \@StorageData)) {
				push ( @Storageinsdata, $hrStorageDetails);
			}
			else {
				Utils::vxlog("INFO", "Skipping mount point [$mountpoint] since there is already resource configured in VCS");
			}
		} else {
			Utils::vxlog("INFO", "Could not discover storage for [$mountpoint]");
		}
		$iCnt++;
	}
END:
	return ( $Storage_ret, $msg, @Storageinsdata );
}

sub doDiscovery
{
	my $self = shift;
	my $sMyName = ( caller 0 )[3];
	my $msg = "Discovery successful.";
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my ($ret, $xmlstr) = ();
	$ret = CONST_SUCCESS;
	$xmlstr = "";

	Utils::vxlog("INFO", "Determining the WebSphereMQ version ...");
	my $MQVersion = "7.0";
	my $hrQueueManagers = ();
        my $version_str = "";
        my $sMQInstallationPath = "";
	my $bMultipleInstallationSupported = 0;

	# Command: rpm -qa --qf "%{NAME} %{INSTPREFIXES} %{VERSION} %{RELEASE}\n" | grep MQSeriesRuntime
	
	my ($CmdRet, $op) = Utils::runCmd("rpm -qa --qf \"%{NAME} %{INSTPREFIXES} %{VERSION} %{RELEASE}\n\" | grep MQSeriesRuntime"); 
	if ($CmdRet != CONST_SUCCESS) {
		$msg = "WebSphere MQ SeriesRuntime RPM not found.";
		Utils::vxlog("ERROR", $msg);
		$ret = CONST_FAILURE;
		goto END;
	}

        my @lines = split(/\n/, $op);
        foreach my $line (@lines) {
                if ($line =~ /^MQSeriesRuntime/) {
			# e.g. 1) MQSeriesRuntime_m7 /opt/customLocation 7.1.0 0
			# e.g. 2) MQSeriesRuntime /opt/mqm 7.1.0 0
			Utils::vxlog("INFO", "WebSphere MQ installation discovered. [$line]");

			my @fields = split(/\s/, $line);
                        $version_str = $fields[2];

			if ( $version_str =~ /^6/ ) {
				$MQVersion = "6.0";
			} elsif ( $version_str =~ /^7.0/ ) {
				$MQVersion = "7.0";
			} else {
				$MQVersion = "7.1";
				$bMultipleInstallationSupported = 1;
				$sMQInstallationPath = $fields[1];		
				Utils::vxlog("INFO", "WebSphere MQ version [$version_str] is installed here: [$sMQInstallationPath]");
				last;
			}
		}
	}
	
	# From WebSphere MQ version 7.1 onwards following changes are supported:
	# 1. Multiple installations (Default installation path is /opt/mqm)
	# 2. Custom DataPath location for Queue Manager (Default DataPath is /var/mqm/qmgrs/QM_name
	# 3. Custom LogPath location for Queue Manager (Default LogPath is /var/mqm/log/QM_name
	
	# MQ 7.0 example
	# /opt/mqm/bin/dspmq 
	# QMNAME(QM1)                                               STATUS(Ended unexpectedly)
	# QMNAME(Paint.queue.manager)                               STATUS(Ended normally)

	# MQ 7.1 example
	# /opt/mqm/bin/dspmq -o installation
	# QMNAME(QM2) INSTNAME(Installation1) INSTPATH(/opt/mqm) INSTVER(7.1.0.0)
	# QMNAME(QM1)   INSTNAME(Installation1) INSTPATH(/opt/mqm) INSTVER(7.1.0.0)
	
	my $sCommand = "su mqm -c \'/opt/mqm/bin/dspmq\'"; # MQ 7.0
	if ( $bMultipleInstallationSupported ) {
		$sCommand = "su mqm -c \'$sMQInstallationPath/bin/dspmq -o installation\'"; # MQ 7.1 onwards
	}
	my ( $dspmqOutput, $dspmqinfOutput ) = ( "", "" );
	($CmdRet, $dspmqOutput) = Utils::runCmd("$sCommand"); 
	if ($CmdRet != CONST_SUCCESS) {
		$msg = "Unable to display WebSphere MQ instances using dspmq command.";
		Utils::vxlog("ERROR", $msg);
		$ret = CONST_FAILURE;
		goto END;
	}
	my @dspmqOutputlines = split(/\n/, $dspmqOutput);
	my ( $qmName, $tmpqmName, $qmInstPath, $qmInstVer, $qmDataPath ) =( "", "", "", "", "" );
	foreach my $qmLine (@dspmqOutputlines) {
		Utils::vxlog("INFO", "Processing [$qmLine].");
		if ( $qmLine =~ /QMNAME\((\S+)\)\s+INSTNAME\((\S+)\)\s+INSTPATH\((.+?)\)\s+INSTVER\((\S+)\)/){
			$qmName = $1;
			if ( not defined $hrQueueManagers -> { $qmName } { INSTPATH } ) {
				$hrQueueManagers -> { $qmName } { INSTPATH } = $qmInstPath = $3;
				$hrQueueManagers -> { $qmName } { INSTVER } = $qmInstVer = $4;
				# An installation of MQ vesion 7.0.1 can co-exists on the system, 
				# along with MQ version 7.1 or later
				if ( $qmInstVer =~ /^7.0/ ) {
					$hrQueueManagers -> { $qmName } { INSTVER } = "7.0";
				}
				if ( $qmInstVer =~ /^7.1/ ) {
					$hrQueueManagers -> { $qmName } { INSTVER } = "7.1";
				}
				if ( $qmInstVer =~ /^7.5/ ) {
					$hrQueueManagers -> { $qmName } { INSTVER } = "7.5";
				}
			}
		} elsif ( $qmLine =~ /QMNAME\((\S+)\)\s+STATUS/){ 
			$qmName = $1;
                        $hrQueueManagers -> { $qmName } { INSTPATH } = "/opt/mqm";
		} else {
			Utils::vxlog("INFO", "Skipping line [$qmLine] from output of dspmq command.");
			next;
		}

		$dspmqinfOutput = "";
		if ( $qmInstPath eq "" ){
			$qmInstPath = "/opt/mqm";
		}
		if ( -f "$qmInstPath/bin/dspmqinf" ) {
			Utils::vxlog("INFO", "[$qmInstPath/bin/dspmqinf] is present.");
			# Find the DataPath for QM 
			($CmdRet, $dspmqinfOutput) = Utils::runCmd("su mqm -c \'$qmInstPath/bin/dspmqinf -s QueueManager -o stanza $qmName\'"); 
			if ($CmdRet != CONST_SUCCESS) {
				$msg = "Error occurred while displaying information for queue manager [$qmName]. Verify that, as 'mqm' user, [dspmqinf] command correctly displays the queue manager [$qmName] information.";
				Utils::vxlog("ERROR", $msg);
				$ret = CONST_FAILURE;
				goto END;
			}
			#$ dspmqinf -s QueueManager -o stanza -o stanza QM789
			#QueueManager:
			   #Name=QM789
			   #Directory=QM789
			   #Prefix=/var/mqm
			   #DataPath=/var/mqm/789/qmgrs/QM789
			   #InstallationName=Installation1

			my @dspmqinfOut = split(/\n/, $dspmqinfOutput);
			foreach my $infLine (@dspmqinfOut) {
				#   DataPath=/var/mqm/123/qmgrs/QM123
				if ($infLine =~ /DataPath=(.+)/) {
					$hrQueueManagers -> { $qmName } { DATAPATH } =$qmDataPath = $1;
					Utils::vxlog("INFO", "Found DataPath [$qmDataPath].");
					last;
				}
			}
		} else {
			Utils::vxlog("INFO", "[$qmInstPath/bin/dspmqinf] not found.");
		}
		# Handle special character in queue manager name.
		$tmpqmName = $qmName;
		$tmpqmName =~ s/\./\!/g;
		$tmpqmName =~ s/\//\&/g;
		if ( not defined $hrQueueManagers -> { $qmName } { DATAPATH } ) {
			$hrQueueManagers -> { $qmName } { DATAPATH } = $qmDataPath = "/var/mqm/qmgrs/$tmpqmName" ;
		}

		# Find the LogPath for QM 
		my $sQMiniFile = "$qmDataPath/qm.ini";
		if ( ! -e $sQMiniFile ){
			$msg = "File [$sQMiniFile] for [$tmpqmName] not found.";
			Utils::vxlog("INFO", $msg);
			next;
		}
		my $iReturn = open (FH,"$sQMiniFile"); 
		if ( ! $iReturn )
		{
			$msg = "Wizard is unable to access [$sQMiniFile] file.";
			Utils::vxlog("ERROR", $msg);
			$ret = CONST_FAILURE;
			goto END;
		} else {
			my @FA = <FH> ;
			foreach my $sLine ( @FA ) {
				chomp $sLine;
				if ($sLine =~ /^\s*LogPath=(.+)/) {
					$hrQueueManagers -> { $qmName } { LOGPATH } = $1;
					Utils::vxlog("INFO", "Found LogPath [$1].");
				}
			}
			close FH;
		}
		if ( not defined $hrQueueManagers -> { $qmName } { LOGPATH } ) {
			$hrQueueManagers -> { $qmName } { LOGPATH } = "/var/mqm/log/$tmpqmName" ;
		}
	}
	Utils::vxlog("INFO", Dumper( $hrQueueManagers ));

	# Discover existing config
	my $resRef = $self->discoverExistingAppConfig();

	# Get available WebSphereMQ instances for processing ...	
	# skip already configured WebSphereMQ Instances
	my $AvailableMQ = $self->_discover_qmgr_names($resRef);

	# Find out Hostname ...
	use Net::Domain qw(hostname hostfqdn hostdomain);
	my $hostname = hostname ;
	my @cols = split(/\./, $hostname, 2);
	$hostname = $cols[0];
	Utils::vxlog("INFO", "Hostname: [$hostname]");

	my @insdata = ();	
	my @allStorageData = ();
	my @SysDiskIDs = ();
	my $SysDiskIDdata = ();	

	foreach my $MQInst ( @$AvailableMQ ) {
		if ( 	( $bMultipleInstallationSupported ) and 
			( not defined $hrQueueManagers -> { $MQInst } ) ) {
			Utils::vxlog("WARNING", "Discovery skipped for queue manager [$MQInst]. Could not get details from 'dspmq' command.");
			next;
		}
		if ( 	( $bMultipleInstallationSupported ) and 
			( not defined $hrQueueManagers -> { $MQInst } { LOGPATH } ) ) {
			Utils::vxlog("INFO", "Discovery skipped for queue manager [$MQInst]. LOGPATH is not found for queue manager [$MQInst].");
			next;
		}
		my $data = ();
		my (@instanceStorageData) = ();
		my ($arApplicationPath, $arApplicationMounts) = ();
		$data -> { Instance } = $MQInst;
		$data -> { MQVersion } = $MQVersion;
		if ( defined $hrQueueManagers -> { $MQInst } { INSTVER } ) {
			$data -> { MQVersion } = $hrQueueManagers -> { $MQInst } { INSTVER };
		}
		$data -> { ListenerOption } = 1;
		$data -> { ListenerPort } = "";
		$data -> { EnvFile } = "";

		$tmpqmName = $MQInst;
		$tmpqmName =~ s/\./\!/g;
		$tmpqmName =~ s/\//\&/g;
		my $sDataPath = "/var/mqm/qmgrs/" . $tmpqmName; 
		my $sLogPath  = "/var/mqm/log/" . $tmpqmName;   
		my $sInstPath = "/opt/mqm";

		# From WebSphere MQ version 7.1 onwards following changes are supported:
		# 1. Multiple installations (Default installation path is /opt/mqm)
		# 2. Custom DataPath location for queue manager (Default DataPath is /var/mqm/qmgrs/QM_name
		# 3. Custom LogPath location for queue manager (Default LogPath is /var/mqm/log/QM_name
		
		if ( defined $hrQueueManagers -> { $MQInst } { DATAPATH } ) {
			$sDataPath = $hrQueueManagers -> { $MQInst } { DATAPATH };
		}

		if ( defined $hrQueueManagers -> { $MQInst } { LOGPATH } ) {
			$sLogPath = $hrQueueManagers -> { $MQInst } { LOGPATH };
		}
		if ( defined $hrQueueManagers -> { $MQInst } { INSTPATH } ) {
			$sInstPath = $hrQueueManagers -> { $MQInst } { INSTPATH };
			$data -> { MQInstallationPath } = $sInstPath;
		}

		Utils::vxlog("INFO", "InstPath: [$sInstPath] ");
		Utils::vxlog("INFO", "DataPath: [$sDataPath] ");
		Utils::vxlog("INFO", "LogPath : [$sLogPath] ");
		Utils::vxlog("INFO", "Hostname: [$hostname]");

		# If the application binaries are on LVM, active-active setup not feasible. Hence we should not add InstPath for configuration. 

		# Storage discovery for the instance
		my ($hrStorageDetails) = ();
		my ($sMountPoint, $sBlockDevice, $sFSType, $sDiskGroup, $sVolume, $sVolumeGroup, $sLogicalVolume) = ();
		Utils::vxlog("INFO", "Discover storage for Queue Manager [$MQInst] with DataPath [$sDataPath]");

		my $Storage_ret = CONST_SUCCESS;
		my $arDiskIDs;
		my $iCnt =1;
		foreach my $sApplicationPath ( @$arApplicationPath) { 
			chomp $sApplicationPath; 
			$sApplicationPath =~ s/\/$//; # Removed the / character at the end of $sApplicationPath

			($Storage_ret, $hrStorageDetails) = $self->GetStorageDetails ($sApplicationPath);

			if ( CONST_SUCCESS == $Storage_ret ) {
				push(@$arApplicationMounts,$hrStorageDetails->{MountPoint});
				$hrStorageDetails -> { Instance } = "$MQInst" . "_" . "$iCnt";

				# Below calls getVxVMDgDisks() and getLVMVgDisks() perform following tasks:
				# 1. get list of disks for dg
				# 2. get controller_port information (DiskID) for each disk.
				if ( defined $hrStorageDetails -> { DiskGroup } ) {
					$sDiskGroup = $hrStorageDetails -> { DiskGroup };
					# DiskID is of the form [Controller:Port] 
					($Storage_ret, $msg, $arDiskIDs) = getVxVMDgDisks($sDiskGroup);
					if ($Storage_ret != CONST_SUCCESS) {
						$ret = $Storage_ret;
						goto END;
					}
				}
				if ( defined $hrStorageDetails -> { VolumeGroup } ) {
					$sVolumeGroup = $hrStorageDetails -> { VolumeGroup };
					($Storage_ret, $msg, $arDiskIDs) = getLVMVgDisks($sVolumeGroup);
					if ($Storage_ret != CONST_SUCCESS) {
						$ret = $Storage_ret;
						goto END;
					}
				}

				$hrStorageDetails -> { DiskID }  = $arDiskIDs; 
				# Generate uniq DiskIDs
				foreach my $DiskID ( @$arDiskIDs ) {
					$SysDiskIDdata -> { $DiskID } = 1;
				}

				push ( @instanceStorageData, $hrStorageDetails);
			} else {
				Utils::vxlog("INFO", "Shared storage not found for [$sApplicationPath]");
			}
			$iCnt++;
		} # for loop for all application mounts

		# Skipping the queue manager instances depending on already configured storage 
		if (CONST_SUCCESS == $self->isStorageConfigured($resRef, \@instanceStorageData)) {
			$logStr = "Skipping the WebSphereMQ queue manager: [$MQInst], because the storage is already configured for an application running under VCS control.";
			Utils::vxlog("INFO", $logStr);
			next;
		}
		@allStorageData = (@allStorageData, @instanceStorageData);

		push ( @insdata, $data);
	}
	my @Storageinsdata = ();
	($ret, $msg, @Storageinsdata) = $self->getAllMountPoints();
	if ($ret != CONST_SUCCESS) {
		goto END;
	}
	if (scalar (@insdata) >= 1) {
		$xmlstr .= "<DiscoveryData>";
			$xmlstr .= $self->createApplicationXML( \@insdata, "Instance" );
			$xmlstr .= "<StorageList>";
			$xmlstr .= $self->createApplicationXML( \@Storageinsdata, "Storage" );
			$xmlstr .= "</StorageList>";
		$xmlstr .= "</DiscoveryData>";
	} else {
		return (APPCONTROL_WEBSPHEREMQ_DISCOVER_NEWDISCOVERYFAILURE, "No new WebSphere MQ queue manager instance found. All instances are either already configured or use the storage which is already configured for an application running under VCS control.", "");
	}

END:
	return ( $ret, $msg, $xmlstr );
}

#---------------------------------------------
# Overriding the createCommitSGDetails method
#---------------------------------------------

sub createCommitSGDetails
{
	my $self = shift;
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
	#-------------------
	# Announce ourselves
	#-------------------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	# Initialize return parameters
	my ( $ret, $msg, $xml, $insxml, $lsnrxml, $insResName) = (CONST_SUCCESS, "", "", "","", "");
	my ( $hrInsResXMLs, $hrInsResNames ) = ();

	Utils::vxlog("INFO", "Commit input XML: $params");
	my $tree = Utils::convertXMLTextToHash( $params );
	Utils::vxlog("INFO", Dumper( $tree ));

	$xml =  "<xml>";

#------------------------------------------------------
	my @inst = ();
	if ( defined ($tree->{Instance})) {
		@inst = Utils::convertHashValuesToArray( $tree->{Instance} );

		# loop for Application instances 
		foreach my $ins (@inst) {
			my $Name = $ins -> { Name };
			push ( @gInstances, $Name);
			my $ListenerOption	= $ins -> { ListenerOption } { Value };
			my $ListenerPort = $ins -> { ListenerPort }	 { Value };
			my $MQVersion       = $ins -> { MQVersion }	     { Value };
			my $EnvFile         = $ins -> { EnvFile }        { Value };
			my $sInstPath = "/opt/mqm";
			if ( ( defined $ins -> { MQInstallationPath } ) and 
			     ( defined $ins -> { MQInstallationPath } { Value } ) ) {
				$sInstPath = $ins -> { MQInstallationPath } { Value };
			}
			if ( $EnvFile ne "" and not -f $EnvFile ) {
				$ret = APPCONTROL_WEBSPHEREMQ_COMMIT_ENVFILE_NOT_PRESENT; 
				$msg = "Specified EnvFile [$EnvFile] not present.";
				goto END;
			}
			if ($ListenerOption == 3) {
				Utils::vxlog("INFO", "Checking ListenerPort [$ListenerPort]");
				if ( $ListenerPort ne "" ) {
					if ( $ListenerPort =~ /\D/ ) {
						$ret = APPCONTROL_WEBSPHEREMQ_COMMIT_EMPTY_LISTENER_IP;
						$msg = "Listener Port should be positive integer value.";
						goto END;
					} else {
						$ListenerPort = "-p " . $ListenerPort;
					}
				}
			}
			if (( defined ($ins->{MountPointList})) and 
			    ($ins -> {MountPointList} ne "") and
			    ( defined ($ins->{MountPointList}{MountPoint})) ) {
				my @sMountPoints = Utils::convertHashValuesToArray ($ins ->{MountPointList}{MountPoint});
				foreach my $sMountPoint (@sMountPoints) {
					my $selected = $sMountPoint->{ Selected };
                                        if ( $selected !~ /true/i ) {
                                                next;
                                        }
					push ( @{$hrInstanceMounts->{ $Name }} , $sMountPoint->{Value} );
					push ( @{$hrMountInstances->{ $sMountPoint->{Value} }} , $Name );
				}
			}

				Utils::vxlog("INFO", "Adding resource for [$Name]");
				$insResName =  $self->GetUniqueResName($Name);
				Utils::vxlog("DEBUG", "WebSphere MQ resource Name:$insResName");
				$insxml = "<Resource>";
				$insxml .= "<Name>$insResName</Name>";
				$insxml .= "<Type>WebSphereMQ</Type>";

				$insxml .= "<Attributes>";
				$insxml .= "<QueueManager><global>$Name</global></QueueManager>";
				$insxml .= "<MQVer><global>$MQVersion</global></MQVer>";
				$insxml .= "<EnvFile><global>$EnvFile</global></EnvFile>";
				if ( $ListenerOption == 2) {
					$insxml .= "<MonitorListener><global>1</global></MonitorListener>";
				}	
				if ( ( defined $ins -> { MQInstallationPath } ) and 
				     ( defined $ins -> { MQInstallationPath } { Value } ) ) {
					$insxml .= "<MQInstallationPath><global>$sInstPath</global></MQInstallationPath>";
				}
				$insxml .= "</Attributes>";
				$insxml .= "</Resource>";

			$hrResXMLs -> { $Name } = $insxml;
			$hrResNames -> { $Name } = $insResName;

			if ( $ListenerOption == 3 ) {
				Utils::vxlog("INFO", "Creating application resource to monitor the listener independent of the queue manager.");
				$lsnrxml = "<Resource>";
				my $LsnrName = $Name . "_Listener_res";
				my $LsnrResName =  $self->GetUniqueResName($LsnrName);
				Utils::vxlog("DEBUG", "WebSphere MQ resource Name:$LsnrResName");
				$lsnrxml .= "<Name>$LsnrResName</Name>";
				$lsnrxml .= "<Type>Application</Type>";
				$lsnrxml .= "<Attributes>";
				$lsnrxml .= "<User><global>mqm</global></User>";
				$lsnrxml .= "<StartProgram><global>$sInstPath/bin/runmqlsr -t tcp $ListenerPort -m $Name &amp;</global></StartProgram>";
				$lsnrxml .= "<StopProgram><global>$sInstPath/bin/endmqlsr -m $Name</global></StopProgram>";
				$lsnrxml .= "<MonitorProcesses><global>$sInstPath/bin/runmqlsr -t tcp $ListenerPort -m $Name</global></MonitorProcesses>";

				$lsnrxml .= "</Attributes>";
				$lsnrxml .= "</Resource>";

				$hrResXMLs -> { $LsnrName } = $lsnrxml;
				$hrResNames -> { $LsnrName } = $LsnrResName;

				push ( @{$hrInsChildParents->{ $Name }} , $LsnrName );
				push ( @{$hrInsParentChildren->{ $LsnrName }} , $Name );
			}
		}

	} else {
		$ret = APPCONTROL_WEBSPHEREMQ_COMMIT_EMPTY_INSTACE_DATA;
		$msg = "Empty instance xml in 'params' is provided";
		goto END; 
	}
#---------------------------------------------------------------------
	my $sSystemNICSGxml = "";
	($ret, $sSystemNICSGxml) = $self->getNetworkXML($tree);
	if ( $ret == CONST_FAILURE) {
		$msg = "Error occurred while generating the network configuration XML.";
		Utils::vxlog("ERROR", $msg);
		goto END; 
	}
	$xml .= $sSystemNICSGxml;
	my $sgXML = $self->createSGXML ($tree, $hrInsResXMLs, $hrInsResNames, "MQ","WebSphereMQ","WebSphere MQ: Queue manager");
	$xml .= $sgXML;
	($ret, $msg) = $self->checkAndAddApplicationResTypes();
END:
	$xml .= "</xml>";
	Utils::vxlog("INFO", "Commit output XML: $xml");

	my $retRef = ();

	$retRef -> { ret } = $ret;
	$retRef -> { msg } = $msg;
	$retRef -> { xml } = $xml;

	return $retRef;
}

sub findApplicationMounts
{
        my $self = shift;
        my $xmlstr = shift;

        my $ret = CONST_SUCCESS;
        my $sMyName = ( caller 0 )[3];
        #-------------------
        # Announce ourselves
        #-------------------
        my $logStr = "Subroutine $sMyName called";
        Utils::vxlog("INFO", $logStr);
        Utils::vxlog("INFO", "Input XML: $xmlstr ");

        my $tree = Utils::convertXMLTextToHash( $xmlstr );

        my $retXML = "";
        my @aDiskIds = ();
        $retXML = "<StorageInfo>";
        # Find the mount list array  for all applications
        # Find the diskID for each of the mountpoint in storage list
        my @arMountPointList = ();
	my @instances = Utils::convertHashValuesToArray($tree -> { Instance });
	foreach my $ins (@instances) {
		if (($ins -> {Selected} =~ /true/ ) &&
			(defined ($ins -> {MountPointList})) && ($ins -> {MountPointList} ne "") &&
			(defined ($ins -> {MountPointList}{MountPoint})) ) {
			my @selectedMountPoints = Utils::convertHashValuesToArray($ins -> {MountPointList}{MountPoint});
			foreach(@selectedMountPoints) {
				if ( $_->{ Selected } !~ /true/i ) {
					next;
				}
				push(@arMountPointList, $_->{Value});
			}
		}
	}
	my @arStorage = ();
	my @diskIds = ();
	my %uniqDisks = ();
	if ((defined ($tree -> {StorageList})) && 
		($tree -> {StorageList} ne "") &&
		(defined ($tree -> {StorageList}{Storage}))) {
		@arStorage = Utils::convertHashValuesToArray($tree -> {StorageList}{Storage});
	}

	# find diskID of all the mount points
	foreach my $storage (@arStorage) {
		if ( (grep { $_ eq $storage->{MountPoint}{Value} } @arMountPointList)) {
			if ((defined $storage->{DiskIDList}) &&
				($storage->{DiskIDList} ne "") &&
				(defined $storage->{DiskIDList}{DiskID})) {
				my @tmpDiskList = Utils::convertHashValuesToArray($storage->{DiskIDList}{DiskID});
				foreach (@tmpDiskList) {
					if (!defined $uniqDisks{$_->{Value}}) {
        					$retXML .= "<DiskID>$_->{Value}</DiskID>";
						$uniqDisks{$_->{Value}} = 1;
					}
				}
			}	
		}	
	}
	
        #$retXML .= "<DiskID>0:1</DiskID><DiskID>0:6</DiskID>";

        $retXML = $retXML . "</StorageInfo>";
        Utils::vxlog("INFO", "Output XML: $retXML");
        return ( $ret , "" , $retXML);
}

sub updateNetworkInstanceInfo
{
	my $sMyName = ( caller 0 )[3];
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my ($self , $arInstances, $sIPAddress ) = (@_);
	my $ret = CONST_SUCCESS;
	if ( $sIPAddress =~ /\S/ ) {
		$sIPAddress = " -i " . $sIPAddress;
	} else {
		Utils::vxlog("ERROR", "IP address information missing for listener resource");
		$ret = CONST_FAILURE;
		goto END; 
	}

	my $runmqlsrCmd = "runmqlsr -t tcp";
	my $runmqlsrCmdWithIP = "runmqlsr -t tcp" . $sIPAddress;
	my ( $lsnrxml ) = ("", "" );

	foreach (@{$arInstances}) {
		my $LsnrName = $_ . "_Listener_res";
		$_ = $LsnrName;
		$lsnrxml = $hrResXMLs -> { $LsnrName }; 
		Utils::vxlog("DEBUG", "Listener resource XML without IP information [$lsnrxml]");
		$lsnrxml =~ s/$runmqlsrCmd/$runmqlsrCmdWithIP/g;
		$hrResXMLs -> { $LsnrName } = $lsnrxml;
		Utils::vxlog("DEBUG", "Listener resource XML with IP information [$lsnrxml]");
	}
END:
	return $ret;
}

sub checkAndAddApplicationResTypes
{
	my $self = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";

	my $vcsobj = new VCS();
	$vcsobj -> makeConfigWritable();

	# check if passed type is present or not
	my $CmdRet = $vcsobj -> isResTypeAvailable ( $WebSphereMQResType );
	if ( $CmdRet != CONST_YES ) {
		$CmdRet = $self -> addResType( $WebSphereMQResType );
		if ($CmdRet != CONST_SUCCESS) {
			$ret = APPCONTROL_WEBSPHEREMQ_COMMIT_RESTYPEADDFAILURE;
			$msg = "Failed to add [$WebSphereMQResType] type";
			goto END;
		}
	}

	$vcsobj -> dumpAndMakeConfigReadOnly();

END:
	return ($ret, $msg);
}


sub addResType
{
	my $self = shift;
	my $resType = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $file = "/etc/VRTSagents/ha/conf/WebSphereMQ/WebSphereMQTypes.cf";
	my $tmpDir = "/tmp/WebSphereMQTypes";
	my $hacf = "/opt/VRTSvcs/bin/hacf";

	my $CmdRet = CONST_SUCCESS;
	my $op = "";

	my $cmd = "mkdir -p $tmpDir";
	($CmdRet, $op) = Utils::runCmd($cmd);
	if ( $CmdRet != CONST_SUCCESS ) {
		return $CmdRet;
	}

	$cmd = "cp $file $tmpDir/main.cf";
	($CmdRet, $op) = Utils::runCmd($cmd);
	if ( $CmdRet != CONST_SUCCESS ) {
		return $CmdRet;
	}

	$cmd = "$hacf -cftocmd $tmpDir";
	($CmdRet, $op) = Utils::runCmd($cmd);
	if ( $CmdRet != CONST_SUCCESS ) {
		return $CmdRet;
	}

	$cmd = "sed s/main.cf/WebSphereMQTypes.cf/g $tmpDir/main.cmd > $tmpDir/WebSphereMQTypes.cmd";
	($CmdRet, $op) = Utils::runCmd($cmd);
	if ( $CmdRet != CONST_SUCCESS ) {
		return $CmdRet;
	}

	$cmd = "$tmpDir/WebSphereMQTypes.cmd";
	my $chmod = "/bin/chmod";
	$op = `$chmod +x $cmd`;
        $CmdRet = $? / 256;
        if ( $CmdRet != 0 ) {
		Utils::vxlog("INFO", "Could not set execute permission for file. [$CmdRet] [$op]");
		return $CmdRet;
        }

	($CmdRet, $op) = Utils::runCmd($cmd);
	if ( $CmdRet != CONST_SUCCESS ) {
		Utils::vxlog("INFO", "File run failed [$CmdRet] [$op]");
		return $CmdRet;
	}

	$cmd = "rm -rf $tmpDir";
	($CmdRet, $op) = Utils::runCmd($cmd);

	return $CmdRet;
}

sub getIPParentInstance
{
	my $self = shift;
	my $resRef = shift;
	my $lsnrResName= shift;

	my $sMySubName = ( caller 0 )[3];
	my $announceStr = "Subroutine $sMySubName called";
	Utils::vxlog("INFO", $announceStr);

	if (defined $resRef-> { $lsnrResName } {StartProgram} {global}) {
		my $lsnrVal = $resRef-> { $lsnrResName } {StartProgram} {global};
		if ( $lsnrVal =~ /runmqlsr.* -m (\S+) /) {
			$lsnrVal = $1;
			Utils::vxlog("DEBUG", "Found queue manager [$lsnrVal] ");
			return $lsnrVal;
		} else {
			Utils::vxlog("INFO", "Found an Application type resource [$lsnrVal] that is not a WebSphereMQ listener.");
		}
	} 
	return "";
}

# This function would be called discoverExistingNetworkConfig to populate
# the instances to be shown on Network page
sub getDiscoveredInstances
{
    my $self = shift;
    my $resRef = shift;

    my $sMySubName = ( caller 0 )[3];
    my $announceStr = "Subroutine $sMySubName called";
    Utils::vxlog("INFO", $announceStr);

    my @resList = ();
    foreach my $resName (keys %$resRef) { 
	if ($resRef->{$resName}{Type}{global} !~ /^Application$/) {
            next;
        }
	if (defined $resRef->{ $resName }{StartProgram}) {
		my $lsnrVal = $resRef-> { $resName } {StartProgram} {global};
		if ( $lsnrVal =~ /runmqlsr.* -m (\S+) /) {
			$lsnrVal = $1;
			push (@resList, $lsnrVal);
		} 
        }
    }
    return \@resList;
}

# get the list of type and attributes which should be queried to get the
# details about the existing IP and instance mapping.
sub getAppTypeAndAttrs
{
	my $self = shift;
	my $sMySubName = ( caller 0 )[3];
	my $announceStr = "Subroutine $sMySubName called";
	Utils::vxlog("INFO", $announceStr);

	my @type = ();
	my @Attrs = ();
	if (VCS::isResourceOfTypeConfigured ("Application")) {
		push (@type, "Application");
		push (@Attrs, "StartProgram");
	}
	
	return (\@type, \@Attrs);
}

sub isQueueManagerPresent
{
	my $self = shift;
	my $sMyName = ( caller 0 )[3];
	my $qmgrName = shift;
	my $resRef = shift;
	my $ret = CONST_FAILURE;

	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	if ((ref $resRef) !~ /HASH/) {
		return $ret;
	}
	foreach my $resName (keys %$resRef) { 
		if ($resRef->{$resName}{Type}{global} !~ /^WebSphereMQ$/) {
			next;
	    	}
		if ((defined $resRef->{ $resName }{QueueManager}) and 
		    ($resRef->{ $resName }{QueueManager}{global} eq $qmgrName)) {
			$logStr = "Resource for WebSphereMQ Queue Manager [" . $qmgrName . "] is already created.";
			Utils::vxlog("INFO", $logStr);
			$ret = CONST_SUCCESS;
			return $ret;
		}
	}
	return $ret;
}

# Shallow discovery is called by validate from the node where wizard is running.
# shallow discovery happens on all the other nodes where wizard is not running.
sub shallowDiscover
{
	my $self = shift;
	my $params = shift;
	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $xml = "<xml></xml>";

	my $packageName = "" ;
	my $packageVersion = "5.1.14.0" ;
	
	($ret,$packageName) = PlatUtils::isAgentPackageInstalled("WebSphereMQ", $packageVersion);
	
	if ( $ret != PlatUtils::PACKAGE_INSTALLED ) {
		$msg = "Appropriate version of [$packageName] package is not installed. Install version [$packageVersion] or higher and then retry.";
		Utils::vxlog("INFO", $msg);
		$ret = CONST_FAILURE;
		return ($ret, $msg, $xml);
	}	
	else
	{
		$logStr = "Package [$packageName] with version [$packageVersion] or higher, is installed";
		Utils::vxlog("INFO", $logStr);
	}
	
	$ret = PlatUtils::isAcclibPackageInstalled("WebSphereMQ");

	if ( $ret != PlatUtils::PACKAGE_INSTALLED ) {
		$msg = "Package [VRTSacclib] for the WebSphereMQ agent is not installed. Install the package and then retry";
		Utils::vxlog("INFO", $msg);
		$ret = CONST_FAILURE;
		return ($ret, $msg, $xml);
	}
	else
	{
		$logStr = "Package [VRTSacclib] is installed.";
		Utils::vxlog("INFO", $logStr);
	} 
	
	my ($cmdRet, $op) = Utils::runCmd("rpm -qa --qf \"%{NAME} %{INSTPREFIXES} %{VERSION} %{RELEASE}\n\" | grep MQSeriesRuntime"); 
	if ($cmdRet != CONST_SUCCESS) {
		$msg = "WebSphere MQ SeriesRuntime package not found. Install the WebSphere MQ packages and then retry.";
		Utils::vxlog("WARNING", $msg);
	} else {
		Utils::vxlog("INFO", "WebSphere MQ SeriesRuntime RPM found.");
	}

	return ($ret, $msg, $xml);
}

# This function is called by discoverExistingNetowrkConfig during add node wizard.
# it returns the WebSphereMQ instances list to the wizard.

sub getValidationInput
{
	my $self = shift;
	my $sMyName = ( caller 0 )[3];
	my $resRef = shift;
	my $clusRef = shift;
	my $ret = CONST_SUCCESS;
	my $QueueManager  = "";
	my $outXML = "<ValidationInfo>";
	foreach my $resName (keys %$resRef) { 
		my $data = ();
		if ($resRef->{$resName}{Type}{global} !~ /^WebSphereMQ$/) {
			next;
		}
		if ((defined $resRef->{ $resName }{QueueManager}) and
		    (defined $resRef->{ $resName }{QueueManager}{global} ) and
		    ( $resRef->{ $resName }{QueueManager}{global} ne "" )) {
			$QueueManager = $resRef->{ $resName }{QueueManager}{global};
			$outXML .= "<Instance>";
			$outXML .= "<InstanceName>" . $QueueManager . "</InstanceName>";
			$outXML .= "</Instance>";
		}
	}
	$outXML .= "</ValidationInfo>";
	return ($ret, "", $outXML);
}

1; # so that perl doesn't compalin!
