#!/opt/VRTSsfmh/bin/perl

#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Id: licenseManager.pl,v 1.2 2012/07/10 13:46:12 stripath Exp $
#

#------------------------
# Common Header
#------------------------
use strict;
use warnings;

use Data::Dumper;
use CGI;
use Config;
use POSIX;

# Push include paths!
my $LibPath1 = "";

BEGIN
{
	if ( $^O =~ m/win/i ) {
		require VRTS::Paths;

		my $xprtlInstallDir= VRTS::Paths::get_path("InstallDir");
		my $regquery = "\"$xprtlInstallDir"."\\bin\\regquery.exe\" query SOFTWARE\\VERITAS\\VCS\\Base HomeDirectory";
		my $vcshome = `$regquery`;
		$LibPath1 = "$vcshome\\portal\\admin\\lib\\";
		$ENV{'VCS_HOME'} = $vcshome;

	} else {
		my $APPDISCOVER_HOME = "/opt/VRTSvcs/portal/admin";
	
		$LibPath1 = "$APPDISCOVER_HOME/lib";
	}
}

use lib $LibPath1;
#------------------------
# End of Common Header
#------------------------

use Constants;
use Utils;
use VCS;
use MessageUtils;
use constant MSG_FILE => "clusterop.msg";
use LicenseManager;

# Before doing anything else, let us set log config ...
# Params:
#	. FileName
#	. File size (in MBs)
Utils::setLogConfig("LicManager", 10);
Utils::vxlog("INFO", ">>>> licenseManager.pl invoked <<<<");
#------------------------
# End of Header
#------------------------

my ($params, $op)	 = ();
my $licMgr = ();
my $lockFile = Utils::getLockDir() . "LicMgr.l";
$licMgr = new LicenseManager();

#-----------------------
# caller subroutines
#-----------------------

#--------------------------
# Main code starts here...
#--------------------------
my $query = CGI->new();
print $query->header('text/xml');

$params  = Utils::parseArguments($query);
$op		 = $params -> { 'OP' } || "";

my $xmlstr = "";
my $msg    = "";
my $ret = CONST_SUCCESS;
my $installedLicInfo = "";

eval {
	if ( $op =~ /^getLicInfo$/i ) {
		($ret, $xmlstr) = $licMgr -> getLicInfo();
	} elsif ( $op =~ /^installLic$/ ) {
		my $cmdop = "";
		$installedLicInfo = "";
		($ret, $msg, $cmdop, $installedLicInfo) = $licMgr -> installLic ( $params -> { params } );
		$installedLicInfo = "" if ( not defined $installedLicInfo );
		$xmlstr = "<Debug><![CDATA[$cmdop]]></Debug>$installedLicInfo";
	} else {
		($ret, $msg) = getMessage(MSG_FILE, "LIC_ALL_INVALIDOP");
	}
};
if ( $@ ) {
	Utils::vxlog("EXCEPTION", "ret=[$ret], msg=[$msg], xmlstr=[$xmlstr]");
	$@		= Utils::strTrim( $@ );
	$ret	= CONST_EXCEPTION;
	$msg	= $@;
	$xmlstr = "";
}

$xmlstr = Utils::getCmdResponseXML("License Manager", $ret, $msg, $xmlstr);
Utils::vxlog("INFO", $xmlstr);
print "$xmlstr\n";

Utils::vxlog("INFO", "Exiting from operation");
exit CONST_SUCCESS;

