#!/opt/VRTSsfmh/bin/perl

#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Id: licenseNotification.pl,v 1.3 2012/02/22 07:37:57 mkarmuse Exp $
#

#------------------------
# Common Header
#------------------------
use strict;
use warnings;


# Push include paths!
my $LibPath1 = "";
my $LibPath2 = "";

use vars qw($vcshome);

BEGIN
{
	if ( $^O =~ m/win/i ) {
		require VRTS::Paths;

		my $xprtlInstallDir= VRTS::Paths::get_path("InstallDir");
		my $regquery = "\"$xprtlInstallDir"."\\bin\\regquery.exe\" query SOFTWARE\\VERITAS\\VCS\\Base HomeDirectory";

		$vcshome = `$regquery`;

		$LibPath1 = "$vcshome\\portal\\admin\\plugins\\common";
		$LibPath2 = "$vcshome\\portal\\admin\\plugins\\windows";
	} else {
		my $APPDISCOVER_HOME = "/opt/VRTSvcs/portal/admin";
		$vcshome = "/opt/VRTSvcs";

		$LibPath1 = "$APPDISCOVER_HOME/plugins/unix";
		$LibPath2 = "$APPDISCOVER_HOME/plugins/common";
	}
}

use lib $LibPath1;
use lib $LibPath2;

use AppControlCodes;
use VCSUtilities;
use LicenseManager;
use Data::Dumper;
use VCSUtilities;

my $licMgr = ();
$licMgr = new LicenseManager();


#---------
# Announce
#---------
my $logStr = "Script license notification called";
VCSUtilities::vxlog("INFO", $logStr);

my $ret = APPCONTROL_SUCCESS;
my $xmlstr = "";

my ($fret, $msg, $LicInfo) = $licMgr -> getInstalledLicenses();
if ( $fret != APPCONTROL_SUCCESS ) {
	
	exit 1;
}

my @appha_lics = ();
my @non_appha_lics = ();

#first check is any permanent license present if yes quite!

foreach my $lic ( keys %$LicInfo ) {
	if ($LicInfo -> { $lic } { ApplicationHAFeature } =~ m/1|yes/i){

		if ( $LicInfo -> { $lic } { LicenseType } =~ /perm/i )	{
			VCSUtilities::vxlog("INFO", "permanent license exists.");
			exit 0;
		}
	} 
}

my $aboutToExpireList = "";
my $isExpired = 0;
foreach my $lic ( keys %$LicInfo ) {
	if ($LicInfo -> { $lic } { ApplicationHAFeature } =~ m/1|yes/i){

		
		if($LicInfo -> { $lic } { Expired } == 1){
		
			$isExpired = 1;
			
		}
		elsif($LicInfo -> { $lic } { ExpiryDate } !~ m/-NA-/i) {

			$aboutToExpireList = $aboutToExpireList. " The license ".  $lic . "will expire on ". $LicInfo -> { $lic } { ExpiryDate }. ".";
		}
	} 
}

if($aboutToExpireList ne ""){

	#raise notification
	my $hInput = ();
	$hInput -> { ID } = "1.3.6.1.4.1.1302.3.24.10.2.3.0.2";
	$hInput -> { AppName } = "";
	$hInput -> { Severity } = "Warning";
	$hInput -> { Message } = "The ApplicationHA license is about to expire.". $aboutToExpireList;
	
	VCSUtilities::sendNotification( $hInput );
}
elsif($isExpired == 1)
{
	#raise notification
	my $hInput = ();
	$hInput -> { ID } = "1.3.6.1.4.1.1302.3.24.10.2.3.0.1";
	$hInput -> { AppName } = "";
	$hInput -> { Severity } = "SevereError";
	$hInput -> { Message } = "The ApplicationHA license has expired.";
	$hInput -> { Force } = 1;

	VCSUtilities::sendNotification( $hInput );
}
