#!/opt/VRTSsfmh/bin/perl
#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Id: proxy.pl,v 1.7 2012/09/19 17:12:00 hshinde Exp $
#

#------------------------
# Common Header
#------------------------
use strict;
use warnings;

use Data::Dumper;
use CGI;
use Config;
use POSIX;

# Push include paths!
my $LibPath1 = "";

BEGIN
{
	if ( $^O =~ m/win/i ) {
		require VRTS::Paths;

		my $xprtlInstallDir= VRTS::Paths::get_path("InstallDir");
		my $regquery = "\"$xprtlInstallDir"."\\bin\\regquery.exe\" query SOFTWARE\\VERITAS\\VCS\\Base HomeDirectory";
		my $vcshome = `$regquery`;
		$LibPath1 = "$vcshome\\portal\\admin\\lib\\";
		$ENV{'VCS_HOME'} = $vcshome;

	} else {
		my $APPDISCOVER_HOME = "/opt/VRTSvcs/portal/admin";
	
		$LibPath1 = "$APPDISCOVER_HOME/lib";
	}
}

use lib $LibPath1;
#------------------------
# End of Common Header
#------------------------

use Constants;
use Utils;
use MessageUtils;
use constant MSG_FILE => "clusterop.msg";

# Before doing anything else, let us set log config ...
# Params:
#	. FileName
#	. File size (in MBs)
Utils::setLogConfig("proxy", 10);
Utils::vxlog("INFO", ">>>> proxy.pl invoked <<<<");
#------------------------
# End of Header
#------------------------

my $xprtlc = ();

sub setup_env
{
	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	if ( $^O =~ /win/i ) {
		my $xprtlInstallDir= VRTS::Paths::get_path("InstallDir");
		$xprtlc = "\"$xprtlInstallDir\\bin\\xprtlc.exe\"";
	} else {
		$xprtlc = "/opt/VRTSsfmh/bin/xprtlc";
	}


	Utils::vxlog("INFO", "xprtlc=[$xprtlc]");
}


sub main
{
	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $cmdRetCode = CONST_SUCCESS;
	my $debug = "";

	setup_env();

	my $query = CGI->new();
	print $query->header('text/xml');

	my $params  = Utils::parseArguments($query);
	my $sURL = $params -> { TargetURL } || "";
	my $sUser = $params -> { TargetUser } || "";
	my $sPasswd = $params -> { TargetPassword } || "";

	if ( $sURL eq "" ) {
		($ret, $msg) = getMessage(MSG_FILE, "PROXY_URL_EMPTY");
		goto END;
	}

	if ( $sUser eq "" ) {
		($ret, $msg) = getMessage(MSG_FILE, "PROXY_USER_EMPTY");
		goto END;
	}

	if ( $sPasswd eq "" ) {
		($ret, $msg) = getMessage(MSG_FILE, "PROXY_PASSWD_EMPTY");
		goto END;
	}

	my $cmd = "$xprtlc -l \"$sURL\"";
	foreach my $param ( keys %$params ) {
		if ( $param eq "TargetURL" or 
		     $param eq "TargetUser" or
		     $param eq "TargetPassword") {
			next;
		}

		$cmd .= " -d $param=" . $params -> { $param };
	}
	# User names do not contain special characters with $.
	$cmd .= " -u \"$sUser\"";
	Utils::vxlog("INFO", "executing [$cmd -p XXXX]");
	
	# Windows can not support single quote on command line 
	# If $ is present in password, linux command line considers it as variable, which is avoided by using single quotes in linux and double quotes in windows.
	if ( $^O =~ m/win/i ) {
		$sPasswd =~ s/"/\"/g;
		$cmd .= " -p \"$sPasswd\"";
	} elsif ( $^O =~ m/linux/i ) {
		$sPasswd =~ s/'/'"'"'/g;
		$cmd .= " -p \'$sPasswd\'";
	}

	($ret, $msg, $cmdRetCode) = Utils::runCmd($cmd, { doNotLogCmd => 1 });
	Utils::vxlog("INFO", "runCmd return Code = [$cmdRetCode], Message = [$msg] ");
	if ($ret != CONST_SUCCESS) {
		$debug = "<Debug><![CDATA[$msg]]></Debug>";
		$msg = "";	#TODO: Return a generic message (from message file).
	}
END:
	if ($ret != CONST_SUCCESS) {
		$msg = Utils::getCmdResponseXML("proxy.pl", $cmdRetCode, $msg, $debug);
	}
	print $msg;
	return $ret;
}

my $ret = main();
exit ($ret);

