#!/opt/VRTSsfmh/bin/perl
#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Id: settings.pl,v 1.3 2012/07/10 13:46:12 stripath Exp $
#

#------------------------
# Common Header
#------------------------
use strict;
use warnings;

use Data::Dumper;
use CGI;
use Config;
use POSIX;

# Push include paths!
my $LibPath = "";

BEGIN
{
	if ( $^O =~ m/win/i ) {
		require VRTS::Paths;

		my $xprtlInstallDir= VRTS::Paths::get_path("InstallDir");
		my $regquery = "\"$xprtlInstallDir"."\\bin\\regquery.exe\" query SOFTWARE\\VERITAS\\VCS\\Base HomeDirectory";
		my $vcshome = `$regquery`;
		$LibPath = "$vcshome\\portal\\admin\\lib\\";
		$ENV{'VCS_HOME'} = $vcshome;

	} else {
		my $APPDISCOVER_HOME = "/opt/VRTSvcs/portal/admin";
	
		$LibPath = "$APPDISCOVER_HOME/lib/";
	}
}

use lib $LibPath;
#------------------------
# End of Common Header
#------------------------

use Constants;
use Utils;
use VCS;
use MessageUtils;
use SettingManager;

use constant MSG_FILE => "appmon.msg";


#---------------------
# Subroutines
#---------------------
sub setSettings
{
	my $paramsStr = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

#	Utils::vxlog("INFO", "Acquiring lock ...");
#	my $lockfd = Utils::acquire_lock($lockFile, 1);
#	Utils::vxlog("INFO", "Got lock, proceeding ...");

	my $msg = "";
	my $ret = CONST_SUCCESS;

	if ( $paramsStr eq "" ) {
		($ret, $msg) = getMessage( MSG_FILE, "INVALID_PARAMS_XML_SCHEMA" );
		goto END;
	}

	my $tree = Utils::convertXMLTextToHash($paramsStr);

	$logStr = Dumper($tree);
	Utils::vxlog("DEBUG", $logStr);

	# validates
	if (not defined $tree or $tree eq "") {
		($ret, $msg) = getMessage( MSG_FILE, "INVALID_PARAMS_XML_SCHEMA" );
		goto END;
	}

	my @Settings = ();
	my $grpRef = $tree -> { Group };
	if (not defined $grpRef or $grpRef eq "") {
		($ret, $msg) = getMessage( MSG_FILE, "PARAMS_XML_TAG_MISSING", "Group" );
		goto END;
	}

	my @groups = Utils::convertHashValuesToArray($grpRef);
	my $settingMgr = new SettingManager();

	foreach my $grp ( @groups ) {
		my $grpName 	= $grp -> { Name    } || "";
		my $settingsRef = $grp -> { Setting } || "";

		if ( $grpName eq "" ) {
			($ret, $msg) = getMessage( MSG_FILE, "PARAMS_XML_TAG_MISSING", "Group::Name" );
			goto END;
		}

		if ( $settingsRef eq "") {
			($ret, $msg) = getMessage( MSG_FILE, "PARAMS_XML_TAG_MISSING", "Setting" );
			goto END;
		}

		my @settings = Utils::convertHashValuesToArray($settingsRef);
	
		foreach my $s ( @settings ) {

			my $Name  = $s -> { Name  } || "";

			my $Value;
			if ( defined $s -> { Value } ) {
				$Value = $s -> { Value };
			}
			else {
				$Value = "";
			}
	
			if ( $Name eq "" ) {
				($ret, $msg) = getMessage( MSG_FILE, "PARAMS_XML_TAG_MISSING", "Setting::Name" );
				goto END;
			}
	
			Utils::vxlog("INFO", "Applying [$Name] setting");
	
			my $function = "Apply_" . $Name;
			$function =~ s/\./_/;
	
			if ( $Name eq "App.RestartAttempts" or $Name eq "App.StartStopTimeout" or $Name eq "App.DisplayName" )
			{
				($ret, $msg) = $settingMgr -> $function ( $grpName, $Value );
			}
	
			else
			{
				($ret, $msg) = getMessage( MSG_FILE, "SET_INVALID_SETTING", $Name );
				goto END;
			}
		}
	}

END:

#	Utils::vxlog("INFO", "Releasing lock ...");
#	Utils::release_lock( $lockfd );

	return ($ret, $msg);
}


sub getSettings
{
	my $paramsStr = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $xml = "";

	if ( $paramsStr eq "" ) {
		($ret, $msg) = getMessage( MSG_FILE, "INVALID_PARAMS_XML_SCHEMA" );
		goto END;
	}

	my $tree = Utils::convertXMLTextToHash($paramsStr);

	$logStr = Dumper($tree);
	Utils::vxlog("DEBUG", $logStr);

	# validates
	if (not defined $tree or $tree eq "") {
		($ret, $msg) = getMessage( MSG_FILE, "INVALID_PARAMS_XML_SCHEMA" );
		goto END;
	}

	my $grpRef = $tree -> { Group };
	if (not defined $grpRef or $grpRef eq "") {
		($ret, $msg) = getMessage( MSG_FILE, "PARAMS_XML_TAG_MISSING", "Group" );
		goto END;
	}

	my @groups = Utils::convertHashValuesToArray($grpRef);
	my $settingMgr = new SettingManager();

	#$xml = "<ServiceGroups>\n";
	foreach my $grp ( @groups ) {

		my $grpName = $grp -> { Name } || "";
		my $str = "";

		if ( $grpName eq "" ) {
			($ret, $msg) = getMessage( MSG_FILE, "PARAMS_XML_TAG_MISSING", "Name" );
			goto END;
		}
	
		($ret, $str) = $settingMgr -> getSettingsXML($grpName);
		$xml .= $str;
	}
	#$xml .= "</ServiceGroups>\n";

END:
	return ($ret, $xml);
}

sub main
{
	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $query = CGI->new();
	print $query->header('text/xml');

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $xml = "";

	my $params = parseArguments($query);

	my $OP = $params -> { OP } || "";
	my $ID = $params -> { ID } || "";

	if ( $OP eq "" ) {
		($ret, $msg) = getMessage( MSG_FILE, "URL_SYNTAX_INVALID_OP" );
		goto END;
	}

	eval {
		if ( $OP =~ /^getSettings$/i ) {
			($ret, $xml) = getSettings($params -> { params } || "");
		} elsif ( $OP =~ /^setSettings$/ ) {
			($ret, $msg) = setSettings($params -> { params } || "");
		} else {
			($ret, $msg) = getMessage( MSG_FILE, "URL_SYNTAX_UNAVAILALE_OP");
		}
	};
	if ( $@ ) {
		$xml = "<Debug><![CDATA[$@]]></Debug>";
		($ret, $msg) = getMessage( MSG_FILE, "CONFAPP_OP_EXECUTION_EXCEPTION", $OP, $ID);
	}

END:
	my $xmlstr = Utils::getCmdResponseXML($ID, $ret, $msg, $xml);
	print "$xmlstr\n";
	return $ret;	
}



my $ret = main();
exit $ret;

