#!/opt/VRTSperl/bin/perl

#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Id: settings_upgrade.pl,v 1.3 2011/04/21 06:32:37 rjog Exp $
#

use strict;
use warnings;

use vars qw($AppControlLogDir $libPath1 $libPath2);

BEGIN
{
	if ( $^O =~ m/win/i ) {
		require VRTS::Paths;

		my $xprtlInstallDir= VRTS::Paths::get_path("InstallDir");
		my $regquery = "\"$xprtlInstallDir"."\\bin\\regquery.exe\" query SOFTWARE\\VERITAS\\VCS\\Base HomeDirectory";

		my $AppControlHome = `$regquery`;
		chomp $AppControlHome;
		$AppControlLogDir = "$AppControlHome\\log\\";
		$libPath1 = "$AppControlHome\\portal\\admin\\plugins\\common";
		$libPath2 = "$AppControlHome\\portal\\admin\\plugins\\windows";

	} else {
		$AppControlLogDir = "/var/VRTSvcs/log/";
		$libPath1 = "/opt/VRTSvcs/portal/admin/plugins/common";
		$libPath2 = "/opt/VRTSvcs/portal/admin/plugins/unix";
	}
}

use lib $libPath1;
use lib $libPath2;

use AppControlCodes;
use VCSUtilities;
use AppControlLogger;
use SettingManager;

use Data::Dumper;

#---------------------
# Global vars
#---------------------
use constant {
	APPHA_UPGRADE_SUCCESS => [0, ""],
	APPHA_UPGRADE_FAILURE => [1, ""],
	APPHA_UPGRADE_EXCEPTION => [5, ""],

	APPHA_UPGRADE_XML_SYNTAX_CHECK_FAILED => [2, "Syntax check failed"],
	APPHA_UPGRADE_XML_FILE_NOT_FOUND => [3, "File not found."],
	APPHA_UPGRADE_XML_SAVE_FILE_FAILED => [4, "Error occurred while saving the file"],
};


my $logFileName = "AppHAXMLFileUpgrade";
my $logFileSize = 5;
my ( $logger, $settingsMgr ) = ();

eval
{
	$logger = new AppControlLogger($logFileName, $AppControlLogDir, $logFileSize);
	VCSUtilities::setLogConfig($logFileName, $logFileSize);
};
if ( $@ ) {
	print "ERROR while initializing logger.\n";
	exit 1;
}

$logger -> vxlog("INFO", "\n\n----------------- XML Upgrade Invoked -----------------------");

#------------------
# Subriutines ...
#------------------
sub usage
{
	print "\n";
	print "perl settings_upgrade.pl <old xml file name>";
	print "\n";
	print "\n";
}


sub check_param_syntax
{
	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	$logger -> vxlog("INFO", $logStr);

	$logger -> vxlog("INFO", "Number of arguments: $#ARGV");
	if ($#ARGV != 0) {
		return 1;
	}

	return 0;
}


sub sync_xmls
{
	my $hnref = shift;
	my $horef = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	$logger -> vxlog("INFO", $logStr);

	my $href = ();

	my $ref = $hnref -> { Setting };
    my @settings = ();

    if ( "$ref" =~ /ARRAY/ ) {
        @settings = @$ref;
    } elsif ( "$ref" =~ /HASH/) {
        push(@settings, $ref);
    }

	$ref = $horef -> { Setting };
    my @osettings = ();

    if ( "$ref" =~ /ARRAY/ ) {
        @osettings = @$ref;
    } elsif ( "$ref" =~ /HASH/) {
        push(@osettings, $ref);
    }

    foreach my $s ( @settings ) {
		my $SName  = $s -> { Name };
		my $SValue = $s -> { Value };

		$logger -> vxlog("INFO", "Processing settings [$SName]: [$SValue]");
		my $os = ();
		my $found = 0;
		my $OValue = "";
		foreach $os ( @osettings ) {
			my $OName  = $os -> { Name };
			if ( $OName eq $SName ) {
				$OValue = $os -> { Value };
				$logger -> vxlog("INFO", "Found in old settings, value is [$OValue]");
				$found = 1;
				last;
			}
		}

		if ( $found == 1 ) {
			$s -> { Value } = $OValue;
		}

		my $val = $s -> { Value };
		$logger -> vxlog("INFO", "Value is : $val");
	}

	$href = $hnref;

END:
	return $href;
}


sub main
{
	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	$logger -> vxlog("INFO", $logStr);

	my $ret = APPHA_UPGRADE_SUCCESS;

	eval {
		if ( check_param_syntax() ) {
			usage();
			$ret = APPHA_UPGRADE_XML_SYNTAX_CHECK_FAILED;
			goto END;
		}
    
		my $oldFile = shift @ARGV;
    
		# check if files present or not
		if ( not -f $oldFile ) {
			$ret = APPHA_UPGRADE_XML_FILE_NOT_FOUND;
			goto END;
		}
    
		my $horef = SettingManager::readSettingsConfFile ( $oldFile );
		my $hnref = SettingManager::readSettingsConfFile ( );
    
		my $hsref = sync_xmls( $hnref, $horef );
    
		if ( SettingManager::dumpSettingsConf( $hsref ) ) {
			$ret = APPHA_UPGRADE_XML_SAVE_FILE_FAILED;
			goto END;
		}
	};
	if ( $@ ) {
		$ret = APPHA_UPGRADE_EXCEPTION;
		$ret -> [1] = $@;
	}

END:
	return $ret;
}


#----------------------
# Call the main routine
#----------------------
my $ret = main();
if ($ret != APPHA_UPGRADE_SUCCESS) {
	print $ret -> [1];
}

$logger -> vxlog("INFO", "Exeting with return codes op=[". $ret->[1]. "], code=[" . $ret->[0] . "]");
exit $ret -> [0];

