#!/opt/VRTSperl/bin/perl

#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
# 
# $Id: synchronize_guest_config.pl,v 1.15 2011/08/17 09:27:09 rjog Exp $
#


#------------------------
# Common Header
#------------------------
use strict;
use warnings;

# Push include paths!
my $LibPath1 = "";
my $LibPath2 = "";

use vars qw($vcshome $worldPath);
use vars qw($xprtlc $sEAT_DATA_DIR $vssat $sSFM_DATA_DIR $sEAT_HOME_DIR);
use vars qw($xprtlInstallDir);

BEGIN
{
	if ( $^O =~ m/win/i ) {
		require VRTS::Paths;

		$xprtlInstallDir= VRTS::Paths::get_path("InstallDir");
		my $regquery = "\"$xprtlInstallDir"."\\bin\\regquery.exe\" query SOFTWARE\\VERITAS\\VCS\\Base HomeDirectory";

		$vcshome = `$regquery`;
		$LibPath1 = "$vcshome\\portal\\admin\\plugins\\common";
		$LibPath2 = "$vcshome\\portal\\admin\\plugins\\windows";

		$worldPath = "$vcshome\\portal\\world\\";
		$ENV{'VCS_HOME'} = $vcshome;

		$xprtlc = "\"$xprtlInstallDir"."\\bin\\xprtlc.exe\"";

		$regquery = "\"$xprtlInstallDir"."\\bin\\regquery.exe\" query SOFTWARE\\Symantec\\SFMH\\xprtl\\config var_dir";
		$sSFM_DATA_DIR = `$regquery`;
		$sEAT_DATA_DIR = $sSFM_DATA_DIR;
		$sSFM_DATA_DIR .= "\\sec";
		$sEAT_DATA_DIR .= "\\ApplicationHA\\sec";

		$vssat = "\"$xprtlInstallDir\\bin\\vssatbin.exe\"";
		$sEAT_HOME_DIR = $xprtlInstallDir;
		
	} else {
		my $APPDISCOVER_HOME = "/opt/VRTSvcs/portal/admin";
		$vcshome = "/opt/VRTSvcs";
		$LibPath1 = "$APPDISCOVER_HOME/plugins/unix";
		$LibPath2 = "$APPDISCOVER_HOME/plugins/common";

		$worldPath = "$vcshome/portal/world/";
		$xprtlc = "/opt/VRTSsfmh/bin/xprtlc";

		$sEAT_HOME_DIR = "/opt/VRTSsfmh";
		$sSFM_DATA_DIR = "/var/opt/VRTSsfmh/sec";
		$sEAT_DATA_DIR = "/var/opt/VRTSsfmh/ApplicationHA/sec";
		$vssat = "/opt/VRTSsfmh/bin/vssat";
	}
}

use lib $LibPath1;
use lib $LibPath2;

use AppControlCodes;
use VCSUtilities;
use Data::Dumper;
use VCS;
use CGI;
use ApplicationConfigManager;
use SFMUtils;

use File::Path qw(mkpath);
use File::Copy;

my $HBRes = "VCSAppMonHBRes";
my $HBSGAttr = "ServiceGroupName";
my $GuestConfigMXL = "GuestConfig.xml";

my $sGenAppID = "";
if ( $^O =~ /win/i ) {
	$sGenAppID = "CustomApplication";
} else {
	$sGenAppID = "GenericApplication";
}


my $testVMwareFile = "";
if ($^O =~ /win/i) {
	$testVMwareFile = "$xprtlInstallDir\\bin\\testVMware.exe";
} else {
	$testVMwareFile = "/opt/VRTSsfmh/bin/testVMware";
}

my $createAppMonHBSGURL = "https://localhost:5634/vcs/admin/createAppMonHBSG.pl";

VCSUtilities::vxlog("INFO", ">>>> synchronize_guest_config is invoked <<<<");
#------------------------
# End of Header
#------------------------

sub synchronize_vcs_sysconf
{
	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	VCSUtilities::vxlog("INFO", $logStr);

	my $ret = APPCONTROL_SUCCESS;
	my $vcs_sysconf = "/etc/sysconfig/vcs";
	my $vcs_sysconf_bk = "/etc/sysconfig/vcs.applicationha.save";
	my $sed = "/bin/sed";
	if ( not -x $sed) {
		$sed = "/usr/bin/sed";
	}

	if (-f $vcs_sysconf and -x $sed) {
		my $mv_file = "mv $vcs_sysconf $vcs_sysconf_bk";
		VCSUtilities::runCmd($mv_file);

		my $sed_cmd = "$sed s/ONENODE=.*/ONENODE=yes/ $vcs_sysconf_bk | " .
			"$sed s/VCS_START=.*/VCS_START=1/ | " .
			"$sed s/VCS_STOP=.*/VCS_STOP=1/ " .
			" > $vcs_sysconf";
		VCSUtilities::runCmd($sed_cmd);
	}

END:
	return $ret;
}



sub create_app_conf_entries
{
	my $vcs = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	VCSUtilities::vxlog("INFO", $logStr);

	my $ret = APPCONTROL_SUCCESS;
	my $msg = "";

	# first call createAppMonHBSG with SG names
	# It will create other required resources and also updates xprtld scheduler if applicable.

	($ret, $msg) = $vcs -> getResAttrValue($HBRes, $HBSGAttr);
	if ( $ret != APPCONTROL_SUCCESS ) {
		goto END;
	}

	$msg =~ s/^\s+//; $msg =~ s/\s+$//;
	my @Names = split(/\s+/, $msg);

	my $sSGNameXML = "<xml><ServiceGroups>";
	foreach my $sg ( @Names ) {
		$sSGNameXML .= "<Name>$sg</Name>";
	}
	$sSGNameXML .= "</ServiceGroups></xml>";

	my $cmd = "$xprtlc -l $createAppMonHBSGURL -d params=\"$sSGNameXML\"";
	VCSUtilities::runCmd($cmd);

	# now check app.conf entries for un-added SG's
	my $hAppMgr = new ApplicationConfigManager();
	
	my @hGroupsToAdd = ();
	foreach my $sg ( @Names ) {
		my $hApp = $hAppMgr -> getApplicationBySGName( $sg );
		if (not defined $hApp or $hApp eq "") {
			push(@hGroupsToAdd, $sg);
		}
	}

	my $sapph = VCSUtilities::getSupportedTypes();
	my $sapph_id = VCSUtilities::getSupportedApplications();
	my $happs = ();

	VCSUtilities::vxlog("INFO", "Supported Applications are \n" . Dumper($sapph_id));
	VCSUtilities::vxlog("INFO", "Supported Types are \n" . Dumper($sapph));

	# create empty happs;
	foreach my $id ( @$sapph_id ) {
		$happs -> { $id } = "";
	}

	my ($VCSclusRef, $resRef, $grpRef, $resDepRef, $grpDepRef, $href) = ();
	if ( $#hGroupsToAdd >= 0 ) {
		my ($retf, $fmsg, $clusRef) = $vcs -> getClusterDetails();
		$VCSclusRef = $clusRef -> { ClusRef };
		$resRef = $clusRef -> { ResRef };
		$grpRef = $clusRef -> { GroupRef };
		$resDepRef = $clusRef -> { ResDepRef };
		$grpDepRef = $clusRef -> { GrpDepRef };
		$href = $clusRef -> { ProcessedRef };
	}

	foreach my $grp ( @hGroupsToAdd ) {

		my $res_id = ();
		foreach my $_id ( @$sapph_id ) {
			$res_id -> { $_id } = 0;
		}

		foreach my $res ( keys %{$href -> { $grp } { Res } } ) {
			my $type = $resRef -> { $res } { Type };
			VCSUtilities::vxlog("INFO", "Resource type: " . $type);
			my $resId = $sapph -> { $type } || $sGenAppID;
			VCSUtilities::vxlog("INFO", "Resource ID: " . $resId);
			$res_id -> { $resId } += 1;
		}

		#
		# give priority always to supported applications.
		# if nothing found, treat it as Custom Application.
		#
		my $grpId = $sGenAppID;
		my $cnt = 0;
		foreach my $_id ( keys %$res_id ) {
			if ( $_id eq $sGenAppID ) {
				next;
			}
			if ( $cnt < $res_id -> { $_id } ) {
				$grpId = $_id;
				$cnt = $res_id -> { $_id }
			}
		}
		# add it to happs.
		VCSUtilities::vxlog("INFO", "Adding [$grp] tp [$grpId]");
		$happs -> { $grpId } .= " $grp ";
	}

	VCSUtilities::vxlog("INFO", Dumper( $happs ));

	foreach my $appID ( keys %$happs ) {

		my $grps = $happs -> { $appID };
		$grps =~ s/^\s+//;
		$grps =~ s/\s+$//;
		if ( $grps eq "" ) {
			next;
		}

		my @SGList = split(/\s+/, $grps);
		my $appName = VCSUtilities::getDisplayName( $appID );
		$hAppMgr -> addAppNode( $appName, $appID, \@SGList );
	}

END:
	return ($ret, $msg);
}


sub copy_dir_recursively {
	my ($from_dir, $to_dir) = @_;
	
	opendir(DIR, $from_dir);
	my @files = readdir(DIR);
	closedir(DIR);

	for my $entry (@files) {
		if ($entry eq "." or $entry eq "..") {
			next;
		}
		
		my $source = "$from_dir/$entry";
		my $destination = "$to_dir/$entry";
		
		if (-d $source) {
			mkdir $destination or die "mkdir '$destination' failed: $!" if not -e $destination;
			VCSUtilities::vxlog("INFO", "creating directory [$destination]");
			copy_dir_recursively($source, $destination);
		} else {
			VCSUtilities::vxlog("INFO", "copying [$source] to [$destination]");
			copy($source, $destination);
		}
	}
	
	return;
}



sub synchronize_sfmh_settings
{
	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	VCSUtilities::vxlog("INFO", $logStr);

	my $ret = APPCONTROL_SUCCESS;
	my $msg = "";

	# update SFM resolv conf file
	my $sAppHAConsoleIP = "";
	my $sGuestConfig = $worldPath . $GuestConfigMXL;
	my $data = "";
	open FH, $sGuestConfig;
	while ( <FH> ) {
		$data .= $_;
	}
	close FH;

	my @AppHAIPs = ();
	if ($data =~ /<ApplicationHA_primary>(.*)<\/ApplicationHA_primary>/i ) {
		$sAppHAConsoleIP = $1;
		push(@AppHAIPs, $sAppHAConsoleIP);
	}

	if ($data =~ /<ApplicationHA_secondary>(.*)<\/ApplicationHA_secondary>/i ) {
		$sAppHAConsoleIP = $1;
		push(@AppHAIPs, $sAppHAConsoleIP);
	}

	foreach my $AppHAIP ( @AppHAIPs ) {
		update_sfm_resolve_file($AppHAIP, $sEAT_DATA_DIR);
	}

	my $dir_already_copied = 0;
	if ( not -d $sEAT_DATA_DIR ) {
		mkpath $sEAT_DATA_DIR;
		copy_dir_recursively($sSFM_DATA_DIR, $sEAT_DATA_DIR);
		$dir_already_copied = 1;
		goto END;
	}

	$ENV { EAT_HOME_DIR } = $sEAT_HOME_DIR;
	$ENV { EAT_DATA_DIR } = $sEAT_DATA_DIR;

	my $cmd = "$vssat showcred" .
			  " -d vx:appha_domain" .
			  " -p appha_guest";

	($ret, $msg) = VCSUtilities::runCmd( $cmd );
	if ( $msg =~ /found:\s+0/i and not $dir_already_copied ) {
		copy_dir_recursively($sSFM_DATA_DIR, $sEAT_DATA_DIR);
	}

END:
	return ($ret, $msg);
}


sub main
{
	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	VCSUtilities::vxlog("INFO", $logStr);

	my $ret = APPCONTROL_SUCCESS;
	my $msg = "";

	my $query = CGI->new();
	print $query->header('text/xml');

	# Step: 
	VCSUtilities::vxlog("INFO", "\n--------------------- SYNCRONIZE VCS CONFIG --------------------\n\n");
	my $vcs = new VCS();
	if ( $vcs -> checkIfVCSRunning() != APPCONTROL_YES ) {
		$ret = APPCONTROL_FAILURE;
		$msg = "ApplicationHA guest components (VCS) are not running";
	} else {
		create_app_conf_entries( $vcs );
	}

	# Step:
	if ( $testVMwareFile ne ""	and -x $testVMwareFile) {
		($ret, $msg) = VCSUtilities::runCmd("\"$testVMwareFile\"");
		if ( $ret == APPCONTROL_SUCCESS ) {
			VCSUtilities::vxlog("INFO", "\n--------------------- SYNCRONIZE SFMH SETTINGS --------------------\n\n");
			synchronize_sfmh_settings();
		}
	}

	# Step:
	if ( $^O !~ /win/i ) {
		VCSUtilities::vxlog("INFO", "\n--------------------- SYNCRONIZE VCS SYSCONFIG SETTINGS --------------------\n\n");
		synchronize_vcs_sysconf();
	}

END:
	my $xmlstr = VCSUtilities::getInCmdXMLFormat("synchronize_guest_config", $ret, $msg, "");
	VCSUtilities::vxlog("INFO", $xmlstr);
	print "$xmlstr\n";

	return $ret;
}


#---------------
# Call main
#---------------
my $ret = main();
VCSUtilities::vxlog("INFO", "Exiting from synchronize_guest_config");
exit $ret ->[0];

