#!/opt/VRTSsfmh/bin/perl

#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Id: unconfigureCluster.pl,v 1.7 2012/08/20 18:55:41 sgupta2 Exp $
#

#------------------------
# Common Header
#------------------------
use strict;
use warnings;

use Data::Dumper;
use CGI;
use Config;
use POSIX;

# Push include paths!
my $LibPath1 = "";

BEGIN
{
	if ( $^O =~ m/win/i ) {
		require VRTS::Paths;

		my $xprtlInstallDir= VRTS::Paths::get_path("InstallDir");
		my $regquery = "\"$xprtlInstallDir"."\\bin\\regquery.exe\" query SOFTWARE\\VERITAS\\VCS\\Base HomeDirectory";
		my $vcshome = `$regquery`;
		$LibPath1 = "$vcshome\\portal\\admin\\lib\\";
		$ENV{'VCS_HOME'} = $vcshome;

	} else {
		my $APPDISCOVER_HOME = "/opt/VRTSvcs/portal/admin";
	
		$LibPath1 = "$APPDISCOVER_HOME/lib";
	}
}

use lib $LibPath1;
#------------------------
# End of Common Header
#------------------------

use Constants;
use Utils;
use VCS;
use PlatUtils;

use Net::Domain qw(hostname);


sub moveFile
{
	my $file = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $op  = "";
	my $cmdRet = 0;

	if ( -e $file) {
		my $date = `date +"\%Y-\%m-\%d_\%H-\%M-\%S"`;
		my $newLocation = $file . "_" . $date;
		my $mvCmd = "mv $file $newLocation";

		($ret, $op, $cmdRet) = Utils::runCmd($mvCmd);

		if ( $ret != CONST_SUCCESS ) {
			Utils:vxlog ("ERROR", "Failed to move $file.");
		}
	}

	return ($ret, $op);
}


sub checkModuleStateAndStop
{
	my $moduleName = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $op  = "";
	my $cmdRet = 0;

	my $statusCmd = "/etc/init.d/$moduleName status";

	($ret, $op, $cmdRet) = Utils::runCmd($statusCmd);
	if ( $ret == CONST_SUCCESS and ( $op !~ /not/ and $op !~ /unloaded/ ) ) {
		Utils::vxlog("INFO", "Module $moduleName is loaded. Trying to unload it.");
		
		my $stopCmd = "/etc/init.d/$moduleName stop";
		($ret, $op, $cmdRet) = Utils::runCmd($stopCmd);
		if ( $ret != CONST_SUCCESS ) {
			Utils:vxlog ("ERROR", "Failed to stop module $moduleName - [$op]");
		}
	}
	else {
		Utils::vxlog("INFO", "Module $moduleName already stopped.");
		$ret = CONST_SUCCESS;
	}

	return ($ret, $op);
}


sub main
{
	my $hParams = shift;

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $cmdRet = 0;
	my $op = "";
	my $backup_files_only = 0;	

	# This variable is used to backup files only if VCS is not running.
	if ( $hParams -> {'BackupFilesIfVCSStopped'} ) {
		$backup_files_only = 1;
	}

	if ( $hParams -> {'NoOp'} ) {
		Utils::vxlog ("INFO", "This is a no-op. Returning success.");
		goto END;
	}

	my $vcsobj = new VCS();
	my $href = PlatUtils::getVCSPaths();
	my $nodeName = hostname();

	# If VCS is running, stop it
	if ( $vcsobj -> checkIfVCSRunning () == CONST_YES ) {
		if ( $backup_files_only == 0 ) {

			my $hastopCmd = $href -> { hastop } . " -local -force";
			($ret, $op, $cmdRet) = Utils::runCmd($hastopCmd);

			if ( $ret != CONST_SUCCESS ) {
				Utils::vxlog ("ERROR", "Failed to stop VCS on $nodeName - [$op].");
				goto END;
			}
		}
		else {
			$ret = CONST_FAILURE;
			Utils::vxlog ("ERROR", "VCS is running on $nodeName. Cannot proceed.");
			goto END;
		}
	}

	if ( $backup_files_only == 0 ) {
		# Waiting for at max 20 second to let 'had' stop cleanly
		my $i = 0;
		while ($i < 20) {
			sleep(1);
			my $VCSProcCmd = "ps -u root -o pid,args | egrep 'had'|grep -v grep";
			($ret, $op, $cmdRet) = Utils::runCmd($VCSProcCmd);
			last if ( $ret != CONST_SUCCESS);
			$i++;
		}

		my $hadprocess = "";
		if ($i == 20) {
			$hadprocess = "had|";
			# Perform cleanup operation that had does when it exit gracefully
			my $rmHadSocCmd = "rm -rf /var/VRTSvcs/uds/.hadsocket";

			($ret, $op, $cmdRet) = Utils::runCmd($rmHadSocCmd);

			if ( $ret != CONST_SUCCESS ) {
				Utils:vxlog ("ERROR", "Failed to remove had uds socket '/var/VRTSvcs/uds/.hadsocket' [$op].");
			}
		}
		# If any VCS process is still running kill them
		my $killVCSProcCmd = "for pid in `ps -u root -o pid,args | egrep '${hadprocess}CmdServer|vcsauthserver|hashadow' | egrep -v grep | sed -e 's/^ //' | cut -d' ' -f1 | xargs`; do kill -9 \$pid ; if [ \$? -ne 0 ] ; then echo -n \"\$pid \"; break ; fi ; done";

		($ret, $op, $cmdRet) = Utils::runCmd($killVCSProcCmd);

		if ( $ret != CONST_SUCCESS ) {
			Utils:vxlog ("ERROR", "Failed to kill the following pids [$op]");
			goto END;
		}
	}

	# Backup main.cf
	($ret, $op) = moveFile ( $href -> { MainCf } );
	if ( $ret != CONST_SUCCESS ) {
		Utils:vxlog ("ERROR", "Failed to move " . $href -> { MainCf } . " - [$op]");
	}

	if ( $backup_files_only == 0 ) {
		# It takes some time to unregister agents from AMF after VCS has been stopped forcefully.
		# Hence, adding sleep here to avoid AMF stop failure.
		sleep 5;

		# Check the status of amf and stop if running/loaded.
		($ret, $op) = checkModuleStateAndStop("amf");
		if ( $ret != CONST_SUCCESS ) {
			Utils:vxlog ("ERROR", "Failed to stop amf - [$op]");
			goto END;
		}

		# Check the status of vxfen and stop if running/loaded.
		($ret, $op) = checkModuleStateAndStop("vxfen");
		if ( $ret != CONST_SUCCESS ) {
			Utils:vxlog ("ERROR", "Failed to stop vxfen - [$op]");
			goto END;
		}
	
	
		# Check the status of gab and stop if running/loaded.
		($ret, $op) = checkModuleStateAndStop("gab");
		if ( $ret != CONST_SUCCESS ) {
			Utils:vxlog ("ERROR", "Failed to stop gab - [$op]");
			goto END;
		}
	
		# Check the status of llt and stop if running/loaded.
		($ret, $op) = checkModuleStateAndStop("llt");
		if ( $ret != CONST_SUCCESS ) {
			Utils:vxlog ("ERROR", "Failed to stop llt - [$op]");
			goto END;
		}
	}


	# Backup gabtab
	($ret, $op) = moveFile ( $href -> { gabtab } );
	if ( $ret != CONST_SUCCESS ) {
		Utils:vxlog ("ERROR", "Failed to move " . $href -> { gabtab } . " - [$op]");
	}

	# Backup llttab
	($ret, $op) = moveFile ( $href -> { llttab } );
	if ( $ret != CONST_SUCCESS ) {
		Utils:vxlog ("ERROR", "Failed to move " . $href -> { llttab } . " - [$op]");
	}

	# Backup llthosts
	($ret, $op) = moveFile ( $href -> { llthosts } );
	if ( $ret != CONST_SUCCESS ) {
		Utils:vxlog ("ERROR", "Failed to move " . $href -> { llthosts } . " - [$op]");
	}

END:
	return $ret;
}

my $query = CGI->new();
print $query->header('text/xml');

my $cmdArgs = Utils::parseArguments($query);

my @paramsArr = split(",",$cmdArgs->{'params'}) if $cmdArgs->{'params'};
my $hParams;
@{$hParams}{@paramsArr} = @paramsArr;

my $ret = main($hParams);
my $msg = "";
my $xml = "";
my $xmlstr = Utils::getCmdResponseXML("unconfigureCluster", $ret, $msg, $xml);
Utils::vxlog("INFO", "$xmlstr");
print "$xmlstr\n";
