#!/opt/VRTSsfmh/bin/perl 

#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Id: verifySingleSignon.pl,v 1.1 2012/04/11 05:23:54 rjog Exp $
#


#------------------------
# Common Header
#------------------------
use strict;
use warnings;

use Data::Dumper;
use CGI;
use Config;
use POSIX;

# Push include paths!
my $LibPath1 = "";

BEGIN
{
	if ( $^O =~ m/win/i ) {
		require VRTS::Paths;

		my $xprtlInstallDir= VRTS::Paths::get_path("InstallDir");
		my $regquery = "\"$xprtlInstallDir"."\\bin\\regquery.exe\" query SOFTWARE\\VERITAS\\VCS\\Base HomeDirectory";

		my $vcshome = `$regquery`;
		$LibPath1 = "$vcshome\\portal\\admin\\lib\\";
	} else {
		my $APPDISCOVER_HOME = "/opt/VRTSvcs/portal/admin";
	
		$LibPath1 = "$APPDISCOVER_HOME/lib";
	}
}

use lib $LibPath1;
#------------------------
# End of Common Header
#------------------------

use Constants;
use Utils;
use Logger;
use CmdUtils;
use MessageUtils;

use constant MSG_FILE => "sec.msg";

my $logDir = PlatUtils::getLogDir();
my $logFileName = "verifySSO";
my $logFileSize = 5;
my ( $logger, $cmdutils ) = ();

eval
{
	$logger = new Logger($logFileName, $logDir, $logFileSize);
	$cmdutils = new CmdUtils( $logger );
};
if ( $@ ) {
	my $str = Utils::getCmdResponseXML("verifySSO.pl", CONST_EXCEPTION, "<Message>$@</Message>", "");
	print "$str\n";
	exit CONST_EXCEPTION;
}


# Before doing anything else, let us set log config ...
# Params:
#   . FileName
#   . File size (in MBs)
Utils::setLogConfig($logFileName, $logFileSize);
$logger -> vxlog("INFO", ">>>> verifySingleSignon is invoked <<<<");
#------------------------
# End of Header
#------------------------


my ($EAT_DATA_DIR, $EAT_HOME_DIR, $xprtlc) = "";
sub setup_env
{

	if ( $^O =~ /win/i ) {
		my $xprtlInstallDir= VRTS::Paths::get_path("InstallDir");
		$EAT_HOME_DIR = $xprtlInstallDir;

		my $regquery = "\"$xprtlInstallDir"."\\bin\\regquery.exe\" query SOFTWARE\\Symantec\\SFMH\\xprtl\\config var_dir";
		$EAT_DATA_DIR = `$regquery`;
		$EAT_DATA_DIR .= "\\ApplicationHA\\sec";

		$xprtlc = "\"$xprtlInstallDir\\bin\\xprtlc.exe\"";
	} else {
		$EAT_DATA_DIR = "/var/opt/VRTSsfmh/ApplicationHA/sec";
		$EAT_HOME_DIR = "/opt/VRTSsfmh";
		$xprtlc = "/opt/VRTSsfmh/bin/xprtlc";
	}

	if ( not -d "$EAT_DATA_DIR" ) {
		mkpath $EAT_DATA_DIR;
	}

	$logger -> vxlog("INFO", "EAT_HOME_DIR=[$EAT_HOME_DIR], EAT_DATA_DIR=[$EAT_DATA_DIR], xprtlc=[$xprtlc]");
	$ENV { EAT_HOME_DIR } = $EAT_HOME_DIR;
	$ENV { EAT_DATA_DIR } = $EAT_DATA_DIR;
}


#-------------
# Subroutines
#-------------
sub check_param_syntax
{
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	$logger -> vxlog("INFO", $logStr);

	my $ret    = CONST_SUCCESS;
	my $msg    = "";
	my $href   = ();
	
	my $ConsoleIP = $params -> { ConsoleIP };

	if (not defined $ConsoleIP) {
		($ret, $msg) = getMessage(MSG_FILE,
		                          "VERIFYSSO_INVALID_SYNTAX_CONSOLEIP",
		                          CONSOLE_PROD_NAME);
		goto END;
	}

	$href -> { ConsoleIP }    = $ConsoleIP;
	
END:
	return ($ret, $msg, $href);
}


sub VirtualMachineToConsoleSSOConfigured
{
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	$logger -> vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $op  = "";

	my $ConsoleIP = $params -> { ConsoleIP };

	my $cmd = "$xprtlc -c appha://" . $ConsoleIP. ":5634/" .
	          " -l https://" . $ConsoleIP . ":5634/appha_console/agent/verifySingleSignon.xml" .
	          " -u vxss://" . $ConsoleIP . ":14153/appha_guest/appha_domain/vx";

	($ret, $op) = $cmdutils -> runCmd( $cmd );
	if ($ret != CONST_SUCCESS) {
		($ret, $msg) = getMessage( MSG_FILE, "VERIFYSSO_SSOXML_FETCH_FAILED",
		                           CONSOLE_PROD_NAME, $ConsoleIP);
	}

END:
	return ($ret, $msg, $op);
}


sub main
{
    my $sMyName = ( caller 0 )[3];
    #---------
    # Announce
    #---------
    my $logStr = "Subroutine $sMyName called";
    $logger -> vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $op  = "";

	my $query = CGI->new();
	print $query->header('text/xml');
	my $params = parseArguments($query);

	setup_env();

	$logStr = Dumper( $params );
	$logger -> vxlog ("INFO", "params are:\n$logStr");

	($ret, $msg) = check_param_syntax( $params );
	if ( $ret != CONST_SUCCESS ) {
		goto END;
	}

	($ret, $msg, $op) = VirtualMachineToConsoleSSOConfigured( $params );
	if ( $ret != CONST_SUCCESS ) {
		$logger -> vxlog("INFO", "Virtual Machine to Console SSO is not configured.");
		goto END;
	}

END:
	my $debug = "<Debug>$op</Debug>";
    my $xmlstr = Utils::getCmdResponseXML("verifySingleSignon", $ret, $msg, $debug);
    print "$xmlstr\n";
    $logger -> vxlog("INFO", $xmlstr);
	return ($ret, $msg);
}


#-----------
# Call main
#-----------
my ($ret, $msg) = main();

$logger -> vxlog("INFO", "Exiting from verifySingleSignon; exit code => " . $ret);
exit $ret;

