#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
package CPIC;
use strict;

sub completion {
    my ($cpic,$edr,$cfg,$prod,$rel,$web);
    $cpic=shift;
    $edr=Obj::edr();

    unless (Cfg::opt('precheck')) {
        $prod=$cpic->prod;
        $cpic->set_reboot_info();
        $prod->completion_messages();
        $rel=$cpic->rel;
        $rel->config_without_reboot_message();

        # Ask feedback
        $rel->cli_ask_feedback();

        # to set right values in responsefile for (precheck +) install+config
        $cfg=Obj::cfg();
        Cfg::set_opt($cpic->{task}) if (($cpic->{task} eq 'install') && Cfg::opt('configure'));

        unless($cpic->{noresponsefile}) {
            # Create files/response files
            $cpic->responsefile_comments();
            $edr->{all_tunables} = $cpic->all_tunables;
            $cfg->create_responsefile();
        }

        if (Obj::webui()) {
            $rel->check_update();
            $web=Obj::web();
            $web->web_script_form('completion');
        }

        # Handle the log files
        $rel->upload_logs();
        $rel->check_update() unless (Obj::webui());
        $rel->completion();
        $rel->reboot_messages();
    }
    $edr->exit_noexitfile();

    return '';
}

sub update_summary {
    my ($cpic,$msg,@items) = @_;
    my $summary;
    $summary="$msg->{msg}\n\n";
    for my $item(@items) { $summary.="\t$item\n"; }
    Msg::add_summary($summary,1);
    return;
}

sub set_reboot_info {
    my (@start_sys,$cpic,$prod,$sys);
    $cpic=shift;
    $cpic->{reboot_systems}=$cpic->systems_need_reboot("check_tunables");
    return unless (@{$cpic->{reboot_systems}});
    return unless ($cpic->{prod});
    $prod=$cpic->prod;
    for my $sys (@{$cpic->{systems}}) {
        $prod->require_start_after_reboot_sys($sys);
        push (@start_sys, $sys->{sys}) if ($sys->{require_start_after_reboot});
    }
    $cpic->{require_start_after_reboot_systems}=\@start_sys;
    return;
}

sub responsefile_comments {
    # Each response file comment is a 4 item list
    # item 1 is the comment, previously translated in the prior line
    # item 2 a 0=optional, 1=required
    # item 3 is 0=scalar, 1=list
    # item 4 is 0=1d, 1=2d is SYSTEM, other=other second dimension
    my ($cpic,$prod,$cmt,$pdfrs,$edr);
    $cpic=shift;
    # for makeresponsefile upgrade where sometimes fake systems are specified and product cannot be detected
    if ($cpic->{prod}) {
        $prod=$cpic->prod;
        $prod->responsefile_comments();
    }
    $pdfrs=Msg::get('pdfrs');
    # As create_responsefile() is in CFG using $edr->{rfc} hash instead of $cpic->{rfc}
    # EDR option comments are moved to EDR::responsefile_comments()
    $edr=Obj::edr();
    #$cmt=Msg::new("This variable defines a list of optional $pdfrs not to be installed on the systems");
    #$edr->{rfc}{donotinstall}=[$cmt->{msg},0,1,0];
    #$cmt=Msg::new("This variable defines a list of $pdfrs not to be removed from the systems during uninstall");
    #$edr->{rfc}{donotremove}=[$cmt->{msg},0,1,0];
    $cmt=Msg::new("This variable defines a list of license keys to be registered on the systems during install", 30, 1000);
    $edr->{rfc}{keys}=[$cmt->{msg},0,1,1];
    $cmt=Msg::new("This variable designates a standard configure task", 30, 1001);
    $edr->{rfc}{opt__configure}=[$cmt->{msg},0,0,0];
    $cmt=Msg::new("This variable installs $pdfrs and starts the products that do not require configuration", 30, 1002, "$pdfrs");
    $edr->{rfc}{opt__install}=[$cmt->{msg},0,0,0];
    $cmt=Msg::new("This variable licenses the product only", 30, 1003);
    $edr->{rfc}{opt__license}=[$cmt->{msg},0,0,0];
    $cmt=Msg::new("This variable installs the product without requiring entry of a license key", 30, 1004);
    $edr->{rfc}{opt__nolic}=[$cmt->{msg},0,0,0];
    $cmt=Msg::new("This variable installs the product with keyless license", 30, 1005);
    $edr->{rfc}{opt__vxkeyless}=[$cmt->{msg},0,0,0];
    $cmt=Msg::new("This variable defines a location, typically an NFS mount, from which all remote systems can install product patches. The location must be accessible from all target systems", 30, 1006);
    $edr->{rfc}{opt__patchpath}=[$cmt->{msg},0,0,0];
    $cmt=Msg::new("This variable defines a location, typically an NFS mount, from which all remote systems can install product $pdfrs. The location must be accessible from all target systems", 30, 1007, "$pdfrs");
    $edr->{rfc}{opt__pkgpath}=[$cmt->{msg},0,0,0];
    $cmt=Msg::new("This variable defines the location where a working directory is created to store temporary files and $pdfrs needed during the install.  The default location is /var/tmp", 30, 1008, "$pdfrs");
    $edr->{rfc}{opt__tmppath}=[$cmt->{msg},0,0,0];
    $cmt=Msg::new("This variable uninstalls a product", 30, 1009);
    $edr->{rfc}{opt__uninstall}=[$cmt->{msg},0,0,0];
    $cmt=Msg::new("This variable installs only minimum product $pdfrs", 30, 1010, "$pdfrs");
    $edr->{rfc}{opt__installminpkgs}=[$cmt->{msg},0,0,0];
    $cmt=Msg::new("This variable installs only recommended product $pdfrs", 30, 1011, "$pdfrs");
    $edr->{rfc}{opt__installrecpkgs}=[$cmt->{msg},0,0,0];
    $cmt=Msg::new("This variable installs all product $pdfrs", 30, 1012, "$pdfrs");
    $edr->{rfc}{opt__installallpkgs}=[$cmt->{msg},0,0,0];
    $cmt=Msg::new("This variable defines the list of systems on which the product is to be installed, uninstalled, or configured", 30, 1013);
    $edr->{rfc}{systems}=[$cmt->{msg},1,1,0];
    $cmt=Msg::new("This variable defines the list of systems to be recognized for configuration when a secure environment prevents all systems from being installed at once", 30, 1014);
    $edr->{rfc}{systemscfg}=[$cmt->{msg},0,1,0];
    $cmt=Msg::new("This variable is an abbreviation that defines the product to be installed, uninstalled, or configured", 30, 1015);
    $edr->{rfc}{prod}=[$cmt->{msg},1,0,0];
    $cmt=Msg::new("This variable defines whether user agrees with EULA.pdf on the media", 30, 1016);
    $edr->{rfc}{accepteula}=[$cmt->{msg},0,0,0];
    $cmt=Msg::new("This variable defines whether user agrees to upload the log files onto Symantec website", 30, 1017);
    $edr->{rfc}{uploadlogs}=[$cmt->{msg},0,0,0];
    return;
}


# EDR completion is coded in implementing module versus the
# EDR module to allow whatever additional customization is necessary
sub edr_completion {
    my $cpic=shift;
    my $web=Obj::web();
    my $cfg=Obj::cfg();
    my $edr=Obj::edr();

    # to set right values in responsefile for (precheck +) install+config
    Cfg::set_opt($cpic->{task}) if (($cpic->{task} eq 'install') && Cfg::opt('configure'));

    unless($cpic->{noresponsefile}) {
        $cpic->responsefile_comments();
        $edr->{all_tunables} = $cpic->all_tunables;
        $cfg->create_responsefile();
    }

    $web->web_script_form('completion') if (Obj::webui());
    $edr->exit_noexitfile();
    return '';
}

sub precheck_report {
    my ($cpic,$cfg,$edr,$msg,$errors,$warnings,$sys,$systems,$task,$tmpsub);
    $cpic=shift;
    $cfg=Obj::cfg();
    $edr=Obj::edr();

    Msg::n();
    $msg=Msg::new("Precheck report completed", 30, 1018);
    $msg->print;

    # count errors and warnings
    for my $sys (@{$cpic->{systems}}) {
        $errors+=scalar(@{$sys->{errors}}) if ($sys->{errors});
        $warnings+=scalar(@{$sys->{warnings}}) if ($sys->{warnings});
    }
    $edr->display_errors_warnings();

    unless ($errors || $warnings) {
        $msg=Msg::new("No issues found in prechecks\n", 30, 1019);
        $msg->print;
        $systems = join(' ',@{$cfg->{systems}});
        if ($cpic->ask_install()) {
            $task = Cfg::opt('upgrade') ? 'upgrade' :
                        Cfg::opt('patchupgrade') ? 'patchupgrade' : 'install';
            $cpic->set_task($task);
            if ($task =~ /^(install|upgrade)$/mx) {
                $tmpsub = $task.'_useragreement';
                $cpic->$tmpsub();
            }
            $cpic->cli_set_title();
            $cpic->options();
            return;
        }
    }
    $cpic->rel->completion();
    $cpic->edr_completion();
    return;
}

sub ask_install {
    my ($ayn,$cfg,$msg,$cpic,$prod,$systems);
    $cpic=shift;
    $prod=$cpic->prod;
    $cfg=Obj::cfg();
    $systems = join(' ',@{$cfg->{systems}});
    if (Cfg::opt('upgrade')) {
        $msg=Msg::new("Would you like to upgrade $prod->{abbr} on $systems?", 30, 1020, "$prod->{abbr}", "$systems");
    } else {
        $msg=Msg::new("Would you like to install $prod->{abbr} on $systems?", 30, 1021, "$prod->{abbr}", "$systems");
    }
    $ayn=$msg->aynn();
    Msg::n();
    return ($ayn eq 'Y') ? 1 : 0;
}

1;
