#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
package CPIC;
use strict;

# EDR initiation is coded in implementing module versus the
# EDR module to allow whatever additional customization is necessary

sub edr_initiation {
    my ($cpic,@vars) = @_;
    my ($edr,$key,$sub);
    $edr=EDR->new(@vars);
    # execute the first level of EDR initiation subs
    for my $sub (qw(init_env create_edr_objects set_interrupt set_localsys
                    set_mediapath init_L10N set_msgs set_tput)) { $edr->$sub(); }

    # copy some EDR vars to CPIC for convenience
    for my $key (qw(rel release padv padvs padvisa mediapath tmppath logpath
                scriptdir script fromdisk installpath perlpath)) { $cpic->{$key}=$edr->{$key}; }
    return;
}

sub initiation {
    my ($cpic,$rel,$padv,$mediapath,$task,$prod)=@_;
    $cpic->edr_initiation($rel,$padv,$mediapath);
    $cpic->{basemediapath}=$cpic->{mediapath};
    my $edr=Obj::edr();
    my $web=Obj::web();
    my $localsys=$edr->{localsys};
    $cpic->{task}=$task;
    $cpic->set_prod($prod);
    $web->web_initiation() if (Obj::webui());
    $cpic->process_bundle_args();
    $cpic->create_cpip_objects($cpic->{padv});
    $cpic->process_args();
    $localsys->padv->verify_root_user();
    $edr->create_local_tempdir();
    $cpic->cli_set_title();
    $edr->cli_display_copyright();
    $cpic->cli_exit_file();
    $cpic->register_logfiles();
    $cpic->expand_exe();
    $cpic->verify_media($cpic->rel);
    $cpic->rel->cli_installer_menu();
    $cpic->cli_set_title();
    $edr->{all_tunables} = $cpic->all_tunables;
    if (Obj::webui()) {
        $web->web_set_script_started();
        if ($web->{args_invalid}) {
            $web->web_script_form('warn_invalid_args');
        }
    }
    return;
}

sub register_logfiles {
    EDR::register_save_logfiles('install.*', 'uninstall.*', 'start.*', 'stop.*');

    if (Cfg::opt('bait')) {
        my $cfg=Obj::cfg();
        my $responsefile=$cfg->{opt}{responsefile};
        EDR::register_save_logfiles('pkg_*', 'patch_*', 'start_*', 'stop_*');
        EDR::register_save_logfiles($responsefile) if ($responsefile);
    }
    return;
}

sub install_useragreement {
    my $cpic=shift;
    my $web=Obj::web();

    return '' unless (Cfg::opt('install'));
    my $prod=$cpic->prod;
    return '' unless ($prod && $prod->{eula});
    $web->web_script_form('useragreement') if (Obj::webui());
    $cpic->cli_useragreement();
    return;
}

sub upgrade_useragreement {
    my $cpic=shift;
    my $web=Obj::web();

    my $cfg=Obj::cfg();
    return '' if ($cfg->{accepteula});
    return '' unless (Cfg::opt('upgrade'));
    my $prod=$cpic->prod;
    return '' unless ($prod && $prod->{eula});
    $web->web_script_form('useragreement') if (Obj::webui());
    $cpic->cli_useragreement();
    return;
}

sub cli_useragreement {
    my($ayn,$cfg,$cpic,$edr,$file,$help,$lang,$msg,$prod);
    $cpic=shift;
    $edr=Obj::edr();
    $cfg=Obj::cfg();
    $prod=$cpic->prod;
    $lang = 'en';
    if ($cpic->{padv} =~ /^Sol\d*sparc/mxi) {
        $lang = ($edr->{envlang} =~ /^ja/m) ? 'ja' :
                ($edr->{envlang} =~ /(^zh$|zh\.|zh_CN)/mx) ? 'zh' : 'en';
    }
    $file=$prod->geteula($lang);
    unless (Cfg::opt('responsefile')) {
        if (Cfg::opt('base_path')) {
            $msg=Msg::new("Do you agree with the terms of the End User License Agreement as specified in the $file file under $cpic->{basemediapath}?", 30, 1051, "$file", "$cpic->{basemediapath}");
        } else {
            $msg=Msg::new("Do you agree with the terms of the End User License Agreement as specified in the $file file present on media?", 30, 1052, "$file");
        }
        $help=Msg::new("If user does not answer 'y' at the prompt to agree with EULA, $cpic->{script} will quit", 30, 1053, "$cpic->{script}");
        $ayn=$msg->ayn(0,$help);
        Msg::n();
    }
    $edr->exit_noexitfile() unless (($ayn eq 'Y')||$cfg->{accepteula});
    $cfg->{accepteula}=1;
    return '';
}

# This sub is to create CPIP objects which already defined in CPI,
# such as Rel/Prod/Proc, and some builtin Pkg modules.
sub init_cpip_object {
    my ($class,$padv) = @_;
    my ($plat,$inherit_padv,$cpic,$codes);

    unless (UNIVERSAL::can("$class\::$padv", 'new')) {
        $plat=Padv::plat($padv);

        # Try to inherit from the first padv listed in CPIC::main()
        # If the first padv is null or "Common" or "$padv", then
        #     inherit from its defined uplevel.
        # else
        #     inherit from the first padv.
        $cpic=Obj::cpic();
        $inherit_padv=$cpic->{padv};
        if (!$inherit_padv || $inherit_padv eq 'Common' || $inherit_padv eq $padv) {
            $inherit_padv=Padv::inherit_uplevel($padv);
        }

        if ($inherit_padv && UNIVERSAL::can("$class\::$inherit_padv", 'new')) {
            $codes = "package $class\::$padv; \@$class\::$padv\::ISA = qw($class\::$inherit_padv);";
        } elsif (UNIVERSAL::can("$class\::$plat", 'new')) {
            $codes = "package $class\::$padv; \@$class\::$padv\::ISA = qw($class\::$plat);";
        }
        eval $codes if ($codes);

        # check again whether new() is available.
        unless (UNIVERSAL::can("$class\::$padv", 'new')) {
            EDR::die ("\"$class\::$plat\" is not defined");
        }
    }
    return "$class\::$padv"->new();
}

sub create_cpip_objects {
    my ($cpic,$padv) = @_;
    my ($rel,$patch,$proc,$prevpkgname,$prodv,$pkgi,$pkglist,@pkgs,$pkg,$patchi,$prod,$media_allpkgs);
    if (!EDRu::inarr($padv, @{EDR::padvs()}, 'Common')) {
        # import anything to operate the rest of the commands OK
        # unsupported padvs will eventually die later
        Msg::log("create_cpip_objects: $padv is not a supported platform");
        return;
    } elsif ($Obj::pool{"Rel\::$cpic->{release}\::$padv"}) {
        #Msg::log("$cpic->{release} $padv CPIP objects already created");
        return $cpic->rel($padv);
    }
    #Msg::log("Creating $cpic->{release}::$padv CPIP objects");
    $rel=$cpic->{rel}{$padv}=init_cpip_object("Rel::$cpic->{release}",$padv);
    push(@pkgs, @{$rel->allpkgs});
    for my $prodv (@{$rel->{prods}}) {
        $prod=init_cpip_object("Prod::$prodv",$padv);
        $prod->{release}=$cpic->{release};
        $prod->set_pkgs;
        push(@pkgs, @{$prod->allpkgs_on_media});
        for my $subprodv(@{$prod->{subprods}}) {
            my $subprod=init_cpip_object("Prod::$subprodv",$padv);
            $subprod->{release}=$cpic->{release};
            push(@pkgs, @{$subprod->allpkgs_on_media});
        }
    }

    $pkglist=EDRu::arruniq(@pkgs);
    for my $pkgi (@{$pkglist}) {
        $pkg=init_cpip_object("Pkg::$pkgi",$padv);
        for my $proc (@{$pkg->allprocs}) { init_cpip_object("Proc::$proc",$padv); }
        for my $prevpkgname (@{$pkg->{previouspkgnames}}) {
            Pkg->new_pkg($prevpkgname, $padv);
        }
    }

    $rel->define_patches();
    for my $pkgi (@{$pkglist}) {
        $pkg=Obj::pkg($pkgi, $padv);
        for my $patchi (@{$pkg->patches_allvers}) {
            # Patches are not built-in defined in CPIP, create it.
            $patch=new_object("Patch::$patchi",$padv);
            $patch->{pkg}=$pkgi;
        }
    }
    $cpic->create_eeb_objects($rel,$padv) if ($rel->{reli}=~/^EEB/);
    $rel->adjust_pkglist();
    return $rel;
}

# This sub is to create CPIP objects which are not defined in CPI,
# such as Obsoleted pkgs, and some patch modules.
sub new_object {
    my ($class,$padv) = @_;
    my ($plat,$inherit_padv,$baseclass,$cpic,$codes);

    # in case some OS packages name have -.+
    $class=~s/[-\=\,\.\+]/_/mg;

    $plat=Padv::plat($padv);
    unless (UNIVERSAL::can("$class\::$padv", 'new')) {
        # Try to inherit from the first padv listed in CPIC::main()
        # If the first padv is null or "Common" or "$padv", then
        #     inherit from its defined uplevel.
        # else
        #     inherit from the first padv.
        $cpic=Obj::cpic();
        $inherit_padv=$cpic->{padv};
        if (!$inherit_padv || $inherit_padv eq 'Common' || $inherit_padv eq $padv) {
            $inherit_padv=Padv::inherit_uplevel($padv);
        }

        if ($inherit_padv && UNIVERSAL::can("$class\::$inherit_padv", 'new')) {
            $codes = "package $class\::$padv; \@$class\::$padv\::ISA = qw($class\::$inherit_padv);";
        } elsif (UNIVERSAL::can("$class\::$plat", 'new')) {
            $codes = "package $class\::$padv; \@$class\::$padv\::ISA = qw($class\::$plat);";
        } elsif (UNIVERSAL::can("$class\::Common", 'new')){
            $codes = "package $class\::$padv; \@$class\::$padv\::ISA = qw($class\::Common);";
        } else {
            $baseclass=$class;
            $baseclass=~s/\::.*$//m;
            $codes = "package $class\::$padv; \@$class\::$padv\::ISA = qw($baseclass);";
        }
        eval $codes;
    }
    return "$class\::$padv"->new();
}

# expand an PAR or perl2exe binary, if necessary
sub expand_exe { }

# Performing the mini-pkg inventory here
sub verify_media {
    my ($cpic,$rel) = @_;
    my ($patch,$prodi,$pkgi,$pkglist,$pkg,$patchi,$prod);
    @$pkglist=();
    push(@$pkglist, @{$rel->allpkgs});
    # Create the list of related packages for the release
    for my $prodi (@{$rel->{prods}}) {
        $prod=$rel->prod($prodi);
        push(@$pkglist,@{$prod->allpkgs_on_media});
    }
    $pkglist=EDRu::arruniq(@$pkglist);
    for my $pkgi (@$pkglist) {
        $pkg=$rel->pkg($pkgi);
        $pkg->verify_media();
        for my $patchi (@{$pkg->patches_allvers($rel->{currentrelease})}) {
            $patch=$rel->patch($patchi);
            $patch->verify_media($pkg);
        }
    }
    return;
}

sub pkg_patch_size {
    my ($cpic,$file)= @_;
    my $size;
    $size=EDR::cmd_local("_cmd_du -sk $file | _cmd_awk '{print \$1}'");
    # assume compression of 3x, so *4 as both
    # tarfile and original are there after gunzip
    $size*=4 if ($file =~ /gz$/m);
    return $size;
}

sub init_args {
    my ($cpic, $bundle_only) = (@_);
    my ($pdfrs,$arguments,$arg_def);
    $pdfrs=Msg::get('pdfrs');

    $cpic->{arguments} = $arguments = {
        'args_task' => [ qw(install configure uninstall license upgrade patchupgrade
                            precheck requirements start stop startstoptest postcheck) ],

        'args_def' => [ qw(browserid keyfile hostfile pkgpath patchpath rootpath
                           prod threadlimit sshpath scppath) ],

        'args_opt' => [ qw(installminpkgs installrecpkgs installallpkgs
                           minpkgs recpkgs allpkgs repkgs missingpkgs
                           pkgset pkgtable pkginfo listpatches
                           makeresponsefile comcleanup version history
                           nolic ignorepatchreqs settunables tunables testuploadurl
                           bait mpok nospacecheck upgradecv askfile nostart ignorechecks) ],

        # The arguments with high priority are that arguments need be handled even if '-h' is provided to show usage.
        'args_high_priority' => [ qw(require responsefile prod install) ],

        # Those arguments have no specific entry here are considered as undocumented arguments.
        # Those arguments have specific entry here but without 'description' attribute are considered as undocumented arguments.
        # Those arguments have specific entry here and with 'undocumented' attribute are considered as undocumented arguments.

        # CPIC task args
        'install' => {
            'description' => Msg::new("The -install option is used to install products on systems", 30, 1054),
            'handler' => \&CPIC::process_install_arg,
        },
        'configure' => {
            'description' => Msg::new("The -configure option is used to configure an unconfigured product following use of the -install option", 30, 1055),
        },
        'uninstall' => {
            'description' => Msg::new("The -uninstall option is used to uninstall products from systems", 30, 1056),
        },
        'license' => {
            'description' => Msg::new("The -license option is used to register product licenses on multiple systems", 30, 1057),
        },
        'upgrade' => {
            'description' => Msg::new("The -upgrade option is used to upgrade all $pdfrs installed on systems", 30, 1058, "$pdfrs"),
        },
        'precheck' => {
            'description' => Msg::new("The -precheck option is used to confirm that systems meet the products install requirements before installing", 30, 1059),
        },
        'requirements' => {
            'description' => Msg::new("The -requirements option displays required OS version, required patches, file system space, and other system requirements in order to install the product", 30, 1060),
        },
        'start' => {
            'description' => Msg::new("The -start option starts all drivers and processes of a product", 30, 1061),
        },
        'stop' => {
            'description' => Msg::new("The -stop option stops all drivers and processes of a product", 30, 1062),
        },
        'postcheck' => {
            'description' => Msg::new("The -postcheck option is used to check any issues after installation or upgrading on the system.", 30, 1063),
        },

        # CPIC definition args
        'prod' => {
            'handler' => \&CPIC::process_prod_arg,
        },
        'hostfile' => {
            'option_description' => Msg::new("<hostfile_path>", 30, 1064),
            'description' => Msg::new("The -hostfile option specifies the location of a file containing the system names for installer", 30, 1065),
            'handler' => \&CPIC::process_hostfile_arg,
        },
        'keyfile' => {
            'option_description' => Msg::new("<ssh_key_file>", 30, 1066),
            'description' => Msg::new("The -keyfile option specifies a key file for SSH. When this option is used, '-i <ssh_key_file>' is passed to every SSH invocation.", 30, 1067),
            'handler' => \&CPIC::process_keyfile_arg,
        },
        'pkgpath' => {
            'option_description' => Msg::new("<pkg_path>", 30, 1068),
            'description' => Msg::new("The -pkgpath option is used to define the complete path of a directory available to all install systems (usually NFS mounted) that contains all $pdfrs to be installed by $cpic->{script}", 30, 1069, "$pdfrs", "$cpic->{script}"),
            'handler' => \&CPIC::process_pkgpath_arg,
        },
        'patchpath' => {
            'option_description' => Msg::new("<patch_path>", 30, 1070),
            'description' => Msg::new("The -patchpath option is used to define the complete path of a directory available to all install systems (usually NFS mounted) that contains all patches to be installed by $cpic->{script}", 30, 1071, "$cpic->{script}"),
            'handler' => \&CPIC::process_patchpath_arg,
        },
        'rootpath' => {
            'option_description' => Msg::new("<root_path>", 30, 1072),
            'description' => Msg::new("The -rootpath option is used to re-root the install of all packages to the given path. On Solaris, -rootpath passes -R <root_path> to pkgadd or pkgrm. The -rootpath option is not supported on AIX, HPUX or Linux.", 30, 1073),
            'handler' => \&CPIC::process_rootpath_arg,
        },
        'base_path' => {
            'option_description' => Msg::new("<base_path>", 30, 1074),
            'description' => Msg::new("The -base_path option is used to define the path of a base level release to be integrated with a maintenance level release in order for the two releases to be simultaneously installed", 30, 1075),
            'handler' => \&CPIC::process_base_path_arg,
        },
        'threadlimit' => {
            'handler' => \&CPIC::process_threadlimit_arg,
        },
        'sshpath' => {
            'handler' => \&CPIC::process_sshpath_arg,
        },
        'scppath' => {
            'handler' => \&CPIC::process_scppath_arg,
        },

        # CPIC option args
        'installminpkgs' => {
            'description' => Msg::new("The -installminpkgs option is used to select the minimum package set", 30, 1076),
        },
        'installrecpkgs' => {
            'description' => Msg::new("The -installrecpkgs option is used to select the recommended package set", 30, 1077),
        },
        'installallpkgs' => {
            'description' => Msg::new("The -installallpkgs option is used to select all $pdfrs", 30, 1078, "$pdfrs"),
        },
        'minpkgs' => {
            'description' => Msg::new("The -minpkgs option is used to display the minimum product $pdfrs (excludes optional $pdfrs) in correct installation order", 30, 1079, "$pdfrs", "$pdfrs"),
            'handler' => \&CPIC::process_pkg_patch_arg,
        },
        'recpkgs' => {
            'description' => Msg::new("The -recpkgs option is used to display the recommended product $pdfrs (excludes optional $pdfrs) in correct installation order", 30, 1080, "$pdfrs", "$pdfrs"),
            'handler' => \&CPIC::process_pkg_patch_arg,
        },
        'allpkgs' => {
            'description' => Msg::new("The -allpkgs option is used to display all product $pdfrs in correct installation order", 30, 1081, "$pdfrs"),
            'handler' => \&CPIC::process_pkg_patch_arg,
        },
        'repkgs' => {
            'handler' => \&CPIC::process_pkg_patch_arg,
        },
        'missingpkgs' => {
            'handler' => \&CPIC::process_pkg_patch_arg,
        },
        'pkgset' => {
            'description' => Msg::new("The -pkgset option is used to discover the package set installed on the systems specified", 30, 1082),
            'handler' => \&CPIC::process_pkg_patch_arg,
        },
        'pkgtable' => {
            'description' => Msg::new("The -pkgtable option is used to display product $pdfrs in correct installation order", 30, 1083, "$pdfrs"),
            'handler' => \&CPIC::process_pkg_patch_arg,
        },
        'pkginfo' => {
            'description' => Msg::new("The -pkginfo option is used to display the correct install order of $pdfrs and patches. This option is available with or without one of following options: -allpkgs, -minpkgs, and -recpkgs.", 30, 1084, "$pdfrs"),
            'handler' => \&CPIC::process_pkg_patch_arg,
        },
        'listpatches' => {
            'description' => Msg::new("The -listpatches option is used to display product patches in correct installation order", 30, 1085),
            'handler' => \&CPIC::process_pkg_patch_arg,
        },
        'makeresponsefile' => {
            'description' => Msg::new("The -makeresponsefile option generates a response file without doing an actual installation. Text displaying install, uninstall, start, and stop actions are simulations. These actions are not being performed on the system.", 30, 1086),
        },
        'comcleanup' => {
            'description' => Msg::new("The -comcleanup option is used to remove the ssh or rsh configuration added by installer on the systems. The option is only required when installation routines that performed auto-configuration of ssh or rsh are abruptly terminated.", 30, 1087),
            'handler' => \&CPIC::process_comcleanup_arg,
        },
        'version' => {
            'description' => Msg::new("The -version option is used to check the status of installed products on the system.", 30, 1088),
            'handler' => \&CPIC::process_version_arg,
        },
        'history' => {
            #'description' => Msg::new("The -history option is used together with -version to list installed packages (and patches) on local systems."),
            'handler' => \&CPIC::process_version_arg,
        },
        'nolic' => {
            'description' => Msg::new("The -nolic option is used to install product $pdfrs on systems without entering product licenses.  Configuration, startup, or installation of license based features are not performed when using this option.", 30, 1089, "$pdfrs"),
        },
        'ignorepatchreqs' => {
            'description' => Msg::new("The -ignorepatchreqs option is used to allow installation or upgrading even if the prerequisite packages or patches are missed on the system.", 30, 1090),
        },
        'settunables' => {
            'description' => Msg::new("The -settunables option is used to set tunable parameters after a product is installed and configured.  Processes of the installed product may need to be restarted for the tunable parameter values to take effect.  This option must be used together with -tunablesfile option.", 30, 1091),
            'handler' => \&CPIC::process_settunables_arg,
        },
        'testuploadurl' => {
            'handler' => \&CPIC::process_testuploadurl_arg,
        },
        'tunables' => {
            'handler' => \&CPIC::process_tunables_arg,
            'description' => Msg::new("The -tunables option is used to list all supported tunables and create a tunables file template.", 30, 1092),
        },
    };

    # modify $arguments hash based on initial $bundle_only call
    my @bundle_args=qw(base_path);
    if ($cpic->maintenance_release && $cpic->{script} !~ /^uninstallmr/) {
        push(@bundle_args, 'base_path');
        push(@{$$arguments{args_def}}, 'base_path');
    } else {
        delete $$arguments{base_path};
    }
    for my $arg (keys %{$arguments}) {
        next if ($arg =~ /^args_/);
         if (EDRu::inarr($arg, @bundle_args)) {
             # delete the bundle_args handler for the normal run
             delete $$arguments{$arg}{handler} if (!$bundle_only);
         } elsif (ref($$arguments{$arg}{handler})) {
             # delete all non bundle args handlers for the bundle_args run
             delete $$arguments{$arg}{handler} if ($bundle_only);
         }
     }
     $cpic->{arguments}=$arguments;
     for my $arg (keys %{$arguments}) {
         $arg_def=$arguments->{$arg};
         if (ref($arg_def) eq 'HASH' &&
             defined $arg_def->{handler} &&
             !defined $arg_def->{handler_args}) {
             $arg_def->{handler_args} = [ $cpic ];
         }
     }
    return 1;
}

sub filter_args {
    my ($cpic,$args_hash)=@_;
    my ($script,$plat,$rel,@unused_args);

    $script=$cpic->{script};
    $plat=Padv::plat($cpic->{padv});

    if ($script=~/^uninstall/) {
        if ($script =~ /^uninstallrp/ || $script =~ /^uninstallmr/) {
            $args_hash->{args_task}=[ qw(rollback) ];
        } else {
            $args_hash->{args_task}=[ qw(uninstall rollback) ];
        }
        #do not show -uninstall for uninstallxxx script 
        $args_hash->{uninstall}->{undocumented}=1;
        @unused_args=qw(tunablesfile pkgpath patchpath nolic tunables
                        settunables upgrade_kernelpkgs upgrade_nonkernelpkgs
                        rolling_upgrade rollingupgrade_phase1 rollingupgrade_phase2
                        installminpkgs installrecpkgs installallpkgs
                        minpkgs recpkgs allpkgs repkgs
                        pkgset pkgtable pkginfo
                        jumpstart flash_archive kickstart yumgroupxml nim ignite
                        security securityonenode securitytrust fips addnode fencing configcps ignorepatchreqs
                        );
        #have to delete both args_opt and args_def.
        $args_hash->{args_opt}=EDRu::arrdel($args_hash->{args_opt}, @unused_args);
        $args_hash->{args_def}=EDRu::arrdel($args_hash->{args_def}, @unused_args);
    } elsif ($script eq 'install_lp') {
        $args_hash->{args_task}=[ qw(install) ];
        @unused_args=qw(installminpkgs installrecpkgs installallpkgs
                        minpkgs recpkgs allpkgs);
        $args_hash->{args_opt}=EDRu::arrdel($args_hash->{args_opt}, @unused_args);
    } elsif ($script eq 'showversion') {
        $args_hash->{args_task}=[];
        $args_hash->{args_def}=[qw(hostfile keyfile sshpath scppath rootpath)];
        $args_hash->{args_opt}=[qw(rsh version history)];

        # Change description for -hostfile
        $args_hash->{hostfile}->{description}=Msg::new("The -hostfile option specifies the location of a file containing the system names for version check", 30, 1093);

        # Change description for -rootpath
        $args_hash->{rootpath}->{description}=Msg::new("The -rootpath option is used to specify the alternative root path for version check. On Solaris, -rootpath passes -R <root_path> to pkginfo. The -rootpath option is not supported on AIX, HPUX or Linux.", 30, 1094);

        # Do not show -version for showversion script
        $args_hash->{version}->{undocumented}=1;
    } elsif ($script eq 'installrp' || $script eq 'installmr') {
        if (Cfg::opt('base_path')) {
            $args_hash->{args_task}=[ qw(patchupgrade precheck install upgrade) ];
        } elsif ($plat ne 'Linux') {
            $args_hash->{args_task}=[ qw(patchupgrade precheck rollback postcheck) ];
        } else {
            $args_hash->{args_task}=[ qw(patchupgrade precheck postcheck) ];
        }
        @unused_args=qw(security securityonenode securitytrust fips addnode fencing configcps
                        installminpkgs installrecpkgs installallpkgs
                        minpkgs recpkgs allpkgs settunables pkgpath tunablesfile tunables);
        $args_hash->{args_opt}=EDRu::arrdel($args_hash->{args_opt}, @unused_args);
        $args_hash->{args_def}=EDRu::arrdel($args_hash->{args_def}, @unused_args);
    } elsif ($script ne 'installer' && $script =~ /^install/) {
        $args_hash->{install}->{undocumented}=1;
        $args_hash->{uninstall}->{undocumented}=1;
    }

    # Only SunOS need -rootpath option
    if ($plat ne 'SunOS') {
        $args_hash->{args_def}=EDRu::arrdel($args_hash->{args_def},qw(rootpath));
    }

    # Only SunOS and HPUX need -ignorepatchreqs option
    if ($plat ne 'SunOS' && $plat ne 'HPUX') {
        $args_hash->{args_opt}=EDRu::arrdel($args_hash->{args_opt},qw(ignorepatchreqs));
    }

    # Only SP/RP releases need -listpatches/-patchpath options
    $rel=$cpic->rel;
    if (!$rel->{patches}) {
        $args_hash->{args_def}=EDRu::arrdel($args_hash->{args_def}, qw(patchpath));
        $args_hash->{args_opt}=EDRu::arrdel($args_hash->{args_opt}, qw(listpatches));
    }

    return 1;
}

# To handle additional arguments in require file
sub init_args2 {}

# process maintenance_path, and hotfix_path args first so code can be imported
# before create_cpip_objects is called
sub process_bundle_args {
    my $cpic=shift;
    my $edr=Obj::edr();
    my $cfg=Obj::cfg();
    my $args_hash = {};
    my $bargs_hash = {};

    # quick dirty responsefile read, just for bundle args
    for my $n(0..$#ARGV) {
        next if ($ARGV[$n] ne '-responsefile');
        my $responsefile=$ARGV[$n+1];
        # requiring here hoses the require in EDR::process_responsefile_arg
        # reading file instead, let any errors get handled there
        if (-f $responsefile) {
            for my $line( split(/\n/, EDRu::readfile($responsefile) ) ) {
                next if ($line=~/^#/);
                for my $barg(qw(base_path hotfix_path hotfix2_path hotfix3_path hotfix4_path hotfix5_path)) {
                    next if ($line!~/{opt}{$barg}/);
                    my (undef,$val,undef)=split(/["']/,$line,3);
                    Cfg::set_opt($barg, $val);
                }
            }
        }
    }


    $cpic->init_args(1);
    $args_hash=$edr->merge_args($args_hash, $cpic->{arguments});
    #only process bundle arguments here
    for my $barg(qw(base_path hotfix_path hotfix2_path hotfix3_path hotfix4_path hotfix5_path)) {
        $bargs_hash->{$barg} = $args_hash->{$barg};
    }
    $bargs_hash->{args_def} = ['base_path', 'hotfix_path', 'hotfix2_path', 'hotfix3_path', 'hotfix4_path', 'hotfix5_path'];

    $edr->process_args($args_hash, 1);
    delete $cfg->{systems};
    return;
}

sub process_args {
    my $cpic=shift;
    my ($edr,$rel,$args_hash);

    $args_hash = {};

    $edr=Obj::edr();
    $edr->init_args();
    $edr->init_args2();
    $args_hash=$edr->merge_args($args_hash, $edr->{arguments});

    $cpic->init_args();
    $cpic->init_args2();
    $args_hash=$edr->merge_args($args_hash, $cpic->{arguments});

    $rel=$cpic->rel;
    $rel->init_args();
    $rel->init_args2();
    $args_hash=$edr->merge_args($args_hash, $rel->{arguments});

    $cpic->filter_args($args_hash);
    $rel->filter_args($args_hash);

    # Save all arguments to Rel level for later using.
    $rel->{args_task} = $args_hash->{args_task};
    $rel->{args_def} = $args_hash->{args_def};
    $rel->{args_opt} = $args_hash->{args_opt};

    $edr->process_args($args_hash);

    # To verify responsefile
    $cpic->process_responsefile_arg();

    return;
}

sub process_prod_arg {
    my ($cpic,$args_hash) = @_;
    my ($cfg,$prod,$rel,$err,$msg);

    $err = 0;
    $cfg = Obj::cfg();
    $prod = $cfg->{opt}{prod} || $cfg->{prod} || $cpic->{prod};
    if ($prod) { # for upgrade in responsefile there is no CFG{prod}
        $rel=$cpic->rel;
        if (EDRu::inarr($prod,@{$rel->{prods}})) {
            $cpic->set_prod($prod);
            $rel->filter_args($args_hash,$prod);
        } else {
            $msg=Msg::new("$prod is not a valid product argument", 30, 1095, "$prod");
            $msg->warning;
            $err++;
        }
    }
    return $err;
}

sub process_install_arg {
    my $cpic = shift;
    my ($prod,$msg);

    if ($cpic->{prod}) {
        $prod=$cpic->prod;
        if (!$prod->hasconfig()) {
            $msg=Msg::new("The -install option is used to install and start products on systems", 30, 1096);
            $cpic->{arguments}->{install}->{description}=$msg;
        }
    }
    return 0;
}

sub process_rootpath_arg {
    my $cpic = shift;
    my ($cfg,$edr,$err,$msg,$lsys,$rootpath);

    $cfg = Obj::cfg();
    $edr = Obj::edr();
    $err=0;

    $rootpath = Cfg::opt('rootpath');
    if ($rootpath) {
        if (EDRu::nonascii($rootpath)) {
             # Not allow NON-ASCII mount point
             $msg=Msg::new("rootpath directory does not allow NON-ASCII mount point", 30, 1097);
             $msg->warning;
             $err++;
        } elsif (! -d $rootpath) {
            # check rootpath on local only when it is on target list: E1665977
            if (defined($cfg->{systems})) {
                $lsys=$edr->{localsys};
                if (EDRu::inarr($lsys->{sys}, @{$cfg->{systems}})) {
                    $msg=Msg::new("rootpath directory: $rootpath not found", 30, 1098, "$rootpath");
                    $msg->warning;
                    $err++;
                }
            } else {
                $msg=Msg::new("rootpath directory: $rootpath not found", 30, 1098, "$rootpath");
                $msg->warning;
                $err++;
            }
        }

        # does not allow -configure and -rootpath at the same time, no config on altroot
        if (Cfg::opt('configure')) {
            $msg=Msg::new("The -configure option cannot be used with -rootpath", 30, 1099);
            $msg->die();
        }

        # if local system is one of the target systems and defined rootpath,
        # or no target system names specified, change the logpath under rootpath
        if ( -d $rootpath) {
            $lsys=$edr->{localsys};
            if ($lsys->sunos() &&
                ($lsys->exists("$rootpath/etc/release"))) {
                if (defined($cfg->{systems})) {
                    $edr->{logpath}="$rootpath$edr->{logpath}"
                        if (EDRu::inarr($lsys->{sys}, @{$cfg->{systems}}));
                } else {
                    $edr->{logpath}="$rootpath$edr->{logpath}";
                }
            }
        }
    }
    return $err;
}

sub process_hostfile_arg {
    my $cpic = shift;
    my ($err,$msg,$hostfile);

    $err=0;
    $hostfile = Cfg::opt('hostfile');
    if ($hostfile) {
        if (! -f $hostfile) {
            $msg=Msg::new("host file: $hostfile not found", 30, 1100, "$hostfile");
            $msg->warning;
            $err++;
        }
    }
    return $err;
}

sub process_sshpath_arg {
    my $cpic = shift;
    my ($cfg,$msg);

    $cfg = Obj::cfg();
    if ($cfg->{opt}{sshpath}) {
        # Since shell will seperate the arguments with space,
        # so need change spaces to '#' in arguments, like:
        # change "-sshpath "<your_sshpath> -<options1> -<options2>" to "-sshpath "<your_sshpath>#-<options1>#-<options2>"
        $cfg->{opt}{sshpath}=~s/#/ /mg;
    } elsif (defined $ENV{'SSHPATH'}) {
        $cfg->{opt}{sshpath}=$ENV{'SSHPATH'};
    }
    if ($cfg->{opt}{sshpath}) {
        $msg=Msg::new("Symantec only tests and supports '$cpic->{script}' using native OS ssh binaries.  Running '$cpic->{script}' operations using independently built ssh binaries should operate correctly, but is not officially supported.", 30, 1101, "$cpic->{script}", "$cpic->{script}");
        $msg->warning;
        sleep 2;
    }
    return 0;
}

sub process_scppath_arg {
    my $cpic = shift;
    my ($cfg,$msg);

    $cfg = Obj::cfg();
    if ($cfg->{opt}{scppath}) {
        # Since shell will seperate the arguments with space,
        # so need change spaces to '#' in arguments, like:
        # change "-scppath "<your_scppath> -<options1> -<options2>" to "-scppath "<your_sshpath>#-<options1>#-<options2>"
        $cfg->{opt}{scppath}=~s/#/ /mg;
    } elsif (defined $ENV{'SCPPATH'}) {
        $cfg->{opt}{scppath}=$ENV{'SCPPATH'};
    }
    if ($cfg->{opt}{scppath}) {
        $msg=Msg::new("Symantec only tests and supports '$cpic->{script}' using native OS scp binaries.  Running '$cpic->{script}' operations using independently built scp binaries should operate correctly, but is not officially supported.", 30, 1102, "$cpic->{script}", "$cpic->{script}");
        $msg->warning;
        sleep 2;
    }
    return 0;
}

sub process_keyfile_arg {
    my $cpic = shift;
    my ($err,$msg,$keyfile);

    $err=0;
    $keyfile = Cfg::opt('keyfile');
    if ($keyfile) {
        if (Cfg::opt('rsh')) {
            $msg=Msg::new("The -keyfile option and -rsh option cannot be used together", 30, 1103);
            $msg->warning;
            $err++;
        } elsif (! -f $keyfile) {
            $msg=Msg::new("key file: $keyfile not found", 30, 1104, "$keyfile");
            $msg->warning;
            $err++;
        }
    }
    return $err;
}

sub process_threadlimit_arg {
    my $cpic = shift;
    my ($err,$msg,$threadlimit);

    $err=0;
    $threadlimit = Cfg::opt('threadlimit');
    if (defined $threadlimit) {
        if (!EDRu::isint($threadlimit) || ($threadlimit < 1)) {
            $msg=Msg::new("The threadlimit value must be a positive integer.", 30, 1105);
            $msg->warning;
            $err++;
        } else {
            $cpic->{threadlimit}=$threadlimit;
        }
    }
    return $err;
}

sub process_pkgpath_arg {
    my $cpic = shift;
    my ($msg,$pkgpath,$err);

    $err=0;
    $pkgpath = Cfg::opt('pkgpath');
    if ($pkgpath && ! -d $pkgpath) {
        if (! -e $pkgpath) {
            $msg=Msg::new("The specified pkgpath '$pkgpath' does not exist", 30, 1106, "$pkgpath");
        } else {
            $msg=Msg::new("The specified pkgpath '$pkgpath' is not a directory", 30, 1107, "$pkgpath");
        }
        $msg->warning;
        $err++;
    }
    return $err;
}

sub process_patchpath_arg {
    my $cpic = shift;
    my ($msg,$patchpath,$err);

    $err=0;
    $patchpath = Cfg::opt('patchpath');
    if ($patchpath && ! -d $patchpath) {
        if (! -e $patchpath) {
            $msg=Msg::new("The specified patchpath '$patchpath' does not exist", 30, 1108, "$patchpath");
        } else {
            $msg=Msg::new("The specified patchpath '$patchpath' is not a directory", 30, 1109, "$patchpath");
        }
        $msg->warning;
        $err++;
    }
    return $err;
}

sub process_pkg_patch_arg {
    my $cpic = shift;
    my ($cfg,$edr);

    $cfg = Obj::cfg();
    $edr = Obj::edr();
    if (Cfg::opt('pkginfo') &&
        (!Cfg::opt('allpkgs')) && (!Cfg::opt('minpkgs')) && (!Cfg::opt('recpkgs'))) {
        $cfg->{opt}{allpkgs} = 1;
        $cfg->{opt}{recpkgs} = 1;
        $cfg->{opt}{minpkgs} = 1;
    }

    if (Cfg::opt(qw(listpatches missingpkgs allpkgs minpkgs recpkgs repkgs))) {
        $cpic->check_pkg_inventory();
        $edr->exit_noexitfile();
    }

    if (Cfg::opt('pkgset')) {
        $cpic->installed_pkgs();
        $edr->exit_noexitfile();
    }

    if (Cfg::opt('pkgtable')) {
        $cpic->display_pkgs();
        $edr->exit_noexitfile();
    }

    return 0;
}

sub process_base_path_arg {
    my $cpic = shift;
    my $msg;
    my $isbase;
    my $br;
    my $basepath=Cfg::opt('base_path');

    my @base_path_exclude_args = qw(postcheck rollback patchupgrade);
    for my $exclude_arg (@base_path_exclude_args) {
        if (Cfg::opt($exclude_arg)) {
            $msg = Msg::new("The -base_path option and -$exclude_arg option cannot be used together", 30, 1110, "$exclude_arg");
            $msg->error;
            return 1;
        }
    }

    if (! -e $basepath) {
        $msg=Msg::new("The base_path '$basepath' does not exist", 30, 1111, "$basepath");
    } elsif (!-d $basepath) {
        $msg=Msg::new("The base_path '$basepath' is not a directory", 30, 1112, "$basepath");
    } elsif (!-d "$basepath/scripts/EDR" ||
             !-d "$basepath/scripts/CPIC" ||
             !-d "$basepath/scripts/CPIP") {
        $msg=Msg::new("The base_path '$basepath' is not a directory that contains a Storage Foundation release", 30, 1113, "$basepath");
    }
    if ($msg) {
        $msg->error;
        return 1;
    } else {
        my $mr=$cpic->maintenance_release;
        if (!$mr) {
            # must be run from a maintenance release only and should never hit this, but just in case
            $msg=Msg::new("The -base_path argument can be executed only with a maintenance release", 30, 1114);
            $msg->error;
            return 1;
        }
        my $padv=$cpic->read_install_script_padv($basepath);
        my $padvs=join(" ", @{$cpic->{padvs}});
        $padv=EDRu::despace($padv);
        $padvs=EDRu::despace($padvs);
        if (!$padv) {
            # should not hit this unless someone alters the install script
            $msg=Msg::new("Cannot determine the platform for the base_path '$basepath' release", 30, 1115, "$basepath");
            $msg->error;
            return 1;
        }
        if ($padv ne $padvs) {
            #$msg=Msg::new("The base_path '$basepath' and $cpic->{script} release are not intended for the same platform");
            $msg=Msg::new("The base_path '$basepath' ($padv) and $cpic->{script} release ($padvs) are not intended for the same platform", 30, 1116, "$basepath", "$padv", "$cpic->{script}", "$padvs");
            $msg->error;
            return 1;
        }
        #workaround to check if base_path is a patch release.
        if ((-f "$basepath/installmr") || (-f "$basepath/installrp")) {
            $msg=Msg::new("The base_path can only contains a Storage Foundation major release", 30, 1117);
            $msg->error;
            return 1;
        }
        #workaround to check baes_release version in 6.0.X releases.
        for my $base_require ($cpic->base_requires) {
            my $version_file = "UXRT" . join('', split(/\./, $base_require)) . ".pm";
            if (-f "$basepath/scripts/CPIP/Rel/$version_file") {
                $cpic->{basemediapath}=$basepath;
                $cpic->{base_path_release} = $base_require;
                return '';
            }
        }
        $msg=Msg::new("The base_path '$basepath' contains a Storage Foundation release which is not compatible with $cpic->{script}", 30, 1118, "$basepath", "$cpic->{script}");
        $msg->error;
        return 1;
    }
}

# assumes that all CPI scripts are a five field call to CPIC::main, return the second field
sub read_install_script_padv {
    my ($cpic,$basepath)=(@_);
    my @files=glob("$basepath/*");
    for my $file(@files) {
        next unless ($file=~/$basepath\/install/);
        my $main;
        my $code=EDRu::readfile($file);
        for my $line(split(/\n/,$code)) {
            next if (!$main) && ($line!~/CPIC::main/);
            $line=~s/^\s+//;
            $line=~s/"//g;
            $main.=$line;
        }
        my (undef,$padv,undef)=split(/,/,$main,3);
        return $padv;
    }
}


sub process_version_arg {
    my $cpic = shift;
    my $edr = Obj::edr();
    if (Cfg::opt('version')) {
        $cpic->version_check();
        $edr->exit_noexitfile();
    }
    return 0;
}

sub process_settunables_arg {
    my $cpic = shift;
    my ($edr,$msg,$tunables,$need_stop,$need_start,$rel,$prodname,$ans,$shutdown);

    $edr = Obj::edr();
    $rel = $cpic->rel;
    if (Cfg::opt('settunables')) {
        if (!Cfg::opt('tunablesfile')) {
            $msg = Msg::new("No tunables file was specified.\n\nA tunables file must be specified using -tunablesfile option when -settunables option is used.", 30, 1119);
            $msg->print;
        } else {
            $cpic->validate_tunables();

            $need_stop = 0;
            $need_start = 0;
            if ( $cpic->{valid_tunables_prestart} && @{$cpic->{valid_tunables_prestart}}) {
                $need_stop = 1;
                $need_start = 1;
            } else {
                if ( $cpic->{valid_tunables_poststart} && @{$cpic->{valid_tunables_poststart}}) {
                    $need_start = 1;
                }
            }

            if (!$cpic->{donotsettunables} && $need_start) {
                $cpic->verify_media($cpic->rel);
                $cpic->{prod} = $rel->get_product_for_settunables;
                for my $sys (@{$cpic->{systems}}) {
                    if ($sys->{settunable_prodvers} ne $cpic->{systems}[0]{settunable_prodvers}) {
                        $msg=Msg::new("Different products or product versions are installed on these systems", 30, 1120);
                        $msg->bold;
                        $msg=Msg::new("Please set tunables for these systems separately", 30, 1121);
                        $msg->bold;
                        Msg::n();
                        $edr->exit_noexitfile();
                        return 1;
                    }
                }

                $prodname = $cpic->prod->{name};
                if ( $need_stop && $need_start ) {
                    $msg=Msg::new("$prodname will be restarted on the specified system(s). Do you want to continue?", 30, 1122, "$prodname");
                    $ans = $msg->ayny;
                    $cpic->{donotsettunables} = 1 if ($ans eq 'N');
                } elsif ($need_start) {
                    $msg=Msg::new("$prodname will be started on the specified system(s). Do you want to continue?", 30, 1123, "$prodname");
                    $ans = $msg->ayny;
                    $cpic->{donotsettunables} = 1 if ($ans eq 'N');
                }
            }

            if (!$cpic->{donotsettunables}) {
                if ( $need_stop && $cpic->{prod}) {
                    # stop product here
                    $cpic->shutdown;
                    $cpic->set_tunables_any();
                    $cpic->set_tunables_prestart();
                }
                if ( $need_start && $cpic->{prod}) {
                    # start product here
                    $cpic->startup;
                    $cpic->set_tunables_any() if (!$need_stop);
                    $cpic->set_tunables_poststart();
                }
                $cpic->set_tunables_any() if ((!$need_stop && !$need_start)||!$cpic->{prod});
                $cpic->print_tunable_failures;

                for my $sys (@{$cpic->{systems}}) {
                    if (defined $sys->{requirereboottunables}) {
                        $shutdown = $sys->padv->{cmd}{shutdown};
                        $tunables=join("\n\t", @{$sys->{requirereboottunables}});
                        $msg=Msg::new("The following tunables require reboot to take effect on $sys->{sys}:\n\t$tunables\n\nExecute '$shutdown' to properly restart your systems", 30, 1124, "$sys->{sys}", "$tunables", "$shutdown");
                        $msg->print;
                        Msg::n();
                    }
                }
            }
        }

        $edr->exit_noexitfile();
    }
    return 0;
}

sub process_comcleanup_arg {
    my $cpic = shift;
    my $edr = Obj::edr();
    if (Cfg::opt('comcleanup')) {
        $edr->{com_setup} = 1;
        $edr->exit_noexitfile();
    }
    return 0;
}

sub process_testuploadurl_arg {
    my $cpic = shift;
    my $edr = Obj::edr();
    if (Cfg::opt('testuploadurl')) {
        $cpic->rel->get_upload_url();
        $edr->exit_noexitfile();
    }
    return 0;
}

sub process_tunables_arg {
    my $cpic = shift;
    my $edr = Obj::edr();
    my $templatefile = '/var/tmp/'.EDRu::plat($cpic->{padv}).'.tunables';
    my ($msg,$msg_tun,$msg_prod,$titleline);
    if (Cfg::opt('tunables')) {
        $edr->{all_tunables} = $cpic->all_tunables;
        # print all supported tunables
        $msg = Msg::new("The following tunables are supported:", 30, 1125);
        $msg->bold;
        $msg_tun= Msg::new("Tunable", 30, 1032);
        $msg_prod= Msg::new("Product", 30, 1126);
        $titleline=sprintf "\t%-32s %s",$msg_tun->{msg},$msg_prod->{msg};
        Msg::bold($titleline);
        for my $tun (sort keys %{$edr->{all_tunables}}) {
            printf "\t%-32s %s\n",$tun,$edr->{all_tunables}{$tun}{define_object}{name};
        }
        Msg::n();
        EDRu::writefile(Obj::cfg()->tunables_template_content(), "$templatefile");
        $msg = Msg::new("Tunables file template is created in $templatefile", 30, 1127, "$templatefile");
        $msg->bold();
        $edr->exit_noexitfile();
    }
    return 0;
}

sub process_responsefile_arg {
    my $cpic = shift;

    if (Cfg::opt('responsefile')) {
        $cpic->verify_responsefile();
        if ($cpic->{prod}) {
            $cpic->prod->verify_responsefile();
        }
    }
    return 1;
}

sub verify_responsefile {
    my $cpic=shift;
    my ($cfg,$prod,$msg);

    $cfg=Obj::cfg();
    # for upgrade in responsefile there is no CFG{prod}
    unless (($cfg->{prod}) || (Cfg::opt(qw(upgrade patchupgrade rollback)))) {
        $msg=Msg::new("\$CFG{prod} should be defined in responsefile", 30, 1128);
        $msg->die;
    }

    # $cpic->{task} mean that user select task from task menu, or specify task in CPIC::main().
    # Then need unset $cpic->{task} if use responsefile
    $cpic->{task}=undef;

    # Set $cpic->{task} as 'install', in case install+config with responsefile
    # which do not require stopping processes.
    $cpic->{task}='install' if (Cfg::opt('install'));

    return;
}

sub display_pkgs {
    my $cpic = shift;
    my ($edr,$pkg,$pkg_set,$pkgname,$pkgs,$pl,$prod,$prodi,$prods,$rel,@pkgs_data);
    my ($format,$seperator,$msg_name,$msg_set,$msg_desc,$msg_min,$msg_rec,$msg_all,$n,$noprtc);

    $edr=Obj::edr();
    $rel=$cpic->rel;

    $prods = ($cpic->{prod}) ? [ $cpic->{prod} ] : $rel->{prods};
    $msg_name=Msg::new("Name", 30, 1129)->{msg};
    $msg_set=Msg::new("Set", 30, 1130)->{msg};
    $msg_desc=Msg::new("Description", 30, 1131)->{msg};
    $msg_min=Msg::new("Minimum", 30, 1132)->{msg};
    $msg_rec=Msg::new("Recommended", 30, 1133)->{msg};
    $msg_all=Msg::new("All", 30, 1134)->{msg};
    Msg::page_reset();
    $noprtc=1 if ($#$prods>=1);
    $seperator='=' x 80 . "\n";
    foreach my $prodi ( sort(@{$prods}) ) {
        $prod=$cpic->prod($prodi);
        next if ($prod->{obsoleted});
        $pkgs = $rel->adjust_pkg_sequence($prod->allpkgs());
        # remove VRTSfsadv for PA host
        if ($edr->{localsys}{padv} =~ /HPUX1131par/ && EDRu::inarr('VRTSfsadv60', @$pkgs)){
            $pkgs = EDRu::arrdel($pkgs, qw(VRTSfsadv60));
        }

        $format = '%-15s %-15s %-s';

        Msg::page_bold("\n$prod->{name} : \n", $noprtc);
        $pl=Msg::string_sprintf($format,$msg_name,$msg_set,$msg_desc);
        Msg::page_bold($pl, $noprtc);
        Msg::page_print($seperator, $noprtc);
        $n=0;
        $edr->{spacelinebreak}=' ' x 32;
        for my $pkgname (@$pkgs) {
            $n++;
            if (EDRu::inarr($pkgname,@{$prod->minpkgs})) {
                $pkg_set=$msg_min;
            } elsif (EDRu::inarr($pkgname,@{$prod->recpkgs})) {
                $pkg_set=$msg_rec;
            } else {
                $pkg_set=$msg_all;
            }
            $pkg=$cpic->pkg($pkgname);
            $pl=Msg::string_sprintf($format,$pkg->{pkg},$pkg_set,$pkg->{name});
            $noprtc=($n==$#$pkgs+1) ? 1 : 0 unless ($noprtc);
            Msg::page_print($pl,$noprtc);

            my @pkg_info = ();
            push(@pkg_info, $pkg->{pkg});
            push(@pkg_info, $pkg_set);
            push(@pkg_info, $pkg->{name});
            push(@pkgs_data, \@pkg_info);
        }
        $edr->{spacelinebreak}='';
    }

    return \@pkgs_data;
}

sub installed_pkgs {
    my $cpic = shift;
    my ($edr,$cfg,$rel,$msg,$sys,$localsys,$sysname,$system_string,$systems);
    my ($pkg,$pkgs,$minpkgs_flag,$recpkgs_flag,$allpkgs_flag,$pl,$prod,$prodi);
    my ($format,$msg_pn,$msg_mp,$msg_rp,$msg_ap,$msg_yes,$msg_no);

    $rel=$cpic->rel;
    $edr=Obj::edr();
    $cfg=Obj::cfg();
    $edr->{tmpdir}=$edr->{tmppath}; # to copy vpvd script
    $localsys=$edr->{localsys};
    $systems=$cfg->{systems};
    unless ($systems) {
        $msg=Msg::new("\n Enter the systems for package check: ", 30, 1135);
        $system_string=$msg->ask($localsys->{sys},'','');
        @{$systems}=split(/\s+/m, $system_string);
    }
    for my $sysname (@{$systems}) {
        $sys=Sys->new($sysname);
        unless ($edr->transport_sys($sys)) {
            $msg=Msg::new("Communication error occured for $sys->{sys}", 30, 1026, "$sys->{sys}");
            $msg->print;
            $systems=EDRu::arrdel($systems,$sys->{sys});
            next;
        }
        $sys->pkgs_patches();
    }

    $format = '%-17s %-18s %-20s %-s';
    $msg_pn=Msg::new("Product Name", 30, 1136)->{msg};
    $msg_mp=Msg::new("Minimum Packages", 30, 1137)->{msg};
    $msg_rp=Msg::new("Recommended Packages", 30, 1138)->{msg};
    $msg_ap=Msg::new("All Packages", 30, 1139)->{msg};
    $msg_yes=Msg::new("Yes", 30, 1037)->{msg};
    $msg_no=Msg::new("No", 30, 1038)->{msg};

    for my $sysname (@{$systems}) {
        $sys=Obj::sys($sysname);
        Msg::bold("$sysname : \n");
        $pl=Msg::string_sprintf($format,$msg_pn,$msg_mp,$msg_rp,$msg_ap);
        Msg::bold($pl);
        Msg::equal_bar();
        for my $prodname (@{$rel->{pkgsetprods}}) {
            $prodi=$prodname;
            $prod=$sys->prod($prodi);
            unless (EDRu::compvers($prod->version_sys($sys),$prod->{vers},2)) {
                $minpkgs_flag=$recpkgs_flag=$allpkgs_flag=1;
                $pkgs=$prod->allpkgs;

                #fix for e3401395, remove VRTSfsadv for PA host 
                if ($sys->{padv} =~ /HPUX1131par/ && EDRu::inarr('VRTSfsadv60', @$pkgs)){ 
                    $pkgs = EDRu::arrdel($pkgs, qw(VRTSfsadv60)); 
                } 

                for my $pkgname(@$pkgs) {
                    $pkg=$sys->pkg($pkgname);
                    #Incident:3388255, fix for -pkgset error
                    next if ($pkg->{mpok});
                    #It is possible that a pkg is splited to two pkgs when installed on system, it is better to check pkg's version
                    unless ($sys->{pkgvers}{$pkg->{pkg}} || $pkg->version_sys($sys)) {
                         EDRu::inarr($pkgname,@{$prod->minpkgs}) ? ($minpkgs_flag=$recpkgs_flag=$allpkgs_flag='') :
                              EDRu::inarr($pkgname,@{$prod->recpkgs}) ? ($recpkgs_flag=$allpkgs_flag='') :
                                  EDRu::inarr($pkgname,@{$prod->allpkgs}) ? ($allpkgs_flag='') : 1;
                    }
                    last unless ($minpkgs_flag || $recpkgs_flag || $allpkgs_flag);
                }
            } else {
                $minpkgs_flag=$recpkgs_flag=$allpkgs_flag='';
            }
            $minpkgs_flag = $minpkgs_flag ? $msg_yes : $msg_no;
            $recpkgs_flag = $recpkgs_flag ? $msg_yes : $msg_no;
            $allpkgs_flag = $allpkgs_flag ? $msg_yes : $msg_no;
            $pl=Msg::string_sprintf($format,$prod->{abbr},$minpkgs_flag,$recpkgs_flag,$allpkgs_flag);
            Msg::print($pl);
        }
        Msg::n();
    }
    return '';
}

sub check_pkg_inventory {
    my $cpic = shift;
    my (%allpkgs,%allpatches,%recpkgs,%recpatches,%minpkgs,%minpatches,%missingpkgs,%missingpatches,%repkgs,%repatches,%pkgpresent,%patchpresent,%undefpkgs,%undefpatches,@listpatches_map,@pkgs,@patches,$listpatches_map,$prods,$prod,$prodi,$pobj,$patchobj,$pkgis,$pkgi,$pkg,$patch,$patchi,$padv,$patchdir,$pkgdir,$pkgstring,$patchstring,$patchver,$rel,$vers,$undef_flag,$prodstr,$pdfrs,$patchstr,$pps,$edr,$msg,$format,$msg_pkg,$msg_patch,$msg_osversion,$patchfile);
    my ($ppmsg);

    $edr=Obj::edr();
    #for $padv(@{$cpic->{padvs}}) {
    $padv=$cpic->{padv};
    {
       $rel=$cpic->rel($padv);
       $prods = ($cpic->{prod}) ? [ $cpic->{prod} ] : $rel->{prods};
       # $prods = $rel->{prods} if (Cfg::opt('listpatches'));

       for my $prodi (@$prods) {
           undef(%pkgpresent);
           undef(%patchpresent);
           $prod=$cpic->prod($prodi,$padv) unless (ref($prodi));
           next if ($prod->{obsoleted});
           $pkgis=$prod->allpkgs;
           for my $pkgi (@$pkgis) {
               $pkg=$cpic->pkg($pkgi, $padv) unless (ref($pkgi));
               if ($padv ne "Linux") {
                   for my $patchi (@{$pkg->patches_allvers($rel->{currentrelease})}) {
                       $patchpresent{$patchi}=1;
                   }
               }
           }

           # ./depot check, all product packages will be there
           unless (Cfg::opt(qw(listpatches allpkgs minpkgs recpkgs))) {
               if ($rel->{pkgs}) {
                   $pkgdir=$rel->pkgs_patches_dir('pkgpath');;
                   EDR::die("Cannot find pkgs dir for product $prod->{abbr} on padv $prod->{padv}, proddir=$prod->{proddir} mediapath=$cpic->{mediapath}") unless ($pkgdir);
               }
               $pkgstring = `ls $pkgdir | grep -v info  | grep -v .toc | grep -v admin`;
               $patchdir=$rel->pkgs_patches_dir('patchpath');;
               if ($patchdir) {
                   $patchstring = `ls $patchdir |  grep -v info  | grep -v .toc | grep -v admin | grep -v README.* |grep -v swagent.log|grep -v catalog`;
               }
               @pkgs=split(/\s+/m,$pkgstring);
               @patches=split(/\s+/m,$patchstring);
               $pkgis=$prod->get_pkgs();
               for my $pobj (@pkgs) {
                   $undef_flag = 1;
                   ($pobj,undef) = split(/-[\dv]/m,$pobj,2); # linux
                   $pobj=~s/.tar.gz//m;
                   $pobj=~s/.bff.*$//m;
                   $pobj=~s/.image.*$//m;
                   $pobj=~s/.pkg.*$//m;
                   $pobj=~s/.p5p.*$//m;
                   for my $pkgi (@$pkgis) {
                       $pkg=$cpic->pkg($pkgi, $padv) unless (ref($pkgi));
                       if ($pkg->{pkg} eq $pobj)   {
                           $pkgpresent{$pkg}=1;
                           $undef_flag = 0;
                       }
                   }
                   $undefpkgs{$prod}.="$pobj " if ($undef_flag);
               }
               $pkgis=$prod->get_pkgs();
               for my $patchobj (@patches) {
                   $undef_flag = 1;
                   # uncomment below after patch definition corrections
                   # ($patchobj,undef) = split(/-[\dv]/,$patchobj,2); # linux
                   $patchobj=~s/-\d+.tar.gz//mx;
                   $patchobj=~s/.tar.gz//m;
                   $patchobj=~s/.bff.*$//m;
                   $patchobj=~s/.image.*$//m;
                   $patchobj=~s/.p5p.*$//m;
                   if ( EDRu::plat($padv) eq 'Linux') {
                       $patchobj=~s/-.*$//m;
                   }
                   # based on assumption package name (not pobj)
                   # and patch name are always same
                   # patches don't have padv binding as well
                   for my $pkgi (@$pkgis) {
                       $pkg=$cpic->pkg($pkgi, $padv) unless (ref($pkgi));
                       for my $patchi (@{$pkg->patches_allvers($rel->{currentrelease})}) {
                           $patch=$cpic->patch($patchi, $padv) unless (ref($patchi));
                           if (($patch->{patchname} eq $patchobj && EDRu::plat($padv) ne 'HPUX')
                                   || ($patch->{patch} eq $patchobj && EDRu::plat($padv) eq 'HPUX')) {
                               $patchpresent{$patchi}+=2;
                               $undef_flag = 0;
                           }
                       }
                   }
                   $undefpatches{$prod}.="$patchobj " if ($undef_flag);
               }
           }

           $pkgis=$prod->get_pkgs();
           # remove VRTSfsadv for PA host
           if ($edr->{localsys}{padv} =~ /HPUX1131par/ && EDRu::inarr('VRTSfsadv60', @$pkgis)){
               $pkgis = EDRu::arrdel($pkgis, qw(VRTSfsadv60));
           }
           # Move CPI pkg to the end of pkglist.
           $pkgis = $rel->adjust_pkg_sequence($pkgis);

           for my $pkgi (@$pkgis) {
               $pkg=$cpic->pkg($pkgi, $padv) unless (ref($pkgi));
               $repkgs{$prod}.="$pkg->{pkg} " if ($pkgpresent{$pkg}==1);
               $allpkgs{$prod}.="$pkg->{pkg} ";
               $missingpkgs{$prod}.="$pkg->{pkg} " unless ($pkgpresent{$pkg}==1);
               # because  requirepkgs = allpkgs - optionalpkgs
               $minpkgs{$prod}.="$pkg->{pkg} "
                   if (EDRu::inarr($pkgi, @{$rel->minpkgs},
                                         @{$prod->minpkgs}));
               $recpkgs{$prod}.="$pkg->{pkg} "
                  if (EDRu::inarr($pkgi, @{$rel->recpkgs}, @{$prod->recpkgs}));
               if (EDRu::plat($padv) ne 'Linux') {
                   for my $patchi (@{$pkg->patches_allvers($rel->{currentrelease})})  {
                       $patch=$cpic->patch($patchi, $padv) unless (ref($patchi));
                       for my $gamp(qw(gapatches sppatches hfpatches rppatches ppatches)) {
                           if (defined($pkg->{$gamp}{$rel->{currentrelease}})) {
                               for my $vers(sort keys %{$pkg->{$gamp}{$rel->{currentrelease}}}) {
                                   if (EDRu::inarr($patchi,@{$pkg->{$gamp}{$rel->{currentrelease}}{$vers}})) {
                                       $patch->{osvers} = $vers;
                                       last;                    
                                   }        
                               }    
                           }    
                       }    
                       # Add patch version on Solaris 10
                       if ($padv=~/Sol10/m) {
                       # process to get the patch revision number
                       #$patchdir=$rel->pkgs_patches_dir("patchpath");;
                       #$patch->{file}=$padv->media_patch_file($patch, $patchdir);
                       #$patch->{vers}=$padv->media_patch_version($patch, $patchdir);
                       #$patch->{patch_vers} = $patch->{patch}."-".$patch->{vers};
                       
                           $patchver=$patch->{patchid};
                       } else {
                           $patchver=$patch->{patch};
                       }
                       $patchver =~ s/_[_0-9]*//mg if (EDRu::plat($padv) =~ /(AIX|Linux)/m || $padv=~/Sol11/m);

                       $allpatches{$prod}.="$patchver ";
                       $repatches{$prod}.="$patchver "
                           if ($patchpresent{$patchi}==1);
                       $recpatches{$prod}.="$patchver "
                           if (EDRu::inarr($pkgi, @{$prod->recpkgs}));
                       $minpatches{$prod}.="$patchver "
                           if (EDRu::inarr($pkgi, @{$prod->minpkgs}));
                       $missingpatches{$prod}.="$patch->{patch} "
                           if ($patchpresent{$patchi}==1);
                   }   
               }
           }

           $prodstr= Cfg::opt('pkginfo') ? "$prod->{name}" : "$prod->{abbr}";

           # pkginfo option
           if (Cfg::opt('pkginfo')) {
               # get pkg msgs
               $pdfrs=Msg::get('pdfrs');
               $patchstr='patches';
               Msg::n();
           } else {
               $pdfrs="$prodstr: PKGS";
               $patchstr="$prodstr: PATCHES";
           }

           $ppmsg = Msg::new("$pdfrs and patches", 30, 1140, "$pdfrs");
           if (Cfg::opt(qw(allpkgs))) {
               if (Cfg::opt('pkginfo')) {
                   $pps= $allpatches{$prod} ? $ppmsg->{msg} : $pdfrs;
                   $msg=Msg::new("The following $prodstr $pps must be installed in the specified order to achieve full functionality. The $pps listed are all the $pps offered by the $prodstr product.", 30, 1141, "$prodstr", "$pps", "$pps", "$pps", "$prodstr");
                   $msg->printn;
               }
               Msg::printn("$pdfrs: $allpkgs{$prod}");
               if ($allpatches{$prod}) {
                   Msg::printn("$patchstr: $allpatches{$prod}");
               }
               Msg::n() if (Cfg::opt('pkginfo'));
           }

           if (Cfg::opt('listpatches')) {
               Msg::print("$prod->{abbr}:");
               @listpatches_map = ();
               if ($padv =~ /Sol10/m) {
                   $format = '%-17s  %-15s  %-s';
                   $msg_pkg = Msg::new("Package", 30, 1142)->{msg};
                   $msg_patch = Msg::new("Patch", 30, 1143)->{msg};
                   $msg_osversion = Msg::new("OS version", 30, 1144)->{msg};
                   push(@listpatches_map, Msg::string_sprintf($format, $msg_pkg, $msg_patch, $msg_osversion));
               }
               for my $pkgi (@$pkgis) {
                   $pkg=$cpic->pkg($pkgi, $padv) unless (ref($pkgi));
                   for my $patchi (@{$pkg->patches_allvers($rel->{currentrelease})})  {
                       $patch=$cpic->patch($patchi, $padv) unless (ref($patchi));
                       $patchver = $patch->{patch_vers} ? $patch->{patch_vers} : $patch->{patch};
                       if ($padv=~/Sol10/m) {
                           $patchver =~ s/_/-/mg;
                       } elsif (EDRu::plat($padv) eq 'AIX' || EDRu::plat($padv) eq 'Linux' || $padv =~/Sol11/) {
                           $patchver =~ s/_[_0-9]*//mg;
                       }
                       $patchfile = EDRu::basename($patch->{file}) if ($padv !~ /Sol10/m);
                       $patchfile ||= $patchver;
                       ($padv =~ /Sol10/m) ?
                       push(@listpatches_map, Msg::string_sprintf($format, $pkg->{pkg}, $patchfile, $patch->{osvers})) :
                       push(@listpatches_map, "$patchfile ");
                       $patchfile = '';
                   }
               }
               $listpatches_map = join("\n", @listpatches_map, '');
               Msg::print("$listpatches_map");
           }
           if (Cfg::opt('recpkgs')) {
               if (Cfg::opt('pkginfo')) {
                   $pps= $recpatches{$prod} ? $ppmsg->{msg} : $pdfrs;
                   $msg=Msg::new("The following $prodstr $pps must be installed in the specified order to achieve recommended functionality. The $pps listed are the recommended $pps for $prodstr offering basic and some advanced functionality for the product.", 30, 1145, "$prodstr", "$pps", "$pps", "$pps", "$prodstr");
                   $msg->printn;
               }
               Msg::printn("$pdfrs: $recpkgs{$prod}");
               if ($recpatches{$prod}) {
                   Msg::printn("$patchstr: $recpatches{$prod}");
               }
               Msg::n() if (Cfg::opt('pkginfo'));
           }

           if (Cfg::opt('minpkgs')) {
               if (Cfg::opt('pkginfo')) {
                   $pps= $minpatches{$prod} ? $ppmsg->{msg} : $pdfrs;
                   $msg=Msg::new("The following $prodstr $pps must be installed in the specified order to achieve basic functionality. The $pps listed provide minimum footprint of the $prodstr product.", 30, 1146, "$prodstr", "$pps", "$pps", "$prodstr");
                   $msg->printn;
               }
               Msg::printn("$pdfrs: $minpkgs{$prod}");
               if ($minpatches{$prod}) {
                   Msg::printn("$patchstr: $minpatches{$prod}");
               }
               Msg::n() if (Cfg::opt('pkginfo'));
           }

           if (Cfg::opt('missingpkgs')) {
               if ($missingpkgs{$prod} && $rel->{pkgs}) {
                   Msg::printn("$pdfrs: $missingpkgs{$prod}");
               }
               if ($missingpatches{$prod}) {
                   if ($padv =~ /Sol10/m) {
                       $missingpatches{$prod} =~ s/_/-/mg;
                   } elsif (EDRu::plat($padv) eq 'AIX' || EDRu::plat($padv) eq 'Linux' || $padv=~/Sol11/) {
                       $missingpatches{$prod} =~ s/_[_0-9]*//mg;
                   }
                   Msg::printn("$patchstr: $missingpatches{$prod}");
               }
           }

           if (Cfg::opt('repkgs')) {
               $msg=Msg::new("$pdfrs: $repkgs{$prod}", 30, 1147, "$pdfrs", "$repkgs{$prod}");
               $msg->printn;
               if ($repatches{$prod}) {
                   Msg::printn("$patchstr: $repatches{$prod}");
               }
           }
       }
   }
    return;
}

sub cli_set_title {
    my ($cpic,$task,$systems) = @_;
    my ($name,$rel,$msg,$cfg,$vers,$len,$prod);
    $rel=$cpic->rel;
    $cfg=Obj::cfg();
    if ($cpic->{prod}) {
       $prod=$cpic->prod;
       $vers=$prod->{titlevers} || $rel->{titlevers};
       $name=$prod->{name}.' '.$vers;
    } else {
       $name=$rel->{reltitle}.' '.$rel->{titlevers};
    }

    $task=Msg::new("Install", 30, 1148);
    if (Cfg::opt('rollback')) {
        $task=Msg::new("Rollback", 30, 1149);
    } elsif (Cfg::opt('uninstall') || ($cpic->{script} =~ /^uninstall/m)) {
        $task=Msg::new("Uninstall", 30, 1150);
    } elsif (Cfg::opt('upgrade')) {
        $task=Msg::new("Upgrade", 30, 1151);
        $task=Msg::new("Live Upgrade", 30, 1152) if Cfg::opt('rootpath');
    } elsif (Cfg::opt('patchupgrade')) {
        $task=Msg::new("Patch Upgrade", 30, 1153);
    } elsif (Cfg::opt("addnode")) {
        $task=Msg::new("Add Node", 30, 1154);
    } elsif (Cfg::opt('configure')) {
        $task=Msg::new("Configure", 30, 1155);
    } elsif (Cfg::opt('start')) {
        $task=Msg::new("Start", 30, 1156);
    } elsif (Cfg::opt('stop')) {
        $task=Msg::new("Stop", 30, 1157);
    } elsif (Cfg::opt('license')) {
        $task=Msg::new("License", 30, 1158);
    } elsif (Cfg::opt('precheck')) {
        $task=Msg::new("Precheck", 30, 1159);
    } elsif (Cfg::opt('postcheck')) {
        $task=Msg::new("Postcheck", 30, 1160);
    } elsif (Cfg::opt("hotfix")) {
        $task=Msg::new("EEB", 30, 1161);
    } elsif (Cfg::opt("restore")) {
        $task=Msg::new("EEB Restore", 30, 1162);
    }

    $msg=Msg::new("$name $task->{msg} Program", 30, 1163, "$name", "$task->{msg}");
    $msg->msg('title');

    # Only systems which total length < 60 are shown in title
    $systems='';
    if ($cfg->{systems}) {
        $len=0;
        for my $name (@{$cfg->{systems}}) {
            $len+=length($name)+1;
            if ($len>60) {
               $systems.='...';
               last;
            } else {
               $systems.=" $name";
            }
        }
    }
    Msg::key_msg('systems', $systems);
    return '';
}

sub set_prod {
    my ($cpic,$prod) = @_;
    my ($cfg);
    return unless ($prod);

    $cpic->{orig_prod}=$prod;

    $cfg=Obj::cfg();
    $cpic->{prod}=$prod;
    $cfg->{prod}||=$cpic->{orig_prod};
    return;
}

sub cli_exit_file {
    my ($ayn,$cfg,$cpic,$edr,$eprod,$esystems,$etask);
    my ($prod,$prodi,$msg,$exitfile,$help,$io,$line,$opt,$opts,$localsys);
    return '' if (Cfg::opt(qw(responsefile makeresponsefile)));

    $cpic=shift;
    $edr=Obj::edr();
    $cfg=Obj::cfg();
    $localsys=$edr->{localsys};
    $esystems = $eprod = $etask = '';

    $exitfile = $edr->{logpath}."/exitfile-".$edr->{release}."-*";
    $edr->{exitpath} = $localsys->path($exitfile);
    $exitfile = $localsys->lsfile($edr->{exitpath});
    $edr->{exitfile}=$exitfile='noexitfile' if ((Cfg::opt(qw(security addnode fencing configcps))) || ($ENV{NOEXITFILE}));
    if (-f $exitfile) {
        $io = EDRu::readfile($exitfile);
        for my $line (split(/\n/, $io)) {
            if (!$esystems && ($line =~ /CFG{systems}/mx)) {
                $line =~ s/\$CFG{systems}=\[\ qw\(//m;
                $line =~ s/\)\ \];//m;
                $esystems = $line;
            }
            if (!$eprod && ($line =~ /CFG{prod}/m)) {
                $prodi = $1 if($line =~ /=\s*["]?(\w+)/mx);
                return '' if (!$prodi);
                $prod = $cpic->prod($prodi);
                $eprod = $prod->{abbr};
                #$line =~ s/\$CFG{prod}=\"//;
                #$line =~ s/\d+\";//;
                #$eprod = $line;
            }
            if (!$etask && ($line =~ /{opt}{(install|patchupgrade|upgrade|uninstall|configure|precheck|license|start|stop)}/mx)) {
                $line =~ s/\$CFG{opt}{//mx;
                $line =~ s/}=1;//m;
                $etask = $line;
            }
        }
        unless ($etask && $esystems) {
            return "";
        }
        if (($etask ne 'upgrade') && ($eprod eq "")) {
            return "";
        }

        if (($etask ne 'uninstall') && ($cpic->{script} =~ /^uninstall/m)) {
            return "";
        }
        if ($eprod) {
            # show exitfile prompt only if at least task is selected
            $msg=Msg::new("The installer has discovered an existing installer process. The process exited while performing $etask of $eprod on $esystems. Do you want to resume this process?", 30, 1164, "$etask", "$eprod", "$esystems");
        } else {
            # For upgrade scenario, CFG{prod} is not set, in that case we do not show $eprod
            $msg=Msg::new("The installer has discovered an existing installer process. The process exited while performing $etask on $esystems. Do you want to resume this process?", 30, 1165, "$etask", "$esystems");
        }
        $help=Msg::new("This message indicates that a previous installer instance failed or exited incorrectly.\nThe message indicates the following information:\n- whether the installer was performing an installation or an upgrade\n- which Veritas product was selected\n- the system names\n\n If you have corrected the issues which caused the installer to exit, you can resume the process. Select y at the prompt. The installer continues the selected task from the point when the installation or upgrade failed.\n If you are performing a different task or do not wish to restart the previous installer process, select n at the prompt", 30, 1166);
        $ayn=$msg->ayny($help, 0);
        Msg::n();
        if ($ayn eq 'Y') {
            $edr->{exitfile}=$exitfile;
            Cfg::set_opt('responsefile',$exitfile);
            our %CFG;
            require($cfg->{opt}{responsefile});
            $cpic->set_task($etask);
            $opts=$cfg->{opt};
            %$cfg = (%$cfg, %CFG);
            # merging the hashes with the line above doesn't merge $cfg{opt} correctly
            for my $opt (keys %$opts) { $cfg->{opt}{$opt}=$$opts{$opt}; }
        } else {
            $localsys->rm("$edr->{logpath}/exitfile-*");
        }
    }
    return '';
}

sub version_check {
    my ($cpic) = @_;
    my ($edr, $cfg, $msg, $padv, $rel);
    my ($sys, $localsys, $systems, $inputsystems, $sysname);

    $edr = Obj::edr();
    $cfg = Obj::cfg();
    $rel = $cpic->rel;
    $edr->{tmpdir} = $edr->{tmppath};
    $localsys=$edr->{localsys};

    $rel->require_versionmatrix_lib();
    $rel->init_releasematrix();

    # check local system if -history specified for version history
    if (Cfg::opt('history')) {
        if ($cfg->{systems} && @{$cfg->{systems}}) {
            $sys = $cfg->{systems}[0];
            $sys = Sys->new($sys);
            if (! $edr->transport_sys($sys)) {
                Msg::n();
                $msg = Msg::new("Communication error occured for $sys->{sys}", 30, 1026, "$sys->{sys}");
                $msg->warning();
                return;
            }
        } else {
            $sys = $localsys;
        }

        $sys->pkgs_patches();
        if ( $rel->can('output_installed_pkgs_patches_for_version_history')) {
            $rel->output_installed_pkgs_patches_for_version_history($sys);
        } elsif ($cpic->can('output_installed_pkgs_patches_for_version_history')) {
            $cpic->output_installed_pkgs_patches_for_version_history($sys);
        }
        return '';
    }

    # check network connection to SORT server for Patch/Hotfix check
    if (EDRu::is_port_connectable("sort.symantec.com",443)) {
        $cpic->{sortready} = 1;
    } else {
        $cpic->{sortready} = 0;
    }

    $systems = $cfg->{systems};
    if (!$systems || !@$systems) {
        if ($cfg->{opt}{hostfile})    {
            $inputsystems = EDRu::readfile($cfg->{opt}{hostfile});
        } else {
            $msg = Msg::new("\nEnter the system names for version check: ", 30, 1167);
            $inputsystems = $msg->ask($localsys->{sys}, '', '');
        }
        @{$systems} = @{EDRu::arruniq(split(/\s+/m, $inputsystems))};
    }

    while ($edr->validate_systemnames(@{$systems})) {
        $msg = Msg::new("\nEnter the system names for version check: ", 30, 1167);
        $inputsystems = $msg->ask($localsys->{sys}, '', '');
        @{$systems} = @{EDRu::arruniq(split(/\s+/m, $inputsystems))};
    }

    for my $sysname (@{$systems}) {
        $sys = Sys->new($sysname);
        unless ($edr->transport_sys($sys)) {
            Msg::n();
            $msg = Msg::new("Communication error occured for $sys->{sys}", 30, 1026, "$sys->{sys}");
            $msg->warning();
            $systems = EDRu::arrdel($systems, $sys->{sys});
            next;
        }
        $sys->pkgs_patches();
        Msg::n();
        $rel->set_inst_vrtsprods_sys($sys);
        $cpic->check_prods_install_status_sys($sys);
        $cpic->output_prods_install_status_sys($sys);
    }
    return '';
}

sub get_fallback_releases {
    my ($cpic, $mpvers) = @_;
    my ($majorvers, $rel, $release, @releases, $rellvl, $relnum);
    $rel=$cpic->rel;
    $majorvers = $rel->get_major_vers($mpvers);

    @releases = ();
    $release = $mpvers;
    while ($release && $release !~ /^\d+\.\d*$/mx && $release !~ /^\d+$/mx && $release ne $majorvers) {
        if ( $release =~ /_/mx) {
            $release =~ s/_.*$//mx;
        } elsif ($release =~ /[^\d]+$/ ) {
            $release =~ s/[^\d]+$//mx;
        } else {
            $rellvl = $release;
            $relnum = $release;
            $rellvl =~ s/\d+$//mx;
            $relnum =~ s/^.*[a-zA-Z](\d+)$/$1/mx;
            if ( ! $relnum || $relnum == 0 ) {
                $release = $rellvl;
                $release =~ s/[a-zA-Z]+$//mx;
            } else {
                $relnum--;
                $release = "$rellvl"."$relnum";
            }
        }
        push @releases, $release;
        last if (scalar @releases > 32);
    }
    return @releases;
}

sub get_prod_required_pkgs {
    my ($cpic, $prod, $mpvers) = @_;
    my ($rel, @requiredpkgs, $release);
    $rel=$cpic->rel;
    if ( $prod eq "at" ) {
        return $rel->get_at_required_pkgs($mpvers);
    }
    if (exists($rel->{rel}{$mpvers})
        && exists($rel->{rel}{$mpvers}{prod})
        && exists($rel->{rel}{$mpvers}{prod}{$prod})) {
        @requiredpkgs = @{$rel->{rel}{$mpvers}{prod}{$prod}{rpkgs}};
    } else {
        for my $release ($cpic->get_fallback_releases($mpvers)) {
            if (exists($rel->{rel}{$release})
                && exists($rel->{rel}{$release}{prod})
                && exists($rel->{rel}{$release}{prod}{$prod})) {
                @requiredpkgs = @{$rel->{rel}{$release}{prod}{$prod}{rpkgs}};
                last;
            }
        }
    }
    return EDRu::arruniq(sort @requiredpkgs);
}

sub get_prod_optional_pkgs {
    my ($cpic, $prod, $mpvers) = @_;
    my ($rel, @optionalpkgs, $release);
    $rel=$cpic->rel;
    if ( $prod eq "at" ) {
        return $rel->get_at_optional_pkgs($mpvers);
    }
    if (exists($rel->{rel}{$mpvers})
        && exists($rel->{rel}{$mpvers}{prod})
            && exists($rel->{rel}{$mpvers}{prod}{$prod})) {
        @optionalpkgs = @{$rel->{rel}{$mpvers}{prod}{$prod}{opkgs}};
    } else {
        for my $release ($cpic->get_fallback_releases($mpvers)) {
            if (exists($rel->{rel}{$release})
                && exists($rel->{rel}{$release}{prod})
                && exists($rel->{rel}{$release}{prod}{$prod})) {
                @optionalpkgs = @{$rel->{rel}{$release}{prod}{$prod}{opkgs}};
                last;
            }
        }
    }
    return EDRu::arruniq(sort @optionalpkgs);
}

sub get_prod_all_pkgs {
    my ($cpic, $prod, $mpvers) = @_;
    my ($rel, @allpkgs, $release);
    $rel=$cpic->rel;
    if ( $prod eq "at" ) {
        return $rel->get_at_all_pkgs($mpvers);
    }
    if (exists($rel->{rel}{$mpvers})
        && exists($rel->{rel}{$mpvers}{prod})
        && exists($rel->{rel}{$mpvers}{prod}{$prod})) {
        push @allpkgs, @{$rel->{rel}{$mpvers}{prod}{$prod}{rpkgs}};
        push @allpkgs, @{$rel->{rel}{$mpvers}{prod}{$prod}{opkgs}};
    } else {
        for my $release ($cpic->get_fallback_releases($mpvers)) {
            if (exists($rel->{rel}{$release})
                && exists($rel->{rel}{$release}{prod})
                && exists($rel->{rel}{$release}{prod}{$prod})) {
                push @allpkgs, @{$rel->{rel}{$release}{prod}{$prod}{rpkgs}};
                push @allpkgs, @{$rel->{rel}{$release}{prod}{$prod}{opkgs}};
                last;
            }
        }
    }
    return EDRu::arruniq(sort @allpkgs);
}

sub get_rel_pkgs_mapping_patches {
    my ($cpic, $relvers, $pkgs, $osvers) = @_;
    my ($rel, $pkg, $patch, @patches, @supp_osvers);
    $rel=$cpic->rel;
    for my $patch (keys %{$rel->{rel}{$relvers}{patch}}) {
        if ($osvers) {
            # Solaris allows undefined {osvers} values which default to ALL (5.8 5.9 5.10)
            @supp_osvers = qw(5.8 5.9 5.10);
            if (exists($rel->{rel}{$relvers}{patch}{$patch}{osvers})) {
                @supp_osvers = @{$rel->{rel}{$relvers}{patch}{$patch}{osvers}};
            }
            next unless (EDRu::inarr($osvers, @supp_osvers));
        }
        for my $pkg (@{$rel->{rel}{$relvers}{patch}{$patch}{pkgs}}) {
            push @patches, $patch if (EDRu::inarr($pkg, @{$pkgs}));
        }
    }
    return EDRu::arruniq(sort @patches);
}

sub get_sort_padv{
    my ($cpic, $padv) = @_;
    my $sort_padv;
    $sort_padv = $padv;
    if ( $padv =~ /^(RHEL[0-9]+)x8664/) {
        $sort_padv = $1."_x86_64";
    } elsif ($padv =~ /^(RHEL[0-9]+)ppc64/) {
        $sort_padv = $1."_ppc64";
    } elsif ( $padv =~ /^(SLES[0-9]+)x8664/) {
        $sort_padv = $1."_x86_64";
    } elsif ($padv =~ /^(SLES[0-9]+)ppc64/) {
        $sort_padv = $1."_ppc64";
    } elsif ( $padv =~ /^AIX(\d*)/) {
        $sort_padv = "AIX$1";
    } elsif ( $padv =~ /^Sol([0-9]*)sparc/) {
        $sort_padv = "sol$1_sparc";
    } elsif ( $padv =~ /^Sol([0-9]*)x64/) {
        $sort_padv = "sol$1_x64";
    } elsif ( $padv =~ /^HPUX1131/) {
        $sort_padv = "hpux1131";
    } elsif ($padv =~ /^HPUX1123/) {
        $sort_padv = "hpux1123";
    }
    return lc "$sort_padv";
}

sub get_sort_os{
    my ($cpic, $padv) = @_;
    my $sort_os;
    if ( $padv =~ /^RHEL/) {
        $sort_os = 'linux';
    } elsif ( $padv =~ /^SLES/) {
        $sort_os = 'linux';
    } elsif ( $padv =~ /^AIX/) {
        $sort_os = 'aix';
    } elsif ( $padv =~ /^Sol/) {
        $sort_os = 'solaris';
    } elsif ( $padv =~ /^HPUX/) {
        $sort_os = 'hp-ux';
    } else {
        $sort_os = '';
    }
    return $sort_os;
}

sub get_sort_arch{
    my ($cpic, $padv) = @_;
    my $sort_arch;
    if ( $padv =~ /^RHEL[0-9]+x8664/) {
        $sort_arch = 'x86-64';
    } elsif ( $padv =~ /^SLES[0-9]+x8664/) {
        $sort_arch = 'x86-64';
    } elsif ( $padv =~ /^AIX/) {
        $sort_arch = 'power';
    } elsif ( $padv =~ /^Sol[0-9]*sparc/) {
        $sort_arch = 'sparc';
    } elsif ( $padv =~ /^Sol[0-9]*x64/) {
        $sort_arch = 'x86-64';
    } elsif ( $padv =~ /^HPUX[0-9]+ia64/) {
        $sort_arch = 'ia-64 ia-64/pa-risc';
    } elsif ( $padv =~ /^HPUX[0-9]+par/) {
        $sort_arch = 'pa-risc ia-64/pa-risc';
    } else {
        $sort_arch = '';
    }
    return $sort_arch;
}

sub get_sort_osvers{
    my ($cpic, $padv) = @_;
    my $sort_osvers;
    if ( $padv =~ /^RHEL([0-9]+)x8664/) {
        $sort_osvers = 'rhel'.$1;
    } elsif ( $padv =~ /^SLES([0-9]+)x8664/) {
        $sort_osvers = 'sles'.$1;
    } elsif ( $padv =~ /^AIX71/) {
        $sort_osvers = '7.1';
    } elsif ( $padv =~ /^AIX/) {
        $sort_osvers = '6.1';
    } elsif ( $padv =~ /^Sol([0-9]+)sparc/) {
        $sort_osvers = $1;
    } elsif ( $padv =~ /^Sol([0-9]+)x64/) {
        $sort_osvers = $1;
    } elsif ( $padv =~ /^HPUX([0-9][0-9])([0-9][0-9])/) {
        $sort_osvers = $1.'.'.$2;
    } else {
        $sort_osvers = '';
    }
    return $sort_osvers;
}

sub get_patchinfo_via_sort_webservice {
    my ($cpic, $patchid) = @_;
    my ($json,$url,$response,$sort_patches);

    if (!$cpic->{sortready}) {
        return;
    }

    if ( $patchid =~ /^https:\/\/.*\/([0-9]+)$/) {
        $patchid = $1;
    }

    if ($cpic->{sort_patchinfo}{$patchid}) {
        return $cpic->{sort_patchinfo}{$patchid};
    }

    $url="https://sort.symantec.com/vos_services/rest/patch_services/1.0/patchRelease/$patchid";
    $response = EDRu::download_page($url);
    if ($response) {
        require JSON;
        $json = JSON->new->allow_nonref;
        $sort_patches = undef;
        eval {$sort_patches = $json->decode($response);};
        if ( $sort_patches && (ref($sort_patches) eq "HASH") &&
             (grep { /patchDetailsDTO/ } (keys %$sort_patches))) {
            $cpic->{sort_patchinfo}{$patchid} =  $sort_patches->{patchDetailsDTO}[0];
            return $cpic->{sort_patchinfo}{$patchid};
        }
    }
    return;
}

# sub get_uptodate_patches {
#     my ($cpic, $opatch) = @_;
#     my (@patches,$opatchinfo,$patch,$patchinfo);
#     @patches = ();
#     if ( $opatch->{is_obsolete} eq "Y") {
#         $opatchinfo = $cpic->get_patchinfo_via_sort_webservice($opatch->{release_id});
#         for my $patch (@{$opatchinfo->{osri}{obsolete}}) {
#             $patchinfo = $cpic->get_patchinfo_via_sort_webservice($patch->{id});
#             push @patches, $cpic->get_uptodate_patches($patchinfo);
#         }
#     } else {
#         push @patches, $opatch;
#     }
#     return @patches;
# }

sub get_patches_via_sort_webservice {
    my ($cpic, $prod, $padv,$vers) = @_;
    my ($json,$url,$response,$sort_padv,$sort_patches,$sort_prod,$rel);

    if (!$cpic->{sortready}) {
        return;
    }

    # convert prod name for SORT
    $rel=$cpic->rel;
    $sort_prod = $rel->get_sort_product_name($prod);
    $sort_padv = $cpic->get_sort_padv($padv);

    $url = "https://sort.symantec.com/vos_services/rest/patch_services/1.0/patch?productAbbr=$sort_prod&padv=$sort_padv";
    if ( $vers) {
        if ( $vers =~ /[0-9]P[0-9]$/ ) {
            $vers =~ s/^(.*[0-9])P[0-9]$/$1/;
        }
        $url .= "&productVersion=$vers";
    }
    $response = EDRu::download_page($url);
    if ($response) {
        require JSON;
        $json = JSON->new->allow_nonref;
        $sort_patches = undef;
        eval {$sort_patches = $json->decode($response);};
        if ( $sort_patches && (ref($sort_patches) eq "HASH") &&
             (grep { /patchDetailsDTO/ } (keys %$sort_patches))) {
            return $sort_patches->{patchDetailsDTO};
        }
    }
    return;
}

# get latest major/minor version
sub get_latest_majorminor_version_via_sort_webservice {
    my ($cpic, $prod, $padv) = @_;
    my ($json,$url,$response,$sort_result,$versions);
    my ($sort_prod,$sort_os,$sort_archs,$sort_osvers);
    $sort_prod = $cpic->rel()->get_sort_product_name_for_latest_version($prod);
    $sort_os = $cpic->get_sort_os($padv);
    $sort_archs = $cpic->get_sort_arch($padv);
    $sort_osvers = $cpic->get_sort_osvers($padv);

    $versions=[];
    if ( $sort_prod && $sort_os && $sort_archs && $sort_osvers) {
        for my $sort_arch (split(/\s/, $sort_archs)) {
            $url = "https://sort.symantec.com/SORTServices/rest/version/1.0/latest?&source=cpi_6.0&productAbbr=".$sort_prod."&os=".$sort_os."&arch=".$sort_arch."&osVersion=".$sort_osvers;
            $response = EDRu::download_page($url);
            if ($response) {
                require JSON;
                $json = JSON->new->allow_nonref;
                $sort_result = undef;
                eval {$sort_result = $json->decode($response);};
                if ( $sort_result && (ref($sort_result) eq "HASH") &&
                         defined($sort_result->{version}) && $sort_result->{version}) {
                    push @$versions, $sort_result->{version};
                }
            }
        }
    }
    return $versions;
}

# returns (
#           $sort_status
#           $prod_patches_for_current_version,
#           $cpi_patches_for_current_version,
#           $latest_version_in_current_branch,
#           $prod_patches_for_latest_version_in_current_branch,
#           $cpi_patches_for_latest_version_in_current_branch,
#           $latest_version_in_latest_branch,
#           $prod_patches_for_latest_version_in_latest_branch,
#           $cpi_patches_for_latest_version_in_latest_branch
#         )
sub get_releases_patches_via_sort_webservice {
    my ($cpic, $prod, $padv, $vers) = @_;
    my ($sort_status, $prod_patches_for_current_version, $cpi_patches_for_current_version, $latest_version_in_current_branch, $prod_patches_for_latest_version_in_current_branch, $cpi_patches_for_latest_version_in_current_branch, $latest_version_in_latest_branch, $prod_patches_for_latest_version_in_latest_branch, $cpi_patches_for_latest_version_in_latest_branch);
    my ($patches, $majorvers, $current_majorvers, @all_versions, @reversed_all_versions, $latestsortversions);

    $latest_version_in_current_branch = '';
    $latest_version_in_latest_branch = '';

    if (EDRu::is_port_connectable("sort.symantec.com",443)) {
        $sort_status = 1;
        $cpic->{sortready} = 1;
        $patches = $cpic->get_patches_via_sort_webservice($prod, $padv);
        for my $patch (@$patches) {
            next if ($patch->{is_obsolete} eq 'Y');
            next if ($patch->{product_version} !~ /^[1-9]\.[0-9]/);
            push @all_versions, $patch->{product_version};
        }

        # check latest major release version from SORT and put it info @all_versions
        $latestsortversions = $cpic->get_latest_majorminor_version_via_sort_webservice($prod,$padv);
        if ($latestsortversions && ref($latestsortversions) eq 'ARRAY' ) {
            push @all_versions, @$latestsortversions;
        }

        if (@all_versions) {
            @all_versions = map({my $v = $_;$v =~ s/MP/SO/;$v =~ s/_11\.31//;$v} @all_versions);
            @reversed_all_versions = reverse sort @all_versions;
            @all_versions = map({my $v = $_;$v =~ s/SO/MP/;$v} @all_versions);
            @reversed_all_versions = map({my $v = $_;$v =~ s/SO/MP/;$v} @reversed_all_versions);
            $current_majorvers = $vers;
            $current_majorvers =~ s/^([0-9\.]*).*$/$1/;
            # check latest major version
            $majorvers = $reversed_all_versions[0];
            $majorvers =~ s/^([0-9\.]*).*$/$1/;
            if ( $majorvers && $majorvers > $current_majorvers) {
                $latest_version_in_latest_branch = $reversed_all_versions[0];
                $patches = $cpic->get_patches_via_sort_webservice($prod, $padv, $latest_version_in_latest_branch);
                for my $patch (@$patches) {
                    if ($patch->{is_obsolete} eq "N") {
                        if ($patch->{product_abbr} eq "cpi") {
                            push @{$cpi_patches_for_latest_version_in_latest_branch}, $patch;
                        } else {
                            push @{$prod_patches_for_latest_version_in_latest_branch}, $patch;
                        }
                    }
                }
            }

            # check latest version in current branch
            for my $ver (@reversed_all_versions) {
                $majorvers = $ver;
                $majorvers =~ s/^([0-9\.]*).*$/$1/;
                if ( $majorvers > $current_majorvers) {
                    next;
                } elsif ($majorvers < $current_majorvers) {
                    last;
                } else {
                    $ver =~ s/MP/SO/;
                    $vers =~ s/MP/SO/;
                    if ($ver gt $vers) {
                        $ver =~ s/SO/MP/;
                        $vers =~ s/SO/MP/;
                        $latest_version_in_current_branch = $ver;
                        $patches = $cpic->get_patches_via_sort_webservice($prod, $padv, $latest_version_in_current_branch);
                        for my $patch (@$patches) {
                            if ($patch->{is_obsolete} eq "N") {
                                if ($patch->{product_abbr} eq "cpi") {
                                    push @{$cpi_patches_for_latest_version_in_current_branch}, $patch;
                                } else {
                                    push @{$prod_patches_for_latest_version_in_current_branch}, $patch;
                                }
                            }
                        }
                    }
                    $ver =~ s/SO/MP/;
                    $vers =~ s/SO/MP/;
                    last;
                }
            }

            # check updates for current version
            $patches = $cpic->get_patches_via_sort_webservice($prod, $padv, $vers);
            for my $patch (@$patches) {
                if ($patch->{is_obsolete} eq "N") {
                    if ($patch->{product_abbr} eq "cpi") {
                        push @{$cpi_patches_for_current_version}, $patch;
                    } else {
                        push @{$prod_patches_for_current_version}, $patch;
                    }
                }
            }
        }
    } else {
        $sort_status = 0;
    }

    return ($sort_status, $prod_patches_for_current_version, $cpi_patches_for_current_version, $latest_version_in_current_branch, $prod_patches_for_latest_version_in_current_branch, $cpi_patches_for_latest_version_in_current_branch, $latest_version_in_latest_branch, $prod_patches_for_latest_version_in_latest_branch, $cpi_patches_for_latest_version_in_latest_branch);
}


sub check_sort_patch_installed_sys {
    my ($cpic, $sys, $prod,$patch) = @_;
    my $patchids = $patch->{patchIds}{patch_id};
    my ($pkg, $patched_pkgs_patches,$pkg_vers,@vers);
    my @infra_pkgs = qw /VRTSperl VRTSat VRTSatClient VRTSatServer VRTSspt VRTScpi VRTSvlic/;
    $patched_pkgs_patches = [];
    return $patched_pkgs_patches if (!@$patchids);
    return $patched_pkgs_patches if ($patch->{release_type} eq 'mp' || $patch->{release_type} eq 'sp' || $patch->{release_type} eq 'rp' || $patch->{release_type} eq 'pr');
    if ( $sys->linux()) {
        for my $pkg (keys %{$sys->{pkgvers}}) {
            next if (EDRu::inarr($pkg, @infra_pkgs));
            next if (!EDRu::inarr($pkg, @{$cpic->get_prod_all_pkgs($prod, $sys->{iprod}{$prod}{defined_vers})}));
            $pkg_vers = "$pkg"."-".$sys->{pkgvers}{$pkg}.'[-$]';
            $pkg_vers =~ s/\./\./g;
            if (grep {/^$pkg_vers/} @$patchids ) {
                push @$patched_pkgs_patches, "$pkg $sys->{pkgvers}{$pkg}";
            }
        }
    } elsif ($sys->aix()) {
        for my $pkg (keys %{$sys->{pkgvers}}) {
            next if (EDRu::inarr($pkg, @infra_pkgs));
            next if (!EDRu::inarr($pkg, @{$cpic->get_prod_all_pkgs($prod, $sys->{iprod}{$prod}{defined_vers})}));
            @vers = split /\./, $sys->{pkgvers}{$pkg};
            $vers[0] = 0 if (!$vers[0]);
            $vers[1] = 0 if (!$vers[1]);
            $vers[2] = 0 if (!$vers[2]);
            $vers[3] = 0 if (!$vers[3]);
            $pkg_vers = sprintf("%s-%02d.%02d.%04d.%04d",$pkg,$vers[0],$vers[1],$vers[2],$vers[3]);
            $pkg_vers =~ s/\./\./g;
            if (grep {/^$pkg_vers/} @$patchids ) {
                push @$patched_pkgs_patches, "$pkg $sys->{pkgvers}{$pkg}";
            }
        }
    } else {
        for my $patchid (@$patchids) {
            if (EDRu::inarr($patchid, @{$sys->{patches}})) {
                push @$patched_pkgs_patches, $patchid;
            }
        }
    }
    return $patched_pkgs_patches;
}


sub check_prods_install_status_sys {
    my ($cpic, $sys) = @_;
    my ($rel,$patchid);

    $rel=$cpic->rel;
    $rel->check_prods_install_status_sys($sys);

    # check installed P/PP/HF for prod
    if ($cpic->{sortready}) {
        for my $prod (sort keys %{$sys->{iprod}}) {
            # Get all releases which have updates available
            my (@all_versions,@reversed_all_versions,$latest_majorvers,$latest_branchvers,$latestsortversions);
            $sys->{iprod}{$prod}{sort_patches_all} = $cpic->get_patches_via_sort_webservice($prod, $sys->{padv});
            for my $patch (@{$sys->{iprod}{$prod}{sort_patches_all}}) {
                next if ($patch->{is_obsolete} eq 'Y');
                next if ($patch->{product_version} !~ /^[1-9]\.[0-9]/);
                push @all_versions, $patch->{product_version};
            }

            # check latest major release version from SORT and put it info @all_versions
            $latestsortversions = $cpic->get_latest_majorminor_version_via_sort_webservice($prod,$sys->{padv});
            if ($latestsortversions && ref($latestsortversions) eq 'ARRAY' ) {
                push @all_versions, @$latestsortversions;
            }

            @all_versions=@{EDRu::arruniq(@all_versions)};
            if (@all_versions) {
                # check updates for latest major version
                @all_versions = map({my $v = $_;$v=~s/(\d)\.(\d)\.(\d)(.*)/$1\.$2TP$3$4/;$v =~ s/MP/SO/;$v =~ s/_11\.31//;$v=~s/PR1/.010/;$v} @all_versions);
                @reversed_all_versions = reverse sort @all_versions;
                @all_versions = map({my $v = $_;$v =~ s/SO/MP/;$v=~s/(\d)\.(\d)TP(\d)(.*)/$1\.$2\.$3$4/;$v} @all_versions);
                @reversed_all_versions = map({my $v = $_;$v =~ s/SO/MP/;$v=~s/(\d)\.(\d)TP(\d)(.*)/$1\.$2\.$3$4/;$v=~s/.010/PR1/;$v} @reversed_all_versions);
                $latest_majorvers = $reversed_all_versions[0];
                $latest_majorvers =~ s/^([0-9\.]*).*$/$1/;
                if ( $latest_majorvers && $latest_majorvers > $sys->{iprod}{$prod}{mjvers}) {
                    $sys->{iprod}{$prod}{latest_majorvers} = $latest_majorvers;
                    $sys->{iprod}{$prod}{latest_majorvers_patchvers} = $reversed_all_versions[0];

                    $sys->{iprod}{$prod}{sort_patches_latestmajorvers} = $cpic->get_patches_via_sort_webservice($prod, $sys->{padv}, $sys->{iprod}{$prod}{latest_majorvers_patchvers});
                    for my $patch (@{$sys->{iprod}{$prod}{sort_patches_latestmajorvers}}) {
                        if ($patch->{is_obsolete} eq "N") {
                            if ($patch->{product_abbr} eq "cpi") {
                                push @{$sys->{iprod}{$prod}{latestmajorvers_sort_patches_cpi}}, $patch;
                            } else {
                                push @{$sys->{iprod}{$prod}{latestmajorvers_sort_patches}}, $patch;
                            }
                        }
                    }
                }

                # check updates for latest SP/MP/RP of current installed version
                for my $vers (@reversed_all_versions) {
                    $latest_branchvers = $vers;
                    $latest_branchvers =~ s/^([0-9\.]*).*$/$1/;
                    if ( $latest_branchvers > $sys->{iprod}{$prod}{mjvers}) {
                        next;
                    } elsif ($latest_branchvers < $sys->{iprod}{$prod}{mjvers}) {
                        last;
                    } else {
                        my $tmpver = $sys->{iprod}{$prod}{mprpvers};
                        $tmpver =~ s/MP/SO/;
                        $vers =~ s/MP/SO/;
                        if ( $vers gt $tmpver) {
                            $vers =~ s/SO/MP/;
                            $sys->{iprod}{$prod}{latest_branchvers_patchvers} = $vers;
                            $sys->{iprod}{$prod}{sort_patches_latestbranchvers} = $cpic->get_patches_via_sort_webservice($prod, $sys->{padv}, $sys->{iprod}{$prod}{latest_branchvers_patchvers});
                            for my $patch (@{$sys->{iprod}{$prod}{sort_patches_latestbranchvers}}) {
                                if ($patch->{is_obsolete} eq "N") {
                                    if ($patch->{product_abbr} eq "cpi") {
                                        push @{$sys->{iprod}{$prod}{latestbranchvers_sort_patches_cpi}}, $patch;
                                    } else {
                                        push @{$sys->{iprod}{$prod}{latestbranchvers_sort_patches}}, $patch;
                                    }
                                }
                            }
                        }
                        $vers =~ s/SO/MP/;
                        last;
                    }
                }
            }

            # check updates for current installed version
            $sys->{iprod}{$prod}{sort_patches_currvers} = $cpic->get_patches_via_sort_webservice($prod, $sys->{padv}, $sys->{iprod}{$prod}{mprpvers});

            for my $patch (@{$sys->{iprod}{$prod}{sort_patches_currvers}}) {
                if ($patch->{product_abbr} eq "cpi") {
                    if ($patch->{is_obsolete} eq "N") {
                        push @{$sys->{iprod}{$prod}{available_sort_patches_cpi}}, $patch;
                    }
                    next;
                }

                $patch->{release_version} = $patch->{release_name};
                $patch->{release_version} =~ s/^.*-//;
                next if ($patch->{release_version} eq $sys->{iprod}{$prod}{mprpvers});

                $patchid = $patch->{release_id};
                $sys->{iprod}{$prod}{installed_sort_patch}{$patchid} = $cpic->check_sort_patch_installed_sys($sys,$prod,$patch);
                if (@{$sys->{iprod}{$prod}{installed_sort_patch}{$patchid}}) {
                    push @{$sys->{iprod}{$prod}{installed_sort_patches}}, $patch;
                } elsif ($patch->{is_obsolete} eq "N") {
                    if (!$sys->{iprod}{$prod}{available_sort_patches}|| !EDRu::inarr($patch->{release_id}, map {$_->{release_id}} @{$sys->{iprod}{$prod}{available_sort_patches}})) {
                        push @{$sys->{iprod}{$prod}{available_sort_patches}}, $patch;
                    }
                }
            }
        }
    }
    return '';
}

sub output_prods_install_status_sys {
    my ($cpic,$sys) = @_;
    my ($script,$msg,$rel,%count,$pkgs,$patch,$patchid,$relvers,$pkg,$prod,$mjvers,$pstamp,$rootpath,$pkgvers,@patchids,$patchlist,$prodname,$prodvers,$installednum,$totalnum,$msgnone,$msginstaller);
    $rel=$cpic->rel;

    unless (keys %{$sys->{iprod}}) {
        $msg = Msg::new("No products installed on $sys->{sys}", 30, 1168, "$sys->{sys}");
        $msg->print();
        return '';
    }

    $msg = Msg::new("Installed product(s) on $sys->{sys}:", 30, 1169, "$sys->{sys}");
    $msg->bold();
    for my $prod (sort keys %{$sys->{iprod}}) {
        $count{unsupport_prods}++ if ($sys->{iprod}{$prod}{unsupport});
        $msg = Msg::new("\t$sys->{iprod}{$prod}{name} - $sys->{iprod}{$prod}{mprpvers}", 30, 1170, "$sys->{iprod}{$prod}{name}", "$sys->{iprod}{$prod}{mprpvers}");
        $msg->print();
    }
    Msg::n();

    if ($count{unsupport_prods}) {
        $script = EDRu::basename($0);
        for my $prod (sort keys %{$sys->{iprod}}) {
            if ($sys->{iprod}{$prod}{unsupport}) {
                if ($sys->aix()||$sys->linux()) {
                    $msg = Msg::new("$script cannot check the installed and missing packages of $sys->{iprod}{$prod}{name} $sys->{iprod}{$prod}{mprpvers}", 30, 1171, "$script", "$sys->{iprod}{$prod}{name}", "$sys->{iprod}{$prod}{mprpvers}");
                } else {
                    $msg = Msg::new("$script cannot check the installed and missing packages and patches of $sys->{iprod}{$prod}{name} $sys->{iprod}{$prod}{mprpvers}", 30, 1172, "$script", "$sys->{iprod}{$prod}{name}", "$sys->{iprod}{$prod}{mprpvers}");
                }
                $msg->warning();
            }
        }
        Msg::n() unless ($count{unsupport_prods} == scalar(keys %{$sys->{iprod}}));
    }

    for my $prod (sort keys %{$sys->{iprod}}) {
        next if ($sys->{iprod}{$prod}{unsupport});
        Msg::n() if ($count{instprods}++);
        # Product report
        $msg = Msg::new("Product:", 30, 1173);
        $msg->bold();
        $msg = Msg::new("\t$sys->{iprod}{$prod}{name} - $sys->{iprod}{$prod}{mprpvers}", 30, 1170, "$sys->{iprod}{$prod}{name}", "$sys->{iprod}{$prod}{mprpvers}");
        $msg->print();
        Msg::n();

        # Packages report
        $msg = Msg::new("Packages:", 30, 1174);
        $msg->bold();

        $msg = Msg::new("\tInstalled Required packages for $sys->{iprod}{$prod}{name} $sys->{iprod}{$prod}{mprpvers}:", 30, 1175, "$sys->{iprod}{$prod}{name}", "$sys->{iprod}{$prod}{mprpvers}");
        $msg->print();
        if ( $sys->sunos() && $sys->{padv} !~ /Sol11/) {
            $msg = Msg::new("\t  #PACKAGE     #VERSION     #PSTAMP", 30, 1176);
        } else {
            $msg = Msg::new("\t  #PACKAGE     #VERSION", 30, 1177);
        }
        $msg->print();
        for my $pkg (@{$sys->{iprod}{$prod}{irpkgs}}) {
            if ($sys->sunos() && $sys->{padv} !~ /Sol11/) {
                $rootpath = Cfg::opt('rootpath')?Cfg::opt('rootpath'):'/';
                $pstamp = $sys->cmd("_cmd_pkginfo -l -R $rootpath $pkg | _cmd_grep '^[ \t]*PSTAMP:'");
                $pstamp =~ s/^\s*PSTAMP:\s*//mx;
                $pkgvers = $sys->{pkgvers}{$pkg} || $sys->{pkgvers}{$rel->{pkgtopobj}{$pkg}};
                $msg = Msg::string_sprintf("\t  %-12s %-12s (%s)", $pkg, $pkgvers, $pstamp);
            } elsif($sys->aix() && $pkg eq 'VRTSat') {
                $pkgvers = $sys->{pkgvers}{'VRTSat.client'};
                $msg = Msg::string_sprintf("\t  %-12s %s", $pkg, $pkgvers);
            } else {
                $pkgvers = $sys->{pkgvers}{$pkg} || $sys->{pkgvers}{$rel->{pkgtopobj}{$pkg}};
                $msg = Msg::string_sprintf("\t  %-12s %s", $pkg, $pkgvers);
            }
            Msg::print($msg);
            $count{$prod}{pkgs}{arpkgs}++;
            $count{$prod}{pkgs}{irpkgs}++;
        }
        Msg::n();

        if (exists($sys->{iprod}{$prod}{mrpkgs})
            && @{$sys->{iprod}{$prod}{mrpkgs}}) {
            $msg = Msg::new("\tMissing Required packages for $sys->{iprod}{$prod}{name} $sys->{iprod}{$prod}{mprpvers}:", 30, 1178, "$sys->{iprod}{$prod}{name}", "$sys->{iprod}{$prod}{mprpvers}");
            $msg->print();
            $msg = Msg::new("\t  #PACKAGE", 30, 1179);
            $msg->print();
            for my $pkg (@{$sys->{iprod}{$prod}{mrpkgs}}) {
                $msg = Msg::new("\t  $pkg", 30, 1180, "$pkg");
                $msg->print();
                $count{$prod}{pkgs}{arpkgs}++;
            }
            Msg::n();
        }

        if (exists($sys->{iprod}{$prod}{iopkgs})
            && @{$sys->{iprod}{$prod}{iopkgs}}) {
            $msg = Msg::new("\tInstalled Optional packages for $sys->{iprod}{$prod}{name} $sys->{iprod}{$prod}{mprpvers}:", 30, 1181, "$sys->{iprod}{$prod}{name}", "$sys->{iprod}{$prod}{mprpvers}");
            $msg->print();
            if ( $sys->sunos() && $sys->{padv} !~ /Sol11/) {
                $msg = Msg::new("\t  #PACKAGE     #VERSION     #PSTAMP", 30, 1176);
            } else {
                $msg = Msg::new("\t  #PACKAGE     #VERSION", 30, 1177);
            }
            $msg->print();
            for my $pkg (@{$sys->{iprod}{$prod}{iopkgs}}) {
                if ($sys->sunos() && $sys->{padv} !~ /Sol11/) {
                    $rootpath = Cfg::opt('rootpath')?Cfg::opt('rootpath'):'/';
                    $pstamp = $sys->cmd("_cmd_pkginfo -l -R $rootpath $pkg | _cmd_grep '^[ \t]*PSTAMP:'");
                    $pstamp =~ s/^\s*PSTAMP:\s*//mx;
                    $pkgvers = $sys->{pkgvers}{$pkg} || $sys->{pkgvers}{$rel->{pkgtopobj}{$pkg}};
                    $msg = Msg::string_sprintf("\t  %-12s %-12s (%s)", $pkg, $pkgvers, $pstamp);
                } elsif($sys->aix() && $pkg eq 'VRTSat') {
                    $pkgvers = $sys->{pkgvers}{'VRTSat.client'};
                    $msg = Msg::string_sprintf("\t  %-12s %s", $pkg, $pkgvers);
                } else {
                    $pkgvers = $sys->{pkgvers}{$pkg} || $sys->{pkgvers}{$rel->{pkgtopobj}{$pkg}};
                    $msg = Msg::string_sprintf("\t  %-12s %s", $pkg, $pkgvers);
            }
                Msg::print($msg);
                $count{$prod}{pkgs}{aopkgs}++;
                $count{$prod}{pkgs}{iopkgs}++;
            }
            Msg::n();
        }

        if (exists($sys->{iprod}{$prod}{mopkgs})
            && @{$sys->{iprod}{$prod}{mopkgs}}) {
            $msg = Msg::new("\tMissing Optional packages for $sys->{iprod}{$prod}{name} $sys->{iprod}{$prod}{mprpvers}:", 30, 1182, "$sys->{iprod}{$prod}{name}", "$sys->{iprod}{$prod}{mprpvers}");
            $msg->print();
            $msg = Msg::new("\t  #PACKAGE", 30, 1179);
            $msg->print();
            for my $pkg (@{$sys->{iprod}{$prod}{mopkgs}}) {
                $msg = Msg::new("\t  $pkg", 30, 1180, "$pkg");
                $msg->print();
                $count{$prod}{pkgs}{aopkgs}++;
            }
            Msg::n();
        }

        # Patches report
        if (exists($sys->{iprod}{$prod}{ipatches}) || exists($sys->{iprod}{$prod}{mpatches})) {
            $msg = Msg::new("Patches:", 30, 1183);
            $msg->bold();

            for my $relvers (reverse @{$rel->{releases}}) {
                next if ($relvers lt $sys->{iprod}{$prod}{mprpvers});
                if (exists($sys->{iprod}{$prod}{ipatches}{$relvers})) {
                    $msg = Msg::new("\tInstalled patches for $sys->{iprod}{$prod}{name} $relvers:", 30, 1184, "$sys->{iprod}{$prod}{name}", "$relvers");
                    $msg->print();
                    $msg = Msg::new("\t  #PATCH          #PACKAGE", 30, 1185);
                    $msg->print();
                    for my $patch (sort keys %{$sys->{iprod}{$prod}{ipatches}{$relvers}}) {
                        $pkgs = join(' ', @{$sys->{iprod}{$prod}{ipatches}{$relvers}{$patch}});
                        $msg = Msg::string_sprintf("\t  %-16s%s", $patch, $pkgs);
                        Msg::print($msg);
                        $count{$prod}{patch}{$relvers}{all}++;
                        $count{$prod}{patch}{$relvers}{inst}++;
                    }
                    Msg::n();
                }
                if (exists($sys->{iprod}{$prod}{mpatches}{$relvers})) {
                    $msg = Msg::new("\tMissing patches for $sys->{iprod}{$prod}{name} $relvers:", 30, 1186, "$sys->{iprod}{$prod}{name}", "$relvers");
                    $msg->print();
                    $msg = Msg::new("\t  #PATCH          #PACKAGE", 30, 1185);
                    $msg->print();
                    for my $patch (sort keys %{$sys->{iprod}{$prod}{mpatches}{$relvers}}) {
                        $pkgs = join(' ', @{$sys->{iprod}{$prod}{mpatches}{$relvers}{$patch}});
                        $msg = Msg::string_sprintf("\t  %-16s%s", $patch, $pkgs);
                        Msg::print($msg);
                        $count{$prod}{patch}{$relvers}{all}++;
                    }
                    Msg::n();
                }
            }
        }

        # Summary
        $msg = Msg::new("Summary:", 30, 1187);
        $msg->bold();
        Msg::n();

        # Installed packages summary
        $msg = Msg::new("Packages:", 30, 1174);
        $msg->bold();

        # Installed required packages statistics
        $prodname = $sys->{iprod}{$prod}{name};
        $prodvers = $sys->{iprod}{$prod}{mprpvers};
        $installednum = Msg::string_sprintf("%-2s", $count{$prod}{pkgs}{irpkgs});
        $totalnum = Msg::string_sprintf("%-2s", $count{$prod}{pkgs}{arpkgs});

        $msg = Msg::new("\t$installednum of $totalnum required $prodname $prodvers packages installed", 30, 1188, "$installednum", "$totalnum", "$prodname", "$prodvers");
        Msg::print($msg);

        # Installed optional packages statistics
        if (exists($count{$prod}{pkgs}{aopkgs}) && $count{$prod}{pkgs}{aopkgs} > 0) {
            $count{$prod}{pkgs}{iopkgs} ||= 0;

            $installednum = Msg::string_sprintf("%-2s", $count{$prod}{pkgs}{iopkgs});
            $totalnum = Msg::string_sprintf("%-2s", $count{$prod}{pkgs}{aopkgs});

            $msg = Msg::new("\t$installednum of $totalnum optional $prodname $prodvers packages installed", 30, 1189, "$installednum", "$totalnum", "$prodname", "$prodvers");
            Msg::print($msg);
        }

        # Installed patches summary
        if (exists($sys->{iprod}{$prod}{ipatches}) || exists($sys->{iprod}{$prod}{mpatches})) {
            $msg = Msg::new("Patches:", 30, 1183);
            $msg->bold();

            for my $relvers (reverse @{$rel->{releases}}) {
                if (exists($count{$prod}{patch}{$relvers}{all}) && $count{$prod}{patch}{$relvers}{all} > 0) {
                    $count{$prod}{patch}{$relvers}{inst} ||= 0;

                    $installednum = Msg::string_sprintf("%-2s", $count{$prod}{patch}{$relvers}{inst});
                    $totalnum = Msg::string_sprintf("%-2s", $count{$prod}{patch}{$relvers}{all});

                    $msg = Msg::new("\t$installednum of $totalnum $prodname $relvers patches installed", 30, 1190, "$installednum", "$totalnum", "$prodname", "$relvers");
                    Msg::print($msg);
                }
            }
        }

        # Installed/available P/PP/HF
        $msgnone = Msg::new("None", 30, 1191);
        $msginstaller = Msg::new("Installer", 30, 1192);

        if ($cpic->{sortready}) {
            $prodname = $sys->{iprod}{$prod}{name};
            $relvers = $sys->{iprod}{$prod}{mprpvers};
            Msg::n();
            $msg = Msg::new("Installed updates for $prodname $relvers:", 30, 1193, "$prodname", "$relvers");
            $msg->bold();

            if ($sys->{iprod}{$prod}{installed_sort_patches} && @{$sys->{iprod}{$prod}{installed_sort_patches}}) {
                for my $patch (@{$sys->{iprod}{$prod}{installed_sort_patches}}) {
                    $patchid = $patchid = $patch->{release_id};
                    $msg = Msg::string_sprintf("\t%-20s: \n\t\t%s", $patch->{release_name}, join("\n\t\t", @{$sys->{iprod}{$prod}{installed_sort_patch}{$patchid}}));
                    Msg::print($msg);
                }
            } else {
                $msg = Msg::string_sprintf("\t%s", $msgnone->{msg});
                Msg::print($msg);
            }

            Msg::n();
            $msg = Msg::new("Available updates for $prodname $relvers:", 30, 1194, "$prodname", "$relvers");
            $msg->bold();

            if ($sys->{iprod}{$prod}{available_sort_patches} && @{$sys->{iprod}{$prod}{available_sort_patches}}||$sys->{iprod}{$prod}{available_sort_patches_cpi} && @{$sys->{iprod}{$prod}{available_sort_patches_cpi}}) {
                for my $patch (@{$sys->{iprod}{$prod}{available_sort_patches}}) {
                    @patchids = @{$patch->{patchIds}{patch_id}};
                    if (@patchids){
                        $patchlist = join ("\n\t\t", @patchids);
                        $msg = Msg::string_sprintf("\t%-20s:", $patch->{release_name});
                        Msg::bold($msg);
                        $msg = Msg::string_sprintf("\t\t%s", $patchlist);
                        Msg::print($msg);
                    } else {
                        $msg = Msg::string_sprintf("\t%-20s", $patch->{release_name});
                        Msg::bold($msg);
                    }
                }
                for my $patch (@{$sys->{iprod}{$prod}{available_sort_patches_cpi}}) {
                    $msg = Msg::string_sprintf("\t(%s) %-20s", $msginstaller->{msg}, $patch->{release_name});
                    Msg::bold($msg);
                }

            } else {
                $msg = Msg::string_sprintf("\t%s", $msgnone->{msg});
                Msg::print($msg);
            }

            if ($sys->{iprod}{$prod}{latest_branchvers_patchvers}) {
                $relvers = $sys->{iprod}{$prod}{latest_branchvers_patchvers};
                $mjvers = $relvers;
                $mjvers =~ s/^([0-9\.]*).*$/$1/;
                Msg::n();

                $msg = Msg::new("The latest version of $prodname $mjvers release is: $relvers", 30, 1195, "$prodname", "$mjvers", "$relvers");
                $msg->bold();
                $msg = Msg::new("\tAvailable updates for $prodname $relvers:", 30, 1196, "$prodname", "$relvers");
                $msg->bold();

                if ($sys->{iprod}{$prod}{latestbranchvers_sort_patches} && @{$sys->{iprod}{$prod}{latestbranchvers_sort_patches}}||$sys->{iprod}{$prod}{latestbranchvers_sort_patches_cpi} && @{$sys->{iprod}{$prod}{latestbranchvers_sort_patches_cpi}}) {
                    for my $patch (@{$sys->{iprod}{$prod}{latestbranchvers_sort_patches}}) {
                        $msg = Msg::string_sprintf("\t  %-20s", $patch->{release_name});
                        Msg::print($msg);
                    }
                    for my $patch (@{$sys->{iprod}{$prod}{latestbranchvers_sort_patches_cpi}}) {
                        $msg = Msg::string_sprintf("\t  (%s) %-20s", $msginstaller->{msg}, $patch->{release_name});
                        Msg::print($msg);
                    }
                } else {
                    $msg = Msg::string_sprintf("\t  %s", $msgnone->{msg});
                    Msg::print($msg);
                }
            }

            if ($sys->{iprod}{$prod}{latest_majorvers_patchvers}) {
                $relvers = $sys->{iprod}{$prod}{latest_majorvers_patchvers};
                $mjvers = $relvers;
                $mjvers =~ s/^([0-9\.]*).*$/$1/;
                Msg::n();
                $msg = Msg::new("The latest version of $prodname is: $relvers", 30, 1197, "$prodname", "$relvers");
                $msg->bold();
                $msg = Msg::new("\tAvailable updates for $prodname $relvers:", 30, 1196, "$prodname", "$relvers");
                $msg->bold();

                if ($sys->{iprod}{$prod}{latestmajorvers_sort_patches} && @{$sys->{iprod}{$prod}{latestmajorvers_sort_patches}}||$sys->{iprod}{$prod}{latestmajorvers_sort_patches_cpi} && @{$sys->{iprod}{$prod}{latestmajorvers_sort_patches_cpi}}) {
                    for my $patch (@{$sys->{iprod}{$prod}{latestmajorvers_sort_patches}}) {
                        $msg = Msg::string_sprintf("\t  %-20s", $patch->{release_name});
                        Msg::print($msg);
                    }
                    for my $patch (@{$sys->{iprod}{$prod}{latestmajorvers_sort_patches_cpi}}) {
                        $msg = Msg::string_sprintf("\t  (%s) %-20s", $msginstaller->{msg}, $patch->{release_name});
                        Msg::print($msg);
                    }
                } else {
                    $msg = Msg::string_sprintf("\t  %s", $msgnone->{msg});
                    Msg::print($msg);
                }
            }
            Msg::n();
            $msg = Msg::new("Please visit https://sort.symantec.com for more information.", 30, 1198);
            $msg->bold();

        } else {
            Msg::n();
            $msg = Msg::new("Attempted to connect to https://sort.symantec.com to check for product updates, but connection failed.", 30, 1199);
            $msg->bold();
            $msg = Msg::new("Please visit https://sort.symantec.com to check for available product updates and information.", 30, 1200);
            $msg->bold();
        }

    }
    return '';
}

1;
