#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
package CPIC;
use strict;

sub install {
    my $cpic=shift;
    my ($cfg,$msg,$failmsg,$prod,$steps,$web);
    return '' if (!$cpic->{prod});

    $cfg=Obj::cfg();
    $cpic->preinstall_messages() unless (Cfg::opt(qw(patchupgrade upgrade)));

    $prod=$cpic->prod;
    $prod->preinstall_tasks if $prod->can('preinstall_tasks');
    $web = Obj::web();
    $web->web_script_form('installing') if (Obj::webui());

    Msg::title();

    my $edr = Obj::edr();
    # because upgrade = uninstall then install, steps are counted in uninstall
    $steps = $cpic->install_systems_steps();
    $edr->set_progress_steps($steps);
    # we need 2 phase install because of the incident 2086295
    Cfg::opt('serial') ? $cpic->install_systems_serial() :
                         $cpic->install_systems_threaded();

    $msg=Msg::new("$prod->{name} Install completed successfully", 30, 1201, "$prod->{name}");
    $failmsg=Msg::new("$prod->{name} Install did not complete successfully", 30, 1202, "$prod->{name}");
    $msg->display_completion($cpic->failures('installfailmsg'), $msg, $failmsg);
    $cpic->update_summary_install();
    $cpic->ask_install_continue();
    return '';
}

# displays CLI messages
sub preinstall_messages {
    my $cpic = shift;
    my $prod=$cpic->prod;
    $prod->preinstall_messages();
    return;
}

sub install_systems_steps {
    my $cpic=shift;
    my($rel,$prod,$pkg,$patch,$steps,$sys);
    $rel=$cpic->rel;
    $prod=$cpic->prod;
    $steps=0;
    $cpic->{falsefailures}=0;
    $cpic->{installpkgs}||=[];
    $cpic->{installpatchpkgs}||=[];
    $cpic->{installpatches}||=[];
    $rel->{batchinstall}=0;

    if (Cfg::opt('serial')) {
        for my $sys (@{$cpic->{systems}}) {
            for my $pkgi (@{$sys->{installpkgs}}) {
                $pkg=$sys->pkg($pkgi);
                if ($pkg->{batchinstall}) {
                    $rel->{batchinstall}=1;
                } else {
                    $steps++;
                }
            }
            for my $patchi (@{$sys->{installpatches}}) {
                $patch=$sys->patch($patchi);
                if ($patch->{batchinstall}) {
                    $rel->{batchinstall}=1;
                } else {
                    $steps++;
                }
            }
            $steps+=$rel->{batchinstall};
            $steps+=2; # for prod_preinstall_sys and prod_postinstall_sys
        }
    } else {
        $cpic->{installpkgs}=$cpic->verify_list('installpkgs');
        #$ipp = ($#{$cpic->{padvs}}) ? 'installpatchpkgs' : 'installpatches';
        #$cpic->{$ipp}=$cpic->verify_list($ipp,'installpatches');
        $cpic->{installpatches}=$cpic->verify_list('installpatches');

        for my $pkgi (@{$cpic->{installpkgs}}) {
            $pkg=$cpic->pkg($pkgi);
            if ($pkg->{batchinstall}) {
                $rel->{batchinstall}=1;
            } else {
                $steps++;
            }
        }

        for my $patchi (@{$cpic->{installpatches}}) {
            $patch=$cpic->patch($patchi);
            if ($patch->{batchinstall}) {
                $rel->{batchinstall}=1;
            }
        }

        $steps+=$rel->{batchinstall};
        if (!$rel->{batchinstall}) {
            $steps += ($#{$cpic->{padvs}}) ?
                   scalar(@{$cpic->{installpatchpkgs}}) : scalar(@{$cpic->{installpatches}});
        }
        $steps+=2; # for prod_preinstall_sys and prod_postinstall_sys
    }
    return $steps;
}

sub install_systems_serial {
    my $cpic=shift;
    my($failmsg,$passmsg,$completemsg,$msg,$stage,$patch,$pdfr,$pdfrs,$rel,$pkg,$prod,$edr,$tmpsub);
    $rel=$cpic->rel;
    $prod=$cpic->prod;
    $pdfr=Msg::get('pdfr');
    $edr=Obj::edr();
    return '' unless ($edr->{steps});

    $failmsg=Msg::new("Failed", 30, 1203);
    $passmsg=Msg::new("Done", 30, 1204);
    $completemsg=Msg::new("Done", 30, 1204);

    Msg::title();
    $stage = Msg::new("Installing $prod->{abbr}", 30, 1205, "$prod->{abbr}");
    $stage->log();
    for my $sys (@{$cpic->{systems}}) {
        $msg=Msg::new("Performing $prod->{abbr} preinstall tasks on $sys->{sys}", 30, 1206, "$prod->{abbr}", "$sys->{sys}");
        $stage->display_left($msg);
        $cpic->prod_preinstall_sys($sys);
        $stage->display_right($completemsg);
    }

    $prod->preinstall_errors($cpic) if ($cpic->sys_error);
    EDR::reset_errors_warnings();

    for my $sys (@{$cpic->{systems}}) {
        if ($rel->{batchinstall} && @{$sys->{installpkgs}}) {
            $pdfrs=Msg::get('pdfrs');
            $msg=Msg::new("Installing $prod->{abbr} $pdfrs on $sys->{sys}", 30, 1207, "$prod->{abbr}", "$pdfrs", "$sys->{sys}");
            $stage->display_left($msg);
            if ($cpic->pkgs_install_sys($sys)) {
                $stage->display_right($passmsg);
            } else {
                $stage->display_right($failmsg);
            }
        }

        if ($rel->{batchinstall} && @{$sys->{installpatches}}) {
            $pdfrs=Msg::get('pdfrs');
            $msg=Msg::new("Installing $prod->{abbr} $pdfrs on $sys->{sys}", 30, 1207, "$prod->{abbr}", "$pdfrs", "$sys->{sys}");
            $stage->display_left($msg);
            if ($cpic->patches_install_sys($sys)) {
                $stage->display_right($passmsg);
            } else {
                $stage->display_right($failmsg);
            }
        }

        for my $pkgi (@{$sys->{installpkgs}}) {
            $pkg=$sys->pkg($pkgi);
            next if ($pkg->{installpkgslast} || $pkg->{batchinstall});
            $pdfr=Msg::get('pdfr');
            $msg=Msg::new("Installing $pkg->{pkg} $pdfr on $sys->{sys}", 30, 1207, "$pkg->{pkg}", "$pdfr", "$sys->{sys}");
            $stage->display_left($msg);
            if ($cpic->pkg_install_sys($sys, $pkg)) {
                $stage->display_right($passmsg);
                pkg_install_passed_sys($sys, $pkg);
            } else {
                $stage->display_right($failmsg);
                pkg_install_failed_sys($sys, $pkg);
            }
        }

        for my $patchi (@{$sys->{installpatches}}) {
            $patch=$sys->patch($patchi);
            $pkg=$patch->pkg($patch->{pkg});
            next if ($pkg->{installpatcheslast} || $patch->{batchinstall});
            $msg=Msg::new("Installing $patch->{patch_vers} patch on $sys->{sys}", 30, 1208, "$patch->{patch_vers}", "$sys->{sys}");
            $stage->display_left($msg);
            if ($cpic->patch_install_sys($sys, $patch)) {
                $stage->display_right($passmsg);
                patch_install_passed_sys($sys, $patch);
            } else {
                $stage->display_right($failmsg);
                patch_install_failed_sys($sys, $patch);
            }
        }
        $sys->{installpkgslast}=$rel->adjust_pkg_sequence($sys->{installpkgslast});

        for my $pkgi (@{$sys->{installpkgslast}}) {
            $pkg=$sys->pkg($pkgi);
            next if ($pkg->{batchinstall});

            $pdfr=Msg::get('pdfr');
            $msg=Msg::new("Installing $pkg->{pkg} $pdfr on $sys->{sys}", 30, 1207, "$pkg->{pkg}", "$pdfr", "$sys->{sys}");
            $stage->display_left($msg);
            if ($cpic->pkg_install_sys($sys, $pkg)) {
                $stage->display_right($passmsg);
                pkg_install_passed_sys($sys, $pkg);
            } else {
                $stage->display_right($failmsg);
                pkg_install_failed_sys($sys, $pkg);
            }
        }

        for my $patchi (@{$sys->{installpatcheslast}}) {
            $patch=$sys->patch($patchi);
            $pkg=$patch->pkg($patch->{pkg});
            next if (!$pkg->{installpatcheslast});
            next if ($patch->{batchinstall});
            $msg=Msg::new("Installing $patch->{patch_vers} patch on $sys->{sys}", 30, 1208, "$patch->{patch_vers}", "$sys->{sys}");
            $stage->display_left($msg);
            if ($cpic->patch_install_sys($sys, $patch)) {
                $stage->display_right($passmsg);
                patch_install_passed_sys($sys, $patch);
            } else {
                $stage->display_right($failmsg);
                patch_install_failed_sys($sys, $patch);
            }
        }
    }

    for my $sys (@{$cpic->{systems}}) {
        $msg=Msg::new("Performing $prod->{abbr} postinstall tasks on $sys->{sys}", 30, 1209, "$prod->{abbr}", "$sys->{sys}");
        $stage->display_left($msg);
        $cpic->prod_postinstall_sys($sys);
        $stage->display_right($completemsg);
    }

    $prod->postinstall_errors($cpic) if ($cpic->sys_error);
    EDR::reset_errors_warnings();
    return;
}

sub install_systems_threaded {
    my $cpic=shift;
    my ($copythr,$failmsg,$passmsg,$completemsg,$msg,$stage,$nsys,$nthr,$patch,$pdfr,$pdfrs,$pkg,$prod,$rtns,$thr,$rel);
    $prod=$cpic->prod;
    $rel=$cpic->rel;
    $pdfr=Msg::get('pdfr');

    $failmsg=Msg::new("Failed", 30, 1203);
    $passmsg=Msg::new("Done", 30, 1204);
    $completemsg=Msg::new("Done", 30, 1204);

    $stage = Msg::new("Installing $prod->{abbr}", 30, 1205, "$prod->{abbr}");
    $stage->log();
    if ($cpic->nsystems*2<=$cpic->{threadlimit}) {
        $copythr=1;
        $copythr=0 if ($rel->{batchinstall});
    } else {
        $nsys=$cpic->nsystems;
        Msg::log("No copy threads as $nsys systems*2 would exceed the thread limit of $cpic->{threadlimit}");
    }

    for my $sys (@{$cpic->{systems}}) {
        for my $pkgi (@{$sys->{installpkgslast}}) {
            push(@{$cpic->{installpkgslast}},$pkgi) unless EDRu::inarr($pkgi,@{$cpic->{installpkgslast}});
        }
    }
    #fix for e2567015
    $cpic->{installpkgslast}=$rel->adjust_pkg_sequence($cpic->{installpkgslast});

    $thr=Obj::thr();
    $nthr = ($copythr) ? $cpic->nsystems*2 : $cpic->nsystems;
    $thr->mq_create_threads($nthr, $cpic->{threadlimit});
    $cpic->copy_pkg_patch_subs($thr) if ($copythr);

    $msg=Msg::new("Performing $prod->{abbr} preinstall tasks", 30, 1210, "$prod->{abbr}");
    $stage->display_left($msg);

    $thr->mq_add_sub_allsys('CPIC', 'prod_preinstall_sys');
    $thr->mq_wait_for_completion($cpic->nsystems);
    $thr->mq_read_rtns();

    if ($cpic->sys_error) {
        $stage->display_right($failmsg);
        $prod->preinstall_errors($cpic);
    } else {
        $stage->display_right($completemsg);
    }
    EDR::reset_errors_warnings();

    # install packages batchly
    if ($rel->{batchinstall} && @{$cpic->{installpkgs}}) {
        $pdfrs=Msg::get('pdfrs');
        $msg=Msg::new("Installing $prod->{abbr} $pdfrs", 30, 1211, "$prod->{abbr}", "$pdfrs");
        $stage->display_left($msg);
        $thr->mq_add_sub_allsys('CPIC', 'pkgs_install_sys');
        $thr->mq_wait_for_completion();
        $rtns=$thr->mq_read_rtns();
        $msg=$thr->mq_check_rtns($rtns) ? $passmsg : $failmsg;
        $stage->display_right($msg);
    }

    # install patches batchly
    if ($rel->{batchinstall} && @{$cpic->{installpatches}}) {
        $msg=Msg::new("Installing $prod->{abbr} patches", 30, 1212, "$prod->{abbr}");
        $stage->display_left($msg);
        $thr->mq_add_sub_allsys('CPIC', 'patches_install_sys');
        $thr->mq_wait_for_completion();
        $rtns=$thr->mq_read_rtns();
        $msg=$thr->mq_check_rtns($rtns) ? $passmsg : $failmsg;
        $stage->display_right($msg);
    }

    # install packages
    for my $pkgi (@{$cpic->{installpkgs}}) {
        $pkg=$cpic->pkg($pkgi);
        next if ($pkg->{installpkgslast} || $pkg->{batchinstall});
        $msg=Msg::new("Installing $pkg->{pkg} $pdfr", 30, 1211, "$pkg->{pkg}", "$pdfr");
        $stage->display_left($msg);
        $thr->mq_add_sub_allsys('CPIC', 'pkg_install_sys', $pkgi, $copythr);
        $thr->mq_wait_for_completion($cpic->nsystems);
        $rtns=$thr->mq_read_rtns('01', 'CPIC::pkg_install_passed_sys',
                                       'CPIC::pkg_install_failed_sys', $pkgi);
        # if failures happen on any machine, print 'Failed'
        $msg=$thr->mq_check_rtns($rtns) ? $passmsg : $failmsg;
        $stage->display_right($msg);
    }

    # install patches
    if ($#{$cpic->{padvs}}) {
        # multi-padv must display "Installing $pkg patches" and install
        # the appropriate patch per the padv of the system
        for my $pkgi (@{$cpic->{installpatchpkgs}}) {
            $pkg=$cpic->pkg($pkgi);
            next if ($pkg->{installpatcheslast} || $pkg->{batchinstall});
            $msg=Msg::new("Installing $pkg->{pkg} patch", 30, 1213, "$pkg->{pkg}");
            $stage->display_left($msg);
            $cpic->multipadv_patch_install_subs($thr, $pkgi, $copythr);
            $thr->mq_wait_for_completion($cpic->nsystems);
            $cpic->read_multipadv_patch_install_returns($thr, $pkgi);
            $stage->display_right();
        }
    } else {
        # single-padv can explicitely display "Installing $patch patch"
        for my $patchi (@{$cpic->{installpatches}}) {
            $patch=$cpic->patch($patchi);
            $pkg=$patch->pkg($patch->{pkg});
            next if ($pkg->{installpatcheslast} || $patch->{batchinstall});
            $msg=Msg::new("Installing $patch->{patch_vers} patch", 30, 1213, "$patch->{patch_vers}");
            $stage->display_left($msg);
            $thr->mq_add_sub_allsys('CPIC', 'patch_install_sys', $patchi, $copythr);
            $thr->mq_wait_for_completion($cpic->nsystems);
            $rtns=$thr->mq_read_rtns('01','CPIC::patch_install_passed_sys',
                                          'CPIC::patch_install_failed_sys', $patchi);
            # if failures happen on any machine, print 'Failed'
            $msg=$thr->mq_check_rtns($rtns) ? $passmsg : $failmsg;
            $stage->display_right($msg);
        }
    }

    # install packages round II
    for my $pkgi (@{$cpic->{installpkgslast}}) {
        $pkg=$cpic->pkg($pkgi);
        next if ($pkg->{batchinstall});

        $msg=Msg::new("Installing $pkg->{pkg} $pdfr", 30, 1211, "$pkg->{pkg}", "$pdfr");
        $stage->display_left($msg);
        $thr->mq_add_sub_allsys('CPIC', 'pkg_install_sys', $pkgi, $copythr);
        $thr->mq_wait_for_completion($cpic->nsystems);
        $rtns=$thr->mq_read_rtns('01', 'CPIC::pkg_install_passed_sys',
                                       'CPIC::pkg_install_failed_sys', $pkgi);
        # if failures happen on any machine, print 'Failed'
        $msg=$thr->mq_check_rtns($rtns) ? $passmsg : $failmsg;
        $stage->display_right($msg);
    }

    # install patches round II
    if ($#{$cpic->{padvs}}) {
        # multi-padv must display "Installing $pkg patches" and install
        # the appropriate patch per the padv of the system
        for my $pkgi (@{$cpic->{installpatchpkgslast}}) {
            $pkg=$cpic->pkg($pkgi);
            next if (!$pkg->{installpatcheslast});
            next if ($pkg->{batchinstall});
            $msg=Msg::new("Installing $pkg->{pkg} patch", 30, 1213, "$pkg->{pkg}");
            $stage->display_left($msg);
            $cpic->multipadv_patch_install_subs($thr, $pkgi, $copythr);
            $thr->mq_wait_for_completion($cpic->nsystems);
            $cpic->read_multipadv_patch_install_returns($thr, $pkgi);
            $stage->display_right();
        }
    } else {
        for my $patchi (@{$cpic->{installpatcheslast}}) {
            $patch=$cpic->patch($patchi);
            $pkg=$patch->pkg($patch->{pkg});
            next if (!$pkg->{installpatcheslast});
            next if ($patch->{batchinstall});
            $msg=Msg::new("Installing $patch->{patch_vers} patch", 30, 1213, "$patch->{patch_vers}");
            $stage->display_left($msg);
            $thr->mq_add_sub_allsys('CPIC', 'patch_install_sys', $patchi, $copythr);
            $thr->mq_wait_for_completion($cpic->nsystems);
            $rtns=$thr->mq_read_rtns('01','CPIC::patch_install_passed_sys',
                                     'CPIC::patch_install_failed_sys', $patchi);
            # if failures happen on any machine, print 'Failed'
            $msg=$thr->mq_check_rtns($rtns) ? $passmsg : $failmsg;
            $stage->display_right($msg);
        }
    }

    $msg=Msg::new("Performing $prod->{abbr} postinstall tasks", 30, 1214, "$prod->{abbr}");
    $stage->display_left($msg);
    $thr->mq_add_sub_allsys('CPIC', 'prod_postinstall_sys');
    $thr->mq_wait_for_completion($cpic->nsystems);
    $thr->mq_read_rtns();
    if ($cpic->sys_error) {
        $stage->display_right($failmsg);
        $prod->postinstall_errors($cpic);
    } else {
        $stage->display_right($completemsg);
    }
    EDR::reset_errors_warnings();

    $thr->mq_join_threads();

    return '';
}

# call /opt/VRTS/install/bin/TRAINVERS/add_install_scripts on $sys
sub add_install_scripts_sys {
    my ($cpic,$sys) = @_;
    my ($script,$rootpath);
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $rootpath=Cfg::opt('rootpath');
    $script=$rootpath.$cpic->{installpath}.'/bin/'.$cpic->{release}.'/add_install_scripts';
    if ($sys->exists($script)) {
        $sys->cmd("CALLED_BY=CPI $script");
    } else {
        Msg::log("$script not found on $sys->{sys}");
    }
    return;
}

# this whole thing is probably going away, but saving to compare the two
# upper copy works, this is merge from final 5.1SP1
#
# copy the CPI libraries to /opt/VRTSperl/lib/site_perl/TRAINVERS
# 5.0 copies base libraries in place
# install scripts are copied in completion
sub copy_cpi_sys {
    my ($cpic,$sys) = @_;
    my ($rootpath,$prodx,@allprods,$rel,$vxgettext_dir,$installscript_prod,$bmcdomaindir,$dir,$mod,$scripts,$installscript_name,$prodname,$prods,$cpidir,$installscript,$prod,$localsys,$patchrelease_file);
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $cpic=Obj::cpic() unless (ref($cpic) eq 'CPIC');
    return 1 if (Cfg::opt('makeresponsefile'));
    return '' if ($cpic->{prod} && $sys->prod->{nocpionsys});
    $localsys=$cpic->localsys;
    $rel=$sys->rel;
    $rel->copy_cpi_sys($sys);  #for extras
    if ($cpic->{prod}) {
        $sys->prod->copy_cpi_sys($sys);  #for product binaries and other scripts
    }

    # condition for -copyinstallscripts option (product not defined yet)
    $prods = ($cpic->{prod}) ? [ $cpic->{prod} ] : $rel->{prods};

    $rootpath=Cfg::opt('rootpath');
    $cpidir=$rootpath.$cpic->{perlpath}.'/lib/site_perl/'.$cpic->{release};

    # copy CPI libraries
    $bmcdomaindir=$cpic->{mediapath}.'/scripts';
    for my $dir ('CPIC','CPIP','EDR') {
        next unless (-e "$bmcdomaindir/$dir");
        $sys->mkdir("$cpidir/$dir");
        $localsys->copy_to_sys($sys, "$bmcdomaindir/$dir/*", "$cpidir/$dir");
    }
    if ($rel->{type} eq "P") {
        $patchrelease_file = $rel->{ppatchname}.'.pm';
        $localsys->copy_to_sys($sys, "$cpic->{mediapath}/$patchrelease_file", "$cpidir/CPIP/$patchrelease_file");
    }
      

    # copy CPI message catalogs
    $vxgettext_dir = "$bmcdomaindir/bin";
    # Make sure this image includes messages and a way to decode them.
    if (-x $vxgettext_dir && -d "$bmcdomaindir/messages") {
        $localsys->copy_to_sys($sys, "$vxgettext_dir", "$cpidir");
        $sys->chmod("$cpidir/bin", "+x");
        $sys->rm("$cpidir/messages");
        $sys->mkdir("$cpidir/messages");
        $localsys->copy_to_sys($sys, "$bmcdomaindir/messages/*", "$cpidir/messages");
        # FIX: need to sort out unique directories per train as each family
        # will have its own bmcmap file
    }

    # copy CPI installer scripts
    $sys->mkdir("$rootpath$cpic->{installpath}");
    $sys->createfile("$rootpath$cpic->{installpath}/.cpi5");

    if (Cfg::opt('upgrade') && (defined $rel->{obsolete_scripts})){
        $scripts=join(' ', @{$rel->{obsolete_scripts}});
        unless ($scripts =~ /^\s*$/m) {
            $sys->cmd("cd $rootpath$cpic->{installpath}; _cmd_rmr $scripts");
        }
    }
    if ($rel->{pkgs}) {
        for my $prodname (@$prods) {
            $prod=$sys->prod($prodname);
            @allprods=($prodname);
            push(@allprods,@{$prod->{subprods}}) if ($cpic->{prod} && $prod->{subprods});
            for my $prodx (@allprods) {
                $prod=$sys->prod($prodx);

                # copy product binaries and other scripts
                $prod->copy_cpi_sys($sys);
                next if ($prod->{nocpionsys});

                # copy other modules
                for my $mod (@{$prod->{cpimodules}}) {
                    $localsys->copy_to_sys($sys, "$bmcdomaindir/$mod", $cpidir);
                }

                # copy task scripts
                $installscript_prod = $prod->{installscript_prod} || $prodx;
                $installscript_name = lc($prod->{installscript_name} || $prod->{prod});
                $installscript_name=~s/\s+//m;
                $installscript = "$rootpath$cpic->{installpath}/install" . $installscript_name;
                $cpic->create_task_script_sys($sys,'configure',$installscript_prod,$installscript) if (EDRu::inarr('configure',@{$rel->{args_task}}));
                $installscript = "$rootpath$cpic->{installpath}/uninstall" . $installscript_name;
                $cpic->create_task_script_sys($sys,'uninstall',$installscript_prod,$installscript) if (EDRu::inarr('uninstall',@{$rel->{args_task}}));
            }
        }
    } else {
        unless (Padv::plat($cpic->{padv}) eq "Linux") {
            $installscript_prod = '';
            $installscript_name = $rel->{ppatchname};
            $installscript = "$rootpath/opt/VRTS/install/uninstall" . $installscript_name;
            $cpic->create_task_script_sys($sys,'rollback',$installscript_prod,$installscript);
        }
    }

    $sys->chown_root("$rootpath$cpic->{installpath}");
    return;
}

sub create_task_script_sys {
    my ($cpic,$sys,$task,$prodname,$filename) = @_;
    my ($script,$rel,$edr,$padvs);
    $rel=$sys->rel;
    $edr=Obj::edr();
    $padvs=join(' ',@{$edr->{padvs}});
    $script=<<'SCRIPT';
#!/bin/sh

#
# Copyright (c) 2014 Symantec Corporation. All rights reserved.
# Symantec and the Symantec Logo are trademarks or registered trademarks
# of Symantec Corporation or its affiliates in the U.S. and other countries.
# Other names may be trademarks of their respective owners.
#
# The Licensed Software and Documentation are deemed to be "commercial
# computer software" and "commercial computer software documentation"
# as defined in FAR Sections 12.212 and DFARS Section 227.7202.
#

PDIR=/opt/VRTSperl
SDIR=/opt/VRTSperl/lib/site_perl/RELEASE
PERLVERS=`$PDIR/bin/perl -v | grep This.is | awk '{print $4}' | sed 's/v//'`
PERLFOR=`$PDIR/bin/perl -v | grep This.is | awk '{print $7}'`
IOPT="-I$PDIR/lib/$PERLVERS/$PERLFOR -I$PDIR/lib/$PERLVERS -I$PDIR/lib/site_perl/$PERLVERS/$PERLFOR -I$PDIR/lib/site_perl/$PERLVERS -I$PDIR/lib/site_perl"
IOPT="-I$SDIR/EDR -I$SDIR/CPIC -I$SDIR/CPIP $IOPT"

if [ -z "$PDIR" ]; then
        echo "Cannot find perl to execute $0"
        exit 1
fi

eval 'exec $PDIR/bin/perl -x -S $IOPT $0 ${1+"$@"}'
        if 0;
#!$PDIR/bin/perl

use RELEASE;
use PATCHNAME;

CPIC::main( "RELEASE",
            "PADV",
            "",
            "TASK",
            "PROD");
SCRIPT

    $script=~s/RELEASE/$cpic->{release}/mg;
    if ($rel->{type} eq "P") {
        $script=~s/PATCHNAME/$rel->{ppatchname}/mg;
    } else {
        $script=~s/'use PATCHNAME;'//mg;
    } 
    $script=~s/PADV/$padvs/mg;
    $script=~s/TASK/$task/mg;
    $script=~s/PROD/$prodname/mg;

    $sys->cmd("_cmd_rmr $filename");
    $sys->writefile($script, $filename);
    $sys->chmod($filename, '+x');
    return '' if(EDR::cmdexit());
    return 1;
}


# push the copy_pkg_patch_sys on the upper range of threads
sub copy_pkg_patch_subs {
    my ($cpic,$thr) = @_;
    my ($sys,$tid,$pkgi);
    $tid=$cpic->nsystems;
    for my $sys (@{$cpic->{systems}}) {
        $tid++;
        $thr->mq_add_sub($tid, 'CPIC', 'copy_pkg_patch_sys', $sys->{sys});
    }
    return;
}

# copy the packages and patches in parallel with running the installs
# each routine copies three packages and then waits for the install to
# remove the package it has completed installing before installing the next
sub copy_pkg_patch_sys {
    my ($cpic,$sys) = @_;
    my ($n,@copypp,$patch,$pkgi,$pkg,$patchi);
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    return 1 if (Cfg::opt('makeresponsefile'));;

    # create the list of everything we need to copy, packages
    for my $pkgi (@{$cpic->{installpkgs}}) {
        push(@copypp, $sys->pkg($pkgi))
            if (EDRu::inarr($pkgi, @{$sys->{installpkgs}}) && !(EDRu::inarr($pkgi, @{$sys->{installpkgslast}})));
    }

    # then patches
    if ($#{$cpic->{padvs}}) {
        for my $pkgi (@{$cpic->{installpatchpkgs}}) {
            $pkg=$sys->pkg($pkgi);
            for my $patchi (@{$pkg->patches_sys($sys)}) {
                push(@copypp, $sys->patch($patchi))
                    if (EDRu::inarr($patchi, @{$sys->{installpatches}}) && !(EDRu::inarr($patchi, @{$sys->{installpatcheslast}})));
            }
        }
    } else {
        for my $patchi (@{$cpic->{installpatches}}) {
            push(@copypp, $sys->patch($patchi))
                if (EDRu::inarr($patchi, @{$sys->{installpatches}}) && !(EDRu::inarr($patchi, @{$sys->{installpatcheslast}})));
        }
    }

    # packages for install round II should be last in copypp array
    for my $pkgi (@{$cpic->{installpkgslast}}) {
        push(@copypp, $sys->pkg($pkgi)) if (EDRu::inarr($pkgi, @{$sys->{installpkgslast}}));
    }

    # then patches in round II
    if ($#{$cpic->{padvs}}) {
        for my $pkgi (@{$cpic->{installpatchpkgslast}}) {
            $pkg=$sys->pkg($pkgi);
            for my $patchi (@{$pkg->patches_sys($sys)}) {
                push(@copypp, $sys->patch($patchi))
                    if (EDRu::inarr($patchi, @{$sys->{installpatcheslast}}));
            }
        }
    } else {
        for my $patchi (@{$cpic->{installpatcheslast}}) {
            push(@copypp, $sys->patch($patchi))
                if (EDRu::inarr($patchi, @{$sys->{installpatcheslast}}));
        }
    }
    # now do the copies, copying no more than 3 ahead of the install
    # may be a pkg or a patch, but methods and references are identical
    for my $n (0..$#copypp) {
        if ($n>=4) {
            $pkg=$copypp[$n-4];
            EDRu::wait_for_file_rm($pkg->copy_mark_sys($sys));
        }
        $pkg=$copypp[$n];
        $pkg->copy_sys($sys);
    }
    return '';
}

# can't use $thr->mq_add_sub_allsys() due to $pkg->$patch dereference
sub multipadv_patch_install_subs {
    my ($cpic,$thr,$pkgi,$copythr) = @_;
    my ($sys,$tid,$pkg,$patchi);
    for my $sys (@{$cpic->{systems}}) {
        $tid++;
        $pkg=$sys->pkg($pkgi);
        for my $patchi (@{$pkg->patches_sys($sys)}) {
            $thr->mq_add_sub($tid, 'CPIC', 'patch_install_sys', $sys->{sys},
                $patchi, $copythr);
        }
    }
    return;
}

# can't use $thr->mq_read_rtns() due to $pkg->$patch dereference
sub read_multipadv_patch_install_returns {
    my ($cpic,$thr,$pkgi) = @_;
    my ($sys,$ttid,$patch,$tid,$rtn,$pkg,$patchi);
    for my $sys (@{$cpic->{systems}}) {
        $tid++;
        $pkg=$sys->pkg($pkgi);
        for my $patchi (@{$pkg->patches_sys($sys)}) {
            $patch=$sys->patch($patchi);
            $ttid=(($tid-1)%$thr->{nthreads})+1;
            $rtn=shift(@{$thr->{$ttid}{rtns}});
            next unless (defined $rtn);
            if ($rtn) {
                patch_install_passed_sys($sys, $patch);
            } else {
                patch_install_failed_sys($sys, $patch);
            }
        }
    }
    return;
}

sub pkgs_install_sys {
    my ($cpic,$sys)= @_;
    my ($pkgs,$pkgis,$pkg,$pkg_file,$pkg_file_r,$tmpdir,$localsys,$begintime,$clocktime,$duration);
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');

    $pkgs='';
    $pkgis='';
    $pkg_file='';
    for my $pkgi (@{$cpic->{installpkgs}}) {
        next if (!EDRu::inarr($pkgi, @{$sys->{installpkgs}}));
        $pkg=$sys->pkg($pkgi);
        next if ($pkg->{installpkgslast} || !$pkg->{batchinstall});
        $pkgs.=" $pkg->{pkg}";
        $pkgis.=" $pkgi";
        $pkg_file||=$pkg->{file};
    }
    for my $pkgi (@{$sys->{installpkgslast}}) {
        $pkg=$sys->pkg($pkgi);
        next if (!$pkg->{batchinstall});
        $pkgs.=" $pkg->{pkg}";
        $pkgis.=" $pkgi";
        $pkg_file||=$pkg->{file};
    }
    return 1 unless ($pkgs);

    $begintime=time();

    unless (Cfg::opt('makeresponsefile')) {
        $localsys=$cpic->localsys;
        if (!$sys->{islocal} && $pkg_file) {
            $tmpdir=EDR::tmpdir();
            $pkg_file_r="$tmpdir/" . EDRu::basename($pkg_file);
            if (!$sys->exists($pkg_file_r)) {
                $localsys->copy_to_sys($sys,$pkg_file,$tmpdir);
            }
            $pkg_file=$pkg_file_r;
        }
        $sys->install_pkgs($pkg_file,$pkgs);

        my $iof=EDRu::outputfile('install', $sys->{sys}, "pkgs");
        $sys->copy_to_sys($localsys,$iof) unless ($sys->{islocal});
    }

    $duration=time()-$begintime;
    $clocktime=EDRu::clocktime($duration);
    Msg::log("Install packages '$pkgs' on $sys->{sys} took $duration seconds($clocktime)");

    return 1 if ($cpic->pkgs_install_verify_sys($sys,$pkgis));
    return 0;
}

sub pkgs_install_verify_sys {
    my ($cpic,$sys,$pkgis)= @_;
    my ($fails,$pkg,$pdfr,$msg);
    $pkgis=~s/^\s*//;
    $pkgis=~s/\s*$//;
    $fails=0;
    $pdfr=Msg::get('pdfr');
    for my $pkgi (split(/\s+/,$pkgis)) {
        $pkg=$sys->pkg($pkgi);
        if (Cfg::opt('makeresponsefile') || $pkg->install_success_sys($sys)) {
            $msg=Msg::new("$pkg->{pkg} $pdfr installed successfully on $sys->{sys}", 30, 1215, "$pkg->{pkg}", "$pdfr", "$sys->{sys}");
            $msg->log;
            $sys->set_value('pkginstallpass', 'push', $pkg->{pkg});
            $sys->set_value("pkgvers,$pkg->{pkg}",$pkg->{vers});
            EDRu::appendfile($pkg->{pkg}, EDR::tmpdir()."/pkg_install_pass.$sys->{sys}")
                if (Cfg::opt('bait'));
        } else {
            $msg=Msg::new("$pkg->{pkg} $pdfr failed to install on $sys->{sys}", 30, 1216, "$pkg->{pkg}", "$pdfr", "$sys->{sys}");
            $msg->log;
            $sys->set_value('installfailmsg', 'push', $msg->{msg});
            $sys->set_value('pkginstallfail', 'push', $pkg->{pkg});
            EDRu::appendfile($pkg->{pkg}, EDR::tmpdir()."/pkg_install_fail.$sys->{sys}")
                if (Cfg::opt('bait'));
            $fails++;
        }
    }
    return 0 if ($fails);
    return 1;
}

sub patches_install_sys {
    my ($cpic,$sys)= @_;
    my ($patches,$patchis,$patch,$pkg,$patch_file,$patch_file_r,$tmpdir,$localsys,$begintime,$clocktime,$duration);
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');

    $patches='';
    $patchis='';
    $patch_file='';
    for my $patchi (@{$cpic->{installpatches}}) {
        next if (!EDRu::inarr($patchi, @{$sys->{installpatches}}));
        $patch=$sys->patch($patchi);
        $pkg=$patch->pkg($patch->{pkg});
        next if ($pkg->{installpatcheslast} || !$patch->{batchinstall});
        #e3379438:remove driver before install patch
        $patch->preinstall_sys($sys);
        
        $patches.=" $patch->{patchname}";
        $patchis.=" $patchi";
        $patch_file||=$patch->{file};
    }
    for my $patchi (@{$sys->{installpatcheslast}}) {
        $patch=$sys->patch($patchi);
        next if (!$patch->{batchinstall});
        #e3379438:remove driver before install patch
        $patch->preinstall_sys($sys);

        $patches.=" $patch->{patchname}";
        $patchis.=" $patchi";
        $patch_file||=$patch->{file};
    }
    return 1 unless ($patches);

    $begintime=time();

    unless (Cfg::opt('makeresponsefile')) {
        $localsys=$cpic->localsys;
        if (!$sys->{islocal} && $patch_file) {
            $tmpdir=EDR::tmpdir();
            $patch_file_r="$tmpdir/" . EDRu::basename($patch_file);
            if (!$sys->exists($patch_file_r)) {
                $localsys->copy_to_sys($sys,$patch_file,$tmpdir);
            }
            $patch_file=$patch_file_r;
        }
        # Since on Solaris 11, patches and pkgs are using exact same mechanism
        $sys->install_patches($patch_file,$patches);

        my $iof=EDRu::outputfile('install', $sys->{sys}, "patches");
        $sys->copy_to_sys($localsys,$iof) unless ($sys->{islocal});
    }

    #e3379438:add driver after install patch
    for my $patchi (@{$cpic->{installpatches}}) {
        next if (!EDRu::inarr($patchi, @{$sys->{installpatches}}));
        $patch=$sys->patch($patchi);
        $pkg=$patch->pkg($patch->{pkg});
        next if ($pkg->{installpatcheslast} || !$patch->{batchinstall});
        $patch->postinstall_sys($sys);
    }
    for my $patchi (@{$sys->{installpatcheslast}}) {
        $patch=$sys->patch($patchi);
        next if (!$patch->{batchinstall});
        $patch->postinstall_sys($sys);
    }

    $duration=time()-$begintime;
    $clocktime=EDRu::clocktime($duration);
    Msg::log("Install packages '$patches' on $sys->{sys} took $duration seconds($clocktime)");

    return 1 if ($cpic->patches_install_verify_sys($sys,$patchis));
    return 0;
}

sub patches_install_verify_sys {
    my ($cpic,$sys,$patchis)= @_;
    my ($fails,$patch,$pdfr,$msg);
    $patchis=~s/^\s*//;
    $patchis=~s/\s*$//;
    $fails=0;
    $pdfr=Msg::get('pdfr');
    for my $patchi (split(/\s+/,$patchis)) {
        $patch=$sys->patch($patchi);
        if (Cfg::opt('makeresponsefile') || $patch->install_success_sys($sys)) {
            $msg=Msg::new("$patch->{patch} $pdfr installed successfully on $sys->{sys}", 30, 1215, "$patch->{patch}", "$pdfr", "$sys->{sys}");
            $msg->log;
            $sys->set_value('patchinstallpass', 'push', $patch->{patch});
            $sys->set_value("patchvers,$patch->{patch}",$patch->{vers});
            EDRu::appendfile($patch->{patch}, EDR::tmpdir()."/patch_install_pass.$sys->{sys}")
                if (Cfg::opt('bait'));
        } else {
            $msg=Msg::new("$patch->{patch} $pdfr failed to install on $sys->{sys}", 30, 1216, "$patch->{patch}", "$pdfr", "$sys->{sys}");
            $msg->log;
            $sys->set_value('installfailmsg', 'push', $msg->{msg});
            $sys->set_value('patchinstallfail', 'push', $patch->{patch});
            EDRu::appendfile($patch->{patch}, EDR::tmpdir()."/patch_install_fail.$sys->{sys}")
                if (Cfg::opt('bait'));
            $fails++;
        }
    }
    return 0 if ($fails);
    return 1;
}

# used when one thread runs copy, install, remove routine
sub pkg_install_sys {
    my ($cpic,$sys,$pkg,$copythr)= @_;
    my ($begintime,$clocktime,$duration);
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $pkg=$sys->pkg($pkg) unless (ref($pkg));
    return 1 if (Cfg::opt('makeresponsefile'));;
    # pkg install is conditional
    return 1 if (!EDRu::inarr($pkg->{pkgi}, @{$sys->{installpkgs}}));
    if ($pkg->{mpok}) {
        Msg::log("Not installing $pkg->{pkg} on $sys->{sys} as it is missing from the release media");
        return;
    }

    $begintime=time();

    $pkg->preinstall_sys($sys);
    ($copythr) ? $pkg->wait_for_copy_sys($sys) :
                 $pkg->copy_sys($sys);
    $pkg->install_sys($sys);
    $pkg->remove_sys($sys);
    my $iof=EDRu::outputfile('install', $sys->{sys}, $pkg->{pkg});
    $sys->copy_to_sys($cpic->localsys,$iof) unless ($sys->{islocal});
    $pkg->postinstall_sys($sys);
    # Adding temp failures to test for the web script

    $duration=time()-$begintime;
    $clocktime=EDRu::clocktime($duration);
    Msg::log("Install package $pkg->{pkg} on $sys->{sys} took $duration seconds($clocktime)");

    return 0 if ($cpic->false_failure);
    return 1 if ($pkg->install_success_sys($sys));
    return 0;
}

sub patch_install_sys {
    my ($cpic,$sys,$patch,$copythr)= @_;
    my ($begintime,$clocktime,$duration);
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $patch=$sys->patch($patch) unless (ref($patch));
    return 1 if (Cfg::opt('makeresponsefile'));;
    # patch install is conditional
    return 1 if (!EDRu::inarr($patch->{patch}, @{$sys->{installpatches}}));
    return '' unless (EDRu::inarr($patch->{patch}, @{$sys->{installpatches}}));
    if ($patch->{mpok}) {
        Msg::log("Not installing $patch->{patch_vers} on $sys->{sys} as it is missing from the release media");
        return;
    }

    $begintime=time();

    $patch->preinstall_sys($sys);
    ($copythr) ? $patch->wait_for_copy_sys($sys) :
                 $patch->copy_sys($sys);
    $patch->install_sys($sys);
    $patch->remove_sys($sys);
    my $iof=EDRu::outputfile('install', $sys->{sys}, "patch.$patch->{patch_vers}");
    $sys->copy_to_sys($cpic->localsys,$iof) unless ($sys->{islocal});
    $patch->postinstall_sys($sys);

    $duration=time()-$begintime;
    $clocktime=EDRu::clocktime($duration);
    Msg::log("Install patch $patch->{patch_vers} on $sys->{sys} took $duration seconds($clocktime)");

    return 1 if ($patch->install_success_sys($sys));
    return 0;
}

sub pkg_install_failed_sys {
    my ($sys,$pkg)= @_;
    $pkg=$sys->pkg($pkg) unless (ref($pkg));
    push(@{$sys->{pkginstallfail}}, $pkg->{pkg});
    my $pdfr=Msg::get('pdfr');
    my $io=EDRu::readfile(EDRu::outputfile('install', $sys->{sys}, $pkg->{pkg}));
    my $msg=Msg::new("$pkg->{pkg} $pdfr failed to install on $sys->{sys}", 30, 1216, "$pkg->{pkg}", "$pdfr", "$sys->{sys}");
    $msg->log;
    Msg::log("$io");
    push(@{$sys->{installfailmsg}}, $msg);
    EDRu::appendfile($pkg->{pkg}, EDR::tmpdir()."/pkg_install_fail.$sys->{sys}")
        if (Cfg::opt('bait'));
    return '';
}

sub pkg_install_passed_sys {
    my ($sys,$pkg)= @_;
    $pkg=$sys->pkg($pkg) unless (ref($pkg));
    # pkg install is conditional
    return '' if (!EDRu::inarr($pkg->{pkgi}, @{$sys->{installpkgs}}));
    push(@{$sys->{pkginstallpass}}, $pkg->{pkg});
    $sys->{pkgvers}{$pkg->{pkg}}=$pkg->{vers};
    my $pdfr=Msg::get('pdfr');
    Msg::log("$pkg->{pkg} $pdfr installed successfully on $sys->{sys}");
    EDRu::appendfile($pkg->{pkg}, EDR::tmpdir()."/pkg_install_pass.$sys->{sys}")
        if (Cfg::opt('bait'));
    return '';
}

sub patch_install_failed_sys {
    my ($sys,$patch)= @_;
    $patch=$sys->patch($patch) unless (ref($patch));
    push(@{$sys->{patchinstallfail}}, $patch->{patch_vers});
    my $io=EDRu::readfile(EDRu::outputfile('install', $sys->{sys}, "patch.$patch->{patch_vers}"));
    my $msg=Msg::new("$patch->{patch_vers} patch failed to install on $sys->{sys}", 30, 1217, "$patch->{patch_vers}", "$sys->{sys}");
    $msg->log;
    Msg::log("$io");
    push(@{$sys->{installfailmsg}}, $msg);
    EDRu::appendfile($patch->{patch_vers}, EDR::tmpdir()."/patch_install_fail.$sys->{sys}")
        if (Cfg::opt('bait'));
    return '';
}

sub patch_install_passed_sys {
    my ($sys,$patch)= @_;
    $patch=$sys->patch($patch) unless (ref($patch));
    # patch install is conditional
    return '' if (!EDRu::inarr($patch->{patch}, @{$sys->{installpatches}}));
    push(@{$sys->{patchinstallpass}}, $patch->{patch_vers});
    Msg::log("$patch->{patch_vers} patch installed successfully on $sys->{sys}");
    EDRu::appendfile($patch->{patch_vers},
        EDR::tmpdir()."/patch_install_pass.$sys->{sys}") if (Cfg::opt('bait'));
    return '';
}

sub prod_preinstall_sys {
    my ($cpic,$sys) = @_;
    my ($tmpsub,$prod);
    return '' if (Cfg::opt('makeresponsefile'));;
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');

    $sys->rel->preinstall_sys($sys);
    $sys->rel->preinstall_patch_sys($sys);

    $prod=$sys->prod;
    $tmpsub = upgrade_sub('preinstall_sys');
    if ($prod->can($tmpsub)) {
        $prod->$tmpsub($sys);
    }

    return;
}

sub prod_postinstall_sys {
    my ($cpic,$sys) = @_;
    my ($tmpsub,$prod,$rootpath,$scripts);
    return 1 if (Cfg::opt('makeresponsefile'));
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $prod=$sys->prod;
    $tmpsub = upgrade_sub('postinstall_sys');
    if ($prod->can($tmpsub)) {
        $prod->$tmpsub($sys);
    }
    if ($cpic->rel->{cpipkg}) {
        $rootpath=Cfg::opt('rootpath');
        if (Cfg::opt('upgrade') && (defined $cpic->rel->{obsolete_scripts})){
            $scripts=join(' ', @{$cpic->rel->{obsolete_scripts}});
            unless ($scripts =~ /^\s*$/m) {
                $sys->cmd("cd $rootpath$cpic->{installpath}; _cmd_rmr $scripts");
            }
        }
        if (Cfg::opt('upgrade') && !$sys->{partial_upgrade}){
            $scripts = "uninstallrp uninstallmr";
            #fixed for e3376685, do not show error msg on screen when doing postinstall tasks
            $sys->cmd("cd $rootpath$cpic->{installpath}; _cmd_rmr $scripts"); 
        }
        $cpic->add_install_scripts_sys($sys);
        if (defined $prod->{obsolete_scripts}) {
            $scripts=join(' ', @{$prod->{obsolete_scripts}});
            unless ($scripts =~ /^\s*$/m) {
                $sys->cmd("cd $rootpath$cpic->{installpath}; _cmd_rmr $scripts");
            }
        }
    } else {
        $cpic->copy_cpi_sys($sys);
    }

    $sys->rel->postinstall_sys($sys);
    return;
}

sub update_summary_install {
    my $cpic=shift;
    my ($msg,$pdfrs,$pkgset);

    $pdfrs=Msg::get('pdfrs');
    if ($cpic->{task} eq 'install') {
        if (Cfg::opt('installminpkgs')) {
            $pkgset = Msg::new("The Minimum package set was selected for installation", 30, 1218);
        } elsif (Cfg::opt('installrecpkgs')) {
            $pkgset = Msg::new("The Recommended package set was selected for installation", 30, 1219);
        } elsif (Cfg::opt('installallpkgs')) {
            $pkgset = Msg::new("The All package set was selected for installation", 30, 1220);
        }
    }
    for my $sys (@{$cpic->{systems}}) {
        if ($cpic->{task} eq 'install' && $pkgset) {
            $msg=Msg::new_obj("$pkgset->{msg}");
            $cpic->update_summary($msg);
        }
        if ($#{$sys->{pkginstallpass}}>=0) {
            $msg=Msg::new("The following $pdfrs were installed on $sys->{sys}:", 30, 1221, "$pdfrs", "$sys->{sys}");
            $cpic->update_summary($msg, @{$sys->{pkginstallpass}});
        }
        if ($#{$sys->{pkginstallfail}}>=0) {
            $msg=Msg::new("The following $pdfrs failed to install on $sys->{sys}:", 30, 1222, "$pdfrs", "$sys->{sys}");
            $cpic->update_summary($msg, @{$sys->{pkginstallfail}});
        }
        if ($#{$sys->{patchinstallpass}}>=0) {
          $msg=Msg::new("The following patches were installed on $sys->{sys}:", 30, 1223, "$sys->{sys}");
            $cpic->update_summary($msg, @{$sys->{patchinstallpass}});
        }
        if ($#{$sys->{patchinstallfail}}>=0) {
            $msg=Msg::new("The following patches failed to install on $sys->{sys}:", 30, 1224, "$sys->{sys}");
            $cpic->update_summary($msg, @{$sys->{patchinstallfail}});
        }
    }
    return '';
}

#this function could be called in the "sub install" function as an enhancement, its related incident number is 2663858 and it#could be evaluated in the future.
sub ask_install_continue {
    my $cpic = shift;
    my ($msg, $msg_advice, $pdfrs, $ayn, $pkginstallfail_flag, $patchinstallfail_flag);
    my $edr = Obj::edr();
    for my $sys (@{$cpic->{systems}}) {
        if ($#{$sys->{pkginstallfail}}>=0) {
            $pkginstallfail_flag = 1; 
        }
        if ($#{$sys->{patchinstallfail}}>=0) {
            $patchinstallfail_flag = 1;
        }
    }
    if ($pkginstallfail_flag == 1 || $patchinstallfail_flag == 1){
        $pdfrs=Msg::get('pdfrs');
        $msg = Msg::new("$pdfrs/patches failed to install. Do you want to exit $cpic->{script}?", 30, 1225, "$pdfrs", "$cpic->{script}");
        $ayn = $msg->ayny;
        if ($ayn eq 'Y') {
            $msg_advice = Msg::new("Refer to the installer logs for details and correct the settings. Continue with the installer again, or try to install the packages/patches manually.", 30, 1226);
            $msg_advice->log;
            $msg_advice->print;
            Msg::n();
            $cpic->edr_completion();
        }
    }
    return '';
}

1;
