#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
package CPIC;
use strict;

sub postcheck {
    my($cpic,$edr,$msg,$failmsg,$prod,$steps,$web);
    $cpic=shift;
    return '' if (Cfg::opt(qw(makeresponsefile rootpath)) && !($cpic->{prod}));
    $prod=$cpic->prod;
    return '' if ($prod->{lp});

    $cpic->postcheck_register_items();

    $steps=$cpic->postcheck_steps();
    return if (!$steps);

    $web = Obj::web();
    $edr = Obj::edr();
    $web->web_script_form('postcheck') if (Obj::webui());
    $edr->set_progress_steps($steps);

    Msg::title();

    $cpic->{falsefailures}=0;
    Cfg::opt('serial') ? $cpic->postcheck_serial() :
                         $cpic->postcheck_threaded();

    $cpic->postcheck_summary();

    $msg=Msg::new("$prod->{name} Postcheck completed successfully", 30, 1260, "$prod->{name}");
    $failmsg=Msg::new("$prod->{name} Postcheck did not complete successfully", 30, 1261, "$prod->{name}");

    $edr->display_errors_warnings($msg,$failmsg);
    $web->{complete_failed}=1 if $web->{errors};
    #my $failures=$cpic->failures("postcheckfailmsg");
    #$web->display_completion($failures, $msg, $failmsg);

    return '';
}

sub postcheck_register_items {
    my ($cpic,$prod);
    $cpic=shift;
    $prod=$cpic->prod;

    $prod->register_postchecks_per_system();
    $prod->register_postchecks_per_cluster();
    return 1;
}

sub postcheck_steps {
    my ($cpic,$steps,$sys);
    $cpic=shift;

    $steps=0;
    if (Cfg::opt('serial')) {
        for my $sys (@{$cpic->{systems}}) {
            # for packages/patches checking
            $steps+=1;
            # for processes/drivers checking
            $steps+=1;
            # for registered product specific post-install checkings
            $steps+=scalar(keys %{$cpic->{postchecks_per_system}});
        }
    } else {
        # for packages/patches checking
        $steps+=1;
        # for processes/drivers checking
        $steps+=1;

        # for registered product specific post-install checkings
        $steps+=scalar(keys %{$cpic->{postchecks_per_system}});
    }
    $steps+=scalar(keys %{$cpic->{postchecks_per_cluster}});
    return $steps;
}

sub postcheck_serial {
    my($cpic,$msg,$sys,$failed,$desc,$checki,$prod);
    $cpic=shift;
    $prod=$cpic->prod;
    my $stage = Msg::new("Performing post-install checking for $prod->{abbr}", 30, 1262, "$prod->{abbr}");
    $stage->log();
    for my $sys (@{$cpic->{systems}}) {
        if (EDR::continue_checks()) {
            $msg=Msg::new("Checking status of packages on $sys->{sys}", 30, 1263, "$sys->{sys}");
            $stage->display_left($msg);
            $failed=$cpic->prod_postcheck_pkgs_sys($sys, $cpic->{prod}) ? '' : 'failed';
            $stage->display_right($failed);
        }
    }
    if ($cpic->rel->{rp}) {
        for my $sys (@{$cpic->{systems}}) {
            if (EDR::continue_checks()) {
                $msg=Msg::new("Checking status of patches on $sys->{sys}", 30, 1264, "$sys->{sys}");
                $stage->display_left($msg);
                $failed=$cpic->prod_postcheck_patches_sys($sys, $cpic->{prod}) ? '' : 'failed';
                $stage->display_right($failed);
            }
        }
    }
    for my $sys (@{$cpic->{systems}}) {
        if (EDR::continue_checks()) {
            $msg=Msg::new("Checking processes and drivers status on $sys->{sys}", 30, 1265, "$sys->{sys}");
            $stage->display_left($msg);
            $failed=$cpic->prod_postcheck_procs_sys($sys, $cpic->{prod}) ? '' : 'failed';
            $stage->display_right($failed);
        }
    }

    for my $checki (sort keys %{$cpic->{postchecks_per_system}}) {
        if (EDR::continue_checks()) {
            $desc=$cpic->{postchecks_per_system}{$checki}{description};
            for my $sys (@{$cpic->{systems}}) {
                $msg=Msg::new("Checking $desc on $sys->{sys}", 30, 1266, "$desc", "$sys->{sys}");
                $stage->display_left($msg);
                $failed=$cpic->prod_postcheck_item_sys($sys, $checki) ? '' : 'failed';
                $stage->display_right($failed);
            }
        }
    }

    for my $checki (sort keys %{$cpic->{postchecks_per_cluster}}) {
        if (EDR::continue_checks()) {
            $desc=$cpic->{postchecks_per_cluster}{$checki}{description};
            $msg=Msg::new("Checking $desc", 30, 1267, "$desc");
            $stage->display_left($msg);
            $failed=$cpic->prod_postcheck_item_cluster($checki) ? '' : 'failed';
            $stage->display_right($failed);
        }
    }

    return '';
}

sub postcheck_threaded {
    my($cpic,$msg,$thr,$checki,$desc,$prod);
    $cpic=shift;

    $prod=$cpic->prod;
    my $stage = Msg::new("Performing post-install checking for $prod->{abbr}", 30, 1262, "$prod->{abbr}");
    $stage->log();
    $thr=Obj::thr();
    $thr->mq_create_threads($cpic->nsystems, $cpic->{threadlimit});
    if (EDR::continue_checks()) {
        $msg=Msg::new("Checking status of packages", 30, 1268);
        $stage->display_left($msg);
        $thr->mq_add_sub_allsys('CPIC', 'prod_postcheck_pkgs_sys', $cpic->{prod});
        $thr->mq_wait_for_completion();
        $thr->mq_read_rtns();
        $stage->display_right();
    }
    if ($cpic->rel->{rp} && EDR::continue_checks()) {
        $msg=Msg::new("Checking status of patches", 30, 1269);
        $stage->display_left($msg);
        $thr->mq_add_sub_allsys('CPIC', 'prod_postcheck_patches_sys', $cpic->{prod});
        $thr->mq_wait_for_completion();
        $thr->mq_read_rtns();
        $stage->display_right();
    }

    if (EDR::continue_checks()) {
        $msg=Msg::new("Checking processes and drivers status", 30, 1270);
        $stage->display_left($msg);
        $thr->mq_add_sub_allsys('CPIC', 'prod_postcheck_procs_sys', $cpic->{prod});
        $thr->mq_wait_for_completion();
        $thr->mq_read_rtns();
        $stage->display_right();
    }

    for my $checki (sort keys %{$cpic->{postchecks_per_system}}) {
        if (EDR::continue_checks()) {
            $desc=$cpic->{postchecks_per_system}{$checki}{description};
            $msg=Msg::new("Checking $desc", 30, 1267, "$desc");
            $stage->display_left($msg);
            $thr->mq_add_sub_allsys('CPIC', 'prod_postcheck_item_sys', $checki);
            $thr->mq_wait_for_completion();
            $thr->mq_read_rtns();
            $stage->display_right();
        }
    }

    $thr->mq_join_threads();

    for my $checki (sort keys %{$cpic->{postchecks_per_cluster}}) {
        if (EDR::continue_checks()) {
            $desc=$cpic->{postchecks_per_cluster}{$checki}{description};
            $msg=Msg::new("Checking $desc", 30, 1267, "$desc");
            $stage->display_left($msg);
            $cpic->prod_postcheck_item_cluster($checki);
            $stage->display_right();
        }
    }

    return '';
}

sub prod_postcheck_procs_sys {
    my ($cpic,$sys,$prod) = @_;
    my ($msg,$proci,@failed_procs,$running,$startprocs,$proc,$procs);

    # scalars must be passed to threaded call
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $prod=$sys->prod($prod) unless (ref($prod));

    return 1 if (Cfg::opt('makeresponsefile'));
    return '' if ($sys->{stop_checks});

    $startprocs=$prod->startprocs_sys($sys);
    for my $proci (@{$startprocs}) {
        $proc=$sys->proc($proci);
        next if $proc->{donotstart};
        $running=$proc->check_sys($sys, 'poststart');
        if (!$running) {
            push(@failed_procs, $proc->{proc});
        }
    }

    if (@failed_procs) {
        $sys->set_value('postcheckfailprocs','list',@failed_procs);
        $procs=join("\n\t", @failed_procs);
        $msg=Msg::new("The following processes or drivers are not started on $sys->{sys}:\n\t$procs", 30, 1271, "$sys->{sys}", "$procs");
        $sys->push_error($msg);
        return 0;
    }
    return 1;
}

sub prod_postcheck_pkgs_sys {
    my ($cpic,$sys,$prod) = @_;
    my ($msg,$pkg,$pkgi,$pkgs,$pkgvers,%invalidpkgs,@notinstalledpkgs,@partialpkgs,$vers,$rel);
    my ($msg_missing_pkgs, $msg_invalid_pkgs, $msg_partial_pkgs);
    # scalars must be passed to threaded call
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $prod=$sys->prod($prod) unless (ref($prod));

    return 1 if (Cfg::opt('makeresponsefile'));
    return '' if ($sys->{stop_checks});

    $rel = Obj::rel();
    $pkgs = $prod->minpkgs();
    for my $pkgi (@{$pkgs}) {
        $cpic->create_cpip_objects($sys->{padv});
        $pkg = $sys->pkg($pkgi);
        next if ($pkg->donotinstall_sys($sys));
        $vers = $pkg->version_sys($sys);
        if (!$vers) {
            push @notinstalledpkgs, $pkg->{pkg};
        } else {
            # todo verify pkg
            if ($sys->padv->pkg_verify_sys($sys, $pkg->{pkg})) {
                push @partialpkgs, $pkg->{pkg};
            }
            # For MR release, we don't check package version, because
            # 1. for some platforms package version is same with base version after MR installed
            # 2. package base version can be various
            next if($rel->{rp});

            if ($cpic->{fromdisk}) {
                $pkgvers = $pkg->{gavers} || $rel->{vers};
            } else {
                $pkgvers = $pkg->{vers};
            }
            if ($pkg->{gavers}){
                if (EDRu::compvers($vers,$pkgvers,4) == 2) {
                    $invalidpkgs{$pkg->{pkg}}="$vers|$pkgvers";
                    next;
                }
            } else {
                if (EDRu::compvers($vers,$pkgvers,3) != 0) {
                    $invalidpkgs{$pkg->{pkg}}="$vers|$pkgvers";
                    next;
                }
            }
        }
    }

    if (@notinstalledpkgs) {
        $msg_missing_pkgs = join("\n\t", @notinstalledpkgs);
        $msg=Msg::new("The following required packages are not installed on $sys->{sys}:\n\t$msg_missing_pkgs", 30, 1272, "$sys->{sys}", "$msg_missing_pkgs");
        $sys->push_error($msg);
    }

    if (keys %invalidpkgs ) {
        $msg_invalid_pkgs = "";
        for my $pkg (sort keys %invalidpkgs) {
            ($vers,$pkgvers) = split(/\|/,$invalidpkgs{$pkg});
            $msg_invalid_pkgs .= sprintf("\n\t%-12s  installed: %-16s  expected: %-16s", $pkg, $vers, $pkgvers);
        }
        $msg=Msg::new("The following required packages are not installed with the correct version on $sys->{sys}:$msg_invalid_pkgs", 30, 1273, "$sys->{sys}", "$msg_invalid_pkgs");
        $sys->push_error($msg);
    }
    
    if (@partialpkgs) {
        $msg_partial_pkgs = join("\n\t", @partialpkgs);
        $msg=Msg::new("Verification of following packages failed on $sys->{sys}:\n\t$msg_partial_pkgs\nDetailed information can be found in the log files.", 30, 1274, "$sys->{sys}", "$msg_partial_pkgs");
        $sys->push_warning($msg);
    }
    
    if ( %invalidpkgs || @notinstalledpkgs || @partialpkgs) {
        return 0;
    } else {
        return 1;
    }
}

sub prod_postcheck_patches_sys {
    my ($cpic,$sys,$prod) = @_;
    my ($pkg,$patch,$cv,$padv,$rel);
    my (@notinstalledpatches, $msg_missing_patches, $msg);
    # scalars must be passed to threaded call
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $prod=$sys->prod($prod) unless (ref($prod));
    $rel=$prod->rel;
    $padv=$sys->padv;

    return 1 if (Cfg::opt('makeresponsefile'));
    return '' if ($sys->{stop_checks}) || (!$rel->{patches});

    for my $pkgi (@{$prod->allpkgs}) {
        $pkg=$prod->pkg($pkgi);
        next unless ($pkg->version_sys($sys));
        for my $patchi (@{$pkg->patches_sys($sys)})  {
            $patch=$sys->patch($patchi) unless (ref($patchi));
            if ($patch->{vers}) {
                if (EDRu::compvers($padv->patch_version_sys($sys, $patch),$patch->{vers}) != 0) {
                    # Patch not installed. Need to record.
                    push @notinstalledpatches, $patch->{showform};
                }
            }
        }
    }

    if( @notinstalledpatches ) {
        $msg_missing_patches=join("\n\t", @notinstalledpatches);
        $msg=Msg::new("The following patches are not installed on $sys->{sys}:\n\t$msg_missing_patches", 30, 1275, "$sys->{sys}", "$msg_missing_patches");
        $sys->push_error($msg);
        return 0;
    }

    return 1;
}

sub prod_postcheck_item_sys {
    my ($cpic,$sys,$checki) = @_;
    my ($check,$rtn,$handler,$prod);

    # scalars must be passed to threaded call
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');

    return 1 if (Cfg::opt('makeresponsefile'));
    return '' if ($sys->{stop_checks});

    $check=$cpic->{postchecks_per_system}{$checki};
    $prod=$check->{product};
    $handler=$check->{handler};
    $rtn=&$handler($prod,$sys);
    return $rtn?1:0;

}

sub prod_postcheck_item_cluster {
    my ($cpic,$checki) = @_;
    my ($check,$rtn,$handler,$prod);

    return 1 if (Cfg::opt('makeresponsefile'));

    $check=$cpic->{postchecks_per_cluster}{$checki};
    $prod=$check->{product};
    $handler=$check->{handler};
    $rtn=&$handler($prod);
    return $rtn?1:0;
}

sub postcheck_summary {
    my ($cpic,$prod,$steps,$sys);
    $cpic=shift;

    $prod=$cpic->prod;
    $prod->postcheck_summary();
    return;
}

1;
