#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
package CPIC;
use strict;

sub rollback {
    my ($cpic,$msg,$failmsg,$prod,$steps,$web,$errors);
    $cpic=shift;
    return '' if (Cfg::opt('makeresponsefile') && !($cpic->{prod}));
    $prod=$cpic->prod;
    $prod->preremove_tasks if $prod->can('preremove_tasks');
    $steps=$cpic->rollback_systems_steps();
    return if (!$steps); # not a partial upgrade

    my $edr = Obj::edr();
    $web = Obj::web();
    $web->web_script_form('uninstalling') if (Obj::webui()); 
    $edr->set_progress_steps($steps);

    Msg::title();

    Cfg::opt('serial') ? $cpic->rollback_systems_serial() :
                         $cpic->rollback_systems_threaded();

    $msg=Msg::new("$prod->{name} Rollback completed successfully", 30, 1276, "$prod->{name}");
    $failmsg=Msg::new("$prod->{name} Rollback did not complete successfully", 30, 1277, "$prod->{name}");
    $errors=$cpic->failures('uninstallfailmsg');
    $msg->display_completion($errors, $msg, $failmsg);     
    # bail out uninstallrp/uninstallmr if there is any patches failed to be rollback
    if ($#$errors>=0) {
        $msg=Msg::new("Manually uninstall the failed patches and reboot the systems to start the processes.\n", 30, 1278);
        $msg->print();
        $cpic->completion();
    }
    return '';
}

sub rollback_systems_steps {
    my($cpic,$prod,$steps,$sys); 
    $cpic=shift;
    $prod=$cpic->prod;
    $cpic->{falsefailures}=0;
    $steps=0;
    $cpic->{uninstallpatches}=$cpic->verify_list('uninstallpatches');
    return 0 unless (scalar(@{$cpic->{uninstallbundles}}) + scalar(@{$cpic->{uninstallpatches}}));
    if (Cfg::opt('serial')) {
        for $sys(@{$cpic->{systems}}) {
            $steps++ if ($prod->can(upgrade_sub('preremove_sys')));
            $steps+=scalar(@{$sys->{uninstallpatches}}) if ($sys->{uninstallpatches});
            $steps++ if ($prod->can(upgrade_sub('postremove_sys')));
        }
    } else {       
        $steps++ if ($prod->can(upgrade_sub('preremove_sys')));
        $steps+=scalar(@{$cpic->{uninstallpatches}}) if ($cpic->{uninstallpatches});
        $steps++ if ($prod->can(upgrade_sub('postremove_sys')));
    }
    return $steps;
}       

sub rollback_systems_serial {
    my $cpic=shift;
    my($fail,$failmsg,$passmsg,$completemsg,$msg,$pass,$patch,$patchi,$prod,$stage,$sys,$bundle,$rel_vers,$upgraded_sub);
    $prod=$cpic->prod;
    $stage = Msg::new("Uninstalling $prod->{abbr}", 30, 1279, "$prod->{abbr}");
    $stage->log();

    $failmsg=Msg::new("Failed", 30, 1203);
    $passmsg=Msg::new("Done", 30, 1204);
    $completemsg=Msg::new("Done", 30, 1204);

    $rel_vers=$cpic->rel->{vers};
    $upgraded_sub = upgrade_sub('preremove_sys');
    if ($prod->can($upgraded_sub)) {
       for $sys(@{$cpic->{systems}}) {
            $msg=Msg::new("Performing $prod->{abbr} preremove tasks on $sys->{sys}", 30, 1280, "$prod->{abbr}", "$sys->{sys}");
            $stage->display_left($msg);
            $sys->rel->preremove_sys($sys);
            $prod->$upgraded_sub($sys);
            $stage->display_right($completemsg);
        }
    }

    $prod->preremove_errors($cpic) if ($cpic->sys_error); 
    EDR::reset_errors_warnings();

    for $sys(@{$cpic->{systems}}) {
        for $patchi(@{$cpic->{uninstallpatches}}) {
            next if (!EDRu::inarr($patchi, @{$sys->{uninstallpatches}}));
            $patch=$sys->patch($patchi);
            $msg=Msg::new("Uninstalling $patch->{showform} patch on $sys->{sys}", 30, 1281, "$patch->{showform}", "$sys->{sys}");
            $stage->display_left($msg);
            if ($cpic->patch_uninstall_sys($sys, $patch, $rel_vers)) {
                patch_uninstall_passed_sys($sys, $patch);
                $stage->display_right($passmsg);
                $pass++;
            } else {
                patch_uninstall_failed_sys($sys, $patch);
                $stage->display_right($failmsg);
                $fail++;
            }
        }
    }

    $upgraded_sub = upgrade_sub('postremove_sys');
    if ($prod->can($upgraded_sub)) {
        for $sys(@{$cpic->{systems}}) {
            $msg=Msg::new("Performing $prod->{abbr} postremove tasks on $sys->{sys}", 30, 1282, "$prod->{abbr}", "$sys->{sys}");
            $stage->display_left($msg);
            $prod->$upgraded_sub($sys);
            $sys->rel->postremove_sys($sys);            
            $stage->display_right($completemsg);
        }
    }

    $prod->postremove_errors($cpic) if ($cpic->sys_error); 
    EDR::reset_errors_warnings();

    return '';
}

sub rollback_systems_threaded {
    my $cpic=shift;
    my($failmsg,$passmsg,$completemsg,$msg,$patch,$patchi,$prod,$stage,$sys,$rtn,$thr,$bundle,$rtns);
    $prod=$cpic->prod;    
    $thr=Obj::thr();

    $failmsg=Msg::new("Failed", 30, 1203);
    $passmsg=Msg::new("Done", 30, 1204);
    $completemsg=Msg::new("Done", 30, 1204);

    $stage = Msg::new("Uninstalling $prod->{abbr}", 30, 1279, "$prod->{abbr}");
    $stage->log();

    $thr->mq_create_threads($cpic->nsystems, $cpic->{threadlimit});

    if ($prod->can(upgrade_sub('preremove_sys'))) {
        $msg=Msg::new("Performing $prod->{abbr} preremove tasks", 30, 1283, "$prod->{abbr}");
        $stage->display_left($msg);
        $thr->mq_add_sub_allsys('CPIC', 'prod_preremove_sys',$cpic->{prod});
        $thr->mq_wait_for_completion();
        $thr->mq_read_rtns();
        $stage->display_right($completemsg);
    }

    $prod->preremove_errors($cpic) if ($cpic->sys_error); 

    for $patchi(@{$cpic->{uninstallpatches}}) {
        $patch=$cpic->patch($patchi);
        $msg=Msg::new("Uninstalling $patch->{showform} patch", 30, 1284, "$patch->{showform}");
        $stage->display_left($msg);
        $cpic->mq_add_uninstall_subs($thr, $patchi);
        $thr->mq_wait_for_completion();
        $rtns=$thr->mq_read_rtns('01','CPIC::patch_uninstall_passed_sys', 
                                       'CPIC::patch_uninstall_failed_sys', $patch);
        # if failures happen on any machine, print 'Failed'
        $msg=$thr->mq_check_rtns($rtns) ? $passmsg : $failmsg;
        $stage->display_right($msg);
    }

    if ($prod->can(upgrade_sub('postremove_sys'))) {
        $msg=Msg::new("Performing $prod->{abbr} postremove tasks", 30, 1285, "$prod->{abbr}");
        $stage->display_left($msg);
        $thr->mq_add_sub_allsys('CPIC', 'prod_postremove_sys', 
                   $cpic->{prod});
        $thr->mq_wait_for_completion();
        $thr->mq_read_rtns();
        $stage->display_right($completemsg);
    }

    $prod->postremove_errors($cpic) if ($cpic->sys_error);

    $thr->mq_join_threads();
    return '';
}

# can't use $thr->tsub("mq_add_sub_allsys") as patch uninstall is conditionaL
sub mq_add_uninstall_subs {
    my($cpic,$patchi,$sys,$tid,$thr,$rel_vers);
    ($cpic,$thr,$patchi)=(@_);
    $rel_vers=$cpic->rel->{vers};
    for $sys(@{$cpic->{systems}}) {
        $tid++;
        $thr->mq_add_sub($tid, 'CPIC', 'patch_uninstall_sys', $sys->{sys}, $patchi, $rel_vers)
            if (EDRu::inarr($patchi, @{$sys->{uninstallpatches}}));
    }
}

sub patch_uninstall_sys {
    my ($cpic,$sys,$patch,$rel_vers)=(@_);
    my ($begintime,$clocktime,$duration);
    # scalars must be passed to threaded call
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $patch=$sys->patch($patch) unless (ref($patch));
    return 1 if (Cfg::opt('makeresponsefile'));
    Msg::log("BEGIN-patch_uninstall_sys($patch->{patch_vers},$sys->{sys})");
    $begintime=time();
    # $patch->tsub("preremove_sys", $sys);
    $patch->uninstall_sys($sys);
    my $uof=EDRu::outputfile('uninstall', $sys->{sys}, "patch.$patch->{patch_vers}");
    $sys->copy_to_sys($cpic->localsys,$uof) unless ($sys->{islocal});
    $patch->postremove_sys($sys);
    Msg::log("END-patch_uninstall_sys($patch->{patch_vers},$sys->{sys})");
    # Adding temp failures to test for web script
    $duration=time()-$begintime;
    $clocktime=EDRu::clocktime($duration);
    Msg::log("Uninstall patch $patch->{patch_vers} on $sys->{sys} took $duration seconds($clocktime)");

    return 0 if ($cpic->false_failure);
    return 1 if ($patch->uninstall_success_sys($sys, $rel_vers));
    return 0;
}

sub patch_uninstall_failed_sys {
    my($sys,$patch)=(@_);
    push(@{$sys->{patchuninstallfail}}, $patch->{patch_vers});
    my $pdfr=Msg::get('pdfr');
    my $io=EDRu::readfile(EDRu::outputfile('uninstall', $sys->{sys}, "patch.$patch->{patch_vers}"));
    my $msg=Msg::new("$patch->{patch_vers} $pdfr failed to uninstall on $sys->{sys}", 30, 1286, "$patch->{patch_vers}", "$pdfr", "$sys->{sys}");
    $msg->log;
    Msg::log("$io");
    push(@{$sys->{uninstallfailmsg}}, $msg);
    EDRu::appendfile($patch->{patch_vers}, 
        EDR::tmpdir()."/patch_uninstall_fail.$sys->{sys}") if (Cfg::opt('bait'));
    return '';
}

sub patch_uninstall_passed_sys {
    my($sys,$patch)=(@_);
    push(@{$sys->{patchuninstallpass}}, $patch->{patch_vers});
    my $pdfr=Msg::get('pdfr');
    Msg::log("$patch->{patch_vers} $pdfr uninstalled successfully on $sys->{sys}");
    EDRu::appendfile($patch->{patch_vers}, 
        EDR::tmpdir()."/patch_uninstall_pass.$sys->{sys}") if (Cfg::opt('bait'));
    return '';
}

1;
