#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
package CPIC;
use strict;

sub shutdown {
    my $cpic=shift;
    my ($failures,$edr,$cfg,$msg,$failmsg,$prod,$steps,$web);
    return '' if (Cfg::opt('makeresponsefile') && !($cpic->{prod}));

    $edr=Obj::edr();
    $prod=$cpic->prod;
    $cfg=Obj::cfg();

    # if fresh install + configure, then no need to shutdown.
    # for upsell install, if user choose to configure after upsell, and choose to re-configure VCS,
    # need to go through shutdown process
    return '' if (($cpic->{task} eq 'install') && Cfg::opt('configure') && !$cfg->{reconfigurevcs});
    return '' if ($prod->{lp});
    return '' if ($prod->donotreconfig());

    # define the product stop procs lists for all systems and per-system
    if (Cfg::opt('rootpath')) {
        # only 1 step for Live Upgrade.
        $steps=1;
    } else {
        $cpic->{stopprocs}=$prod->stopprocs();
        for my $sys (@{$cpic->{systems}}) {
            $sys->{stopprocs}=$prod->stopprocs_sys($sys);
        }
        $steps=$cpic->stop_processes_steps();
    }
    return if (!$steps); # not a partial upgrade

    $prod->pre_shutdown_question();

    unless (Cfg::opt('rootpath')) {
        if(Cfg::opt('upgrade_kernelpkgs')) {
            $cpic->rel->ru_switch_groups($cpic->{systems});
        }
        $edr->exit_noexitfile() if ($cpic->cli_ask_stop_processes($prod));
    }
    
    $web = Obj::web();
    $web->web_script_form('stopprocess') if (Obj::webui());
    $edr->set_progress_steps($steps);

    Msg::title();

    Cfg::opt('serial') ? $cpic->stop_processes_serial() :
                         $cpic->stop_processes_threaded();

    # do we need $cpic->reboot_messages here?
    if (Cfg::opt('rootpath')) {
        $msg=Msg::new("$prod->{name} configuration backup completed successfully", 30, 1287, "$prod->{name}");
        $failmsg=Msg::new("$prod->{name} configuration backup did not complete successfully", 30, 1288, "$prod->{name}");
    } else {
        $msg=Msg::new("$prod->{name} Shutdown completed successfully", 30, 1289, "$prod->{name}");
        $failmsg=Msg::new("$prod->{name} Shutdown did not complete successfully", 30, 1290, "$prod->{name}");
    }
    $failures=$cpic->failures('stopfailmsg');
    $msg->display_completion($failures, $msg, $failmsg);
    if ($#$failures>=0) {
        EDR::die('stop failures') if (Cfg::opt('startstoptest'));
        $cpic->ask_reboot_if_fatal_stop_failures();
    }
    $cpic->update_summary_stop();
    return '';
}

sub cli_ask_stop_processes {
    my ($cpic, $prod) = @_;
    return ($cpic->ask_stop_processes($prod));
}

sub stop_processes_steps {
    my $cpic=shift;
    my ($prod,$steps);

    $prod=$cpic->prod;
    $cpic->{falsefailures}=0;
    if (Cfg::opt('serial')) {
        $steps=0;
        for my $sys (@{$cpic->{systems}}) {
            next if ((Cfg::opt('uninstall')) && (!$sys->{uninstallpkgs}));
            $steps+=scalar(@{$sys->{stopprocs}}) if ($sys->{stopprocs});
        }
        return 0 if ($steps==0);
        $steps+=scalar(@{$cpic->{systems}}) if ($prod->can(upgrade_sub('prestop_sys')));
        $steps+=scalar(@{$cpic->{systems}}) if ($prod->can(upgrade_sub('poststop_sys')));
    } else {
        $cpic->{stopprocs}=$cpic->verify_list('stopprocs');
        $steps=scalar(@{$cpic->{stopprocs}});
        return 0 if ($steps==0);
        $steps++ if ($prod->can(upgrade_sub('prestop_sys')));
        $steps++ if ($prod->can(upgrade_sub('poststop_sys')));
    }
    return $steps;
}

sub stop_processes_serial {
    my $cpic=shift;
    my ($failmsg,$passmsg,$completemsg,$msg,$prod,$proc,$stage,$tmpsub);

    $prod=$cpic->prod;

    $failmsg=Msg::new("Failed", 30, 1203);
    $passmsg=Msg::new("Done", 30, 1204);
    $completemsg=Msg::new("Done", 30, 1204);

    if (Cfg::opt('rootpath')) {
        $stage = Msg::new("Backing up $prod->{abbr} configuration", 30, 1291, "$prod->{abbr}");
    } else {
        $stage = Msg::new("Stopping $prod->{abbr}", 30, 1292, "$prod->{abbr}");
    }
    $stage->log();

    if ($prod->can(upgrade_sub('prestop_sys'))) {
        for my $sys (@{$cpic->{systems}}) {
            $msg=Msg::new("Performing $prod->{abbr} prestop tasks on $sys->{sys}", 30, 1293, "$prod->{abbr}", "$sys->{sys}");
            $stage->display_left($msg);
            $tmpsub = upgrade_sub('prestop_sys');
            $prod->$tmpsub($sys);
            $stage->display_right($completemsg);
        }
    }

    if ($cpic->sys_error) {
        $prod->prestop_errors($cpic);
        # clean the screen
        Msg::title();
    }
    EDR::reset_errors_warnings();
    return '' if (Cfg::opt('rootpath'));

    for my $proci (@{$cpic->{stopprocs}}) {
        $proc=$cpic->proc($proci);
        for my $sys (@{$cpic->{systems}}) {
            next unless EDRu::inarr($proci,@{$sys->{stopprocs}});
            next if EDRu::inarr($proci,@{$sys->{donotstopprocs}});
            $msg=Msg::new("Stopping $proc->{proc} on $sys->{sys}", 30, 1294, "$proc->{proc}", "$sys->{sys}");
            $stage->display_left($msg);
            if ($cpic->proc_stop_sys($sys, $proc)) {
                $stage->display_right($passmsg);
                proc_stop_passed_sys($sys, $proc);
            } else {
                $stage->display_right($failmsg);
                proc_stop_failed_sys($sys, $proc);
            }
        }
    }

    if ($prod->can(upgrade_sub('poststop_sys'))) {
        for my $sys (@{$cpic->{systems}}) {
            $msg=Msg::new("Performing $prod->{abbr} poststop tasks on $sys->{sys}", 30, 1295, "$prod->{abbr}", "$sys->{sys}");
            $stage->display_left($msg);
            $tmpsub = upgrade_sub('poststop_sys');
            $prod->$tmpsub($sys);
            $stage->display_right($completemsg);
        }

        $prod->poststop_errors($cpic) if ($cpic->sys_error);
        EDR::reset_errors_warnings();
    }

    return '';
}

sub stop_processes_threaded {
    my $cpic=shift;
    my ($failmsg,$passmsg,$completemsg,$msg,$prod,$proc,$thr,$stage,$rtns);

    $prod=$cpic->prod;

    $failmsg=Msg::new("Failed", 30, 1203);
    $passmsg=Msg::new("Done", 30, 1204);
    $completemsg=Msg::new("Done", 30, 1204);

    if (Cfg::opt('rootpath')) {
        $stage = Msg::new("Backing up $prod->{abbr} configuration", 30, 1291, "$prod->{abbr}");
    } else {
        $stage = Msg::new("Stopping $prod->{abbr}", 30, 1292, "$prod->{abbr}");
    }
    $stage->log();

    $thr=Obj::thr();
    $thr->mq_create_threads($cpic->nsystems, $cpic->{threadlimit});

    if ($prod->can(upgrade_sub('prestop_sys'))) {
        $msg=Msg::new("Performing $prod->{abbr} prestop tasks", 30, 1296, "$prod->{abbr}");
        $stage->display_left($msg);
        $thr->mq_add_sub_allsys('CPIC', 'prod_prestop_sys');
        $thr->mq_wait_for_completion();
        $thr->mq_read_rtns();
        $stage->display_right($completemsg);
    }

    if ($cpic->sys_error) {
        $prod->prestop_errors($cpic);
        # clean the screen
        Msg::title();
    }
    EDR::reset_errors_warnings();
    return '' if (Cfg::opt('rootpath'));

    for my $proci (@{$cpic->{stopprocs}}) {
        $proc=$cpic->proc($proci);
        $msg=Msg::new("Stopping $proc->{proc}", 30, 1292, "$proc->{proc}");
        $stage->display_left($msg);
        $thr->mq_add_sub_allsys('CPIC', 'proc_stop_sys', $proci, 1);
        $thr->mq_wait_for_completion();
        $rtns=$thr->mq_read_rtns('01', 'CPIC::proc_stop_passed_sys',
                                       'CPIC::proc_stop_failed_sys', $proci, 1);
        # if failures happen on any machine, print 'Failed'
        $msg=$thr->mq_check_rtns($rtns) ? $passmsg : $failmsg;
        $stage->display_right($msg);
    }

    if ($prod->can(upgrade_sub('poststop_sys'))) {
        $msg=Msg::new("Performing $prod->{abbr} poststop tasks", 30, 1297, "$prod->{abbr}");
        $stage->display_left($msg);
        $thr->mq_add_sub_allsys('CPIC', 'prod_poststop_sys');
        $thr->mq_wait_for_completion();
        $thr->mq_read_rtns();
        $stage->display_right($completemsg);
    }

    $prod->poststop_errors($cpic) if ($cpic->sys_error);
    EDR::reset_errors_warnings();

    $thr->mq_join_threads();
    return '';
}

sub ask_stop_processes {
    my ($cpic,$prod) = @_;
    my ($stop_prod,$ayn,$help,$msg,$cfg);
    return '' if ($prod->{autostop} || Cfg::opt(qw(stop responsefile startstoptest)));
    $stop_prod=$prod->{stop_prod} || $prod->{abbr};
    $msg=Msg::new("All $stop_prod processes that are currently running must be stopped", 30, 1298, "$stop_prod");
    $msg->print;
    $msg=Msg::new("Do you want to stop $stop_prod processes now?", 30, 1299, "$stop_prod");
    $help=Msg::new("Enter 'y' if you want to stop processes, otherwise, enter 'n' and then installer will exit", 30, 1300);
    Msg::n();
    $ayn=$msg->ayny($help,0);
    Msg::n();
    $cfg = Obj::cfg();
    if ($ayn eq 'N') {
       if ($cfg->{reconfigurevcs}) {
           $msg=Msg::new("If you do not stop $stop_prod processes, you will lose all the changed configuration and installer will exit. Do you want to stop the processes now?", 30, 1301, "$stop_prod");
           $ayn=$msg->ayny($help,0);
           Msg::n();
           if ($ayn eq 'Y') {
               return '';
           }
       }
       if (Cfg::opt('uninstall')) {
           $msg=Msg::new("You have to stop processes if you want to uninstall $stop_prod.", 30, 1302, "$stop_prod");
           $msg->print;
           Msg::n();
       }
       return 1;
    }
    return '';
}

sub ask_reboot_if_fatal_stop_failures {
    my $cpic=shift;
    my (@systems,$systems,$msg,$edr,$ayn);

    # check whether any systems have fatal stop failures
    @systems=();
    for my $sys (@{$cpic->{systems}}) {
        if (defined $sys->{fatalstopfailprocs}) {
            push @systems, $sys->{sys};
        }
    }

    if (@systems && Cfg::opt(qw(patchupgrade upgrade rollback))) {
        $systems=join(' ', @systems);
        $msg=Msg::new("It is recommended to reboot the systems $systems to resolve the failures and then retry. If issues persist after reboot, contact Symantec technical support or refer to installation guide for further troubleshooting.\nDo you want to continue?", 30, 1303, "$systems");
        $ayn=$msg->aynn();
        $cpic->edr_completion() if ($ayn eq 'N');
    }
    return;
}

sub proc_stop_sys {
    my ($cpic,$sys,$proc,$checkproclist) = @_;
    my ($running,$rtn,$sleep,$edr);
    my ($begintime,$clocktime,$duration);
    $edr=Obj::edr();
    # scalars must be passed to threaded call
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $proc=$sys->proc($proc) unless (ref($proc));
    return 1 if (Cfg::opt('makeresponsefile'));
    # proc stop is conditional
    return 1 if ($checkproclist &&
        (!EDRu::inarr($proc->{proci}, @{$sys->{stopprocs}}) ||
        EDRu::inarr($proc->{proci},@{$sys->{donotstopprocs}})));

    $begintime=time();

    $edr->{logfile2}="$edr->{tmpdir}/stop."."$proc->{proc}."."$sys->{sys}";
    Msg::log("## Stopping $proc->{proc} on $sys->{sys}");
    # initial check
    Msg::log("## Begin prestop checking for $proc->{proc} on $sys->{sys}");
    $running=$proc->check_sys($sys, 'prestop');
    Msg::log("## The prestop checking return '$running'");
    if ($running) {
        # save process status
        $proc->set_value("running,$sys->{sys}",1);
        Msg::log("## $proc->{proc} is running on $sys->{sys}");
        Msg::log("## Begin stopping $proc->{proc} on $sys->{sys}");
        $rtn=$proc->stop_sys($sys);
        Msg::log("## The stop_sys() return '$rtn'");
        if ($rtn) {
            # confirm shutdown, may retry check for a defined period
            Msg::log("## Begin poststop checking for $proc->{proc} on $sys->{sys}");
            $running=$proc->check_sys($sys, 'poststop');
            Msg::log("## The poststop checking return '$running'");
            if ($running) {
                $sleep=0;
                while ($sleep<$proc->{stop_period}) {
                    $sleep+=10;
                    sleep 10;
                    Msg::log('## Sleep 10, and do poststop checking again');
                    if ($proc->{retry_stop}) {
                        Msg::log("## The previous stop failed, retry stopping $proc->{proc}");
                        $proc->stop_sys($sys);
                    }
                    $running=$proc->check_sys($sys, 'poststop');
                    Msg::log("## The poststop checking return '$running'");
                    last if (!$running);
                }
                if ($running && $proc->can('force_stop_sys')) {
                    Msg::log("## Begin force stop $proc->{proc} on $sys->{sys}");
                    $proc->force_stop_sys($sys);
                    $running=$proc->check_sys($sys, 'poststop');
                    Msg::log("## The poststop checking after force stop return '$running'");
                }
            }
            if ($running) {
                Msg::log("## $proc->{proc} is not successfully stopped on $sys->{sys}");
                # To show debug informations in stop_failed_sys().
                $proc->stop_failed_sys($sys);
                $sys->push_fatal_stop_proc($proc);
                $rtn=0;
            } else {
                Msg::log("## $proc->{proc} is successfully stopped on $sys->{sys}");
                $rtn=1;
            }
        } else {
            Msg::log('## The stop_sys() directly failed');
            $rtn=0;
        }
    } else {
        Msg::log("## $proc->{proc} is not running on $sys->{sys}");
        $rtn=1;
    }
    Msg::log("## The proc_stop_sys() return '$rtn'");
    $edr->{logfile2}='';

    $duration=time()-$begintime;
    $clocktime=EDRu::clocktime($duration);
    Msg::log("Stop $proc->{proc} on $sys->{sys} took $duration seconds($clocktime)");

    # adding temp failures to test the web script
    return 0 if ($cpic->false_failure);
    return $rtn;
}

# $cpic ref not passed when called through $thr->mq_read_rtns
sub proc_stop_failed_sys {
    my($sys,$proc)= @_;
    $proc=$sys->proc($proc) unless (ref($proc));
    push(@{$sys->{stopfail}}, $proc->{proc});
    $sys->{proc_reboot}=1 if (!$proc->{noreboot_failstop});
    my $msg=Msg::new("$proc->{proc} failed to stop on $sys->{sys}", 30, 1304, "$proc->{proc}", "$sys->{sys}");
    $msg->log;
    push(@{$sys->{stopfailmsg}}, $msg);
    EDRu::appendfile($proc->{proc}, EDR::tmpdir()."/stop_fail.$sys->{sys}")
        if (Cfg::opt('bait'));
    return '';
}

sub proc_stop_passed_sys {
    my($sys,$proc,$checkproclist)= @_;
    $proc=$sys->proc($proc) unless (ref($proc));
    # proc stop is conditional
    return '' if ($checkproclist &&
        (!EDRu::inarr($proc->{proci}, @{$sys->{stopprocs}}) ||
        EDRu::inarr($proc->{proci},@{$sys->{donotstopprocs}})));
    push(@{$sys->{stoppass}}, $proc->{proc});
    Msg::log("$proc->{proc} stopped successfully on $sys->{sys}");
    EDRu::appendfile($proc->{proc}, EDR::tmpdir()."/stop_pass.$sys->{sys}")
        if (Cfg::opt('bait'));
    return '';
}

sub prod_prestop_sys {
    my ($cpic,$sys) = @_;
    my ($tmpsub);
    # scalars must be passed to threaded call
    return '' if (Cfg::opt('makeresponsefile'));
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $tmpsub = upgrade_sub('prestop_sys');
    $sys->prod->$tmpsub($sys);
    return;
}

sub prod_poststop_sys {
    my ($cpic,$sys) = @_;
    my ($tmpsub);
    # scalars must be passed to threaded call
    return '' if (Cfg::opt('makeresponsefile'));
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $tmpsub = upgrade_sub('poststop_sys');
    $sys->prod->$tmpsub($sys);
    return;
}

sub update_summary_stop {
    my $cpic=shift;
    my $msg;

    for my $sys (@{$cpic->{systems}}) {
        if ($#{$sys->{stoppass}}>=0) {
            $msg=Msg::new("The following processes were stopped on $sys->{sys}:", 30, 1305, "$sys->{sys}");
            $cpic->update_summary($msg, @{$sys->{stoppass}});
        }
        if ($#{$sys->{stopfail}}>=0) {
            $msg=Msg::new("The following processes failed to stop on $sys->{sys}:", 30, 1306, "$sys->{sys}");
            $cpic->update_summary($msg, @{$sys->{stopfail}});
        }
    }
    return '';
}
1;
