#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
package CPIC;
use strict;

sub startup {
    my $cpic=shift;
    my ($failures,$msg,$failmsg,$prod,$edr,$web,$steps,$reboot_systems);
    return '' if (Cfg::opt('makeresponsefile') && !($cpic->{prod}));
    #return "" if (Cfg::opt(qw(rootpath)));

    $prod=$cpic->prod;
    return '' if ($prod->{lp});

    $reboot_systems=$cpic->systems_need_reboot();
    $cpic->{reboot}=1 if (@{$reboot_systems} || Cfg::opt('rootpath'));

    # define the product start proc lists for all systems and per-system
    if ($cpic->{reboot} || (Cfg::opt("nostart")) || (Cfg::opt(qw/upgrade patchupgrade/) && ($cpic->{prod_not_enabled_before_upgrade} || $cpic->{prod_not_running_before_upgrade}))) {
        # only 1 step for configuration.
        $steps=1;
    } else {
        $cpic->{startprocs}=$prod->startprocs();
        for my $sys (@{$cpic->{systems}}) {
            $sys->{startprocs}=$prod->startprocs_sys($sys);
        }
        $steps = $cpic->start_processes_steps();
    }
    return if (!$steps);

    $web = Obj::web();
    $edr = Obj::edr();
    $web->web_script_form('startprocess') if (Obj::webui());
    $edr->set_progress_steps($steps);

    Msg::title();

    Cfg::opt('serial') ? $cpic->start_processes_serial() :
                         $cpic->start_processes_threaded();

    if ($cpic->{reboot} || (Cfg::opt("nostart")) || (Cfg::opt(qw/upgrade patchupgrade/) && ($cpic->{prod_not_enabled_before_upgrade} || $cpic->{prod_not_running_before_upgrade}))) {
        $msg=Msg::new("$prod->{name} Configure completed successfully", 30, 1307, "$prod->{name}");
        $failmsg=Msg::new("$prod->{name} Configure did not complete successfully", 30, 1308, "$prod->{name}");
        $msg=$failmsg if ($prod->{has_poststart_config} &&
            (Cfg::opt('configure')));
    } else {
        $msg=Msg::new("$prod->{name} Startup completed successfully", 30, 1309, "$prod->{name}");
        $failmsg=Msg::new("$prod->{name} Startup did not complete successfully", 30, 1310, "$prod->{name}");
    }
    $failures=$cpic->failures('startfailmsg');
    $msg->display_completion($failures, $msg, $failmsg);
    $edr->display_notes();
    EDR::die('start failures') if ((Cfg::opt('startstoptest')) &&
                                   ($#$failures>=0));
    $cpic->update_summary_start();
    return '';
}

sub start_processes_steps {
    my $cpic=shift;
    my ($proc,$prod,$steps);
    $steps=0;
    $prod=$cpic->prod;
    $cpic->{falsefailures}=0;
    if (Cfg::opt('serial')) {
        for my $sys (@{$cpic->{systems}}) {
            if ($sys->{startprocs}) {
                $steps+=scalar(@{$sys->{startprocs}});
                for my $proci (@{$sys->{startprocs}}) {
                    $proc=$cpic->proc($proci);
                    $steps-- if ($proc->{donotstart});
                }
            }
            $steps++ if ((Cfg::opt('configure')) &&
                         ($prod->can('configure_sys')));
            $steps++ if ((Cfg::opt('upgrade')) &&
                         ($prod->can('upgrade_configure_sys')));
            $steps++ if ((Cfg::opt('patchupgrade')) && $cpic->{reuse_config}
                            && ($prod->can(upgrade_sub('configure_sys'))));
            $steps++ if ((Cfg::opt('patchupgrade')) && !$cpic->{reuse_config}
                            && ($prod->can('configure_sys')));
            $steps++ if ($prod->can(upgrade_sub('poststart_sys')));
        }
    } else {
        $cpic->{startprocs}=$cpic->verify_list('startprocs');
        $steps+=scalar(@{$cpic->{startprocs}}) if ($cpic->{startprocs});
        for my $proci (@{$cpic->{startprocs}}) {
            $proc=$cpic->proc($proci);
            $steps-- if ($proc->{donotstart});
        }
        $steps++ if ((Cfg::opt('configure')) && ($prod->can('configure_sys')));
        $steps++ if ((Cfg::opt('upgrade')) &&
                     ($prod->can('upgrade_configure_sys')));
        $steps++ if ((Cfg::opt('patchupgrade')) && $cpic->{reuse_config}
                        && ($prod->can(upgrade_sub('configure_sys'))));
        $steps++ if ((Cfg::opt('patchupgrade')) && !$cpic->{reuse_config}
                        && ($prod->can('configure_sys')));
        $steps++ if ($prod->can(upgrade_sub('poststart_sys')));
    }
    return $steps;
}

sub start_processes_serial {
    my $cpic=shift;
    my ($failmsg,$passmsg,$completemsg,$abortmsg,$msg,$prod,$proc,$reboot_systems,$stage,$tmpsub,$rtn);

    $failmsg=Msg::new("Failed", 30, 1203);
    $passmsg=Msg::new("Done", 30, 1204);
    $completemsg=Msg::new("Done", 30, 1204);
    $abortmsg=Msg::new("Aborted", 30, 1311);

    $prod=$cpic->prod;
    if ($cpic->{reboot} || (Cfg::opt(qw/upgrade patchupgrade/) && ($cpic->{prod_not_enabled_before_upgrade} || $cpic->{prod_not_running_before_upgrade}))) {
        $stage = Msg::new("Configuring $prod->{abbr}", 30, 1312, "$prod->{abbr}");
    } else {
        $stage = Msg::new("Starting $prod->{abbr}", 30, 1313, "$prod->{abbr}");
    }
    $stage->log();

    if (Cfg::opt(qw(upgrade patchupgrade))) {
        for my $sys (@{$cpic->{systems}}) {
            # only support reusing configure when patchupgrade
            if ((Cfg::opt('upgrade') || $cpic->{reuse_config}) &&
                ($prod->can(upgrade_sub('configure_sys')))) {
                # if upgrade scenario or product already configured when patchupgrade
                $msg=Msg::new("Performing $prod->{abbr} upgrade configuration on $sys->{sys}", 30, 1314, "$prod->{abbr}", "$sys->{sys}");
                $stage->display_left($msg);
                $tmpsub = upgrade_sub('configure_sys');
                $prod->$tmpsub($sys);
            } elsif ($prod->can('configure_sys')) {
                # which means product not configured when patchupgrade
                $msg=Msg::new("Performing $prod->{abbr} configuration on $sys->{sys}", 30, 1315, "$prod->{abbr}", "$sys->{sys}");
                $stage->display_left($msg);
                $prod->configure_sys($sys);
            }
            $stage->display_right($completemsg);
        }
    }

    if ((Cfg::opt('configure')) && ($prod->can('configure_sys'))) {
        for my $sys (@{$cpic->{systems}}) {
            $msg=Msg::new("Performing $prod->{abbr} configuration on $sys->{sys}", 30, 1315, "$prod->{abbr}", "$sys->{sys}");
            $stage->display_left($msg);
            $prod->configure_sys($sys);
            $stage->display_right($completemsg);
        }
    }

    $prod->config_errors($cpic) if ($cpic->sys_error);
    EDR::reset_errors_warnings();

    unless ($cpic->{reboot} || (Cfg::opt("nostart")) || (Cfg::opt(qw/upgrade patchupgrade/) && ($cpic->{prod_not_enabled_before_upgrade} || $cpic->{prod_not_running_before_upgrade}))) {
        for my $proci (@{$cpic->{startprocs}}) {
            $proc=$cpic->proc($proci);
            next if $proc->{donotstart};
            for my $sys (@{$cpic->{systems}}) {
                next unless EDRu::inarr($proci,@{$sys->{startprocs}});
                next if EDRu::inarr($proci,@{$sys->{donotstartprocs}});
                $msg=Msg::new("Starting $proc->{proc} on $sys->{sys}", 30, 1316, "$proc->{proc}", "$sys->{sys}");
                $stage->display_left($msg);
                $rtn=$cpic->proc_start_sys($sys, $proc);
                if ($rtn) {
                    $stage->display_right($passmsg);
                    proc_start_passed_sys($sys, $proc);
                } elsif (defined $rtn) {
                    $stage->display_right($failmsg);
                    proc_start_failed_sys($sys, $proc);
                } else {
                    $stage->display_right($abortmsg);
                    proc_start_aborted_sys($sys, $proc);
                }
            }
        }

        $reboot_systems=$cpic->systems_need_reboot();
        unless (@{$reboot_systems}) {
            if ($prod->can(upgrade_sub('poststart_sys'))) {
                for my $sys (@{$cpic->{systems}}) {
                    $msg=Msg::new("Performing $prod->{abbr} poststart tasks on $sys->{sys}", 30, 1317, "$prod->{abbr}", "$sys->{sys}");
                    $stage->display_left($msg);
                    $cpic->prod_poststart_sys($sys);
                    $stage->display_right($completemsg);
                }
            }
        }

        $prod->poststart_errors($cpic) if ($cpic->sys_error);
        EDR::reset_errors_warnings();
    }
    return '';
}

sub start_processes_threaded {
    my $cpic=shift;
    my ($failmsg,$passmsg,$completemsg,$msg,$partialabortmsg,$abortmsg,$abortstate);
    my ($prod,$proc,$thr,$reboot_systems,$logfileid,$edr,$rtns,$rtn,$success,$stage,$tmpsub);

    $prod=$cpic->prod;
    $thr=Obj::thr();
    $edr=Obj::edr();
    if (($cpic->{reboot}) || (Cfg::opt('rootpath')) || (Cfg::opt("nostart")) || (Cfg::opt(qw/upgrade patchupgrade/) && ($cpic->{prod_not_enabled_before_upgrade} || $cpic->{prod_not_running_before_upgrade}))) {
        $stage = Msg::new("Configuring $prod->{abbr}", 30, 1312, "$prod->{abbr}");
    } else {
        $stage = Msg::new("Starting $prod->{abbr}", 30, 1313, "$prod->{abbr}");
    }
    $stage->log();

    $failmsg=Msg::new("Failed", 30, 1203);
    $passmsg=Msg::new("Done", 30, 1204);
    $completemsg=Msg::new("Done", 30, 1204);
    $partialabortmsg=Msg::new("Partially Aborted", 30, 1318);
    $abortmsg=Msg::new("Aborted", 30, 1311);

    $thr->mq_create_threads($cpic->nsystems, $cpic->{threadlimit});

    if (Cfg::opt(qw(upgrade patchupgrade rollback))) {
        if ((Cfg::opt('upgrade') || $cpic->{reuse_config})
            && ($prod->can(upgrade_sub('configure_sys')))) {
            if (Cfg::opt('rollback')) {
                $msg=Msg::new("Performing $prod->{abbr} configuration", 30, 1319, "$prod->{abbr}");
            } else {
                $msg=Msg::new("Performing $prod->{abbr} upgrade configuration", 30, 1320, "$prod->{abbr}");
            }
            $stage->display_left($msg);
            $thr->mq_add_sub_allsys('CPIC', 'prod_upgrade_configure_sys');
            $thr->mq_wait_for_completion();
            $thr->mq_read_rtns();
        } elsif ($prod->can('configure_sys')) {
            $msg=Msg::new("Performing $prod->{abbr} configuration", 30, 1319, "$prod->{abbr}");
            $stage->display_left($msg);
            $thr->mq_add_sub_allsys('CPIC', 'prod_configure_sys');
            $thr->mq_wait_for_completion();
            $thr->mq_read_rtns();
        }
        $stage->display_right($completemsg);
    }

    if ((Cfg::opt('configure')) && ($prod->can('configure_sys'))) {
        $msg=Msg::new("Performing $prod->{abbr} configuration", 30, 1319, "$prod->{abbr}");
        $stage->display_left($msg);
        $thr->mq_add_sub_allsys('CPIC', 'prod_configure_sys');
        $thr->mq_wait_for_completion();
        $thr->mq_read_rtns();
        $stage->display_right($completemsg);
    }

    $prod->config_errors($cpic) if ($cpic->sys_error);
    return '' if (Cfg::opt(qw(rootpath)));

    unless ($cpic->{reboot} || (Cfg::opt("nostart")) || (Cfg::opt(qw/upgrade patchupgrade/) && ($cpic->{prod_not_enabled_before_upgrade} || $cpic->{prod_not_running_before_upgrade}))) {
        for my $proci (@{$cpic->{startprocs}}) {
            $proc=$cpic->proc($proci);
            next if $proc->{donotstart};
            $msg=Msg::new("Starting $proc->{proc}", 30, 1313, "$proc->{proc}");
            $stage->display_left($msg);
            $success=1;
            $abortstate=$cpic->proc_abort_startup_status($proc);
            if (($#{$cpic->{systems}}>0) && ($proc->{multisystemserialstart})) {
                $logfileid=0;
                for my $sys (@{$cpic->{systems}}) {
                    $logfileid++;
                    next unless EDRu::inarr($proci,@{$sys->{startprocs}});
                    next if EDRu::inarr($proci,@{$sys->{donotstartprocs}});
                    $edr->{logfileid}=$logfileid;
                    $rtn=$cpic->proc_start_sys($sys, $proci);
                    $edr->{logfileid}=0;
                    if ($rtn) {
                        proc_start_passed_sys($sys, $proci);
                    } elsif (defined $rtn) {
                        proc_start_failed_sys($sys, $proci);
                        $success=0;
                    } else {
                        proc_start_aborted_sys($sys, $proci);
                    }
                }
            } else {
                $thr->mq_add_sub_allsys('CPIC', 'proc_start_sys', $proci, 1);
                $thr->mq_wait_for_completion();
                $rtns=$thr->mq_read_rtns('01u', 'CPIC::proc_start_passed_sys',
                                                'CPIC::proc_start_failed_sys',
                                                'CPIC::proc_start_aborted_sys', $proci, 1);
                # if failures happen on any machine, print 'Failed'
                $success=0 if (!$thr->mq_check_rtns($rtns));
            }
            if ($abortstate==2) {
                $msg=$abortmsg;
            } elsif ($abortstate==1) {
                $msg=$partialabortmsg;
            } else {
                $msg=$success ? $passmsg : $failmsg;
            }
            $stage->display_right($msg);
        }

        $reboot_systems=$cpic->systems_need_reboot();
        unless (@{$reboot_systems}) {
            if ($prod->can(upgrade_sub('poststart_sys'))) {
                $msg=Msg::new("Performing $prod->{abbr} poststart tasks", 30, 1321, "$prod->{abbr}");
                $stage->display_left($msg);
                if (($#{$cpic->{systems}}>0) && ($prod->{multisystemserialpoststart})) {
                    $logfileid=0;
                    for my $sys (@{$cpic->{systems}}) {
                        $logfileid++;
                        $edr->{logfileid}=$logfileid;
                        $cpic->prod_poststart_sys($sys);
                        $edr->{logfileid}=0;
                    }
                } else {
                    $thr->mq_add_sub_allsys('CPIC', 'prod_poststart_sys');
                    $thr->mq_wait_for_completion();
                    $thr->mq_read_rtns();
                }
                $stage->display_right($completemsg);
            }

            $prod->poststart_errors($cpic) if ($cpic->sys_error);
        }
    }

    $thr->mq_join_threads();
    return;
}

#sub ask_start_processes {
#    my($ayn,$cpic,$prod,$help,$msg);
#    ($cpic,$prod)=@_;
#    return "" if ($prod->{autostart} || Cfg::opt(qw(start responsefile)));
#    $msg=Msg::new("Do you want to start $prod->{abbr} processes now?");
#    $help=Msg::new("Enter 'y' if want to start processes, otherwise, enter 'n'");
#    Msg::n();
#    $ayn=$msg->ayny($help,0);
#    return 1 if ($ayn eq "N");
#    return "";
#}

sub prod_poststart_sys {
    my ($cpic,$sys) = @_;
    my ($tmpsub);
    return '' if (Cfg::opt('makeresponsefile'));
    # scalars must be passed to threaded call
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $tmpsub = upgrade_sub('poststart_sys');
    $sys->prod->$tmpsub($sys);
    return;
}

# not using OO call, $cpic not needed and for thread/serial consistency
sub proc_start_sys {
    my ($cpic,$sys,$proc,$checkproclist) = @_;
    my ($sleep,$edr,$running,$rtn,$msg);
    my ($begintime,$clocktime,$duration);
    $edr=Obj::edr();
    # scalars must be passed to threaded call
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $proc=$sys->proc($proc) unless (ref($proc));
    return 1 if (Cfg::opt('makeresponsefile'));
    # proc start is conditional
    return 1 if ($checkproclist &&
        (!EDRu::inarr($proc->{proci}, @{$sys->{startprocs}}) ||
         EDRu::inarr($proc->{proci},@{$sys->{donotstartprocs}})));

    $edr->{logfile2}="$edr->{tmpdir}/start."."$proc->{proc}."."$sys->{sys}";

    if ($sys->{abort_startup} ||
        EDRu::inarr($proc->{proc}, @{$sys->{abort_procs}})) {
        Msg::log("## Aborted to start $proc->{proc} on $sys->{sys}");
        return;
    }

    $begintime=time();

    Msg::log("## Starting $proc->{proc} on $sys->{sys}");
    # initial check
    Msg::log("## Begin prestart checking for $proc->{proc} on $sys->{sys}");
    $running=$proc->check_sys($sys, 'prestart');
    Msg::log("## The prestart checking return '$running'");
    if ($running) {
        Msg::log("## $proc->{proc} is already running on $sys->{sys}");
        $rtn=1;
    } else {
        # start the process
        Msg::log("## Begin starting $proc->{proc} on $sys->{sys}");
        $rtn=$proc->start_sys($sys);
        Msg::log("## The start_sys() return '$rtn'");
        if ($rtn) {
            # confirm startup, may retry check for a defined period
            Msg::log("## Begin poststart checking for $proc->{proc} on $sys->{sys}");
            $running=$proc->check_sys($sys, 'poststart');
            Msg::log("## The poststart checking return '$running'");
            if (!$running) {
                $sleep=0;
                while ($sleep<$proc->{start_period}) {
                    $sleep+=10;
                    sleep 10;
                    Msg::log('## Sleep 10, and do poststart checking again');
                    $running=$proc->check_sys($sys, 'poststart');
                    Msg::log("## The poststart checking return '$running'");
                    last if ($running);
                }
            }
            if ($running) {
                Msg::log("## $proc->{proc} is successfully started on $sys->{sys}");
                $rtn=1;
            } else {
                Msg::log("## $proc->{proc} is not successfully started on $sys->{sys}");
                $rtn=0;
            }
        } else {
            Msg::log('## The start_sys() directly failed');
            $rtn=0;
        }

        if ($rtn==0) {
            # Start failed.
            # To show debug informations in start_failed_sys().
            $proc->start_failed_sys($sys);

            # abort those processes which depend on this proc.
            $sys->set_value('abort_startup', 1) if ($proc->{abort_startup});
            if (defined $proc->{abort_procs}) {
                $sys->set_value('abort_procs', 'push', @{$proc->{abort_procs}});
            }
        }
    }

    Msg::log("## The proc_start_sys() return '$rtn'");
    $edr->{logfile2}='';

    $duration=time()-$begintime;
    $clocktime=EDRu::clocktime($duration);
    Msg::log("Startup $proc->{proc} on $sys->{sys} took $duration seconds($clocktime)");

    # adding temp failures to test the web script
    return 0 if ($cpic->false_failure);
    return $rtn;
}

# $cpic ref not passed when called through $thr->mq_read_rtns
sub proc_start_failed_sys {
    my($sys,$proc)= @_;
    $proc=$sys->proc($proc) unless (ref($proc));
    push(@{$sys->{startfail}}, $proc->{proc});
    $sys->{proc_reboot}=1 if (!$proc->{noreboot_failstart});
    my $msg=Msg::new("$proc->{proc} failed to start on $sys->{sys}", 30, 1322, "$proc->{proc}", "$sys->{sys}");
    $msg->log;
    push(@{$sys->{startfailmsg}}, $msg);
    EDRu::appendfile($proc->{proc}, EDR::tmpdir()."/start_fail.$sys->{sys}")
        if (Cfg::opt('bait'));
    return '';
}

sub proc_start_passed_sys {
    my($sys,$proc,$checkproclist)= @_;
    $proc=$sys->proc($proc) unless (ref($proc));
    # proc start is conditional
    return '' if ($checkproclist &&
        (!EDRu::inarr($proc->{proci}, @{$sys->{startprocs}}) ||
        EDRu::inarr($proc->{proci},@{$sys->{donotstartprocs}})));
    push(@{$sys->{startpass}}, $proc->{proc});
    Msg::log("$proc->{proc} started successfully on $sys->{sys}");
    EDRu::appendfile($proc->{proc}, EDR::tmpdir()."/start_pass.$sys->{sys}")
        if (Cfg::opt('bait'));
    return '';
}

sub proc_start_aborted_sys {
    my($sys,$proc)= @_;
    $proc=$sys->proc($proc) unless (ref($proc));
    push(@{$sys->{startfail}}, $proc->{proc});
    $sys->{proc_reboot}=1 if (!$proc->{noreboot_failstart});
    my $msg=Msg::new("$proc->{proc} aborted to start on $sys->{sys}", 30, 1323, "$proc->{proc}", "$sys->{sys}");
    $msg->log;
    push(@{$sys->{startfailmsg}}, $msg);
    EDRu::appendfile($proc->{proc}, EDR::tmpdir()."/start_fail.$sys->{sys}")
        if (Cfg::opt('bait'));
    return '';
}

sub startstoptest {
    my $cpic=shift;
    my $n=0;
    while(1) {
        $n++;
        Msg::print("startstoptest $n:");
        $cpic->startup();
        $cpic->shutdown();
    }
    return;
}

# Return 0 meaning no abort.
# Return 1 meaning the proc is partially aborted on some systems.
# Return 2 meaning the proc is aborted on all systems.
sub proc_abort_startup_status {
    my ($cpic,$proc)=@_;
    my ($num,$nsystems);

    $num=0;
    $nsystems=scalar @{$cpic->{systems}};
    for my $sys (@{$cpic->{systems}}) {
        if ($sys->{abort_startup} ||
            EDRu::inarr($proc->{proc}, @{$sys->{abort_procs}})) {
            $num++;
        }
    }
    if ($num) {
        return 1 if ($num < $nsystems);
        return 2 if ($num == $nsystems);
    }
    return 0;
}

sub update_summary_start {
    my $cpic=shift;
    my $msg;

    for my $sys (@{$cpic->{systems}}) {
        if ($#{$sys->{startpass}}>=0) {
            $msg=Msg::new("The following processes were started on $sys->{sys}:", 30, 1324, "$sys->{sys}");
            $cpic->update_summary($msg, @{$sys->{startpass}});
        }
        if ($#{$sys->{startfail}}>=0) {
            $msg=Msg::new("The following processes failed to start on $sys->{sys}:", 30, 1325, "$sys->{sys}");
            $cpic->update_summary($msg, @{$sys->{startfail}});
        }
    }
    return '';
}

1;
