#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
package CPIC;
use strict;

sub uninstall {
    my $cpic=shift;
    my ($msg,$failmsg,$prod,$steps,$web);
    return '' if (Cfg::opt('makeresponsefile') && !($cpic->{prod}));

    $prod=$cpic->prod;
    $prod->preremove_tasks if $prod->can('preremove_tasks');
    $steps=$cpic->uninstall_systems_steps();
    return if (!$steps); # not a partial upgrade

    $web = Obj::web();
    my $edr = Obj::edr();
    $web->web_script_form('uninstalling') if (Obj::webui());
    $edr->set_progress_steps($steps);

    Msg::title();

    Cfg::opt('serial') ? $cpic->uninstall_systems_serial() :
                         $cpic->uninstall_systems_threaded();

    $msg=Msg::new("$prod->{name} Uninstall completed successfully", 30, 1368, "$prod->{name}");
    $failmsg=Msg::new("$prod->{name} Uninstall did not complete successfully", 30, 1369, "$prod->{name}");
    $msg->display_completion($cpic->failures('uninstallfailmsg'), $msg, $failmsg);
    $cpic->update_summary_uninstall();
    $cpic->ask_uninstall_continue();
    return '';
}

sub uninstall_systems_steps {
    my $cpic=shift;
    my ($rel,$prod,$steps,$sys);
    $rel=$cpic->rel;
    $prod=$cpic->prod;
    $cpic->{falsefailures}=0;
    $steps=0;
    $cpic->{uninstallbundles}=$cpic->verify_list('uninstallbundles');
    $cpic->{uninstallpkgs}=$cpic->verify_list('uninstallpkgs');
    return 0 unless (scalar(@{$cpic->{uninstallbundles}}) + scalar(@{$cpic->{uninstallpkgs}}));
    if (Cfg::opt('serial')) {
        for my $sys (@{$cpic->{systems}}) {
            $steps+=scalar(@{$sys->{uninstallbundles}}) if ($sys->{uninstallbundles});
            if ($sys->{uninstallpkgs}) {
                if ($rel->{batchuninstall}) { 
                    $steps++;
                } else {
                    $steps+=scalar(@{$sys->{uninstallpkgs}});
                }
            }
            $steps++ if (Cfg::opt('uninstall') && ((@{$sys->{uninstallbundles}}) || (@{$sys->{uninstallpkgs}})));
            $steps+=2; # for prod_preremove_sys and prod_postremove_sys
        }
    } else {
        $steps+=scalar(@{$cpic->{uninstallbundles}}) if ($cpic->{uninstallbundles});
        if ($cpic->{uninstallpkgs}) {
            if ($rel->{batchuninstall}) {
                $steps++;
            } else {
                $steps+=scalar(@{$cpic->{uninstallpkgs}});
            }
        }
        $steps++ if (Cfg::opt('uninstall') && ((@{$cpic->{uninstallbundles}}) || (@{$cpic->{uninstallpkgs}})));
        $steps+=2; # for prod_preremove_sys and prod_postremove_sys
    }
    return $steps;
}

sub uninstall_systems_serial {
    my $cpic=shift;
    my ($failmsg,$passmsg,$completemsg,$msg,$stage,$pass,$fail,$rel,$pkg,$prod,$pdfrs,$tmpsub);
    $rel=$cpic->rel;
    $prod=$cpic->prod;
    $stage = Msg::new("Uninstalling $prod->{abbr}", 30, 1279, "$prod->{abbr}");
    $stage->log();

    $failmsg=Msg::new("Failed", 30, 1203);
    $passmsg=Msg::new("Done", 30, 1204);
    $completemsg=Msg::new("Done", 30, 1204);

    for my $sys (@{$cpic->{systems}}) {
        $msg=Msg::new("Performing $prod->{abbr} preremove tasks on $sys->{sys}", 30, 1280, "$prod->{abbr}", "$sys->{sys}");
        $stage->display_left($msg);
        $cpic->prod_preremove_sys($sys);
        $stage->display_right($completemsg);
    }

    $prod->preremove_errors($cpic) if ($cpic->sys_error);
    EDR::reset_errors_warnings();

    for my $sys (@{$cpic->{systems}}) {
        for my $bundle (@{$cpic->{uninstallbundles}}) {
            next if (!EDRu::inarr($bundle, @{$sys->{uninstallbundles}}));
            $msg=Msg::new("Uninstalling $bundle bundle on $sys->{sys}", 30, 1370, "$bundle", "$sys->{sys}");
            $stage->display_left($msg);
            if ($cpic->bundle_uninstall_sys($sys, $bundle)) {
                $stage->display_right($passmsg);
                $pass++;
            } else {
                $stage->display_right($failmsg);
                $fail++;
            }
        }
        if ($rel->{batchuninstall}) {
            if ($sys->{uninstallpkgs}) {
                $pdfrs=Msg::get('pdfrs');
                $msg=Msg::new("Uninstalling $prod->{abbr} $pdfrs on $sys->{sys}", 30, 1371, "$prod->{abbr}", "$pdfrs", "$sys->{sys}");
                $stage->display_left($msg);
                if ($cpic->pkgs_uninstall_sys($sys)) {
                    $stage->display_right($passmsg);
                    $pass++;
                } else {
                    $stage->display_right($failmsg);
                    $fail++;
                }
            }
        } else {
            for my $pkgi (@{$cpic->{uninstallpkgs}}) {
                next if (!EDRu::inarr($pkgi, @{$sys->{uninstallpkgs}}));
                $pkg=$sys->pkg($pkgi);
                $msg=Msg::new("Uninstalling $pkg->{pkg} on $sys->{sys}", 30, 1372, "$pkg->{pkg}", "$sys->{sys}");
                $stage->display_left($msg);
                if ($cpic->pkg_uninstall_sys($sys, $pkg)) {
                    $stage->display_right($passmsg);
                    pkg_uninstall_passed_sys($sys, $pkg);
                    $pass++;
                } else {
                    $stage->display_right($failmsg);
                    pkg_uninstall_failed_sys($sys, $pkg);
                    $fail++;
                }
            }
        }
        if (Cfg::opt('uninstall') && ((@{$cpic->{uninstallbundles}}) || (@{$cpic->{uninstallpkgs}}))) {
            $msg=Msg::new("Removing install and uninstall scripts on $sys->{sys}", 30, 1373, "$sys->{sys}");
            $stage->display_left($msg);
            # only remove if there are no failures, but we still have
            # to go into this block to not screw up the step count
            if ($rel->{cpipkg}) {
                $cpic->readd_install_scripts_sys($sys) if (($pass) && (!$fail));
            } else {
                $cpic->cleanup_cpi_sys($sys) if (($pass) && (!$fail));
            }
            $stage->display_right($completemsg);
        }
    }

    for my $sys (@{$cpic->{systems}}) {
        $msg=Msg::new("Performing $prod->{abbr} postremove tasks on $sys->{sys}", 30, 1282, "$prod->{abbr}", "$sys->{sys}");
        $stage->display_left($msg);
        $cpic->prod_postremove_sys($sys);
        $stage->display_right($completemsg);
    }

    $prod->postremove_errors($cpic) if ($cpic->sys_error);
    EDR::reset_errors_warnings();

    return '';
}

sub uninstall_systems_threaded {
    my $cpic=shift;
    my ($failmsg,$passmsg,$completemsg,$msg,$rel,$pkg,$prod,$pdfrs,$sys,$stage,$thr,$rtns);
    $rel=$cpic->rel;
    $prod=$cpic->prod;
    $thr=Obj::thr();

    $failmsg=Msg::new("Failed", 30, 1203);
    $passmsg=Msg::new("Done", 30, 1204);
    $completemsg=Msg::new("Done", 30, 1204);

    $stage = Msg::new("Uninstalling $prod->{abbr}", 30, 1279, "$prod->{abbr}");
    $stage->log();
    $thr->mq_create_threads($cpic->nsystems, $cpic->{threadlimit});

    $msg=Msg::new("Performing $prod->{abbr} preremove tasks", 30, 1283, "$prod->{abbr}");
    $stage->display_left($msg);
    $thr->mq_add_sub_allsys('CPIC', 'prod_preremove_sys');
    $thr->mq_wait_for_completion();
    $thr->mq_read_rtns();
    $stage->display_right($completemsg);

    $prod->preremove_errors($cpic) if ($cpic->sys_error);
    EDR::reset_errors_warnings();

    for my $bundle (@{$cpic->{uninstallbundles}}) {
        $msg=Msg::new("Uninstalling $bundle bundle", 30, 1374, "$bundle");
        $stage->display_left($msg);
        $thr->mq_add_sub_allsys('CPIC', 'bundle_uninstall_sys', $bundle);
        $thr->mq_wait_for_completion();
        $rtns=$thr->mq_read_rtns();
        # if failures happen on any machine, print 'Failed'
        $msg=$thr->mq_check_rtns($rtns) ? $passmsg : $failmsg;
        $stage->display_right($msg);
    }

    if ($rel->{batchuninstall}) {
        if ($cpic->{uninstallpkgs}) {
            $pdfrs=Msg::get('pdfrs');
            $msg=Msg::new("Uninstalling $prod->{abbr} $pdfrs", 30, 1375, "$prod->{abbr}", "$pdfrs");
            $stage->display_left($msg);
            $thr->mq_add_sub_allsys('CPIC', 'pkgs_uninstall_sys');
            $thr->mq_wait_for_completion();
            $rtns=$thr->mq_read_rtns();
            $msg=$thr->mq_check_rtns($rtns) ? $passmsg : $failmsg;
            $stage->display_right($msg);
        }
    } else {
        for my $pkgi (@{$cpic->{uninstallpkgs}}) {
            $pkg=$cpic->pkg($pkgi);
            $msg=Msg::new("Uninstalling $pkg->{pkg}", 30, 1279, "$pkg->{pkg}");
            $stage->display_left($msg);
            $thr->mq_add_sub_allsys('CPIC', 'pkg_uninstall_sys', $pkgi);
            $thr->mq_wait_for_completion();
            $rtns=$thr->mq_read_rtns('01','CPIC::pkg_uninstall_passed_sys',
                                          'CPIC::pkg_uninstall_failed_sys', $pkgi);
            # if failures happen on any machine, print 'Failed'
            $msg=$thr->mq_check_rtns($rtns) ? $passmsg : $failmsg;
            $stage->display_right($msg);
        }
    }

    if (Cfg::opt('uninstall') && ((@{$cpic->{uninstallbundles}}) || (@{$cpic->{uninstallpkgs}}))) {
        $msg=Msg::new("Removing install and uninstall scripts", 30, 1376);
        $stage->display_left($msg);
        if ($rel->{'cpipkg'}) {
            $thr->mq_add_sub_allsys('CPIC', 'readd_install_scripts_sys');
        }else {
            $thr->mq_add_sub_allsys('CPIC', 'cleanup_cpi_sys');
        }
        $thr->mq_wait_for_completion();
        $thr->mq_read_rtns();
        $stage->display_right($completemsg);
    }

    $msg=Msg::new("Performing $prod->{abbr} postremove tasks", 30, 1285, "$prod->{abbr}");
    $stage->display_left($msg);
    $thr->mq_add_sub_allsys('CPIC', 'prod_postremove_sys');
    $thr->mq_wait_for_completion();
    $thr->mq_read_rtns();
    $stage->display_right($completemsg);

    $prod->postremove_errors($cpic) if ($cpic->sys_error);
    EDR::reset_errors_warnings();

    $thr->mq_join_threads();
    return '';
}

sub pkgs_uninstall_sys {
    my ($cpic,$sys)= @_;
    my ($uof,$pkg,$pkgs,$pkgis,$begintime,$clocktime,$duration);
    # scalars must be passed to threaded call
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    return 1 if (Cfg::opt('makeresponsefile'));

    $pkgs='';
    $pkgis='';
    for my $pkgi (@{$cpic->{uninstallpkgs}}) {
        next if (!EDRu::inarr($pkgi, @{$sys->{uninstallpkgs}}));
        $pkg=$sys->pkg($pkgi);
        $pkgs.=" $pkg->{pkg}";
        $pkgis.=" $pkgi";
    }
    return 1 unless ($pkgs);

    # uninstall packages batchly
    Msg::log("BEGIN-pkgs_uninstall_sys($sys->{sys},'$pkgs')");

    $begintime=time();

    $sys->uninstall_pkgs($pkgs);

    $uof=EDRu::outputfile('uninstall', $sys->{sys}, 'pkgs');
    $sys->copy_to_sys($cpic->localsys,$uof) unless ($sys->{islocal});
    Msg::log("END-pkgs_uninstall_sys($sys->{sys},'$pkgs')");

    # Adding temp failures to test for web script
    $duration=time()-$begintime;
    $clocktime=EDRu::clocktime($duration);
    Msg::log("Uninstall package '$pkgs' on $sys->{sys} took $duration seconds($clocktime)");

    return 1 if ($cpic->pkgs_uninstall_verify_sys($sys,$pkgis));
    return 0;
}
 
sub pkgs_uninstall_verify_sys {
    my ($cpic,$sys,$pkgis)= @_;
    my ($fails,$pkg,$pdfr,$msg);
    $pkgis=~s/^\s*//;
    $pkgis=~s/\s*$//;
    $fails=0;
    $pdfr=Msg::get('pdfr');
    for my $pkgi (split(/\s+/,$pkgis)) {
        $pkg=$sys->pkg($pkgi);
        if ($pkg->uninstall_success_sys($sys)) {
            $msg=Msg::new("$pkg->{pkg} $pdfr uninstalled successfully on $sys->{sys}", 30, 1377, "$pkg->{pkg}", "$pdfr", "$sys->{sys}");
            $msg->log;
            $sys->set_value('pkguninstallpass', 'push', $pkg->{pkg});
            EDRu::appendfile($pkg->{pkg}, EDR::tmpdir()."/pkg_uninstall_pass.$sys->{sys}")
                if (Cfg::opt('bait'));
        } else {
            $msg=Msg::new("$pkg->{pkg} $pdfr failed to uninstall on $sys->{sys}", 30, 1286, "$pkg->{pkg}", "$pdfr", "$sys->{sys}");
            $msg->log;
            $sys->set_value('uninstallfailmsg', 'push', $msg->{msg});
            $sys->set_value('pkguninstallfail', 'push', $pkg->{pkg});
            EDRu::appendfile($pkg->{pkg}, EDR::tmpdir()."/pkg_uninstall_fail.$sys->{sys}")
                if (Cfg::opt('bait'));
            $fails++;
        }
    }
    return 0 if ($fails);
    return 1;
}

sub pkg_uninstall_sys {
    my ($cpic,$sys,$pkg)= @_;
    my ($begintime,$clocktime,$duration);
    # scalars must be passed to threaded call
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $pkg=$sys->pkg($pkg) unless (ref($pkg));
    return 1 if (Cfg::opt('makeresponsefile'));
    # pkg uninstall is conditional
    return 1 if (!EDRu::inarr($pkg->{pkgi}, @{$sys->{uninstallpkgs}}));
    Msg::log("BEGIN-pkg_uninstall_sys($pkg->{pkg},$sys->{sys})");

    $begintime=time();

    $pkg->preremove_sys($sys);
    $pkg->uninstall_sys($sys);
    my $uof=EDRu::outputfile('uninstall', $sys->{sys}, $pkg->{pkg});
    $sys->copy_to_sys($cpic->localsys,$uof) unless ($sys->{islocal});
    $pkg->postremove_sys($sys);
    Msg::log("END-pkg_uninstall_sys($pkg->{pkg},$sys->{sys})");
    # Adding temp failures to test for web script

    $duration=time()-$begintime;
    $clocktime=EDRu::clocktime($duration);
    Msg::log("Uninstall package $pkg->{pkg} on $sys->{sys} took $duration seconds($clocktime)");

    return 0 if ($cpic->false_failure);
    return 1 if ($pkg->uninstall_success_sys($sys));
    return 0;
}

sub pkg_uninstall_failed_sys {
    my($sys,$pkg)= @_;
    $pkg=$sys->pkg($pkg) unless (ref($pkg));
    push(@{$sys->{pkguninstallfail}}, $pkg->{pkg});
    my $pdfr=Msg::get('pdfr');
    my $io=EDRu::readfile(EDRu::outputfile('uninstall', $sys->{sys}, $pkg->{pkg}));
    my $msg=Msg::new("$pkg->{pkg} $pdfr failed to uninstall on $sys->{sys}", 30, 1286, "$pkg->{pkg}", "$pdfr", "$sys->{sys}");
    $msg->log;
    Msg::log("$io");
    push(@{$sys->{uninstallfailmsg}}, $msg);
    EDRu::appendfile($pkg->{pkg},
        EDR::tmpdir()."/pkg_uninstall_fail.$sys->{sys}") if (Cfg::opt('bait'));
    return '';
}

sub pkg_uninstall_passed_sys {
    my($sys,$pkg)= @_;
    $pkg=$sys->pkg($pkg) unless (ref($pkg));
    # pkg uninstall is conditional
    return '' if (!EDRu::inarr($pkg->{pkgi}, @{$sys->{uninstallpkgs}}));
    push(@{$sys->{pkguninstallpass}}, $pkg->{pkg});
    my $pdfr=Msg::get('pdfr');
    Msg::log("$pkg->{pkg} $pdfr uninstalled successfully on $sys->{sys}");
    EDRu::appendfile($pkg->{pkg},
        EDR::tmpdir()."/pkg_uninstall_pass.$sys->{sys}") if (Cfg::opt('bait'));
    return '';
}

sub bundle_uninstall_sys {
    my ($cpic,$sys,$bundlei)= @_;
    # scalars must be passed to threaded call
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    return 1 if (Cfg::opt('makeresponsefile'));
    # pkg uninstall is conditional
    return 1 if (!EDRu::inarr($bundlei, @{$sys->{uninstallbundles}}));
    Msg::log("BEGIN-bundle_uninstall_sys($bundlei,$sys->{sys})");
    $sys->padv->bundle_uninstall_sys($sys,$bundlei)
        if ($sys->padv->can('bundle_uninstall_sys'));
    Msg::log("END-bundle_uninstall_sys($bundlei,$sys->{sys})");
    return 1;
}

sub prod_preremove_sys {
    my ($cpic,$sys) = @_;
    my ($tmpsub,$prod);
    return 1 if (Cfg::opt('makeresponsefile'));
    # scalars must be passed to threaded call
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $sys->rel->preremove_sys($sys);
    $prod=$sys->prod;
    $tmpsub = upgrade_sub('preremove_sys');
    if ($prod->can($tmpsub)) {
        $prod->$tmpsub($sys);
    }
    return;
}

sub prod_postremove_sys {
    my ($cpic,$sys) = @_;
    my ($tmpsub,$prod);
    return 1 if (Cfg::opt('makeresponsefile'));
    # scalars must be passed to threaded call
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $prod=$sys->prod;
    $tmpsub = upgrade_sub('postremove_sys');
    if ($prod->can($tmpsub)) {
        $prod->$tmpsub($sys);
    }
    $sys->rel->postremove_sys($sys);
    return;
}

sub cleanup_cpi_sys {
    my ($cpic,$sys) = @_;
    my ($rootpath,$rel,$use,$edr,$subprod,$prodname,$prod);
    return 1 if (Cfg::opt('makeresponsefile'));
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $cpic=Obj::cpic() unless (ref($cpic) eq 'CPIC');
    $rel=$sys->rel;
    $prod=$sys->prod;
    $edr=Obj::edr();
    $rootpath=Cfg::opt('rootpath')||'';

    # remove scripts
    for my $subprod ($cpic->{prod}, @{$prod->{subprods}}) {
        $prod=$sys->prod($subprod);
        $prodname=lc($prod->{installscript_name} || $prod->{prod});
        $prodname=~s/ /_/mg;
        $prodname="_$prodname" if (length($prodname)>8);
        $sys->rm("$rootpath$cpic->{installpath}/install$prodname");
        $sys->rm("$rootpath$cpic->{installpath}/uninstall$prodname");
    }

    # remove perl libs if there are no other versions using it
    # do not remove perl libs if it's running under localized locale
    $use=$sys->grepfile("$cpic->{release}","$rootpath$cpic->{installpath}/*");
    $sys->rm("$rootpath$cpic->{perlpath}/lib/site_perl/$cpic->{release}")
        if ((!$use) && ($edr->{envlang} eq 'C'));
    return;
}

# delete all supported install/uninstall* scripts
# and then add them back for still installed products.
sub readd_install_scripts_sys {
    my ($cpic,$sys) = @_;
    my ($script,$rootpath);
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $rootpath=Cfg::opt('rootpath');
    $script=$rootpath.$cpic->{installpath}.'/bin/'.$cpic->{release}.'/add_install_scripts';
    # if $script doesn't exist, all scripts should have been removed when CPI package was uninstalled.
    if ($sys->exists($script)) {
        $sys->cmd("CALLED_BY=CPI $script delete");
        $sys->cmd("CALLED_BY=CPI DONOTUPDATEVH=1 $script");
    }
    return;
}

sub update_summary_uninstall {
    my $cpic=shift;
    my ($msg,$pdfrs);

    $pdfrs=Msg::get('pdfrs');
    for my $sys (@{$cpic->{systems}}) {
        if ($#{$sys->{pkguninstallpass}}>=0) {
            $msg=Msg::new("The following $pdfrs were uninstalled on $sys->{sys}:", 30, 1378, "$pdfrs", "$sys->{sys}");
            $cpic->update_summary($msg, @{$sys->{pkguninstallpass}});
        }
        if ($#{$sys->{pkguninstallfail}}>=0) {
            $msg=Msg::new("The following $pdfrs failed to uninstall on $sys->{sys}:", 30, 1379, "$pdfrs", "$sys->{sys}");
            $cpic->update_summary($msg, @{$sys->{pkguninstallfail}});
        }
    }
    return '';
}

sub ask_uninstall_continue {
    my $cpic = shift;
    my ($msg, $msg_advice, $pdfrs, $ayn, $pkguninstallfail_flag);
    my $edr = Obj::edr();
    for my $sys (@{$cpic->{systems}}) {
        if ($#{$sys->{pkguninstallfail}}>=0) {
            $pkguninstallfail_flag = 1;
        }
    }
    if ($pkguninstallfail_flag == 1){
        $pdfrs=Msg::get('pdfrs');
        $msg = Msg::new("$pdfrs failed to uninstall. Do you want to exit $cpic->{script}?", 30, 1380, "$pdfrs", "$cpic->{script}");
        $ayn = $msg->ayny;
        if ($ayn eq 'Y') {
            $msg_advice = Msg::new("Refer to the installer logs for details and correct the settings. Continue with the installer again, or try to uninstall the $pdfrs manually", 30, 1381, "$pdfrs");
            $msg_advice->print;
            Msg::n();
            $cpic->edr_completion();
        }
    }
    return '';
}

1;

