#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
package Pkg;
use strict;
@Pkg::ISA = qw(CPIP);

# common Pkg subs and trace-only stubs for optional Pkg subs

# auto set padv references for pkgname and pkgobj
sub pre_init {
    my $pkg=shift;
    (undef,$pkg->{pkgi},$pkg->{padv})=split(/::/m,$pkg->{class},3);
    return '';
}

sub verify_media {
    my ($cpic,$padv,$pkg,$pkgpath,$pkgi,$rel,$edr,$msg, $mediapath);
    $pkg = shift;
    return if ($pkg->nopadvpkg);
    $cpic = Obj::cpic();
    $edr = Obj::edr();
    $rel = $pkg->rel;
    # initiate the padv release object to define file, vers, size, and space
    if ($cpic->{fromdisk} || (!$rel->{pkgs} && !$pkg->{pkgonly})) {
        $pkg->{vers}=$pkg->{gavers};
        $pkg->{vers}||=$rel->{gavers};
        return '';
    } elsif ($rel->{padv} eq 'Common') {
        $pkg->{vers}=$pkg->version();
        return '';
    }
    if ($pkg->{file}) {
        $pkg->{vers} ||= $pkg->padv->media_pkg_version($pkg,EDRu::dirname($pkg->{file}));
        return '';
    }

    $pkgpath=Cfg::opt('pkgpath');
    # only handles packages in base_path at the moment
    $mediapath=$cpic->{basemediapath};
    #for pkgonly packages, we will get the $pkgpath from mr release first
    if ($pkg->{pkgonly} && Cfg::opt('base_path')) {
        $pkgpath||=$rel->pkgs_patches_dir('pkgpath','');
    }
    $pkgpath||=$rel->pkgs_patches_dir('pkgpath', $mediapath);
    $padv=$pkg->padv;
    $pkg->{file}=$padv->media_pkg_file($pkg, $pkgpath);
    if ($pkg->{file}) {
        # common version definitions also defined at the $rel level now
        $pkg->{vers} = $padv->media_pkg_version($pkg,$pkgpath);
        EDR::die("Cannot determine version of $pkg->{padv} $pkg->{pkg}")
            if (!$pkg->{vers} && !$pkg->{batchinstall});
        $pkg->{size}=$cpic->pkg_patch_size($pkg->{file});
        $pkg->{space}=$pkg->space();
        $pkg->{mpok} = 0 if($pkg->{mpok});
    } elsif (Cfg::opt('mpok')) {
        $pkg->{mpok}=1;
    } elsif (!$rel->{verify_media}) {
        Msg::log("Cannot find $pkg->{pkg} for padv $pkg->{padv} in $pkgpath");
    } elsif (Cfg::opt('pkgpath')) {
        $pkgi =Msg::get('pdfr')." ".$pkg->{pkg};
        $msg = Msg::new("Cannot find $pkgi in $pkgpath", 30, 1384, "$pkgi", "$pkgpath");
        $msg->die;
    } elsif ($pkg->{mpok} || Cfg::opt('base_path')){
        Msg::log("$pkg->{pkg} is not in media\n");
    }
    else {
        EDR::die("Cannot find $pkg->{pkg} for padv $pkg->{padv} in $pkgpath");
    }
    return;
}

# Use Pkg::new_pkg() to create object for packages which are not built-in defined in CPI.
sub new_pkg {
    my ($class,$pkgname,$padv,$refpkg) = @_;
    my ($rel,$cpic,$pool,$pkg);
    $pool=join('::',$class,$pkgname,$padv);
    return $Obj::pool{$pool} if (defined($Obj::pool{$pool}));
    $pkg=CPIC::new_object("$class\::$pkgname",$padv);
    $pkg->{pool}=$pool;
    $pkg->{pkg}=$pkgname unless (defined $pkg->{pkg});
    $pkg->{pkgi}=$pkgname;  # use pkg name as pkgi for pkgs not built-in defined
    $pkg->{refpkg}=$refpkg;
    $cpic=Obj::cpic();

    # $rel may not exists for un-supported padvs.
    $rel=$cpic->rel($padv,1);
    $rel->new_pkg_init($pkg) if ($rel);

    $Obj::pool{$pool}=$pkg;
    return $pkg;
}

# get a defined version from $rel->pkg_versions
sub version {
    my $pkg=shift;
    my ($pkgs,$vers,$rel);
    $rel=$pkg->rel;
    $vers=$rel->pkg_versions;
    for my $line (split(/\s+/m,$vers)) {
        ($pkgs,$vers)=split(/=/m,$line,2);
        return $vers if ($pkgs eq $pkg->{pkg});
    }
    EDR::die("version not defined for $pkg->{pkg} in $rel->{vers} pkg_versions\n$vers");
    return;
}

# get a defined space from $rel->pkg_space
sub space {
    my $pkg=shift;
    my ($rel,$space,$opt,$usr,$var,$root);
    $rel=$pkg->rel;
    $space=$rel->pkg_space || $rel->{pkgsspace};
    for my $line (split(/\n/,$space)){
        next unless ($line=~/^$pkg->{pkg}:/mx);
        (undef,$opt,$usr,$var,$root,undef)=split(/[:,]/m,$line,6);
        return [$opt,$usr,$var,$root];
    }
    Msg::log("No space defined for $pkg->{pkg} in $rel->{vers} pkg_space\n$space");
    return ([0,0,0,0]);
}

# add a package to a systems install list
sub add_install_sys {
    my ($pkg,$sys)= @_;
    unless ($pkg->{file}) {
        #workaround for incident:3380450
        if (Cfg::opt('base_path') && $sys->linux()) {
            $pkg->add_patches_sys($sys);
        }
        Msg::log("Package $pkg->{pkg} will not be installed on $sys->{sys} as the file was not found");
        return ''
    }
    return '' if (Cfg::opt('upgrade_kernelpkgs') && $pkg->{unkernelpkg});
    return '' if (Cfg::opt('upgrade_nonkernelpkgs') && (!$pkg->{unkernelpkg}));

    if ($pkg->{installpkgslast}) {
        push(@{$sys->{installpkgslast}}, $pkg->{pkgi});
        #Msg::log("$pkg->{pkgi} has been pushed into \$sys->\{installpkgslast\} array");
    }
    push(@{$sys->{installpkgs}}, $pkg->{pkgi});
    #Msg::log("$pkg->{pkgi} has been pushed into \$sys->\{installpkgs\} array");
    $pkg->previouspkgnames_sys($sys) if ($pkg->{previouspkgnames});
    # do not add patches since the patch is same with rpm on Linux);
    $pkg->add_patches_sys($sys);
    return '';
}

# if we are installing pkg, all patches must be uninstalled
sub add_patches_sys {
    my ($pkg,$sys) = @_;
    my ($patch,$patchi);
    for my $patchi (@{$pkg->patches_sys($sys)}) {
        $patch=$sys->patch($patchi);
        $patch->add_install_sys($sys);
    }
    return '';
}

# add a package to a systems uninstall list
# set upgrade flags if the task is to install
sub add_uninstall_sys {
    my ($pkg,$sys) = @_;
    return '' unless ($pkg->version_sys($sys,0,1));
    return '' if ($pkg->noupgradepkg);
    return '' if (Cfg::opt('upgrade_kernelpkgs') && $pkg->{unkernelpkg});
    return '' if (Cfg::opt('upgrade_nonkernelpkgs') && (!$pkg->{unkernelpkg}));
    
    if (Cfg::opt(qw(upgrade patchupgrade install)) && $pkg->{donotrmonupgrade}) {
        Msg::log("Package $pkg->{pkg} will be upgraded without being uninstalled on $sys->{sys}");
        return '';
    }
    if (!EDRu::inarr($pkg->{pkgi}, @{$sys->{uninstallpkgs}})) {
        #Msg::log("$pkg->{pkgi} has been pushed into \$sys->\{uninstallpkgs\} array");
        push(@{$sys->{uninstallpkgs}}, $pkg->{pkgi});
    }
    return '';
}

sub allprocs {
    my(@procs,$pkg,$procs);
    $pkg=shift;
    for my $procs (qw(definedprocs startprocs stopprocs)) {
        push(@procs, @{$pkg->{$procs}}) if ($pkg->{$procs});
    }
    return EDRu::arruniq(@procs);
}

# stub - determine if a package should not be installed based on external
# hardware or software dependencies or for any other reason
sub donotinstall_sys { return $_[0]->{donotinstall} || EDRu::inarr($_[0]->{pkgi},@{$_[1]->{donotinstallpkgs}}) }

# stub - determine if a package should not be uninstalled based on external
# hardware or software dependencies or for any other reason
sub donotuninstall_sys { return $_[0]->{donotuninstall} || EDRu::inarr($_[0]->{pkgi},@{$_[1]->{donotuninstallpkgs}}) }

# stub - determine if a package should not be upgraded based on external
# hardware or software dependencies or for any other reason
sub donotupgrade_sys { return $_[0]->{donotupgrade} || EDRu::inarr($_[0]->{pkgi},@{$_[1]->{donotupgradepkgs}}) }

# determine if a package's patches should be installed on a system
# does additional version checking to flag more recently installed on AIX
sub set_patches_sys {
    my ($pkg,$sys) = @_;
    my ($mri,$padv,$patch,$cv,$patchi);
    $padv=$sys->padv;
    for my $patchi (@{$pkg->patches_sys($sys)}) {
        $patch=$sys->patch($patchi);
        # AIX patch check is essentially a pkg check
        if ($sys->aix()) {
            $cv=EDRu::compvers($pkg->version_sys($sys),$patch->{vers});
            Msg::log("Current $patch->{patchname} patch version $patch->{vers} is installed on $sys->{sys}") if ($cv==0);
            $mri=1 if ($cv==1);
            $patch->add_install_sys($sys) if ($cv==2);
        } else {
            $cv=EDRu::compvers($patch->{vers},$padv->patch_version_sys($sys, $patch),4);
            $patch->add_install_sys($sys) if ($cv==1);
        }
    }
    return $mri;
}

sub set_uninstall_patches_sys {
    my ($pkg,$sys)=(@_);
    my($cv,$padv,$patch,$patchi,$ipatchvers,$ivers,$prior_rp_patchvers);
    $padv=$sys->padv;
    for $patchi(@{$pkg->patches_sys($sys)}) {
        $patch=$sys->patch($patchi);
        $ipatchvers=$padv->patch_version_sys($sys,$patch);
        next unless ($ipatchvers);
        #$ivers=${$ipatchvers}[$#{$ipatchvers}]; # last version in ipatchvers array
        $ivers=$ipatchvers;
        $prior_rp_patchvers = $pkg->{patchvers}{$sys->{prior_rp}}{$patchi} if ($sys->{prior_rp});
        $prior_rp_patchvers =~ s/_/\./g;
        return '' unless (EDRu::compvers($ivers,$prior_rp_patchvers,4)==1);
        
        $cv=EDRu::compvers($ivers, $patch->{vers});
        $patch->add_uninstall_sys($sys) if ($cv!=2);
    }
    return ;
}

# determine whether a package should be installed/upgraded on a system
# and make the call to determine whether it's patches should be installed
sub set_install_sys {
    my ($pkg,$sys) = @_;
    my ($pkgvers,$rel,$cpic,$cv,$prod);
    #return '' if ($pkg->{mpok});
    return '' if ($pkg->donotinstall_sys($sys));
    return '' if (Cfg::opt('upgrade_kernelpkgs') && $pkg->{unkernelpkg});
    return '' if (Cfg::opt('upgrade_nonkernelpkgs') && (!$pkg->{unkernelpkg}));

    $cpic=Obj::cpic();
    $rel=$sys->rel;
    $prod=$sys->prod;
    $pkgvers=$pkg->version_sys($sys);
    $cv=EDRu::compvers($pkgvers, $pkg->{vers});
    if (Cfg::opt('upgradecv') || $pkg->{upgradecv}) {
          Msg::log("Upgrading current $pkg->{pkg} version $pkgvers to $pkg->{vers} on $sys->{sys}");
          # not uninstall and reinstall the current version if pkg has donotrmonupgrade flag.
          $pkg->add_install_sys($sys) unless (($pkg->{donotrmonupgrade}) ||
              ($sys->aix()) && ($pkg->pkgdep_sys($sys)));
          $pkg->add_uninstall_sys($sys) unless (($sys->aix()) &&
              ($pkg->pkgdep_sys($sys)));
    }
    if (!$pkgvers) {
        # package is not installed
        Msg::log("$pkg->{pkg} is not installed on $sys->{sys}");
        return '' if (Cfg::opt('patchupgrade') && (!$pkg->{installonpatchupgrade}));
        if (Cfg::opt("upgrade")) {
            if ($sys->{upgrade_prior_pkgset}) {
                $pkg->add_install_sys($sys) if (EDRu::inarr($pkg->{pkg},(@{$prod->{installonupgradepkgs}},@{$rel->{installonupgradepkgs}})));
            } else {
                $pkg->add_install_sys($sys);
            }
        } else {
            if (Cfg::opt("patchupgrade") && $sys->linux()) {
                $pkg->set_patches_sys($sys);
            }else{
                $pkg->add_install_sys($sys);
            }
        }
    } elsif ($sys->{pkgpartial}{$pkg->{pkg}}) {
        # package partially installed on SunOS, always clean up
        Msg::log("$pkg->{pkg} is partially installed on $sys->{sys}");
        # return "" if (Cfg::opt("patchupgrade"));
        return '' if (!$rel->{pkgs});
        $pkg->add_install_sys($sys);
        $pkg->add_uninstall_sys($sys);
    } elsif ($cv==0) {
        # current package version is installed, but what about patches?
        Msg::log("Current $pkg->{pkg} version $pkgvers is installed on $sys->{sys}");
        # do not set patches for Linux since patch are same with rpm.
        $pkg->set_patches_sys($sys);
    } elsif ($cv==1) {
        # on AIX, an installed patch version can be higher than the base
        # package version since all patches and packages are the same
        # Or if P patch is installed, pkg version is higer than the base
        if ($sys->aix() || $sys->linux()|| $sys->sunos_sol11()) {
            $pkg->set_patches_sys($sys);
        } else {
            Msg::log("$pkg->{pkg} version $pkgvers, more recent than $pkg->{vers} is installed on $sys->{sys}");
            push(@{$sys->{pkgshigherversioninstalled}}, $pkg->{pkg});
        }
    } else {
        # cv==2, upgrading
        #if (Cfg::opt("patchupgrade")) {
        #    # should probably have bailed out sooner in version check
        #    Msg::log("Cannot install patches for $pkg->{pkg} version $pkgvers on $sys->{sys} using this release");
        #} elsif ($pkg->unupgradable_version_sys($sys)) {
        if ($pkg->unupgradable_version_sys($sys)) {
            Msg::log("Cannot upgrade $pkg->{pkg} version $pkgvers on $sys->{sys}");
            push(@{$sys->{pkgscannotupgrade}}, $pkg);
        } elsif ($pkg->donotupgrade_sys($sys) || $prod->donotupgrade_pkg($pkg)) {
            Msg::log("A condition exists to not upgrade $pkg->{pkg} on $sys->{sys}");
        } elsif ((Cfg::opt('responsefile')) &&
                 (!$prod->{responsefileupgradeok})) {
            Msg::log("Cannot upgrade $pkg->{pkg} on $sys->{sys} using responsefile");
            $sys->{cannotresponsefileupgrade}=1;
        } else {
            # precheck upgrade message
            Msg::log("Upgrading $pkg->{pkg} version $pkgvers on $sys->{sys}");
            # For Patch upgrade, if pkgonly attribute set, we install the package directly.
            if ($pkg->{pkgonly} && Cfg::opt('patchupgrade')) {
                $pkg->add_install_sys($sys);
                return '';
            }
            # on Linux,  if (upgrade) then uninstall old pkg, install new rpm without patch, since patch is same with new rpm.
            #            if (patchupgrade) then install patch with 'rpm -U'.
            # on other platforms,  if (not on AIX or no dependencies) then uninstall old pkgs; fi; install new pkgs and patches.
            if ($sys->linux()) {
                if (Cfg::opt(qw(upgrade install))) {
                    $pkg->add_install_sys($sys);
                    $pkg->add_uninstall_sys($sys);
                } else {
                    $pkg->set_patches_sys($sys);
                }
            } else {
                $pkg->add_install_sys($sys);
                # on AIX and HPUX, don't uninstall pkg if pkg dependency is found
                $pkg->add_uninstall_sys($sys) unless ( ($sys->aix()||$sys->hpux()) && ($pkg->pkgdep_sys($sys)));
            }
        }
    }
    return '';
}

# determine whether a package's language package should be installed on a system
sub languagepkgs_sys {
    my ($pkg,$sys) = @_;
    my ($basevers,$cpic);

    $cpic = Obj::cpic();
    $basevers=$sys->{pkgvers}{$pkg->{basepkg}};
    return '' unless ($basevers);
    Msg::log("Base pkg $pkg->{basepkg} version $basevers is installed on $sys->{sys}");

    Msg::log("Language pkg $pkg->{pkg} version $pkg->{vers} will be installed on $sys->{sys}");
    $pkg->set_install_sys($sys);
    return '';
}

# determine whether a package's required OS patches are installed on a system
sub ospatches_sys {
    my ($pkg,$sys) = @_;
    my ($padv,$patch);
    $padv=$sys->padv;
    for my $patch (@{$pkg->{ospatches}}) {
        push(@{$sys->{pkgospatchesnotinstalled}{$pkg->{pkg}}}, $patch)
# TODO fix SunOS.pm currently expects passed objects
            unless (($padv->patch_installed_sys($sys, $patch)) ||
                    ($padv->patch_obsoleted_sys($sys, $patch)));
    }
    return '';
}

# pad the discovered dependencies with defined softdeps or
# special-case dependencies discovered via another check
sub all_dependencies_sys {
    my ($pkg,$sys) = @_;
    my ($dep,$deps);
    $deps=$pkg->query_pkgdep_sys($sys);
    push(@$deps, split(/\s+/m, $sys->pkgdeps($pkg)));
    for my $dep (@{$pkg->{softdeps}}) {
        push(@$deps, $dep) if ($sys->pkgvers($dep));
    }
    return EDRu::arruniq(@$deps);
}

# determine if the dependency is already on the removed list
# if it is coming later, then the package list is incorrect order
sub removing_dependency_sys {
    my ($pkg,$sys,$dep,$allpkgs) = @_;
    my ($rel,$upkg,$cpic,$upkgi,@allpkgs);
    $cpic=Obj::cpic();
    $rel=$sys->rel;

    # prior CPI had trouble here.  Is the dep string a pkgi or pobj?
    # everything should be coded so that $pkg->{pkg} or $pkg->{pkgname}
    # is the true package string, previously pobj
    return 1 if (($pkg->{pkg} eq $dep) ||
                 ($pkg->{pkgname} eq $dep)); # obscure self-dependency

    # if this pkg has been checked before
    if (defined $sys->{pkg_rm_or_not}{$dep}) {
        return 1 if $sys->{pkg_rm_or_not}{$dep} == 1;
        return '' if $sys->{pkg_rm_or_not}{$dep} == -1;
    }

    # set the hash of pkgname => pkgname_in_CPI
    unless (defined $sys->{pkg_pkgi_hash}{$dep}) {
        push(@allpkgs,@{$sys->{uninstallpkgs}}) if $sys->{uninstallpkgs};
        push(@allpkgs,@$allpkgs) if $allpkgs;
        $sys->{pkg_pkgi_hash} = {};
        for my $upkgi (@allpkgs) {
            $upkg=Pkg->new_pkg($upkgi, $cpic->{padv});
            $upkg=Pkg->new_pkg($upkgi, $sys->{padv});
            $sys->{pkg_pkgi_hash}{$upkg->{pkg}} = $upkgi if $upkg->{pkg};
            $sys->{pkg_pkgi_hash}{$upkg->{pkgname}} = $upkgi if $upkg->{pkgname};
        }
    }

    # if the dep could not be founld from uninstall list, it means that it will not be removed
    if ($sys->{pkg_pkgi_hash}{$dep}) {
        $upkgi = $sys->{pkg_pkgi_hash}{$dep};
        $upkg=Pkg->new_pkg($upkgi, $cpic->{padv});
        $upkg=Pkg->new_pkg($upkgi, $sys->{padv});
        for my $urd (@{$upkg->all_dependencies_sys($sys)}) {
            if (!$upkg->removing_dependency_sys($sys,$urd,$allpkgs)) {
                Msg::log("$dep, dependent on $pkg->{pkg} is not set to be removed");
                $sys->{pkg_rm_or_not}{$dep} = -1;
                return '';
            }
        }

        # if when upgrade_kernelpkgs, the upkg is nonkernel, treat is as to-be-rm pkg
        if ((Cfg::opt('upgrade_kernelpkgs') && $upkg->{unkernelpkg}) ||
               !($upkg->noupgradepkg || $upkg->donotuninstall_sys($sys) ||
                (Cfg::opt(qw(upgrade patchupgrade install)) && $upkg->{donotrmonupgrade}))) {
            Msg::log("$dep, dependent on $pkg->{pkg} is set to be removed");
            $sys->{pkg_rm_or_not}{$dep} = 1;
            return 1;
        }
    }

    # remove the dependency from the L10N pkgs
    for my $upkg (@{$rel->{langpkgs}}) {
        if ($upkg eq $dep) {
            Msg::log("$dep, dependent on $pkg->{pkg} is set to be removed");
            $sys->{pkg_rm_or_not}{$dep} = 1;
            return 1;
        }
    }

    $sys->{pkg_rm_or_not}{$dep} = -1;
    Msg::log("$dep, dependent on $pkg->{pkg} is not set to be removed");
    return '';
}

# assumes vrtspkgversdeps_sys has been called
sub pkgdep_sys {
    my ($pkg,$sys,$allpkgs) = @_;
    my ($msg,$urd,$dpkg,@urdep);
    return '' if (Cfg::opt(qw(upgrade patchupgrade)) && $pkg->{donotrmonupgrade});
    return '' if (Cfg::opt(qw(upgrade_kernelpkgs)) && $sys->hpux());
    for my $urd (@{$pkg->all_dependencies_sys($sys)}) {
        push(@urdep,$urd) if (!$pkg->removing_dependency_sys($sys,$urd,$allpkgs));
    }
    return '' if ($#urdep<0);

    # CPI5 skipped shared pkgs, but ask for others in a semi-chaotic fashion
    # New plan is to always leave pkgs on except for defined exceptions that
    # will always be coming off, asking is too difficult, esp w web, and
    # cascading dependencies across multiple systems and packages
    $urd=join(' ', @urdep);
    if ($pkg->{autoremovedependentpkgs}) {
        # only set for VRTSvcs as of now to remove Enterpise agents
        Msg::log("Uninstalling $pkg->{pkg} dependencies on $sys->{sys}: $urd");
        for my $urd (@urdep) {
            # these packages don't have objects yet
            $dpkg=Pkg->new_pkg($urd, $sys->{padv}, $pkg);
            $dpkg->add_uninstall_sys($sys);
        }
        return '';
    }
    if (Cfg::opt('uninstall')) {
        $msg=Msg::new("Not uninstalling $pkg->{pkg} on $sys->{sys} due to unremoved dependencies: $urd", 30, 1385, "$pkg->{pkg}", "$sys->{sys}", "$urd");
        $msg->print;
        my $titlemsg=Msg::new("The following notes were discovered on the systems:", 30, 1386);
        $titlemsg->addNote($msg->{msg});
    } else {
        Msg::log("$pkg->{pkg} on $sys->{sys} has unremoved dependencies: $urd");
    }
    $pkg->set_donotstopprocs_sys($sys);
    return 1;
}

#If any processes should not be stopped when uninstall due to pkg dependency
#Should add the implementation of this sub in its own pkg pm.
sub set_donotstopprocs_sys {}

# determine whether older versions of the package with a different name
# are installed on a system
sub previouspkgnames_sys {
    my ($pkg,$sys,$allpkgs) = @_;
    my ($ppkg,$deps,$cpic);
    $cpic=Obj::cpic();
    for my $pkgi (@{$pkg->{previouspkgnames}}) {
        $deps=$pkg->query_pkgdep_sys($sys);
        # these packages don't have objects yet
        $ppkg=Pkg->new_pkg_with_previousname($pkgi,$cpic->{padv},$pkg);
        $ppkg=Pkg->new_pkg_with_previousname($pkgi,$sys->{padv},$pkg);
        push (@{$ppkg->{softdeps}},@$deps);
        if ($ppkg->version_sys($sys)) {
            $ppkg->add_uninstall_sys($sys) unless ($ppkg->pkgdep_sys($sys,$allpkgs));
        }
    }
    return '';
}

sub new_pkg_with_previousname {
    my ($class,$previousname,$padv,$pkg)=@_;

    my $ppkg=Pkg->new_pkg($previousname,$padv,$pkg);

    $ppkg->{isprevpkg}=1;
    if ($pkg->{donotrmonupgrade}) {
        $ppkg->{donotrmonupgrade}=1;
    }

    if ($pkg->{stopprocs}) {
        $ppkg->{stopprocs}=$pkg->{stopprocs};
    }

    if ($pkg->{softdeps}) {
        $ppkg->{softdeps}=$pkg->{softdeps};
    }

    return $ppkg;
}

# determine whether a package should be uninstalled on a system
sub set_uninstall_sys {
    my($pkg,$sys,$allpkgs)= @_;
    my ($patch,$cv);
    return '' if ($pkg->{isprevpkg});
    return '' if ($pkg->donotuninstall_sys($sys));
    return '' if (Cfg::opt('upgrade_kernelpkgs') && $pkg->{unkernelpkg});
    return '' if (Cfg::opt('upgrade_nonkernelpkgs') && (!$pkg->{unkernelpkg}));

    $pkg->previouspkgnames_sys($sys,$allpkgs)
        if ($pkg->{previouspkgnames});
    return '' unless ($pkg->version_sys($sys,0,1));
    my $pkgvers=$pkg->version_sys($sys);
    $cv = -1;
    if (Cfg::opt(qw(install upgrade))) {
        if ($pkg->{vers}) {
            $cv=EDRu::compvers($pkgvers, $pkg->{vers});
            #Code change for ET3416167. Sometimes there is no specific package in base_path release, but mr release has its patch. 
        } elsif ($pkg->{patchvers} && $sys->linux() && Cfg::opt('base_path')) {
            for my $patchi (@{$pkg->patches_sys($sys)})  {
                $patch=$sys->patch($patchi) unless (ref($patchi));
                $cv=EDRu::compvers($pkgvers, $patch->{vers});
                last if ($cv==2);
            }
        }
        if ($cv==0) {
            Msg::log("Current $pkg->{pkg} version $pkgvers is installed on $sys->{sys}, it won't be removed");
            return '';
        } elsif ($cv==1) {
            Msg::log("$pkg->{pkg} version $pkgvers, more recent than $pkg->{vers} is installed on $sys->{sys}, it won't be removed");
            return '';
        }
    }
    if (Cfg::opt('rollback') && $pkgvers) {
        $pkg->set_uninstall_patches_sys($sys);
        return '';
    }
    $pkg->add_uninstall_sys($sys)
        unless ($pkg->pkgdep_sys($sys,$allpkgs));
    return '';
}

sub version_sys {
    my ($pkg,$sys,$force_flag,$prevpkgs_flag,$pbe_flag) = @_;
    my ($ppkg,$vers,$pkgi);
    $vers=$sys->{pkgvers}{$pkg->{pkg}};

    if ((!EDRu::vrts_symc($pkg->{pkg})) || (!$sys->{packages}) || ($force_flag)) {
        $vers=$sys->padv->pkg_version_sys($sys, $pkg, $prevpkgs_flag,$pbe_flag);
        $sys->set_value("pkgvers,$pkg->{pkg}", $vers) unless ($pbe_flag);
        return $vers;
    }
    for my $pkgi (@{$pkg->{previouspkgnames}}) {
        last if ($prevpkgs_flag);
        $vers||=$sys->{pkgvers}{$pkgi};
    }
    return $vers;
}

sub mpversion_sys {
    my ($pkg,$sys,$force_flag) = @_;
    my $rel = $sys->rel;
    return $rel->pkg_mpversion_sys($sys,$pkg,$force_flag)
           if $rel->can('pkg_mpversion_sys');
    return '';
}

sub install_success_sys {
    my ($pkg,$sys)= @_;
    return $sys->padv->pkg_install_success_sys($sys, $pkg);
}

sub uninstall_success_sys {
    my ($pkg,$sys)= @_;
    return $sys->padv->pkg_uninstall_success_sys($sys, $pkg);
}

sub patches_allvers {
    my(@patches,@patchtypes,@releases,$edr,$rel,$gamp,$pkg,$vers,$release,$irelease);
    ($pkg,$release,@patchtypes)=@_;
    $rel=$pkg->rel;
    if ($release) {
        push(@releases,$release);
    } else {
        push(@releases,$rel->{currentrelease});
        push(@releases,@{$rel->{base_releases}}) if (defined $rel->{base_releases});
    }
    @patchtypes=qw(gapatches sppatches hfpatches rppatches ppatches) unless (@patchtypes);
    foreach $irelease (@releases) {
        for $gamp(@patchtypes) {
            if (defined($pkg->{$gamp})) {
                for $vers(sort keys %{$pkg->{$gamp}{$irelease}}) {
                    push(@patches, @{$pkg->{$gamp}{$irelease}{$vers}});
                }
            }    
        }    
    }    
    return EDRu::arruniq(@patches);
}

sub patches_sys {
    my(@patches,$edr,$release,$rel,$gamp,$pkg,$sys);
    ($pkg,$sys,$release)=(@_);
    $rel=$pkg->rel;
    $release = $rel->{currentrelease} unless ($release);
    for $gamp(qw(gapatches sppatches rppatches ppatches)) {
        # rppatches assumes either gapatches or mppatches are defined within a release
        push(@patches, @{$pkg->{$gamp}{$release}{all}})
            if (defined($pkg->{$gamp}{$release}{all}));
        push(@patches, @{$pkg->{$gamp}{$release}{$sys->{platvers}}})
            if (defined($pkg->{$gamp}{$release}{$sys->{platvers}}));
    }
    return \@patches;
}   

sub copy_sys {
    my ($pkg,$sys)= @_;
    Msg::log("Copying $pkg->{pkg} media files to $sys->{sys}");
    return $sys->padv->pkg_copy_sys($sys, $pkg);
}

sub copy_mark_sys {
    my ($pkg,$sys)= @_;
    return EDR::tmpdir()."/cp.pkg.$sys->{sys}.$pkg->{pkg}";
}

sub wait_for_copy_sys {
    my ($pkg,$sys)= @_;
    EDRu::wait_for_file($pkg->copy_mark_sys($sys));
    return '';
}

sub install_sys {
    my ($pkg,$sys)= @_;
    Msg::log("Installing $pkg->{pkg} on $sys->{sys}");
    return $sys->padv->pkg_install_sys($sys, $pkg);
}

sub installtime_sys {
    my ($pkg,$sys) = @_;
    return $sys->padv->pkg_installtime_sys($sys,$pkg);
}

sub remove_sys {
    my ($pkg,$sys)= @_;
    Msg::log("Removing $pkg->{pkg} temporary files on $sys->{sys}");
    return $sys->padv->pkg_remove_sys($sys, $pkg);
}

# stub to define additional pkg dependencies per system
# only used by AT at this time
sub query_pkgdep_sys { return []; }

sub patchpreinstall_sys { }

sub patchpostinstall_sys { }

sub preinstall_sys { }

sub preremove_sys { }

sub postinstall_sys { }

sub postremove_sys { }

sub uninstall_sys {
    my ($pkg,$sys)= @_;
    return $sys->padv->pkg_uninstall_sys($sys, $pkg);
}

# a simple inarr query to determine if upgrade of the package is supported
# from the current version using this script
sub unupgradable_version_sys {
    my ($pkg,$sys)= @_;
    return '' if ((!defined($pkg->{ugvers})) ||
                  (EDRu::inarr($sys->{platvers}, @{$pkg->{ugvers}})));
    return 1;
}

sub define_ospatches { }

# marker to indicate the package does not exist at all for the package
sub nopadvpkg { }

# marker to indicate that the previously installed instance is from a tar file
# and not a package and therefore no remove action is required
sub noupgradepkg { }

sub tar { }

sub write_version_file_sys {
    my ($pkg,$sys)=@_;
    # currently uses NB padv strings
    $sys->writefile("$sys->{padv} $pkg->{vers}", $pkg->{version_file});
    return;
}

1;
