#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
package Prod;
use strict;
@Prod::ISA = qw(CPIP);

sub pre_init {
    my $prod=shift;
    (undef,$prod->{prodi},$prod->{padv})=split(/::/m,$prod->{class},3);
    return;
}

sub minpkgs { return $_[0]->{minpkgs} || []; }
sub recpkgs { return $_[0]->{recpkgs} || []; }
sub allpkgs { return $_[0]->{allpkgs} || []; }

# To get obsoleted packages of GA release
sub obsoleted_ga_release_pkgs {
    return ($_[0]->{obsoleted_ga_release_pkgs} || []);
}

# To get obsoleted packages of all previous releases
sub obsoleted_previous_releases_pkgs {
    return ($_[0]->{obsoleted_previous_releases_pkgs} || []);
}

# To get obsoleted packages which need refresh to new version when upgrade if it is installed
sub obsoleted_but_need_refresh_when_upgrade_pkgs {
    return ($_[0]->{obsoleted_but_need_refresh_when_upgrade_pkgs} || []);
}

# To get obsoleted packages which are still supported in current version
sub obsoleted_but_still_support_pkgs {
    return ($_[0]->{obsoleted_but_still_support_pkgs} || []);
}

sub allpkgs_on_media {
    my $prod=shift;
    return EDRu::arruniq(@{$prod->{allpkgs}},
                         @{$prod->{obsoleted_but_need_refresh_when_upgrade_pkgs}},
    );
}

sub set_pkgs {
    my $prod=shift;
    my ($rel,@categories,$category);

    $rel=$prod->rel;
    @categories=qw(minpkgs recpkgs allpkgs
                   obsoleted_ga_release_pkgs
                   obsoleted_but_need_refresh_when_upgrade_pkgs
                   obsoleted_but_still_support_pkgs);
    for my $category (@categories) {
        $prod->{$category}=EDRu::arruniq(@{$rel->{$category}},
                                         @{$prod->{$category}});
    }

    return $prod->{allpkgs};
}

sub get_pkgs {
    my ($prod,$allpkgs_flag)= @_;
    return ($allpkgs_flag) ? $prod->allpkgs :
            Cfg::opt('installminpkgs') ? $prod->minpkgs :
            Cfg::opt('installrecpkgs') ? $prod->recpkgs : $prod->allpkgs;
}

sub pkgs_remove {
    my ($prod,@pkgs) = @_;
    $prod->{minpkgs}=EDRu::arrdel($prod->{minpkgs},@pkgs);
    $prod->{recpkgs}=EDRu::arrdel($prod->{recpkgs},@pkgs);
    $prod->{allpkgs}=EDRu::arrdel($prod->{allpkgs},@pkgs);
    return '';
}

sub geteula {
    my ($prod,$lang)=@_;
    my $cpic = Obj::cpic();
    my $rel = $prod->rel;
    #workaround for incident:3379647, change the name of EULA file if base_path is not same as basetitlevers 
    if (Cfg::opt('base_path') && $cpic->{base_path_release} ne $rel->{basetitlevers}) {
        if($cpic->{base_path_release}) {
            $prod->{eula} =~ s/$rel->{basetitlevers}/$cpic->{base_path_release}/;
        }
    }
    return "$prod->{proddir}/EULA/$lang/$prod->{eula}";
}

# common Prod subs and trace-only stubs for optional Prod subs
sub perform_task_sys { return 1 }
sub perform_task { return 1 }
sub check_config { }

sub configure_alternate_prod { }
sub ask_patchupgrade_configure {
    my ($prod,$sys)=@_;
    return $prod->perform_task('ask_patchupgrade_configure');
}
sub ask_upgrade_configure {
    my ($prod)=@_;
    return $prod->perform_task('ask_upgrade_configure');
}
sub precheck_task { my ($prod,$sys)=@_; return $prod->perform_task('precheck_task'); }
sub preinstall_tasks { my ($prod,$sys)=@_; return $prod->perform_task('preinstall_tasks'); }
sub install_precheck_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'install_precheck_sys'); }
sub patchupgrade_precheck_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'patchupgrade_precheck_sys'); }
sub uninstall_precheck_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'uninstall_precheck_sys'); }
sub upgrade_precheck_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_precheck_sys'); }
sub rollback_precheck_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'rollback_precheck_sys'); }
sub configure_precheck_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'configure_precheck_sys'); }
sub start_precheck_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'start_precheck_sys'); }
sub stop_precheck_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'stop_precheck_sys'); }
sub postcheck_precheck_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'postcheck_precheck_sys'); }
sub require_start_after_reboot_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'require_start_after_reboot_sys'); }
sub licensed { return $_[0]->can('licensed_sys'); }
sub pre_shutdown_question {my ($prod,$sys)=@_; return $prod->perform_task('pre_shutdown_question'); }
sub handle_precheck_issues { my ($prod)=@_; return $prod->perform_task('handle_precheck_issues'); }
# Need add the following commented subs for SF/SFHA/SFCFS/SFCFSRAC/SFRAC
#sub prestop_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'prestop_sys'); }
#sub poststop_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'poststop_sys'); }
#sub preremove_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'preremove_sys'); }
#sub postremove_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'postremove_sys'); }
#sub preinstall_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'preinstall_sys'); }
#sub postinstall_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'postinstall_sys'); }
#sub configure_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'configure_sys'); }
#sub poststart_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'poststart_sys'); }
#
#sub upgrade_prestop_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_prestop_sys'); }
#sub upgrade_poststop_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_poststop_sys'); }
#sub upgrade_preremove_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_preremove_sys'); }
#sub upgrade_postremove_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_postremove_sys'); }
#sub upgrade_preinstall_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_preinstall_sys'); }
#sub upgrade_postinstall_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_postinstall_sys'); }
#sub upgrade_configure_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_configure_sys'); }
#sub upgrade_poststart_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_poststart_sys'); }
#
#sub patchupgrade_prestop_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_prestop_sys'); }
#sub patchupgrade_poststop_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_poststop_sys'); }
#sub patchupgrade_preremove_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_preremove_sys'); }
#sub patchupgrade_postremove_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_postremove_sys'); }
#sub patchupgrade_preinstall_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_preinstall_sys'); }
#sub patchupgrade_postinstall_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_postinstall_sys'); }
#sub patchupgrade_configure_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_configure_sys'); }
#sub patchupgrade_poststart_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_poststart_sys'); }

sub register_postchecks_per_system { my ($prod,$sys)=@_; return $prod->perform_task('register_postchecks_per_system'); }
sub register_postchecks_per_cluster { my ($prod,$sys)=@_; return $prod->perform_task('register_postchecks_per_cluster'); }
sub postcheck_summary { my ($prod,$sys)=@_; return $prod->perform_task('postcheck_summary'); }

sub verify_responsefile { my ($prod,$sys)=@_; return $prod->perform_task('verify_responsefile'); }
sub responsefile_prestart_config { my ($prod,$sys)=@_; return $prod->perform_task('responsefile_prestart_config'); }
sub responsefile_poststart_config { my ($prod,$sys)=@_; return $prod->perform_task('responsefile_poststart_config'); }
sub responsefile_comments { my ($prod,$sys)=@_; return $prod->perform_task('responsefile_comments'); }

sub cli_initiation_questions { }
sub web_initiation_questions { }
sub cli_prod_option { }
sub web_prod_option { }

sub cli_preinstall_messages { my ($prod,$sys)=@_; return $prod->perform_task('cli_preinstall_messages'); }
sub web_preinstall_messages { my ($prod,$sys)=@_; return $prod->perform_task('web_preinstall_messages'); }
sub preinstall_messages { my ($prod,$sys)=@_; return $prod->perform_task('preinstall_messages'); }
sub cli_prestart_config_questions { my ($prod,$sys)=@_; return $prod->perform_task('cli_prestart_config_questions'); }
sub web_prestart_config_questions { my ($prod,$sys)=@_; return $prod->perform_task('web_prestart_config_questions'); }
sub prestart_config_questions { my ($prod,$sys)=@_; return $prod->perform_task('prestart_config_questions'); }
sub cli_poststart_config_questions { my ($prod,$sys)=@_; return $prod->perform_task('cli_poststart_config_questions'); }
sub web_poststart_config_questions { my ($prod,$sys)=@_; return $prod->perform_task('web_poststart_config_questions'); }
sub completion_messages { my ($prod,$sys)=@_; return $prod->perform_task('completion_messages'); }

sub preinstall_errors {return handle_errors(@_);}
sub postinstall_errors {return handle_errors(@_);}
sub preremove_errors {return handle_errors(@_);}
sub postremove_errors {return handle_errors(@_);}
sub config_errors {return handle_errors(@_);}
sub prestop_errors {return handle_errors(@_);}
sub poststop_errors {return handle_errors(@_);}

sub update_prod { }
sub copy_cpi_sys { }
sub other_requirements { }

sub description {
    my $prod=shift;
    my $msg=Msg::new("No description for $prod->{name} defined", 30, 1387, "$prod->{name}");
    $msg->print;
    return;
}

sub obsoleted_bundled_pkgs_sys {
    my ($prod,$sys) = @_;
    my ($msg,@bundled_pkgs,$pkgs);
    return '' unless ($prod->{padv} =~ /HPUX/m);
    # HPUX OEM bundle pkgs handling
    @bundled_pkgs=();
    if (defined $sys->{obsoleted_bundled_pkgs}) {
        push (@bundled_pkgs, @{$sys->{obsoleted_bundled_pkgs}});
    }
    for my $pkg (@{$prod->{obsoleted_bundled_pkgs}}) {
        push (@bundled_pkgs, $pkg) if ($sys->padv->bundle_pkg_sys($sys,$pkg));
    }
    if (@bundled_pkgs) {
        $sys->set_value('obsoleted_bundled_pkgs','list',@bundled_pkgs);
        $pkgs=join(' ',@bundled_pkgs);
        $msg=Msg::new("The following obsolete depot(s) bundled with HP-UX will be uninstalled on $sys->{sys}:\n\t$pkgs", 30, 1388, "$sys->{sys}", "$pkgs");
        if (Cfg::opt(qw(upgrade patchupgrade))) {
            $sys->push_warning($msg);
        } else {
            $sys->push_note($msg);
        }
    }
    return 1;
}

sub obsoleted_bundles_sys {
    my ($prod,$sys) = @_;
    my ($msg,@bundles,$bundles);
    return '' unless ($prod->{padv} =~ /HPUX/m);
    # HPUX OEM bundle handling
    for my $bundle (@{$prod->{obsoleted_bundles}}) {
        push (@bundles, $bundle) if ($sys->padv->bundle_sys($sys, $bundle));
    }
    if (@bundles) {
        $sys->set_value('uninstallbundles','list',@bundles);
        ### push note that some OEM bunldles founded
        $bundles=join(' ',@bundles);
        $msg=Msg::new("The following obsolete bundle(s) with HP-UX will be uninstalled on $sys->{sys}:\n\t$bundles", 30, 1389, "$sys->{sys}", "$bundles");
        if (Cfg::opt(qw(upgrade patchupgrade))) {
            $sys->push_warning($msg);
        } else {
            $sys->push_note($msg);
        }
    }
    return 1;
}

sub disableconfig {
    my $prod=shift;
    return ($prod->{disable_config}) ? 1 : 0;
}

sub hasconfig {
    my $prod=shift;
    return ($prod->{has_config}) ? 1 : 0;
}

sub donotreconfig {
    my $cfg=Obj::cfg();
    return $cfg->{donotreconfigurevcs};
}

sub donotupgrade_pkg {
    my ($prod,$pkg)= @_;
    return EDRu::inarr($pkg->{pkgi}, @{$prod->{donotupgradepkgs}});
}

sub installtime_sys {
    my ($prod,$sys) = @_;
    return if(!$prod->{mainpkg});
    my $pkg=$sys->pkg($prod->{mainpkg});
    return $pkg->installtime_sys($sys);
}

sub version_sys {
    my ($prod,$sys,$force_flag) = @_;
    my ($pkgvers,$rel,$mpvers,$pkg);
    return '' unless ($prod->{mainpkg});
    $pkg=$sys->pkg($prod->{mainpkg});
    $pkgvers=$pkg->version_sys($sys,$force_flag);
    $mpvers=$prod->{vers} if ($pkgvers && $prod->check_installed_patches_sys($sys,$pkgvers));
    $pkgvers= $prod->revert_base_version_sys($sys,$pkg,$pkgvers,$mpvers,$force_flag);
    return ($mpvers || $pkgvers);
}

sub version_with_extra_mainpkgs_sys {
    my ($prod,$sys) = @_;
    my ($extra_mainpkg_version, $extra_mainpkgs, $pkg, $pkgvers, $mpvers, $maxver, $cv);
    $maxver=$prod->version_sys($sys);
    if (defined $prod->{extra_mainpkgs} && $prod->need_extra_mainpkgs_sys($sys)) {
        $extra_mainpkgs = $prod->{extra_mainpkgs};
        foreach my $extra_mainpkg (@$extra_mainpkgs) {
            $pkg = $sys->pkg($extra_mainpkg);
            next unless (defined $sys->{pkgvers}{$pkg->{pkg}});
            $pkgvers = $pkg->version_sys($sys);
            $mpvers = $prod->{vers} if ($pkgvers && $prod->check_installed_patches_sys($sys,$pkgvers));
            $pkgvers = $prod->revert_base_version_sys($sys,$pkg,$pkgvers,$mpvers);
            if ($mpvers){
                $extra_mainpkg_version = $mpvers;
            } else {
                $extra_mainpkg_version = $pkgvers;
            }
            # For Rolling Upgrade phase 2, some extra mainpkgs has already been upgraded to media version.
            # We need to choose the highest version of extra mainpkgs that has not been upgraded to media version comparing prod main pkg.
            # If the extra mainpkg has already been upgraded, we skip this extra mainpkg.
            # E.g. 5.1RP2 SFCFSHA --> 6.0SP1 SFCFSHA, RU phase 2
            # 3 version of package: 5.1RP1(VRTScavf), 5.1RP2(VRTSvcs), 6.0SP1(VRTSvxfs, VRTSvxvm...)
            next if (Cfg::opt('upgrade_nonkernelpkgs') && !EDRu::compvers($extra_mainpkg_version,$pkg->{vers},4));
            $cv = EDRu::compvers($extra_mainpkg_version,$maxver,4);
            $maxver = $extra_mainpkg_version if ($cv == 1);
        }
    }
    
    if ($sys->hpux() && $prod->rp_installed_sys($sys) && $prod->rel->show_withrp($maxver)){
        $maxver .= " (with rp patches)";
    }
    return $maxver;
}

sub need_extra_mainpkgs_sys {
    return 1;
}

sub rp_installed_sys {
    my ($prod, $sys, $pkgi,$pkg);
    ($prod,$sys)=@_;
    $pkgi=$prod->{mainpkg};
    return 0 unless($pkgi);
    
    $pkg = $sys->pkg($pkgi);
    return $sys->padv->pkg_has_rp_sys($sys,$pkg);
}

sub revert_base_version_sys {
    my ($prod,$sys,$pkg,$pkgvers,$mpvers,$force_flag) = @_;
    my ($basevers,$cpic,$pkg_mpvers,$rel);
    $cpic = Obj::cpic();
    $rel=$sys->rel;
    # get the pstamp value of pkg to determine pkg version on Solaris
    $pkg_mpvers = $pkg->mpversion_sys($sys,$force_flag) if ($prod->{padv} =~ /Sol/m);
    $pkgvers = ($pkg_mpvers || $pkgvers);
    # cannot get patch version from disk, hence only check mainpkg version.
    # return mainpkg version if pkg only release
    #fix for e3403280, from disk behavior should be the same as from media
    return $pkgvers if (!$rel->{patches});
    # fix for e1989658, there is no need to revert on HPUX since HP is pkg release
    return $pkgvers if ($mpvers || !$pkgvers || $prod->{padv} =~ /HPUX/m);
    $basevers = ($rel->{rp}) ? $rel->{basevers}[$#{$rel->{basevers}}] : $rel->{basevers};
    return !EDRu::compvers($pkgvers,$prod->{vers},4) ? ($prod->{basevers} || $basevers) : $pkgvers;
}

sub register_postcheck_item {
    my ($prod,$ifpercluster,$sequence_id,$name,$desc,$handler) = @_;
    my ($key,$cpic);

    return unless($Obj::pool{CPIC});
    $cpic=Obj::cpic();

    $key="$sequence_id".'_'."$name";
    $desc = $desc->{msg} if (ref($desc) =~ m/^Msg/);
    if ($ifpercluster) {
        $cpic->{postchecks_per_cluster}{$key} = {
            'product' => $prod,
            'description' => "$desc",
            'handler' => $handler
        };
    } else {
        $cpic->{postchecks_per_system}{$key} = {
            'product' => $prod,
            'description' => "$desc",
            'handler' => $handler
        };
    }

    return 1;
}

sub register_postcheck_item_per_system {
    my $prod=shift;
    $prod->register_postcheck_item(0, @_);
    return;
}

sub register_postcheck_item_per_cluster {
    my $prod=shift;
    $prod->register_postcheck_item(1, @_);
    return;
}

sub handle_errors {
    my ($prod,$cpic) = @_;
    my ($msg,$web,$edr,$errmsg,$exit,$warnings,$errors);
    $edr=Obj::edr();
    $web = Obj::web();

    return 1 if (Cfg::opt(qw(makeresponsefile)) && !Cfg::opt(qw(upgrade patchupgrade)));

    # count errors and warnings
    for my $sys (@{$cpic->{systems}}) {
        $errors+=scalar(@{$sys->{errors}}) if ($sys->{errors});
        $warnings+=scalar(@{$sys->{warnings}}) if ($sys->{warnings});
    }

    # print errors and warnings
    if ($errors) {
        $msg=Msg::new("The following errors were discovered on the systems:", 30, 1390);
        $msg->print;
        $msg->add_summary(1);
        $msg->addError($msg->{msg});

        for my $sys (@{$cpic->{systems}}) {
            for my $errmsg ((@{$sys->{errors}})) {
                Msg::print($errmsg);
                Msg::add_summary($errmsg);
                $msg->addError($errmsg);
            }
        }
        Msg::n();
    }

    if ($warnings) {
        $msg=Msg::new("The following warnings were discovered on the systems:", 30, 1391);
        $msg->print;
        $msg->add_summary(1);
        $msg->addWarning($msg->{msg});

        for my $sys (@{$cpic->{systems}}) {
            for my $errmsg ((@{$sys->{warnings}})) {
                Msg::print($errmsg);
                Msg::add_summary($errmsg);
                $msg->addWarning($errmsg);
                $exit = 1 if ($sys->{warning_exit});
            }
        }
        Msg::n();
    }

    if ($errors || $exit) {
        if(Obj::webui()){
            my $webmsg;
            $webmsg .= $web->{errors} if ($web->{errors});
            $webmsg .= $web->{warnings} if ($web->{warnings});
            $web->before_cli_exit($webmsg);
        }
        EDR::exit_exitfile();
    }
    $edr->cli_handle_warnings() if ($warnings || $errors);
    return '';
}

sub poststart_errors {
    my ($prod,$cpic) = @_;

    return 1 if (Cfg::opt(qw(makeresponsefile)));
    if (Obj::webui()){
        my $msg = Msg::new('');
        for my $sys (@{$cpic->{systems}}) {
            for my $errmsg ((@{$sys->{warnings}})) {
                $msg->addWarning($errmsg);
            }
        }
    }
    for my $sys (@{$cpic->{systems}}) {
        for my $errmsg ((@{$sys->{warnings}}, @{$sys->{errors}})) {
            push(@{$sys->{startfailmsg}}, $errmsg);
        }
    }
    return;
}

# handles errors in CLI
sub cli_handle_errors {
    my ($msg, $ayn) = @_;

    $msg = Msg::new("Do you want to continue?", 30, 1392);
    $ayn = $msg->aynn;
    EDR::exit_exitfile() if ($ayn eq 'N');
    return;
}

# print the os dependency (pkgs/patches/libs)
sub cli_print_os_dependency {
    my ($prod) = @_;
    my (@platvers,$msg,$rel,%os_dependency,$max_length,@os_rpms,@os_libs,@os_build,$pkg,$suffix,$os_with_suffix,%minorverlib);
    $rel=$prod->rel;

    %os_dependency = (
        'ospatches' => Msg::new("OS patches", 30, 1393)->{msg},
        'ospkgs'    => Msg::new("OS packages", 30, 1394)->{msg},
    );
    if (defined $rel->{platvers}) {
        if ($rel->{padv} =~ /HPUX/m) {
            for my $patchindex (@{$rel->{platvers}}) {
                push (@platvers,"${patchindex}IA");
                push (@platvers,"${patchindex}PA");
            }
        } else {
            @platvers = @{$rel->{platvers}};
        }
    }
    push (@platvers,'all');

    if (Padv::plat($rel->{padv}) eq 'Linux') {
        for my $checked (qw(oslibs)) {
            @os_libs = ();
            @os_rpms = ();
            $max_length = 0;
            for my $pkgi(@{$prod->get_pkgs}) {
                $pkg=$prod->pkg($pkgi);
                for my $patchindex (sort keys %{$pkg->{$checked}}) {
                    @os_rpms = ();
                    while (my ($os_lib,$os_rpm) = each %{$pkg->{$checked}{$patchindex}}) {
                        push(@os_libs,$os_lib);
                        my @rpm_split = split(/\s+/,$os_rpm);
                        push(@os_rpms,@rpm_split);
                    }
                    push @{$minorverlib{$patchindex}},@os_rpms;
                }
            }
            for my $minorver(sort keys %minorverlib) {
                $msg = Msg::new("\nRequired OS rpms for $minorver:", 30, 1395, "$minorver");
                $msg->print;
                $minorverlib{$minorver} = EDRu::arruniq(@{$minorverlib{$minorver}});
                if (scalar @{$minorverlib{$minorver}}) {
                    print_ordered_array(\@{$minorverlib{$minorver}});
                } else {
                    $msg = Msg::new("None", 30, 1191);
                    $msg->print;
                }
            }
        }
    } else {
        for my $checked (qw(ospatches ospkgs)) {
            $msg=Msg::new("\nRequired $os_dependency{$checked}:", 30, 1396, "$os_dependency{$checked}");
            $msg->print;
            @os_build = ();
            $max_length = 0;
            for my $pkgi(@{$prod->get_pkgs}) {
                $pkg=$prod->pkg($pkgi);
                for my $patchindex (@platvers) {
                    $suffix = $patchindex eq 'all' ? '' : "($patchindex)";
                    for my $os_dep (@{$pkg->{$checked}{$patchindex}}) {
                        $os_with_suffix = $os_dep . $suffix;
                        push(@os_build,$os_with_suffix);
                        $max_length = length($os_with_suffix) if ($max_length < length($os_with_suffix));
                    }
                }
            }
            @os_build = @{EDRu::arruniq(@os_build)};
            if ($max_length){
                print_ordered_array(\@os_build,$max_length);
            } else {
                $msg = Msg::new("None", 30, 1191);
                $msg->print;
            }
        }
    }
    return;
}

sub print_ordered_array {
    my ($arr_ref,$max_length) = @_;
    my (@target,$length,$num,$format,$count,@div);

    @target = sort @$arr_ref;
    unless ($max_length) {
        $max_length = 0;
        for my $sample (@target) {
            $max_length = $max_length > length($sample) ? $max_length : length($sample);
        }
    }
    $length = $max_length + 1;
    $num = int((EDR::get2('tput','termx') - 3) / $length);
    $num = 1 if $num == 0;
    $format = '%-' . "$length" . 's';
    $format = $format x $num;
    $count = 0;
    while ($count < scalar @target) {
        @div = @target[$count..($count + $num - 1)];
        $count += $num;
        Msg::print(Msg::string_sprintf($format,@div));
    }
    return;
}

sub cli_print_requirements {
    my ($prod) = @_;
    my ($otherreqs,$prodspace,$minspace,$rel,$msg,$platreqs,$platreqs_str,$ps,$ms,$recspace,@space,$cs,$prodname,$pkg,$pkgspace);
    $rel=$prod->rel;

    $prodname = $prod->{name};
    $msg=Msg::new("The requirements for product '$prodname':\n", 30, 1397, "$prodname");
    $msg->print;

    $platreqs = $prod->{platreqs} || $rel->{platreqs};
    if (defined $platreqs) {
        $platreqs_str=join("\n",@{$platreqs});
        $msg=Msg::new("Required minimum OS/Kernel Versions:\n$platreqs_str", 30, 1398, "$platreqs_str");
        $msg->print;
    }

    $prod->cli_print_os_dependency();

    if ($prod->{minimal_memory_requirment}) {
        $msg=Msg::new("\nMinimal Memory Required: $prod->{minimal_memory_requirment}", 30, 1399, "$prod->{minimal_memory_requirment}");
        $msg->print;
    }

    # Determine the space requirement for the product
    for my $pkgi (@{$prod->get_pkgs}) {
        $pkg=$prod->pkg($pkgi);
        $pkg->{space} = $pkg->space();
        $pkg->{space}||=[100,100,100,100];
        @space=@{$pkg->{space}};
        $pkgspace=$space[0]+$space[1]+$space[2]+$space[3];
        $prodspace+=$pkgspace;
        if (EDRu::inarr($pkgi,@{$prod->minpkgs})) {
            $minspace+=$pkgspace;
        }
        if (EDRu::inarr($pkgi,@{$prod->recpkgs})) {
            $recspace+=$pkgspace;
        }
    }
    $ms=int($minspace/1024);
    $cs=int($recspace/1024);
    $ps=int($prodspace/1024);
    $msg=Msg::new("\nSpace Requirements:\n$prod->{abbr} Minimum Package Set - $ms MB\n$prod->{abbr} Recommended Package Set - $cs MB\n$prod->{abbr} All Package Set - $ps MB", 30, 1400, "$prod->{abbr}", "$ms", "$prod->{abbr}", "$cs", "$prod->{abbr}", "$ps");
    $msg->print;

    $otherreqs=$prod->other_requirements||'';
    unless ($otherreqs){
        $otherreqs = Msg::new("None", 30, 1191)->{msg};
    }

    $msg=Msg::new("\nOther Requirements:\n$otherreqs", 30, 1401, "$otherreqs");
    $msg->print;

    return '';
}

sub cpu_number_sys {
    my ($prod,$sys) = @_;
    my ($msg,$minimal_cpu_number,$cpu_number);

    $minimal_cpu_number=$prod->{minimal_cpu_number_requirment};
    return 1 unless ($minimal_cpu_number);
    $cpu_number=$sys->padv->cpu_number_sys($sys);
    return 1 unless ($cpu_number);

    if ($cpu_number < $minimal_cpu_number) {
        if ($cpu_number < 2) {
            $msg=Msg::new("You have only 1 CPU on $sys->{sys}. It is required that you run $prod->{abbr} on servers with more than 1 CPU.", 30, 1402, "$sys->{sys}", "$prod->{abbr}");
        } else {
            $msg=Msg::new("You have only $cpu_number CPUs on $sys->{sys}. It is required that you run $prod->{abbr} on servers with minimal $minimal_cpu_number CPUs.", 30, 1403, "$cpu_number", "$sys->{sys}", "$prod->{abbr}", "$minimal_cpu_number");
        }
        $sys->push_warning($msg);
    }

    return 1;
}

sub cpu_speed_sys {
    my ($prod,$sys) = @_;
    my ($msg,$minimal_cpu_speed,$cpu_speed);

    $minimal_cpu_speed=$prod->{minimal_cpu_speed_requirment};
    return 1 unless ($minimal_cpu_speed);
    $cpu_speed=$sys->padv->cpu_speed_sys($sys);
    return 1 unless ($cpu_speed);

    # Convert the size based on MHz.
    if ($cpu_speed=~/\s*(\d+)\s*[kK]/mx) {
        $cpu_speed=int($1/1024);
    } elsif ($cpu_speed=~/\s*(\d+)\s*[mM]/mx) {
        $cpu_speed=$1;
    } elsif ($cpu_speed=~/\s*(\d+)\s*[gG]/mx) {
        $cpu_speed=$1*1024;
    }

    # Convert the size based on MHz.
    if ($minimal_cpu_speed=~/\s*(\d+)\s*[kK]/mx) {
        $minimal_cpu_speed=int($1/1024);
    } elsif ($minimal_cpu_speed=~/\s*(\d+)\s*[mM]/mx) {
        $minimal_cpu_speed=$1;
    } elsif ($minimal_cpu_speed=~/\s*(\d+)\s*[gG]/mx) {
        $minimal_cpu_speed=$1*1024;
    }

    # In case the product require 2048MHZ CPU, but actually the machine has 2000M.
    if ($cpu_speed < int($minimal_cpu_speed * 0.95)) {
        $msg=Msg::new("The CPU speed on $sys->{sys} is $cpu_speed MHz, the minimal CPU speed required by product $prod->{abbr} is $minimal_cpu_speed MHz", 30, 1404, "$sys->{sys}", "$cpu_speed", "$prod->{abbr}", "$minimal_cpu_speed");
        $sys->push_warning($msg);
    }

    return 1;
}

sub memory_size_sys {
    my ($prod,$sys) = @_;
    my ($msg,$minimal_memory_size,$memory_size);

    $memory_size=$sys->padv->memory_size_sys($sys);
    return 1 unless ($memory_size);

    # Convert the size based on MegaByte.
    if ($memory_size=~/\s*(\d+)\s*[kK]/mx) {
        $memory_size=int($1/1024);
    } elsif ($memory_size=~/\s*(\d+)\s*[mM]/mx) {
        $memory_size=$1;
    } elsif ($memory_size=~/\s*(\d+)\s*[gG]/mx) {
        $memory_size=$1*1024;
    }

    $minimal_memory_size=$prod->{minimal_memory_requirment}||'512M';
    # Convert the size based on MegaByte.
    if ($minimal_memory_size=~/\s*(\d+)\s*[kK]/mx) {
        $minimal_memory_size=int($1/1024);
    } elsif ($minimal_memory_size=~/\s*(\d+)\s*[mM]/mx) {
        $minimal_memory_size=$1;
    } elsif ($minimal_memory_size=~/\s*(\d+)\s*[gG]/mx) {
        $minimal_memory_size=$1*1024;
    }

    # In case the product require 2048M memory, but actually the machine has 2046M.
    if ($memory_size < int($minimal_memory_size * 0.95)) {
        $msg=Msg::new("The RAM size on $sys->{sys} is $memory_size megabytes, the recommended RAM size required by product $prod->{abbr} is $minimal_memory_size megabytes", 30, 1405, "$sys->{sys}", "$memory_size", "$prod->{abbr}", "$minimal_memory_size");
        $sys->push_warning($msg);
    }

    return 1;
}

sub swap_size_sys {
    my ($prod,$sys) = @_;
    my ($msg,$minimal_swap_size,$swap_size);

    $swap_size=$sys->padv->swap_size_sys($sys);
    return 1 unless ($swap_size);

    if ($swap_size=~/\s*(\d+)\s*[kK]/mx) {
        $swap_size=int($1/1024);
    } elsif ($swap_size=~/\s*(\d+)\s*[mM]/mx) {
        $swap_size=$1;
    } elsif ($swap_size=~/\s*(\d+)\s*[gG]/mx) {
        $swap_size=$1*1024;
    }

    $minimal_swap_size=$prod->{minimal_swap_requirment}||'200M';
    # Convert the size based on MegaByte.
    if ($minimal_swap_size=~/\s*(\d+)\s*[kK]/mx) {
        $minimal_swap_size=int($1/1024);
    } elsif ($minimal_swap_size=~/\s*(\d+)\s*[mM]/mx) {
        $minimal_swap_size=$1;
    } elsif ($minimal_swap_size=~/\s*(\d+)\s*[gG]/mx) {
        $minimal_swap_size=$1*1024;
    }

    # In case the product require 2048M swap, but actually the machine has 2046M.
    if ($swap_size < int($minimal_swap_size * 0.95)) {
        $msg=Msg::new("The swap size on $sys->{sys} is $swap_size megabytes, the minimal swap size required by product $prod->{abbr} is $minimal_swap_size megabytes", 30, 1406, "$sys->{sys}", "$swap_size", "$prod->{abbr}", "$minimal_swap_size");
        $sys->push_warning($msg);
        return 0;
    }

    return 1;
}

sub basic_precheck_sys {
    my ($prod,$sys)=@_;

    if (Cfg::opt(qw(install upgrade patchupgrade precheck))) {
        $prod->cpu_number_sys($sys);
        $prod->cpu_speed_sys($sys);
        $prod->memory_size_sys($sys);
        $prod->swap_size_sys($sys);
    }
    if (Cfg::opt(qw(install upgrade patchupgrade configure uninstall precheck))) {
        $prod->rel->verify_commands_sys($sys);
        $sys->padv->installcommand_precheck_sys($sys);
        $prod->check_dir_sys($sys);
    }
    return 1;
}

sub additional_dir_space_sys {
    my ($prod,$sys)=@_;
    return $prod->perform_task_sys($sys, 'additional_dir_space_sys');
}

sub check_dir_sys {
    my ($prod,$sys)=@_;
    my $msg;
    # check /tmp directory
    if (!$sys->is_dir('/tmp')) {
        $msg=Msg::new("There is no /tmp directory on $sys->{sys}", 30, 1407, "$sys->{sys}");
        $sys->push_warning($msg);
    }
    return $prod->additional_dir_space_sys($sys);
}

sub check_installed_patches_sys {
    my ($prod,$sys,$pkg_iv) = @_;
    my ($rel,$vers,$padv,$patch,$cpic,$cv,$pkg);
    $cpic = Obj::cpic();
    return '' if (EDRu::compvers($pkg_iv,$prod->{vers},2));
    $rel=$prod->rel;
    $padv=$sys->padv;
    return '' if (!$rel->{patches});
    for my $pkgi (@{$prod->allpkgs}) {
        $pkg=$prod->pkg($pkgi);
        next unless ($pkg->version_sys($sys));
        for my $patchi (@{$pkg->patches_sys($sys)})  {
            $patch=$sys->patch($patchi) unless (ref($patchi));
            if ($patch->{vers}) {
                $cv=EDRu::compvers($patch->{vers},$padv->patch_version_sys($sys, $patch),4);
                return '' if ($cv==1);
            } else {
                return '' unless ($prod->{padv} =~ /Sol/m);
                return '' unless ($patch->installed_sys($sys));
            }
        }
    }
    return 1;
}

# requires some padv consistency for threaded any to many
# startprocs() and stopprocs() removed only add it if desperately needed for a2a

# again defined if system based control is needed, or cleaner -serial display
sub startprocs {
    my $prod=shift;
    my(@procs,$pkg);
    # what about the hypothetical any to many that differ?
    for my $pkgi (@{$prod->allpkgs}) {
        $pkg=$prod->pkg($pkgi);
        push(@procs, @{$pkg->{startprocs}}) if ($pkg->{startprocs});
    }
    return EDRu::arruniq(@procs);
}

# again defined if system based control is needed, or cleaner -serial display
sub startprocs_sys {
    my ($prod,$sys) = @_;
    my ($pkg,@procs);
    for my $pkgi (@{$prod->get_pkgs}) {
        $pkg=$sys->pkg($pkgi);
        push(@procs, @{$pkg->{startprocs}})
            if (($pkg->{startprocs}) && ($pkg->version_sys($sys)));
    }
    return EDRu::arruniq(@procs);
}

sub stopprocs {
    my $prod=shift;
    my ($cpic,@procs,$pkg);
    # what about the hypothetical any to many that differ?
    $cpic=Obj::cpic() if Obj::created('CPIC');
    @procs=();
    if($cpic && Cfg::opt(qw(patchupgrade upgrade install uninstall))) {
        for my $pkgi (@{$cpic->{uninstallpkgs_seq}}) {
            $pkg=Pkg->new_pkg($pkgi, $prod->{padv});
            push(@procs, @{$pkg->{stopprocs}}) if ($pkg->{stopprocs});
        }
        for my $proci (@procs) {
            CPIC::init_cpip_object("Proc::$proci",$prod->{padv})
                unless (Obj::created('Proc', $proci, $prod->{padv}));
        }

        # TODO:  need check upsell install here.
        #        which need stop procs of all pkgs
        return EDRu::arruniq(@procs);
    }
    for my $pkgi (reverse(@{$prod->allpkgs})) {
        $pkg=$prod->pkg($pkgi);
        push(@procs, @{$pkg->{stopprocs}}) if ($pkg->{stopprocs});
    }
    return EDRu::arruniq(@procs);
}

sub stopprocs_sys {
    my ($prod,$sys) = @_;
    my ($cpic,@procs,$pkg);
    $cpic=Obj::cpic() if Obj::created('CPIC');
    @procs=();
    if($cpic && Cfg::opt(qw(patchupgrade upgrade install uninstall))) {
        for my $pkgi (@{$cpic->{uninstallpkgs_seq}}) {
            $pkg=Pkg->new_pkg($pkgi, $sys->{padv});
            # stop procs for those pkgs exist on the system.
            if ($pkg->{stopprocs} && ($pkg->version_sys($sys))) {
                if (Cfg::opt("install")) {
                    # stop procs of the pkgs which will be uninstalled (cross stack upgrade)
                    push(@procs, @{$pkg->{stopprocs}})
                        if (EDRu::inarr($pkgi,@{$sys->{uninstallpkgs}}));
                } else {
                    push(@procs, @{$pkg->{stopprocs}});
                }
            }
        }
        for my $proci (@procs) {
            CPIC::init_cpip_object("Proc::$proci", $sys->{padv})
                unless (Obj::created('Proc', $proci, $sys->{padv}));
        }

        # TODO:  need check upsell install here.
        #        which need stop procs of all pkgs
        return EDRu::arruniq(@procs);
    }
    for my $pkgi (reverse(@{$prod->get_pkgs})) {
        $pkg=$sys->pkg($pkgi);
        push(@procs, @{$pkg->{stopprocs}})
            if ($pkg->{stopprocs} && ($pkg->version_sys($sys)));
    }
    return EDRu::arruniq(@procs);
}

sub remove_procs_for_prod {
    my ($prod,$procs) = @_;
    for my $pkgi (reverse(@{$prod->get_pkgs})) {
        my $pkg=$prod->pkg($pkgi);
        $procs=EDRu::arrdel($procs, @{$pkg->{stopprocs}});
    }
    return $procs;
}

sub zru_version_sys {
    my ($prod,$sys) = @_;
    my ($vers,$cv);
    return 1 if(Cfg::opt("upgrade_nonkernelpkgs"));
    $vers=$prod->version_sys($sys);
    for my $zru_release (@{$prod->{zru_releases}}) {
        $cv=EDRu::compvers($zru_release,$vers,2);
        next if ($cv != 0);
        $cv=EDRu::compvers($zru_release,$vers);
        return 1 if ($cv != 1);
    }
    return 0;
}

sub upgradeable_version_sys {
    my ($prod,$sys) = @_;
    my ($vers,$cv);
    $vers=$prod->version_sys($sys);
    for my $ugvers (@{$prod->{upgradevers}}) {
        $cv=EDRu::compvers($ugvers,$vers,2);
        next if ($cv != 0);
        $cv=EDRu::compvers($ugvers,$vers);
        return 1 if ($cv != 1);
    }
    return 0;
}

sub pkgsetlist {
    my $cpic = Obj::cpic();
    my %pkgs = ();
    for my $padvi (@{$cpic->{padvs}}) {
        my $rel=$cpic->rel($padvi);
        my $prod=$cpic->prod($cpic->{prod},$padvi);
        my @pkgis = (@{$rel->allpkgs},@{$prod->allpkgs});
        for my $pkgi (@pkgis) {
            my $pkg=$cpic->pkg($pkgi,$padvi);
            unless (EDRu::inarr($pkg,@{$pkgs{$padvi}{allpkgs}})) {
                push @{$pkgs{$padvi}{allpkgs}},$pkg;
                $pkgs{$padvi}{allspace} +=$pkg->{space}[0]+$pkg->{space}[1]+$pkg->{space}[2]+$pkg->{space}[3];
            }
        }
        @pkgis = (@{$rel->recpkgs},@{$prod->recpkgs});
        for my $pkgi (@pkgis) {
            my $pkg=$cpic->pkg($pkgi,$padvi);
            unless (EDRu::inarr($pkg,@{$pkgs{$padvi}{recpkgs}})) {
                push @{$pkgs{$padvi}{recpkgs}},$pkg;
                $pkgs{$padvi}{recspace} +=$pkg->{space}[0]+$pkg->{space}[1]+$pkg->{space}[2]+$pkg->{space}[3];
            }
        }
        @pkgis = (@{$rel->minpkgs},@{$prod->minpkgs});
        for my $pkgi (@pkgis) {
            my $pkg=$cpic->pkg($pkgi,$padvi);
            unless (EDRu::inarr($pkg,@{$pkgs{$padvi}{minpkgs}})) {
                push @{$pkgs{$padvi}{minpkgs}},$pkg;
                $pkgs{$padvi}{minspace} +=$pkg->{space}[0]+$pkg->{space}[1]+$pkg->{space}[2]+$pkg->{space}[3];
            }
        }
    }
    return \%pkgs;
}

sub get_tunables{
    my ($prod) = @_;
    if ($prod->{tunables}) {
        return $prod->{tunables};
    } else {
        return [];
    }
}

sub get_supported_tunables {
    my ($prod) = @_;
    return $prod->get_tunables;
}

sub validate_tunable_sys{
    my ($prod,$sys,$tunable,$value) =@_;
    my ($cpic, $prodname, $sysi, $tunable_info, @values,$msg, $rel);

    # 0 valid
    # 1 Padv is not supported
    # 2 Prod(Ver) is not installed
    # 3 value is invalid

    $cpic = Obj::cpic();
    $rel = Obj::rel();
    $prodname = $prod->{name};
    $sysi = $sys->{sys};

    if ($prod->version_sys($sys) !~ /^6\.0/) {
        $msg = Msg::new("$tunable is a tunable parameter of $prodname $rel->{titlevers}, which is neither installed nor being installed on $sysi", 30, 1408, "$tunable", "$prodname", "$rel->{titlevers}", "$sysi");
        return (2, $msg);
    }

    $tunable_info = $cpic->get_tunable_info($tunable);
    @values = @{$tunable_info->{values}};
    if ( $tunable_info->{type} eq 'enum') {
        if (!EDRu::inarr($value, @values)) {
            $msg = Msg::new("Invalid value $value of $tunable specified for $sysi or all systems", 30, 1409, "$value", "$tunable", "$sysi");
            return (3, $msg);
        }
    } elsif ($tunable_info->{type} eq 'range') {
        if ($value !~ /^-?(0x|0X)?\d+$/) {
            $msg = Msg::new("$tunable requires a numeric value but $value is specified for $sysi or all systems", 30, 1410, "$tunable", "$value", "$sysi");
            return (3, $msg);
        }
        if (($values[0] ne undef && $value < $values[0]) || ($values[1] ne undef && $value > $values[1])) {
            if (!($value == 0 && $tunable_info->{allow_zero})) {
                if ($values[0] ne undef && $values[1] ne undef) {
                    $msg = Msg::new("$tunable requires a numeric value between $values[0] and $values[1] but $value is specified for $sysi or all systems", 30, 1411, "$tunable", "$values[0]", "$values[1]", "$value", "$sysi");
                    return (3, $msg);
                } elsif ($values[0] ne undef) {
                    $msg = Msg::new("$tunable requires a numeric value equal to or greater than $values[0] but $value is specified for $sysi or all systems", 30, 1412, "$tunable", "$values[0]", "$value", "$sysi");
                    return (3, $msg);
                } else {
                    $msg = Msg::new("$tunable requires a numeric value equal to or less than $values[1] but $value is specified for $sysi or all systems", 30, 1413, "$tunable", "$values[1]", "$value", "$sysi");
                    return (3, $msg);
                }
            }
        }

        if ($values[2] ne undef && $value % $values[2]) {
            $msg = Msg::new("Value of $tunable must be a multiple of $values[2] but $value is specified for $sysi or all systems", 30, 1414, "$tunable", "$values[2]", "$value", "$sysi");
            return (3, $msg);
        }
    }

    return 0;
}

sub set_tunable_sys{
    my ($prod,$sys,$tunable,$value) =@_;
    my ($origval,$ret,$out);

    $origval = $prod->get_tunable_value_sys($sys,$tunable);

    if ($origval eq $value) {
        Msg::log("$tunable has already had value $value on $sys->{sys}");
        return 2; # 2 - Not changed
    }

    ($ret, $out) = $prod->set_tunable_value_sys($sys,$tunable,$value);
    if ($ret) {
        return (0, $out); # 0 - Failed to set
    }
    return 1; # 1 - Done
}

sub adjust_pkglist{}
1;
