#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
package Rel;
use strict;
@Rel::ISA = qw(CPIP);

sub pre_init {
    my $rel=shift;
    (undef,$rel->{reli},$rel->{padv})=split(/::/m,$rel->{class},3);
    return;
}

sub allpkgs { return $_[0]->{allpkgs} || []; }
sub minpkgs { return $_[0]->{minpkgs} || []; }
sub recpkgs { return $_[0]->{recpkgs} || []; }

# common Rel subs and trace-only stubs for optional Rel subs

sub platvers_sys { return 1 }
sub upgradevers_sys { return 1 }
sub basic_precheck_sys { }
sub feature_usage_sys{ }
sub copy_cpi_sys { }
sub completion { }
sub detect_prior_rp_sys { }
sub determine_superset { }
sub determine_prodmix_upgrade { }
sub determine_cross_partial_upgrade { }
sub determine_upsell_supported {}
sub determine_rolling_upgrade {}
sub adjust_pkgs { }
sub cli_prod_option { }
sub web_prod_option { }
sub cli_license_option { }
sub cli_language_menu { }
sub preinstall_sys { }
sub preinstall_patch_sys { }
sub postinstall_sys { }
sub preremove_sys { }
sub postremove_sys { }
sub preset_uninstall_pkgs { }
sub upgrade_prod_mix_check { }
sub usage {}
sub process_args {}
sub config_without_reboot_message {}
sub reboot_messages {}
sub ru_switch_groups {}
sub ru_offline_parallel_groups {}
sub minpkgs_nag {return 'Y';}
# should be defined in Pkg and Patch objs but easier to manage batched in Rel
sub pkg_space { }
sub pkg_versions { }
sub patch_space { }
sub patch_versions { }
sub adjust_pkg_sequence { return $_[1] || []; }
sub adjust_options { }
sub rollback_check_task { return 0;}
sub rollback_fsspace_sys {}
sub read_licenses_sys { }
sub new_pkg_init_common { }
sub new_pkg_init_plat { }
sub new_pkg_init_padv { }
sub new_pkg_init {
    my ($rel,$pkg)= @_;
    $rel->new_pkg_init_common($pkg);
    $rel->new_pkg_init_plat($pkg);
    $rel->new_pkg_init_padv($pkg);
    return;
}

sub init_args { }
sub init_args2 { }
sub filter_args { }

sub define_patches {};

sub define_sp_patches {
   my ($rel,$pkgname,$platvers,$release,@patches);
   my (@patchlist,$patchi,$vers);

    ($rel,$pkgname,$platvers,@patches)=(@_);
    my $pkg=Obj::pkg($pkgname,$rel->{padv});
    foreach my $patchvers (@patches){
      ($patchi,$vers)=split(/_/,$patchvers,2);
      push(@patchlist,$patchvers);
      $pkg->{patchvers}{$release}{$patchvers}=$vers if($vers);
      $pkg->{patchname}{$release}{$patchvers}=$patchi if($patchi);
   }
   $pkg->{sppatches}{$release}{"$platvers"} = [ @patchlist ];
}
 
sub define_rp_patches {
   my ($rel,$pkgname,$platvers,$release,@patches)=(@_);
   my (@patchlist,$patchi,$vers);

   my $pkg=Obj::pkg($pkgname,$rel->{padv});
   foreach my $patchvers (@patches){
      ($patchi,$vers)=split(/_/,$patchvers,2);
      push(@patchlist,$patchvers);
      if ($rel->{padv} =~ /HPUX/m) {
          $pkg->{patchvers}{$release}{$patchvers}='1.0';
          $pkg->{patchname}{$release}{$patchvers}=$patchvers;
      } else {
          $pkg->{patchvers}{$release}{$patchvers}=$vers if($vers);
          $pkg->{patchname}{$release}{$patchvers}=$patchi if($patchi);
      }
   }
   $pkg->{rppatches}{$release}{"$platvers"} = [ @patchlist ];
   return; 
}

sub define_p_patches {
   my ($rel,$pkgname,$platvers,$release,@patches)=(@_);
   my (@patchlist,$patchi,$vers);

   my $pkg=Obj::pkg($pkgname,$rel->{padv});
   foreach my $patchvers (@patches){
      ($patchi,$vers)=split(/_/,$patchvers,2);
      push(@patchlist,$patchvers);
      if ($rel->{padv} =~ /HPUX/m) {
          $pkg->{patchvers}{$release}{$patchvers}='1.0';
          $pkg->{patchname}{$release}{$patchvers}=$patchvers;
      } else {
          $pkg->{patchvers}{$release}{$patchvers}=$vers if($vers);
          $pkg->{patchname}{$release}{$patchvers}=$patchi if($patchi);
      }
   }
   $pkg->{ppatches}{$release}{"$platvers"} = [ @patchlist ];
   return;
} 

sub define_hf_patches {
   my ($rel,$pkg,$platvers,@patches)=(@_);
   my (@patchlist,$patchi,$vers,$padv,$release);

   $release=$rel->{currentrelease};
   $padv=Obj::padv($rel->{padv});
   foreach my $patchvers (@patches){
      ($patchi,$vers)=split(/_/,$patchvers,2);
      if ($rel->{padv} =~ /HPUX/m) {
          $pkg->{patchvers}{$release}{$patchvers}='1.0';
          $pkg->{patchname}{$release}{$patchvers}=$patchvers;
      } else {
          $pkg->{patchvers}{$release}{$patchvers}=$vers if($vers);
          $pkg->{patchname}{$release}{$patchvers}=$patchi;
      }
   }
   $pkg->{hfpatches}{$release}{"$platvers"} = [ @patches ];
}

sub define_os_patches {
    my ($rel,$pkgname,$platvers,@patches)= @_;
    my $pkg=$rel->pkg($pkgname,1);
    $pkg->{ospatches}{"$platvers"} = [ @patches ] if($pkg);
    return;
}

# key is pkgpath or patchpath
# defined per release, no longer per product
sub pkgs_patches_dir {
    my ($rel,$key,$patchmediapath) = @_;
    my ($mediapath,$msg,$padv,$dir,$cpic,$path,$prod);
    # to only have to search once
    return $rel->{"ppd-$key-$patchmediapath"} if ($rel->{"ppd-$key-$patchmediapath"});
    return "" if (($key eq "patchpath") && Cfg::opt("missingpkgs") && (!$rel->{patches}));
    $cpic=Obj::cpic();
    $mediapath = ($patchmediapath) ? $patchmediapath : $cpic->{mediapath};


    # can be defined at release level or fall to padv defaults
    $rel->{$key}||=$cpic->{pkgpath}{$rel->{padv}};
    # a2m pkgpath settings are stored in $cpic->{pkgpath}{$padv} by discover_prod_padv
    $rel->{$key}||=$rel->padv->{$key};
    EDR::die("Cannot find $key for $rel") unless ($rel->{$key});
    $dir=$rel->{$key};

    # probably rewrite this section
    # assumes path is raw /$padv and not /$rel->{key} of padv $rel
    $path=Cfg::opt($key);
    if ($path) {
        if ($#{$cpic->{padvs}}>0) {
            # must munge $path and $relpath for multipadv
            # assumes $relpath is path/padv for multi-padv
            $padv=EDRu::basename($dir);
            $path.="/$padv";
        }
        if (-d $path) {
            $rel->{"ppd-$key-$patchmediapath"}=$path;
            return $path
        }
        if ($#{$cpic->{padvs}}>0) {
            $msg=Msg::new("-$key subdirectory not found for $rel->{padv} at $path", 30, 1415, "$key", "$rel->{padv}", "$path");
        } else {
            $msg=Msg::new("-$key directory: $path not found", 30, 1416, "$key", "$path");
        }
    }

    # check in location stored in $dir
    my @mediapathdir;
    if ($^O =~ /Win32/i) {
        @mediapathdir = ("$mediapath\\$dir", "$mediapath\\..\\$dir");
    } else {
        @mediapathdir = ("$mediapath/$dir", "$mediapath/../$dir");
    }


    # check in location stored in $dir
    for my $path (@mediapathdir) {
        if (-d $path) {
            $rel->{"ppd-$key-$patchmediapath"}=$path;
            return $path;
        }
    }
    return;
}

sub pkgdesc {
    my ($rel, $pkg) = @_;

    $pkg = $pkg->{pkg} if (ref($pkg) =~ m/^Pkg/);
    return $rel->{package_descriptions}{$pkg};
}

sub cli_installer_menu {
    my ($cpic,$prod,$rel);
    $rel = shift;
    $cpic=Obj::cpic();
    set_menu_msgs();
    $rel->cli_task_menu();
    $rel->cli_prod_menu();
    if (Cfg::opt(qw(desc requirements))) {
        $prod=$cpic->prod;
        Msg::n();
        (Cfg::opt(qw(desc))) ? $prod->description :
                               $prod->cli_print_requirements;
        Msg::prtc();
        $cpic->{prod}='';
        Cfg::unset_opt('desc');
        Cfg::unset_opt('requirements');
        $rel->cli_installer_menu();
    } elsif (Cfg::opt('requirements')) {
        Cfg::unset_opt('requirements');
        $rel->cli_installer_menu();
    }
    return;
}

sub cli_task_menu {
    my(@mk,@mo,@tasks,$buf,$cpic,$edr,$m,$mkey,$mm1,$mm2,$msg,$rel,$task,$tsl);
    $rel = shift;
    $cpic=Obj::cpic();
    $edr=Obj::edr();
    if($cpic->{task}){
        $cpic->set_task($cpic->{task});
    }
    return if Cfg::opt(@{$rel->{args_task}});

    $msg=Msg::new("Task Menu:\n", 30, 1417);
    $msg->bold;

    @tasks=qw(precheck install configure upgrade license uninstall desc start requirements stop quit help);
    @mo=@tasks;
    while (@mo) {
        $m=shift(@mo);
        next if(!EDRu::inarr($m,@{$rel->{args_task}},'quit','help'));
        $mkey=Msg::get("key_$m");
        push(@mk,$mkey);
        $mm1="$edr->{tput}{bs}$mkey$edr->{tput}{be}) ".Msg::get("menu_$m");
        $m=shift(@mo);
        if ($m) {
            $mkey=Msg::get("key_$m");
            push(@mk,$mkey);
            $mm2="$edr->{tput}{bs}$mkey$edr->{tput}{be}) ".Msg::get("menu_$m");
        } else {
            $mm2='';
        }
        $buf=Msg::string_sprintf('    %-50s %s', $mm1, $mm2);
        $msg=Msg::new_obj($buf);
        $msg->print;
    }
    Msg::n();

    $tsl=join(',',@mk);
    $tsl="[$tsl]";
    while (!EDRu::inarr($task,@mk)) {
        $msg=Msg::new("Enter a Task: $tsl", 30, 1418, "$tsl");
        $task=$msg->ask;
        $task=uc($task);
    }

    if ($task eq Msg::get('key_help')) {
        $rel->task_menu_help();
        $rel->cli_task_menu();
    } else {
        for my $m ('requirements', 'desc') {
            Cfg::set_opt($m) if ($task eq Msg::get("key_$m"));
        }
        for my $m (@{$rel->{args_task}}) {
            if ($task eq Msg::get("key_$m")) {
                $cpic->set_task($m);
                $cpic->{task}=$m;
            }
        }
    }
    return '';
}

sub cli_prod_menu {
    my(@prods,$cpic,$menu,$mprodi,$msg,$prod,$rel);
    $rel = shift;
    $cpic=Obj::cpic();
    return if ($cpic->{prod});

    if($#{$rel->{prods}}==0){
        $cpic->set_prod($rel->{prods}->[0]);
        return;
    }

    for my $mprodi (@{$rel->{prods}}) {
        $prod=$cpic->prod($mprodi);
        push(@prods,$prod->{name});
    }

    if (Cfg::opt('install')) {
        $msg=Msg::new("Select a product to install:", 30, 1419);
    } elsif (Cfg::opt('uninstall')) {
        $msg=Msg::new("Select a product to uninstall:", 30, 1420);
    } elsif (Cfg::opt('configure')) {
        $msg=Msg::new("Select a product to configure:", 30, 1421);
    } elsif (Cfg::opt('start')) {
        $msg=Msg::new("Select a product to start:", 30, 1422);
    } elsif (Cfg::opt('stop')) {
        $msg=Msg::new("Select a product to stop:", 30, 1423);
    } elsif (Cfg::opt('upgrade')) {
        $msg=Msg::new("Select a product to upgrade:", 30, 1424);
    } elsif (Cfg::opt('license')) {
        $msg=Msg::new("Select a product to license:", 30, 1425);
    } elsif (Cfg::opt('precheck')) {
        $msg=Msg::new("Select a product to perform a pre-installation check for:", 30, 1426);
    } elsif (Cfg::opt('desc')) {
        $msg=Msg::new("Select a product to view its product description:", 30, 1427);
    } elsif (Cfg::opt('requirements')) {
        $msg=Msg::new("Select a product to view its product requirements:", 30, 1428);
    } else {
        $msg=Msg::new("Select a product:", 30, 1429);
    }

    Msg::n();
    $menu=$msg->menu(\@prods);
    $cpic->set_prod(${$rel->{prods}}[$menu-1]);
    return;
}

sub task_menu_help {
    menu_help(qw(install uninstall upgrade configure));
    menu_help(qw(start stop precheck license desc requirements));
    return '';
}

sub menu_help {
    my (@mo) = @_;
    my ($edr,$m,$mkey,$mm);
    Msg::title();

    $edr=Obj::edr();

    while (@mo) {
        $m=shift(@mo);
        $mkey=Msg::get("key_$m");
        $mm="$edr->{tput}{bs}$mkey$edr->{tput}{be}) ".Msg::get("menu_$m");
        print "$mm\n\n";
        $mm=Msg::linebreak(Msg::get("help_$m"));
        print "$mm\n";
        print "\n" if (@mo);
    }
    Msg::prtc();
    return;
}

sub set_menu_msgs {
    my ($edr,$msg,$pdfrs);

    # menu license screen msgs
    $msg=Msg::new("Symantec Product", 30, 1430);
    $msg->msg('menu_vendor_prod');
    $msg=Msg::new("Version Installed", 30, 1431);
    $msg->msg('menu_version_installed');
    $msg=Msg::new("Licensed", 30, 1432);
    $msg->msg('menu_version_licensed');

    # menu task selection
    # e1534297:  should not translate shortcut key.
    $edr=Obj::edr();
    #$msg=Msg::new("I");
    #$msg->msg("key_install");
    $edr->{msg}{key_install}='I';
    #$msg=Msg::new("G");
    #$msg->msg("key_upgrade");
    $edr->{msg}{key_upgrade}='G';
    #$msg=Msg::new("C");
    #$msg->msg("key_configure");
    $edr->{msg}{key_configure}='C';
    #$msg=Msg::new("L");
    #$msg->msg("key_license");
    $edr->{msg}{key_license}='L';
    #$msg=Msg::new("P");
    #$msg->msg("key_precheck");
    $edr->{msg}{key_precheck}='P';
    #$msg=Msg::new("U");
    #$msg->msg("key_uninstall");
    $edr->{msg}{key_uninstall}='U';
    #$msg=Msg::new("S");
    #$msg->msg("key_start");
    $edr->{msg}{key_start}='S';
    #$msg=Msg::new("X");
    #$msg->msg("key_stop");
    $edr->{msg}{key_stop}='X';
    #$msg=Msg::new("D");
    #$msg->msg("key_desc");
    $edr->{msg}{key_desc}='D';
    #$msg=Msg::new("?");
    #$msg->msg("key_help");
    $edr->{msg}{key_help}='?';

    $edr->{msg}{key_requirements}='R';
    $edr->{msg}{key_quit}='Q';

    # Installation Menu
    $msg=Msg::new("Install a Product", 30, 1433);
    $msg->msg('menu_install');
    $msg=Msg::new("Upgrade a Product", 30, 1434);
    $msg->msg('menu_upgrade');
    $msg=Msg::new("Configure an Installed Product", 30, 1435);
    $msg->msg('menu_configure');
    $msg=Msg::new("License a Product", 30, 1436);
    $msg->msg('menu_license');
    $msg=Msg::new("Perform a Pre-Installation Check", 30, 1437);
    $msg->msg('menu_precheck');
    $msg=Msg::new("Uninstall a Product", 30, 1438);
    $msg->msg('menu_uninstall');
    $msg=Msg::new("Start a Product", 30, 1439);
    $msg->msg('menu_start');
    $msg=Msg::new("Stop a Product", 30, 1440);
    $msg->msg('menu_stop');
    $msg=Msg::new("View a Product Description", 30, 1441);
    $msg->msg('menu_desc');
    $msg=Msg::new("View Product Requirements", 30, 1442);
    $msg->msg('menu_requirements');
    $msg=Msg::new("Help", 30, 1443);
    $msg->msg('menu_help');
    $msg=Msg::new("Quit", 30, 1444);
    $msg->msg('menu_quit');

    # menu license msgs
    $msg=Msg::new("none", 30, 1445);
    $msg->msg('installed_none');
    $msg=Msg::new("yes", 30, 1446);
    $msg->msg('license_yes');
    $msg=Msg::new("no", 30, 1447);
    $msg->msg('license_no');

    # menu Help msg
    $pdfrs=Msg::get('pdfrs');
    $msg=Msg::new("Installs $pdfrs and patches for an initial installation.", 30, 1448, "$pdfrs");
    $msg->msg('help_install');
    $msg=Msg::new("Starts processes associated with product", 30, 1449);
    $msg->msg('help_start');
    $msg=Msg::new("Stops processes associated with product", 30, 1450);
    $msg->msg('help_stop');
    $msg=Msg::new("Upgrades all $pdfrs and patches installed on the systems .", 30, 1451, "$pdfrs");
    $msg->msg('help_upgrade');
    $msg=Msg::new("Configures the product after installing a product without configuration.", 30, 1452);
    $msg->msg('help_configure');
    $msg=Msg::new("Installs or updates a product license key. This option can be used to update an evaluation key to a permanent key, or to license a feature in a previously installed product.", 30, 1453);
    $msg->msg('help_license');
    $msg=Msg::new("Performs a preinstallation check to determine if systems meet all installation requirements. No products are installed.", 30, 1454);
    $msg->msg('help_precheck');
    $msg=Msg::new("Removes the selected product's $pdfrs, patches, files, and directories from the system providing they are not used by another Veritas product.", 30, 1455, "$pdfrs");
    $msg->msg('help_uninstall');
    $msg=Msg::new("Displays a brief overview of the selected product.", 30, 1456);
    $msg->msg('help_desc');
    $msg=Msg::new("Displays a brief requirements of the selected product.", 30, 1457);
    $msg->msg('help_requirements');

    return '';
}

sub prodlist {
    my $cpic=Obj::cpic();
    my @prods=();
    for my $padvi(@{$cpic->{padvs}}){
        my $rel=$cpic->rel($padvi);
        for my $prodi(@{$rel->{prods}}){
            if(EDRu::inarr($prodi,@{$rel->{menuprods}})){
                my $prod=$cpic->prod($prodi,$padvi);
                push(@prods,$prod);
            }
        }
    }
    return \@prods;
}

# To verify whether the commands exist on the system or not.
sub verify_commands_sys {
    my ($rel,$sys) = @_;
    my ($msg,$cmds,@keys,$key,$padv,$basekey,@cmds,$rtn,@cmds_bin,$cmd_bin,$padv_vers);

    $padv_vers=$sys->{local_platvers}||$sys->{platvers};
    $basekey='commands_need_be_checked';
    push @keys, $basekey;
    $key=$basekey . '_for_' . $padv_vers;
    push @keys, $key;
    for my $opt (@{$rel->{args_task}}) {
        if (Cfg::opt($opt)) {
            $key=$basekey . '_for_' . $opt;
            push @keys, $key;
            $key=$basekey . '_for_' . $opt . '_' . $padv_vers;
            push @keys, $key;
        }
    }

    @cmds=();
    for my $key (@keys) {
        push (@cmds, @{$rel->{$key}}) if (defined $rel->{$key});
    }
    return 1 unless (@cmds);

    $padv=$sys->padv;
    for my $cmd (@cmds) {
        $cmd_bin=$padv->{cmd}{$cmd};
        next unless ($cmd_bin);

        $cmd_bin=~s/\s+.*$//m;
        push (@cmds_bin, $cmd_bin);
    }

    if (@cmds_bin) {
        $cmds=join(' ', @cmds_bin);
        $rtn=$sys->cmd("_cmd_ls -L $cmds 2>&1 1>/dev/null");
        if (EDR::cmdexit()) {
            $msg=Msg::new("The following required OS commands are missing on $sys->{sys}:\n$rtn", 30, 1458, "$sys->{sys}", "$rtn");
            $sys->push_error($msg);
        }
    }
    return '';
}


# return url config from VOS product server
sub get_upload_url {
    my ($rel) = @_;
    my ($upload_conf_file,@url_keys,$key,$url,$rtn,$conf,$line,$msg,$download_errors);

    return if(!$rel->{sort_upload_config_url});

    $upload_conf_file=EDR::tmpdir().'/upload_url.conf';
    $rtn=EDRu::download_file($rel->{sort_upload_config_url}, $upload_conf_file);
    # return "" if failed to get the URL config file.
    if (!$rtn || ! -f "$upload_conf_file") {
        $download_errors=EDR::get('download_errors');
        $msg=Msg::new("Could not download Symantec upload configure file '$rel->{sort_upload_config_url}'!\n\n$download_errors\n", 30, 1459, "$rel->{sort_upload_config_url}", "$download_errors");
        Cfg::opt('testuploadurl') ? $msg->print : $msg->log;
        return;
    }

    $url='';
    @url_keys=();

    # check release specific upload url key, for example:  uxrt60_upload_url=...
    $key=$rel->{release_upload_url_key};
    push @url_keys, $key if (defined $key);

    # check base upload url key, for example:  uxrt_upload_url=...
    $key=$rel->{base_upload_url_key};
    push @url_keys, $key if (defined $key);

    # check the very basic upload url key, for example:  upload_url=...
    push @url_keys, 'upload_url';

    $conf=EDRu::readfile($upload_conf_file);
    Msg::log("The upload configure file's contents:\n$conf");
    for my $line (split(/\n/,$conf)) {
        for my $key (@url_keys) {
            if ($line=~/^\s*$key\s*=\s*(.*)\s*$/mx) {
                $url=$1;
                last;
            }
        }
        last if ($url);
    }

    unless ($url) {
        $msg=Msg::new("Invalid Symantec upload configure url", 30, 1460);
        $msg->print;
    }

    # return $url if the URL get correctly defined.
    if ($url && $url=~/^http/m) {
        $msg=Msg::new("The URL to upload log files: $url", 30, 1461, "$url");
        Cfg::opt('testuploadurl') ? $msg->print : $msg->log;
        return $url;
    }
    return;
}

sub cli_ask_feedback {
    my ($rel) = @_;
    my ($cprod,$url,$cfg,$msg,$help,$ayn);

    $cprod=CPIC::get('prod');
    #TODO: Fred 1
    Cfg::unset_opt('silent') if ($cprod =~ /SVS/);
    return if(!$rel->need_ask_feedback());

    $msg=Msg::new("Would you like to send the information about this installation to Symantec to help improve installation in the future?", 30, 1462);
    $help=Msg::new("The install log files will be uploaded to Symantec for future analysis. The warnings, errors and installation operations are collected in the log.", 30, 1463);
    $ayn=$msg->ayny($help);

    Msg::n();
    return if ($ayn eq 'N');

    $cfg=Obj::cfg();
    $cfg->{uploadlogs}=1;
    return 1;
}

sub upload_logs {
    my ($rel) = @_;
    my ($edr,$cfg,$url,$msg,$rtn,$files,$file,$scriptid,$error,$gzip,$tmppath,$upload_logfiles,$upload_errors);

    $cfg=Obj::cfg();
    return unless ($cfg->{uploadlogs});
    #return "" if (Cfg::opt("makeresponsefile"));
    $url=$rel->{upload_url};
    return unless ($url);

    $scriptid=EDR::get('scriptid');
    $tmppath=EDR::get('tmppath');
    $upload_logfiles=EDR::get('save_logfiles');
    $edr = Obj::edr();
    $edr->dumpmemory();
    $files='';
    for my $file (@{$upload_logfiles}) {
        if ($file=~m{^\/}) {
            $files.="$file ";
        } else {
            $files.="$scriptid/$file ";
        }
    }
    EDR::cmd_local("cd $tmppath; _cmd_tar -cf $scriptid.tar $files 2>/dev/null");

    $file="$tmppath/$scriptid.tar";
    # Compress the file
    $gzip='/usr/bin/gzip';
    if (! -f "$gzip") {
        $gzip=EDR::cmd_local('_cmd_which gzip');
        if (($gzip=~/no gzip/m) || (! -x $gzip)) {
            $gzip='';
        }
    }
    if ($gzip) {
        EDR::cmd_local("$gzip $file");
        $file="$tmppath/$scriptid.tar.gz";
    }

    Msg::log("Uploading $file to Symantec website: $url");
    $rtn=EDRu::upload_file($url, $file, 'application/x-gzip');
    if ($rtn==1) {
        $msg=Msg::new("Upload completed successfully", 30, 1464);
        $msg->bold;
    } else {
        $msg=Msg::new("Upload did not complete successfully due to some issues in Symantec server side", 30, 1465);
        $msg->bold;
 
        $upload_errors=EDR::get('upload_errors');
        $error=EDRu::extract_upload_errors($upload_errors);
        Msg::log($error);
    }

    Msg::n();
    unlink("$file");
    unlink($edr->{memorydumpfile}) if(-f $edr->{memorydumpfile});
    return;
}

# check if the feedback question needs to be asked
sub need_ask_feedback {
    my ($rel) = @_;
    my $url;

    # skip to ask feeback when run uninstaller from '/opt/VRTS/install' since VRTSperl uninstalled at that time.
    return '' if (Cfg::opt('uninstall') && EDR::get('fromdisk'));
    return '' unless (Cfg::opt(qw(install patchupgrade upgrade uninstall configure license makeresponsefile)));

    $url=$rel->get_upload_url();
    return '' unless ($url);
    $rel->{upload_url}=$url;
    return '' if (Cfg::opt('responsefile'));
    return 1;
}

sub get_sort_product_name {
    my ($rel,$prod) = @_;
    return $prod;
}

sub get_sort_product_name_for_latest_version {
    my ($rel,$prod) = @_;
    return $prod;
}

sub check_installed_pkgs_version_sys { return 1; }

sub check_update {
    my ($rel) = @_;
    my ($sort_status,$prod_upd_cv, $cpi_upd_cv, $lv_cb, $prod_upd_lv_cb, $cpi_upd_lv_cb, $lv_mj, $prod_upd_lv_mj, $cpi_upd_lv_mj);
    my ($cpic,$prod,$prodi,$prodname,$relvers,$prodvers,$msg,$patchname,$basevers);
    return if (! Cfg::opt(qw(install upgrade patchupgrade)));
    return if (Cfg::opt(qw(makeresponsefile)));
    $cpic=Obj::cpic();
    return if (! $cpic->{prod});
    return if ($cpic->{eebi});
    $prod = $cpic->prod();
    $prodname = $prod->{name};
    $prodi = lc($prod->{prod});
    $relvers = $rel->{titlevers};
    $prodvers = $relvers;
    $prodvers =~ s/\s//g;
    $basevers = $prodvers;
    $basevers =~ s/^([0-9\.]*).*$/$1/;
    $msg = Msg::new("Checking online updates for $prodname $relvers", 30, 1466, "$prodname", "$relvers");
    $msg->bold;
    ($sort_status,$prod_upd_cv, $cpi_upd_cv, $lv_cb, $prod_upd_lv_cb, $cpi_upd_lv_cb, $lv_mj, $prod_upd_lv_mj, $cpi_upd_lv_mj) = $cpic->get_releases_patches_via_sort_webservice($prodi, $cpic->{padv}, $prodvers);
    $msg = "\n";
    if ($sort_status) {
        # current installed version
        if ($prod_upd_cv && @$prod_upd_cv || $cpi_upd_cv && @$cpi_upd_cv) {
            $msg .= Msg::new("\tAvailable updates for $prodname $relvers:\n", 30, 1467, "$prodname", "$relvers")->{msg};
            for my $patch (@$prod_upd_cv) {
                $patchname = $patch->{release_name};
                $msg .= Msg::new("\t    $patchname\n", 30, 1468, "$patchname")->{msg};
            }
            for my $patch (@$cpi_upd_cv) {
                $patchname = $patch->{release_name};
                $msg .= Msg::new("\t    (Installer) $patchname\n", 30, 1469, "$patchname")->{msg};
            }
        } else {
            $msg .= Msg::new("\tNo updates available for $prodname $relvers\n", 30, 1470, "$prodname", "$relvers")->{msg};
        }
        # latest version in current release branch.
        if ($lv_cb) {
            $msg .= "\n";
            $msg .= Msg::new("\tThe latest version of $prodname $basevers release is: $lv_cb\n", 30, 1471, "$prodname", "$basevers", "$lv_cb")->{msg};
            if ($prod_upd_lv_cb && @$prod_upd_lv_cb || $cpi_upd_lv_cb && @$cpi_upd_lv_cb) {
                $msg .= Msg::new("\t  Available updates for $prodname $lv_cb:\n", 30, 1472, "$prodname", "$lv_cb")->{msg};
                for my $patch (@$prod_upd_lv_cb) {
                    $patchname = $patch->{release_name};
                    $msg .= Msg::new("\t    $patchname\n", 30, 1468, "$patchname")->{msg};
                }
                for my $patch (@$cpi_upd_lv_cb) {
                    $patchname = $patch->{release_name};
                    $msg .= Msg::new("\t    (Installer) $patchname\n", 30, 1469, "$patchname")->{msg};
                }
            } else {
                $msg .= Msg::new("\t  No updates available for $prodname $lv_cb\n", 30, 1473, "$prodname", "$lv_cb")->{msg};
            }
        }
        # latest major release version
        if ($lv_mj) {
            $msg .= "\n";
            $msg .= Msg::new("\tThe latest version of $prodname is: $lv_mj\n", 30, 1474, "$prodname", "$lv_mj")->{msg};
            if ($prod_upd_lv_mj && @$prod_upd_lv_mj || $cpi_upd_lv_mj && @$cpi_upd_lv_mj) {
                $msg .= Msg::new("\t  Available updates for $prodname $lv_mj:\n", 30, 1472, "$prodname", "$lv_mj")->{msg};
                for my $patch (@$prod_upd_lv_mj) {
                    $patchname = $patch->{release_name};
                    $msg .= Msg::new("\t    $patchname\n", 30, 1468, "$patchname")->{msg};
                }
                for my $patch (@$cpi_upd_lv_mj) {
                    $patchname = $patch->{release_name};
                    $msg .= Msg::new("\t    (Installer) $patchname\n", 30, 1469, "$patchname")->{msg};
                }
            } else {
                $msg .= Msg::new("\t  No updates available for $prodname $lv_mj\n", 30, 1473, "$prodname", "$lv_mj")->{msg};
            }
        }

        $msg .= "\n";
        $msg .= Msg::new("\tPlease visit https://sort.symantec.com for more information.\n", 30, 1475)->{msg};
    } else {
        $msg .= Msg::new("\tAttempted to connect to https://sort.symantec.com to check for product updates, but connection failed.\n", 30, 1476)->{msg};
        $msg .= Msg::new("\tPlease visit https://sort.symantec.com to check for available product updates and information.\n", 30, 1477)->{msg};
    }
    Msg::print($msg);
    if (Obj::webui()) {
        my $web = Obj::web();
#        $msg =~ s/(https\:\/\/\S+)\s+/<a href="$1">$1<\/a>/g;
        $msg =~ s/^\t\s{4}/&nbsp;&nbsp;&nbsp;&nbsp;/mg;
        $msg =~ s/^\t\s{2}/&nbsp;&nbsp;/mg;
        $msg =~ s/\t//g;
        $msg =~ s/\n/\\n/g;
        $web->web_script_form('check_update',$msg);
    }
    return;
}

sub get_tunables{
    my ($rel) = @_;
    return $rel->{tunables};
}

sub validate_tunable_sys{
    my ($rel,$sys,$tunable,$value) =@_;
    my ($cpic, $sysi, $tunable_info, @values, $msg);

    # 0 valid
    # 3 value is invalid
    $cpic=Obj::cpic();
    $sysi = $sys->{sys};
    $tunable_info = $cpic->get_tunable_info($tunable);
    @values = @{$tunable_info->{values}};
    if ( $tunable_info->{type} eq 'enum') {
        if (!EDRu::inarr($value, @values)) {
            $msg = Msg::new("Invalid value $value of $tunable specified for $sysi or all systems", 30, 1409, "$value", "$tunable", "$sysi");
            return (3, $msg);
        }
    } elsif ($tunable_info->{type} eq 'range') {
        if ($value !~ /^-?(0x|0X)?\d+$/) {
            $msg = Msg::new("$tunable requires a numeric value but $value is specified for $sysi or all systems", 30, 1410, "$tunable", "$value", "$sysi");
            return (3, $msg);
        }
        if (($values[0] ne undef && $value < $values[0]) || ($values[1] ne undef && $value > $values[1])) {
            if (!($value == 0 && $tunable_info->{allow_zero})) {
                if ($values[0] ne undef && $values[1] ne undef) {
                    $msg = Msg::new("$tunable requires a numeric value between $values[0] and $values[1] but $value is specified for $sysi or all systems", 30, 1411, "$tunable", "$values[0]", "$values[1]", "$value", "$sysi");
                    return (3, $msg);
                } elsif ($values[0] ne undef) {
                    $msg = Msg::new("$tunable requires a numeric value equal to or greater than $values[0] but $value is specified for $sysi or all systems", 30, 1412, "$tunable", "$values[0]", "$value", "$sysi");
                    return (3, $msg);
                } else {
                    $msg = Msg::new("$tunable requires a numeric value equal to or less than $values[1] but $value is specified for $sysi or all systems", 30, 1413, "$tunable", "$values[1]", "$value", "$sysi");
                    return (3, $msg);
                }
            }
        }

        if ($values[2] ne undef && $value % $values[2]) {
            $msg = Msg::new("Value of $tunable must be a multiple of $values[2] but $value is specified for $sysi or all systems", 30, 1414, "$tunable", "$values[2]", "$value", "$sysi");
            return (3, $msg);
        }
    }
    return 0;
}

sub set_tunable_sys{
    my ($rel,$sys,$tunable,$value) =@_;
    my ($origval,$ret,$out);

    $origval = $rel->get_tunable_value_sys($sys,$tunable);

    if ($origval eq $value) {
        Msg::log("$tunable has already had value $value on $sys->{sys}");
        return 2; # 2 - Not changed
    }

    ($ret, $out) = $rel->set_tunable_value_sys($sys,$tunable,$value);
    if ($ret) {
        return (0, $out); # 0 - Failed to set
    }
    return 1; # 1 - Done
}

sub get_product_for_settunables{
    my ($rel) = @_;
    return '';
}

sub get_local_script_version_suffix{
    my ($rel) = @_;
    my $suffix = ($rel->{rp}) ? $rel->{basetitlevers} : $rel->{titlevers};
    $suffix =~ s/\s//mxg;
    $suffix =~ s/\.//mxg;
    return $suffix;
}

sub adjust_pkglist{
    my $rel = shift;
    for my $prodv(@{$rel->{prods}}){
        my $prod = Obj::prod($prodv,$rel->{padv});
        if(UNIVERSAL::can($prod,'adjust_pkglist')){
            $prod->adjust_pkglist();
        }
    } 
}

sub is_patchupgrade_configured {
    my ($rel) = @_;
    my ($cpic,$prod);
    $cpic = Obj::cpic();
    $prod = $cpic->prod();
    return 1 if ($cpic->{prod_running});
    if (Cfg::opt('patchupgrade') && !Cfg::opt('configure') && !$prod->check_config()) {
        return 0;
    }
    return 1;
}

1;
