#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
package EDR;

use strict;
use Expect;

$Expect::Log_Stdout = 0;

#ssh or rsh is already OK, return 0,
#setup ssh or rsh return 1,
#canceled by user  return -1.

sub ask_com_setup {
    my($edr,$systems)=@_;
    my(@com_systems,$sys,$syspasswd,$setup_systems,$msg_systems,$ayn,$com_status,$msg,$webmsg,$addmsg,$ret,$web,$invalid_passwords,$skip_checks,$result);
    if (!$systems) {
        $systems = $edr->{systems};
    }
    $web = Obj::web();
    $skip_checks = 0;
    while (1) { # keep asking passwords until all systems have correct password, or canceld by user by input 'q'
        $setup_systems=[];
        $invalid_passwords=[];
        $syspasswd={};
        $msg_systems='';
        @com_systems=();
        for my $sys (@{$systems}) {
            if ($sys->{com_deny}) {
                push(@com_systems,$sys);
                push(@{$setup_systems}, $sys->{sys});
            }
            if ($sys->{sshrsh_failure}) {
                $skip_checks = 1;
            }
        }
        $msg_systems=join ' ',@{$setup_systems};
        return -1 if ($skip_checks); # ssh or rsh service is not running on at least one of the systems.
        return 0 unless (@{$setup_systems}); # ssh or rsh is ready on all systems.

        Msg::n();
        if (Cfg::opt('rsh')) {
            $addmsg=Msg::new("rsh needs to be set up between the local system and $msg_systems for communication", 20, 1006, "$msg_systems");
            $addmsg->printn;
            $msg=Msg::new("Would you like the installer to setup rsh communication automatically between the systems?\nSuperuser passwords for the systems will be asked.", 20, 1007);
        } else {
            $addmsg=Msg::new("Either ssh or rsh needs to be set up between the local system and $msg_systems for communication", 20, 1008, "$msg_systems");
            $addmsg->printn;
            $msg=Msg::new("Would you like the installer to setup ssh or rsh communication automatically between the systems?\nSuperuser passwords for the systems will be asked.", 20, 1009);
        }
        $ayn=$msg->ayny('','',$addmsg);
        return -1 if ($ayn eq 'N');

        Msg::n();
        $syspasswd = $edr->cli_systems_psw(@com_systems);
        if (Cfg::opt('rsh')) {
            $ret = 2;
        } else {
            $ret = $edr->cli_ssh_rsh();
        }
        next if (EDR::getmsgkey($ret,'back'));
        unless ($ret) {
            $result=$web->web_script_form('systems_psw',@com_systems) if (Obj::webui());
            $ret=delete $result->{selected_ssh_rsh_method};
            $syspasswd=$result;
        }
        $edr->{com_setup_method} = ($ret == 1) ? 'ssh':'rsh';
        $msg=Msg::new("Setting up communication between systems. Please wait.", 20, 1010);
        $msg->print;
        $com_status = ($ret == 1) ? $edr->ssh_setup($syspasswd,$setup_systems,$invalid_passwords,!'try_rsh_on_failure') : $edr->rsh_setup($syspasswd,$setup_systems,$invalid_passwords,!'try_ssh_on_failure');

        EDR::reset_errors_warnings() if ($com_status);
        for my $sys (@{$systems}) {
            if ($sys->{com_deny} && !EDRu::inarr($sys->{sys},@$invalid_passwords)) {
                #e3412739:record systems which rsh/ssh password-less connection sucessful.
                push @{$edr->{transport_setup_systems}}, $sys->{sys};
                $sys->{com_deny}=0;
            }
        }
        if (scalar @$invalid_passwords == 0 ) {
            $msg=Msg::new("Re-verifying systems.", 20, 1011);
            $msg->print;
            sleep 2;
            return 1; # need verify again
        } else {
            # password invalid on one or more systems.
            $msg_systems = join ' ',@$invalid_passwords;
            Msg::n();
            $msg=Msg::new("Failed to login to remote system(s) $msg_systems.\nPlease make sure the password(s) are correct and superuser(root) can login to the remote system(s) with the password(s).", 20, 1012, "$msg_systems");
            $msg->print;
            $webmsg = $msg->{msg}."\n";
            if ($ret==2) {
                if (Obj::localplat() eq 'HPUX') {
                    $msg=Msg::new("If you want to setup rsh on remote system(s), please make sure rsh with command argument ('remsh <host> <command>') is not denied by remote system(s).", 20, 1013);
                } else {
                    $msg=Msg::new("If you want to setup rsh on remote system(s), please make sure rsh with command argument ('rsh <host> <command>') is not denied by remote system(s).", 20, 1014);
                }
                $msg->print;
                $webmsg .= $msg->{msg}."\n";
            }
            $web->web_script_form('alert',$webmsg) if (Obj::webui());
        }
    }
    return;
}

sub cli_systems_psw {
    my ($edr, @com_systems) = @_;
    my ($msg, $passwd, $msg_systems,$sys, $syspasswd, $same_passwd, $passwd0, $sysi, $ayn, $help);
    for my $sys (@com_systems){
        $msg_systems.="$sys->{sys} ";
    }
    # Ask password for each node separately because passwds may include space characters.
    $same_passwd=0;
    # use seperate hash to store sys/passwd values so it's not recorded anywhere
    $syspasswd ={};
    for my $sys (@com_systems) {
        if (!$same_passwd) {
            $sysi=$sys->{sys};
            $msg=Msg::new("Enter the superuser password for system $sysi: ", 20, 1015, "$sysi");
            EDR::cmd_local('stty -echo');
            $msg->bold(1);
            chomp ($passwd = <STDIN>);
            EDR::cmd_local('stty echo');
            Msg::n();
        } else {
            $passwd = $passwd0;
        }
        $syspasswd->{$sys->{sys}} = $passwd;

        if ($sys == $com_systems[0] && $#com_systems > 0) {
            $msg=Msg::new("\n Do you want to use the same password for all systems?", 20, 1016);
            $help = Msg::new("Answer 'Y' if all systems have the same superuser password. Otherwise passwords need to be provided for each system.", 20, 1017);
            $ayn=$msg->ayny;
            if ($ayn eq 'Y') {
                $same_passwd = 1;
                $passwd0 = $passwd;
            }
        }
    }

    $msg->n();
    return $syspasswd;
}

sub cli_ssh_rsh {
    my ($ssh_msg,$rsh_msg,$help,$m,$ret,@menu);
    my $backopt=1;
    $ssh_msg=Msg::new("Setup ssh between the systems", 20, 1018);
    $rsh_msg=Msg::new("Setup rsh between the systems", 20, 1019);
    $help = Msg::new("This is the communication method prompt", 20, 1020);
    $m=Msg::new("Select the communication method", 20, 1021);
    push(@menu,$ssh_msg->{msg},$rsh_msg->{msg});
    $ret=$m->menu(\@menu,1,$help,$backopt);
    return $ret;
}

sub cleanup_com_setup {
    my ($edr, $systems) = @_;
    my(@menu,$ssh_msg,$rsh_msg,$help,$m,$ret,$key_text,$sys,$local_home);
    my($msg,$sysi,$remote_home,$localsys_name,$pubkey,@fields,$cmd,$exp,@exp_ret,$keyfile,$web,$cleanup_method);
    $web=Obj::web();
    $ret = 1 if ($edr->{com_setup_method} eq 'ssh');
    $ret = 2 if ($edr->{com_setup_method} eq 'rsh');
    unless ($ret) {
#        if(Obj::webui()){
#            $web->web_script_form('com_cleanup');
#        } else {
            $ssh_msg=Msg::new("Cleanup ssh between the systems", 20, 1022);
            $rsh_msg=Msg::new("Cleanup rsh between the systems", 20, 1023);
            $help = Msg::new("This is the communication method prompt", 20, 1020);
            $m=Msg::new("Select the communication method", 20, 1021);
            push(@menu,$ssh_msg->{msg},$rsh_msg->{msg});
            $ret=$m->menu(\@menu,1,$help);
#        }
    }
    $sys = EDR::get('localsys');
    $local_home = $sys->get_root_home();
 
    #e3412739:Get what systems need be unsetup transportation
    $systems ||= $edr->{transport_setup_systems};
    if (!$systems) {
        if(Obj::webui()){
            $systems = $web->{com_systems};
        } else {
            $systems = $edr->cli_select_systems();
        }
    }
    return 0 if (!$systems || $#$systems<0);

    $edr->init_sys_objects();
    for my $sysi (@{$systems}) {
        $sys = Obj::sys($sysi);
        $cleanup_method = $ret;
        if ($edr->{com_setup_method}) {
            if (!$sys->{com_setup_method}) {
                next;
            } elsif ($sys->{com_setup_method} eq 'ssh') {
                $cleanup_method = 1;
            } else {
                $cleanup_method = 2;
            }
        }

        #e3430364:if stop_checks was set to 1, these checks below will be recognized as failed.
        $sys->set_value("stop_checks",undef);
        if (!($edr->ping_sys($sys) && $edr->rsh_sys($sys) && $edr->shell_sys($sys) && $edr->padv_ipv_sys($sys))) {
            if ($sys->{com_deny}) {
                if (Cfg::opt('rsh')) {
                    $msg = Msg::new("rsh is not configured in password-less mode on $sysi", 20, 1024, "$sysi");
                } else {
                    $msg = Msg::new("Neither ssh or rsh is configured in password-less mode on $sysi", 20, 1025, "$sysi");
                }
                $msg->print();
                next;
            } elsif ($sys->{sshrsh_failure}||$sys->{stop_checks}) {
                $msg = Msg::new("Failed to connect to $sysi", 20, 1026, "$sysi");
                $msg->print();
                next;
            }
        }
        if ($sys->{islocal}) {
            $msg = Msg::new("$sysi is the local system", 20, 1027, "$sysi");
            $msg->print();
            next;
        }

        # do edr cleanup on $sys now as we will not be able to do that after the ssh/rsh comm is unconfigured
        if ($edr->{com_setup_method} && $edr->{tmpdir} && $edr->{tmpdir} =~ /\d+/mx ) {
            $sys->rm($edr->{tmpdir});
            $sys->{cleanup} = 0;
        }

        $remote_home = $sys->get_root_home();
        if ($cleanup_method==1) {
            if ($sys->{rsh} eq 'rsh' && $sys->{rcp} eq 'rcp') {
                $msg=Msg::new("ssh is not configured in password-less mode on $sysi", 20, 1028, "$sysi");
                $msg->print;
                next;
            }
            $pubkey = undef;
            $keyfile = Cfg::opt('keyfile');
            if ($keyfile && -f $keyfile.'.pub' ) {
                $pubkey = EDRu::readfile($keyfile.'.pub');
                if (scalar (grep {/^ssh/m} split(/\n/,$pubkey)) == 1) {
                    $pubkey = (grep {/^ssh/m} split(/\n/,$pubkey))[0];
                    chomp $pubkey;
                } else {
                    $pubkey = undef;
                }
            } elsif (-f "$local_home/.ssh/id_rsa.pub") {
                $pubkey = EDRu::readfile("$local_home/.ssh/id_rsa.pub");
                if (scalar (grep {/^ssh/m} split(/\n/,$pubkey)) == 1) {
                    $pubkey = (grep {/^ssh/m} split(/\n/,$pubkey))[0];
                    chomp $pubkey;
                } else {
                    $pubkey = undef;
                }
            }
            if ($pubkey) {
                $key_text = $sys->readfile("$remote_home/.ssh/authorized_keys");
                @fields = split /\n/, $key_text;
                $key_text = join("\n", grep {$_ ne $pubkey} @fields);
                $key_text .= "\n";
                $sys->writefile($key_text,"$remote_home/.ssh/authorized_keys");
            }
        } else {
            #e3437549
            $localsys_name = $sys->{transport_setup_rsh_host};
            if (!defined $localsys_name) {
                if (Obj::localplat() eq 'HPUX') {
                    $cmd = "LANG=C LC_ALL=C remsh $sysi";
                } else {
                    $cmd = "LANG=C LC_ALL=C rsh $sysi";
                }
                $exp = Expect->spawn($cmd);
                if (!$exp) {
                    next;
                }
                @exp_ret = $exp->expect(10,
                                        [
                                            qr/#/ => sub {
                                                $exp = shift;
                                                $exp->send("PS1='SYMCPrompt>'\n");
                                                exp_continue;
                                            }
                                        ],
                                        '-re','SYMCPrompt>$',
                                        'assword:'
                                    );
                if ( $exp_ret[2] =~ 'assword:' ) {
                    $msg=Msg::new("rsh is not configured in password-less mode on $sysi", 20, 1024, "$sysi");
                    $msg->print;
                    next;
                }
                if ( $exp_ret[2] ne 'SYMCPrompt>') {
                    $exp->send("PS1='SYMCPrompt>'\n");
                    @exp_ret = $exp->expect(10, '-re', 'SYMCPrompt>$');
                    if ($exp_ret[2] ne 'SYMCPrompt>') {
                        $msg=Msg::new("rsh is not configured in password-less mode on $sysi", 20, 1024, "$sysi");
                        $msg->print;
                        next;
                    }
                }
                if ( $sys->hpux() ) {
                    $exp->send("LANG=C LC_ALL=C who -u am i\n");
                } else {
                    $exp->send("LANG=C LC_ALL=C who am i\n");
                }
                @exp_ret=$exp->expect(10,'-re','^root.*$');
                @fields=split /\s/m, $exp_ret[2];
                $localsys_name=$fields[-1];
                chomp $localsys_name;
                if ( $localsys_name =~ /\([^)]*\)/m) {
                    $localsys_name=~s/\(([^)]*)\)/$1/mx;
                }
    
                if (EDRu::isip($localsys_name)) {
                    $localsys_name = $edr->{localsys}->{sys};
                }
            }
            $key_text = $sys->readfile("$remote_home/.rhosts");
            $key_text =~ s/^\s*$localsys_name\s*\n//mx;
            if ($key_text) {
                $sys->writefile($key_text,"$remote_home/.rhosts");
            } else {
                $sys->rm("$remote_home/.rhosts");
            }
        }
        $msg=Msg::new("Cleaning up the $sys->{rsh} communication mode for $sysi\n", 20, 1029, "$sys->{rsh}", "$sysi");
        $msg->bold;
    }
    return;
}

# ssh or rsh is already OK, return 0,
# setup ssh or rsh, return 1,
# canceled by user, return -1.
sub check_and_setup_transport_sys {
    my ($edr, $sys) = @_;
    my $msg;
    $msg=Msg::new("Checking communication on $sys->{sys}", 20, 1030, "$sys->{sys}");
    $msg->left;
    $msg->display_left($msg) if (Obj::webui());
    $sys->set_value('stop_checks', 0);
    if ($edr->transport_sys($sys)) {
        $msg->right_done;
        $msg->display_right() if (Obj::webui());
        return 0;
    } else {
        if (!EDR::local_windows() &&
            !Cfg::opt("responsefile")) {
            $edr->{com_setup}=$edr->ask_com_setup([$sys]);
            if ($edr->{com_setup} && -1 == $edr->{com_setup}) {
                $msg->right_failed;
                for my $errmsg (@{$sys->{errors}}) {
                   $msg->addError($errmsg);
                   Msg::print("$errmsg\n");
                }
                undef($sys->{errors});
                return -1;
           }
           undef($sys->{errors});
           $sys->set_value('stop_checks', 0);
           $sys->set_value('rsh', $sys->{com_setup_method});
           $sys->{rcp}=($sys->{rsh} eq 'rsh') ? 'rcp' : 'scp';
           return $edr->check_and_setup_transport_sys($sys);
        }
        return 1;
    }
}

sub ssh_setup {
    my ($edr,$syspasswd,$setup_systems,$invalid_passwords,$try_rsh_on_failure) = @_;
    my ($sysi,$local_home,$msg,$remote_home,$exp,$debug,$connected,$sys,$timeout,$localsys,@array_passwd,$keyfile,$rsh_setup_systems,$password_validate,$rem_script,$cmd);
    $timeout=20;
    $localsys=$edr->{localsys};
    $local_home = $localsys->get_root_home();
    $rem_script = $edr->{logdir}.'autocom_remove';
    # Create ssh-key file if no key/pub exist already, specified by "-keyfile" or default.
    $keyfile = Cfg::opt('keyfile');
    if ($keyfile && -f $keyfile.'.pub') {
        EDR::cmd_local("_cmd_rmr $local_home/.ssh/symc_auto.pub ; _cmd_mkdir -p $local_home/.ssh ; _cmd_chmod 0700 $local_home/.ssh ; _cmd_chown root $local_home/.ssh ;_cmd_cp $keyfile.pub $local_home/.ssh/symc_auto.pub");
    } else {
        if ($keyfile) {
            Msg::log("Pubkey missing. Using default ssh key $local_home/.ssh/id_rsa");
            Cfg::set_opt('keyfile',"$local_home/.ssh/id_rsa");
        }
        if (-f "$local_home/.ssh/id_rsa.pub" && -f "$local_home/.ssh/id_rsa") {
            EDR::cmd_local("_cmd_rmr $local_home/.ssh/symc_auto.pub ; _cmd_cp $local_home/.ssh/id_rsa.pub $local_home/.ssh/symc_auto.pub");
        } else {
            EDR::cmd_local("_cmd_rmr $local_home/.ssh/symc_auto $local_home/.ssh/symc_auto.pub ; _cmd_mkdir -p $local_home/.ssh ; _cmd_chmod 0700 $local_home/.ssh ; _cmd_chown root $local_home/.ssh ; _cmd_sshkeygen -t rsa -N \'\' -f $local_home/.ssh/symc_auto ; _cmd_cp -p $local_home/.ssh/symc_auto $local_home/.ssh/id_rsa ; _cmd_cp -p $local_home/.ssh/symc_auto.pub $local_home/.ssh/id_rsa.pub");
        }
    }

    my $error_msg=Msg::new("Cannot find file $local_home/.ssh/symc_auto.pub", 20, 1031, "$local_home");
    return $error_msg->{msg} unless (-f "$local_home/.ssh/symc_auto.pub");

    for my $sysi (@$setup_systems) {
        $sys = Obj::sys($sysi);
        $sys->{com_setup_method} = 'ssh';

        # Validate password on remote system
        $password_validate=0;
        $connected=0;
        $cmd = "LANG=C LC_ALL=C ssh -o 'StrictHostKeyChecking=no' -o 'UserKnownHostsFile=/dev/null' $sysi echo symc_validate_string";
        $exp = Expect->spawn($cmd) or EDR::die("Cannot spawn command: $!\n");

        # To print debug information, set $debug to 1
        $debug=Cfg::opt('trace') || 0;
        $exp->debug($debug);
        $exp->exp_internal($debug);

        $exp->expect($timeout,
              [ qr/Password:/i => sub { $exp = shift;
                                     $connected = 1;
                                     $exp->send("$syspasswd->{$sys->{sys}}\n");
                                     exp_continue; } ],
              [ 'yes/no' => sub { $exp = shift;
                                     $connected = 1;
                                     $exp->send("yes\n");
                                     exp_continue; } ],
              [ 'symc_validate_string' => sub { $exp = shift;
                                     $connected = 1;
                                     $password_validate=1;
                                     exp_continue; } ],
              );
        $exp->hard_close();

        if (!$password_validate) {
            if ($connected) {
                $msg = Msg::new("Failed to setup ssh communication on $sysi: Login denied", 20, 1032, "$sysi");
            } else {
                $msg = Msg::new("Failed to setup ssh communication on $sysi: Connection refused", 20, 1033, "$sysi");
            }
            $msg->print;
            if ($try_rsh_on_failure) {
                $msg = Msg::new("Trying to setup rsh communication on $sysi.", 20, 1034, "$sysi");
                $msg->print;
                $rsh_setup_systems->[0] = $sysi;
                $edr->rsh_setup($syspasswd,$rsh_setup_systems,$invalid_passwords,!'try_ssh_on_failure');
                if ( EDRu::inarr($sysi, @$invalid_passwords)) {
                    Msg::log("Failed to setup rsh communication on $sysi.");
                }
            } else {
                push @$invalid_passwords, $sysi;
            }
            next;
        }

        # Get the value of $HOME on remote system
        $cmd = "LANG=C LC_ALL=C ssh -o 'StrictHostKeyChecking=no' -o 'UserKnownHostsFile=/dev/null' $sysi grep '^root:' /etc/passwd";
        $exp = Expect->spawn($cmd) or EDR::die("Cannot spawn command: $!\n");
        $exp->exp_internal($debug);
        $exp->expect($timeout,
              [ qr/Password:/i => sub { $exp = shift;
                                     $exp->send("$syspasswd->{$sys->{sys}}\n");
                                     exp_continue; } ],
              [ 'yes/no' => sub { $exp = shift;
                                     $exp->send("yes\n");
                                     exp_continue; } ],
              [ qr/root.*/ => sub { $exp = shift;
                                     @array_passwd=split(/:/m,$exp->match());
                                     $remote_home=$array_passwd[5];
                                     exp_continue; } ],
              );
        $exp->hard_close();

        # Transfer key file with scp
        $cmd = "LANG=C LC_ALL=C scp -o 'StrictHostKeyChecking=no' -o 'UserKnownHostsFile=/dev/null' $local_home/.ssh/symc_auto.pub root\@$sys->{sys}:/var/tmp";
        $exp = Expect->spawn($cmd) or EDR::die("Cannot spawn command: $!\n");
        $exp->exp_internal($debug);
        $exp->expect($timeout,
              [ qr/Password:/i => sub { $exp = shift;
                                     $exp->send("$syspasswd->{$sys->{sys}}\n");
                                     exp_continue; } ],
              [ 'yes/no' => sub { $exp = shift;
                                     $exp->send("yes\n");
                                     exp_continue; } ],
              );
        $exp->hard_close();

        # chmod / to 0755
        $cmd = "LANG=C LC_ALL=C ssh -o 'StrictHostKeyChecking=no' -o 'UserKnownHostsFile=/dev/null' $sys->{sys} chmod 0755 /";
        $exp = Expect->spawn($cmd) or EDR::die("Cannot spawn command: $!\n");
        $exp->expect($timeout,
               [ qr/Password:/i => sub { $exp = shift;
                                     $exp->send("$syspasswd->{$sys->{sys}}\n");
                                     exp_continue; } ],
               [ 'yes/no' => sub { $exp = shift;
                                     $exp->send("yes\n");
                                     exp_continue; } ]);
        $exp->hard_close();

        # chown / to root
        $cmd = "LANG=C LC_ALL=C ssh -o 'StrictHostKeyChecking=no' -o 'UserKnownHostsFile=/dev/null' $sys->{sys} chown root /";
        $exp = Expect->spawn($cmd) or EDR::die("Cannot spawn command: $!\n");
        $exp->expect($timeout,
               [ qr/Password:/i => sub { $exp = shift;
                                     $exp->send("$syspasswd->{$sys->{sys}}\n");
                                     exp_continue; } ],
               [ 'yes/no' => sub { $exp = shift;
                                     $exp->send("yes\n");
                                     exp_continue; } ]);
        $exp->hard_close();

        # chmod $remote_home to 0750 if it is not /
        if ($remote_home ne '/') {
            $cmd = "LANG=C LC_ALL=C ssh -o 'StrictHostKeyChecking=no' -o 'UserKnownHostsFile=/dev/null' $sys->{sys} chmod 0750 $remote_home ";
            $exp = Expect->spawn($cmd) or EDR::die("Cannot spawn command: $!\n");
            $exp->expect($timeout,
                         [ qr/Password:/i => sub { $exp = shift;
                                                  $exp->send("$syspasswd->{$sys->{sys}}\n");
                                                  exp_continue; } ],
                         [ 'yes/no' => sub { $exp = shift;
                                                  $exp->send("yes\n");
                                                  exp_continue; } ]);
            $exp->hard_close();

            # chown $remote_home to root
            $cmd = "LANG=C LC_ALL=C ssh -o 'StrictHostKeyChecking=no' -o 'UserKnownHostsFile=/dev/null' $sys->{sys} chown root $remote_home ";
            $exp = Expect->spawn($cmd) or EDR::die("Cannot spawn command: $!\n");
            $exp->expect($timeout,
                         [ qr/Password:/i => sub { $exp = shift;
                                                  $exp->send("$syspasswd->{$sys->{sys}}\n");
                                                  exp_continue; } ],
                         [ 'yes/no' => sub { $exp = shift;
                                                  $exp->send("yes\n");
                                                  exp_continue; } ]);
            $exp->hard_close();
        }

        # mkdir $remotehome/.ssh
        $cmd = "LANG=C LC_ALL=C ssh -o 'StrictHostKeyChecking=no' -o 'UserKnownHostsFile=/dev/null' $sys->{sys} mkdir -p $remote_home/.ssh";
        $exp = Expect->spawn($cmd) or EDR::die("Cannot spawn command: $!\n");
        $exp->expect($timeout,
               [ qr/Password:/i => sub { $exp = shift;
                                     $exp->send("$syspasswd->{$sys->{sys}}\n");
                                     exp_continue; } ],
               [ 'yes/no' => sub { $exp = shift;
                                     $exp->send("yes\n");
                                     exp_continue; } ]);
        $exp->hard_close();

        # chmod $remotehome/.ssh
        $cmd = "LANG=C LC_ALL=C ssh -o 'StrictHostKeyChecking=no' -o 'UserKnownHostsFile=/dev/null' $sys->{sys} chmod 700 $remote_home/.ssh";
        $exp = Expect->spawn($cmd) or EDR::die("Cannot spawn command: $!\n");
        $exp->expect($timeout,
               [ qr/Password:/i => sub { $exp = shift;
                                     $exp->send("$syspasswd->{$sys->{sys}}\n");
                                     exp_continue; } ],
               [ 'yes/no' => sub { $exp = shift;
                                     $exp->send("yes\n");
                                     exp_continue; } ]);
        $exp->hard_close();

        # Add key file as authorized key
        $cmd = "LANG=C LC_ALL=C ssh -o 'StrictHostKeyChecking=no' -o 'UserKnownHostsFile=/dev/null' $sys->{sys} cat /var/tmp/symc_auto.pub \\>\\> $remote_home/.ssh/authorized_keys";
        $exp = Expect->spawn($cmd) or EDR::die("Cannot spawn command: $!\n");
        $exp->expect($timeout,
               [ qr/Password:/i => sub { $exp = shift;
                                     $exp->send("$syspasswd->{$sys->{sys}}\n");
                                     exp_continue; } ],
               [ 'yes/no' => sub { $exp = shift;
                                     $exp->send("yes\n");
                                     exp_continue; } ]);
        $exp->hard_close();

        # Delete temp symc_auto.pub on remote system
        $cmd = "LANG=C LC_ALL=C ssh -o 'StrictHostKeyChecking=no' -o 'UserKnownHostsFile=/dev/null' $sys->{sys} rm -f /var/tmp/symc_auto.pub";
        $exp = Expect->spawn($cmd) or EDR::die("Cannot spawn command: $!\n");
        $exp->expect($timeout,
               [ qr/Password:/i => sub { $exp = shift;
                                     $exp->send("$syspasswd->{$sys->{sys}}\n");
                                     exp_continue; } ],
               [ 'yes/no' => sub { $exp = shift;
                                     $exp->send("yes\n");
                                     exp_continue; } ]);
        $exp->hard_close();

        # chmod $remotehome/.ssh/authorized_keys
        $cmd = "LANG=C LC_ALL=C ssh -o 'StrictHostKeyChecking=no' -o 'UserKnownHostsFile=/dev/null' $sys->{sys} chmod 600 $remote_home/.ssh/authorized_keys";
        $exp = Expect->spawn($cmd) or EDR::die("Cannot spawn command: $!\n");
        $exp->expect($timeout,
               [ qr/Password:/i => sub { $exp = shift;
                                     $exp->send("$syspasswd->{$sys->{sys}}\n");
                                     exp_continue; } ],
               [ 'yes/no' => sub { $exp = shift;
                                     $exp->send("yes\n");
                                     exp_continue; } ]);
        $exp->hard_close();

        # restore SELinux context of $remotehome/.ssh/authorized_keys
        $cmd = "LANG=C LC_ALL=C ssh -o 'StrictHostKeyChecking=no' -o 'UserKnownHostsFile=/dev/null' $sys->{sys} /sbin/restorecon -v $remote_home/.ssh/authorized_keys";
        $exp = Expect->spawn($cmd) or EDR::die("Cannot spawn command: $!\n");
        $exp->exp_internal($debug);
        $exp->expect($timeout,
               [ qr/Password:/i => sub { $exp = shift;
                                     $exp->send("$syspasswd->{$sys->{sys}}\n");
                                     exp_continue; } ],
               [ 'yes/no' => sub { $exp = shift;
                                     $exp->send("yes\n");
                                     exp_continue; } ]);
        $exp->hard_close();
    }
    EDR::cmd_local("_cmd_rmr $local_home/.ssh/symc_auto $local_home/.ssh/symc_auto.pub");
    if (scalar @$invalid_passwords>0) {
        return 0;
    } else {
        return 1;
    }
}

sub rsh_setup {
    my ($edr,$syspasswd,$setup_systems,$invalid_passwords,$try_ssh_on_failure) = @_;
    my ($sysi,$msg,@fields,$remote_home,$ssh_setup_systems,$exp,$debug,$connected,$sys,$timeout,$hostname,$host,@exp_ret,$cmd,$remote_os);
    $timeout=20;
    for my $sysi (@$setup_systems) {
        $sys = Obj::sys($sysi);
        $sys->{com_setup_method} = 'rsh';

        # Login to remote system with rlogin and set PS1 for following commands.
        if (Obj::localplat() eq 'HPUX') {
            $cmd = "LANG=C LC_ALL=C remsh $sysi";
        } else {
            $cmd = "LANG=C LC_ALL=C rsh $sysi";
        }

        $connected = 0;
        $exp = Expect->spawn($cmd) or EDR::die("Cannot spawn command: $!\n");

        # To print debug information, set $debug to 1
        $debug=Cfg::opt('trace') || 0;
        $exp->debug($debug);
        $exp->exp_internal($debug);

        @exp_ret = $exp->expect($timeout,
                                [
                                    qr/Password:/i => sub {
                                        $exp = shift;
                                        $connected = 1;
                                        $exp->send("$syspasswd->{$sys->{sys}}\r");
                                        exp_continue;
                                    }
                                ],
                                [
                                    qr/#/ => sub {
                                        $exp = shift;
                                        $exp->send("PS1='SYMCPrompt>'\n");
                                        exp_continue;
                                    }
                                ],
                                '-re','SYMCPrompt>$',
                            );

        if ( $exp_ret[2] ne 'SYMCPrompt>') {
            $exp->send("$syspasswd->{$sys->{sys}}\r");
            sleep 10;
            $exp->send("PS1='SYMCPrompt>'\n");
            @exp_ret = $exp->expect($timeout, '-re', 'SYMCPrompt>$');
            if ($exp_ret[2] ne 'SYMCPrompt>') {
                if ($connected) {
                    $msg = Msg::new("Failed to setup rsh communication on $sysi: Login denied", 20, 1035, "$sysi");
                } else {
                    $msg = Msg::new("Failed to setup rsh communication on $sysi: Connection refused", 20, 1036, "$sysi");
                }
                $msg->print;
                if ($try_ssh_on_failure && !Cfg::opt('rsh')) {
                    $msg = Msg::new("Trying to setup ssh communication on $sysi.", 20, 1037, "$sysi");
                    $msg->print;
                    $ssh_setup_systems->[0] = $sysi;
                    $edr->ssh_setup($syspasswd,$ssh_setup_systems,$invalid_passwords,!'try_rsh_on_failure');
                    if ( EDRu::inarr($sysi, @$invalid_passwords)) {
                        Msg::log("Failed to setup ssh communication on $sysi.");
                    }
                } else {
                    push @$invalid_passwords, $sysi;
                }
                next;
            }
        }

        # get home dir on remote system.
        $exp->send("LANG=C LC_ALL=C echo HOMEDIR:\$HOME\n");
        @exp_ret=$exp->expect($timeout,'-re','^HOMEDIR:.*$');
        $remote_home=$exp_ret[2];
        chomp $remote_home;
        $remote_home=~s/HOMEDIR://m;
        $remote_home=~s/\r+$//m;
        $remote_home=~s/\/+$//m;

        # get OS on remote system.
        $exp->expect($timeout,'SYMCPrompt>');
        $exp->send("LANG=C LC_ALL=C echo OS:\`uname -s\`\n");
        @exp_ret=$exp->expect($timeout,'-re','^OS:.*$');
        $remote_os=$exp_ret[2];
        chomp $remote_os;
        $remote_os=~s/^OS://m;
        $remote_os=~s/\r+$//m;

        # get the hostname or ip address of local node recognied by remote system.
        $exp->expect($timeout,'SYMCPrompt>');
        if ( $remote_os eq 'HP-UX' ) {
            $exp->send("LANG=C LC_ALL=C who -u am i\n");
        } else {
            $exp->send("LANG=C LC_ALL=C who am i\n");
        }
        @exp_ret=$exp->expect($timeout,'-re','^root.*$');
        @fields=split /\s/m, $exp_ret[2];
        $host=$fields[-1];
        chomp $host;
        if ( $host =~ /\([^)]*\)/m) {
            $host=~s/\(([^)]*)\)/$1/mx;
        }

        # add entry in /etc/hosts on remote system if needed.
        if (EDRu::isip($host)) {
            $hostname = $edr->{localsys}->{sys};
            $exp->expect($timeout,'SYMCPrompt>');
            $exp->send("LANG=C LC_ALL=C echo \"$host $hostname\" >> /etc/hosts \n");
            $host=$hostname;
        }
        
        #e3437549:record the host name when setup rsh password-less connection
        $sys->set_value('transport_setup_rsh_host', $host);
       
        # add entry in ~/.rhosts on remote system
        $exp->expect($timeout,'SYMCPrompt>');
        $exp->send("LANG=C LC_ALL=C echo \"$host\" >> $remote_home/.rhosts \n");
        # set permission mode on ~/.rhosts
        $exp->expect($timeout,'SYMCPrompt>');
        $exp->send("LANG=C LC_ALL=C chmod 0600 $remote_home/.rhosts \n");
        # set ownership on ~/.rhosts
        $exp->expect($timeout,'SYMCPrompt>');
        $exp->send("LANG=C LC_ALL=C chown root $remote_home/.rhosts \n");

        $exp->expect($timeout,'SYMCPrompt>');
        $exp->hard_close();

        # make sure normal rsh (not rlogin) is working.
        $cmd.=" echo 'symc_validate_string'";
        $exp = Expect->spawn($cmd) or EDR::die("Cannot spawn command: $!\n");
        @exp_ret = $exp->expect($timeout,'-re','symc_validate_string');
        if ($exp_ret[1]) {
            $msg = Msg::new("Failed to setup rsh communication on $sysi: 'rsh $sysi <command>' failed", 20, 1038, "$sysi", "$sysi");
            $msg->print;
            if ($try_ssh_on_failure && !Cfg::opt('rsh')) {
                $msg = Msg::new("Trying to setup ssh communication on $sysi.", 20, 1037, "$sysi");
                $msg->print;
                $ssh_setup_systems->[0] = $sysi;
                $edr->ssh_setup($syspasswd,$ssh_setup_systems,$invalid_passwords,!'try_rsh_on_failure');
                if ( EDRu::inarr($sysi, @$invalid_passwords)) {
                    Msg::log("Failed to setup ssh communication on $sysi.");
                }
            } else {
                push @$invalid_passwords, $sysi;
            }
        }
    }
    if ( scalar @$invalid_passwords > 0) {
        return 0;
    } else {
        return 1;
    }
}

1;
