#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
package Sys;
use strict;
@Sys::ISA = qw(Obj);

sub init {
    my $sys=shift;
    $sys->{sys}=shift;
    return;
}

sub aix { return 1 if ($_[0]->{plat} eq 'AIX'); return 0; }
sub hpux { return 1 if ($_[0]->{plat} eq 'HPUX'); return 0; }
sub linux { return 1 if ($_[0]->{plat} eq 'Linux'); return 0; }
sub sunos { return 1 if ($_[0]->{plat} eq 'SunOS'); return 0; }
sub sunos_sol11 {return 1 if (($_[0]->{plat} eq 'SunOS') && ($_[0]->{platvers} eq '5.11')); return 0; }
sub freebsd { return 1 if ($_[0]->{plat} eq 'FreeBSD'); return 0; }
sub macos { return 1 if ($_[0]->{plat} eq 'MacOS'); return 0; }
# modify if padvs change
sub windows { return 1 if ($_[0]->{plat}=~/Win/mx); return 0; }

sub padv {
    my $sys=shift;
    my $pool="Padv::$sys->{padv}";
    EDR::die("No padv object defined for padv $sys->{padv} for system $sys->{sys}")
        unless (Obj::created($pool));
    return Obj::pool($pool);
}

sub padvisa {
    my ($padvisa,$pool,$sys,$tmppadv);
    $sys=shift;
    $padvisa=EDR::get2('padvisa', $sys->{padv});
    # padvisa is not defined for alternate plat localsys in a2m scenario
    $padvisa||=$sys->{padv};
    $pool="Padv::$padvisa";
    # don't die, just create it
    $tmppadv = $sys->{padv};
    return "Padv\::$tmppadv"->new() unless (Obj::created($pool));
    #EDR::die("No padv object defined for padvisa $padvisa for system $sys->{sys}")
    #    unless (Obj::created($pool));
    return Obj::pool($pool);
}

sub obj {
    my ($type,$sys,$obj,$donotdie) = @_;
    my ($padv,$pool);
    # for Single platform releases, return objects initialized to $cpic->{padv}
    # which is the only object were using for CPIP even if $cpic->{padvs} has >1
    # for Any to Many releases, return the object initialized to $cpic->{padvisa}{$sys->{padv}}
    # under the assumption the object has been created by create_system_objects
    $padv=EDR::get('padv');
    $padv=CPIC::get2('padvisa', $sys->{padv}) if ($padv eq 'Common');
    if(!$padv){
        return if($donotdie);
        EDR::die("obj($sys,$obj,$type) cpic->{padvisa}{$sys->{padv}} is not defined");
    }
    $pool=join('::', $type, $obj, $padv);
    if(!Obj::created($pool)){
        return if($donotdie);
        EDR::die("No $type object defined for $type $obj and padv $padv for system $sys->{sys}");
    }
    return Obj::pool($pool);
}

sub rel { return obj('Rel',$_[0], CPIC::get('release')); }

sub prod {
    my($sys,$prod)=@_;
    $prod||=CPIC::get('prod');
    return obj('Prod',$sys, $prod);
}

sub pkg { return obj('Pkg', @_); }
sub proc { return obj('Proc', @_); }
sub patch { return obj('Patch', @_); }
sub eeb { return obj('EEB', @_); }

# return the package description.
sub pkgdesc {
    my ($sys,$pkg)= @_;
    return $sys->padv->pkg_description_sys($sys, $pkg) || '';
}

# returns the version of a package installed on a system
# could use $sys->{pkgvers}{$pkg->{pkg}}, but it is a bit of a char saver
# and a place to hack around things like VRTSvail.VRTSvail
sub pkgvers {
    my ($sys,$pkg)= @_;
    if (ref($pkg)) {
        return $sys->{pkgvers}{$pkg->{pkgname}} if ($pkg->{pkgname});
        return $sys->{pkgvers}{$pkg->{pkg}};
    }
    return $sys->{pkgvers}{$pkg};
}

# doing the same with pkgdeps
sub pkgdeps {
    my ($sys,$pkg)= @_;
    my $pkgdeps=(ref($pkg)) ? $sys->{pkgdeps}{$pkg->{pkg}} : $sys->{pkgdeps}{$pkg};
    return $pkgdeps || '';
}

# returns the patch index
sub patchindex {
    my ($sys,$arch);
    $sys=shift;

    if (($sys->hpux()) && ($sys->{platvers}>=11.31)) {
        $arch=($sys->{arch} eq 'ia64') ? 'IA' : 'PA';
        return "$sys->{platvers}$arch";
    }
    return "$sys->{platvers}";
}

# check if sys is the first system
sub system1 {
    my ($sys,$edr,$sys0);
    $sys = shift;
    $edr=Obj::edr();
    $sys0 = ${$edr->{systems}}[0];
    return 1 if ($sys->{sys} eq $sys0->{sys});
    return 0;
}

sub cmd_bin {
    my ($sys,$key) = @_;
    my ($padv,$cmd);
    for my $padv ($sys->padv, $sys->padvisa) {
        $cmd=$padv->{cmd}{$key};
        return $cmd if ($cmd);
    }
    return '';
}

sub cmdswap {
    my ($sys,$cmd) = @_;
    my ($msg,$padv,$key,$rtn,$swap);
    return $cmd if ($cmd!~/_cmd_/m);
    EDR::die("Cannot execute $cmd on $sys->{sys} with undefined sys->{padv}") if (!$sys->{padv});
    EDR::die("Cannot execute $cmd on $sys->{sys} with uninitialized padv $sys->{padv}")
        unless (Obj::created("Padv::$sys->{padv}"));
    # commands defined within CPIP inits may be defined to lower $sys->padvisa level
    for my $padv ($sys->padv, $sys->padvisa) {
        for my $key (sort keys(%{$padv->{cmd}})) {
            next unless ($cmd=~/_cmd_$key/m);
            $swap=$padv->{cmd}{$key};
            $cmd=~s/_cmd_$key\b/$swap/mxg;
            return $cmd if ($cmd!~/_cmd_/m);
        }
    }
    EDR::die("undefined cmd substitution:\n$cmd\nsys=$sys->{sys}\npadv=$sys->{padv}\npadvisa=".$sys->padvisa);
    return;
}

sub cmd {
    my ($sys,$cmd,$env) = @_;
    my ($edr,$msg,$background,$multicmds,$lang,$comtest);
    my ($rsh_bin,$rsh_options,$ssh,$sshpath,$shell,$rtn_out,$rtn_err,$rtn,$exitcode,$local_exitcode);
    my ($localpadv,$keyfile,$orig_cpu_priority,$begin,$duration,$counter);

    EDR::die("Sys object not passed as first arg to cmd(@_)") if (ref($sys) ne 'Sys');

    $edr=Obj::edr();
    $comtest=1 if ($cmd eq 'echo' || $cmd eq 'echo Symantec');

    $cmd=$sys->cmdswap($cmd) if ($cmd=~/_cmd_/m);
    $cmd=~s/\/dev\/null/nul/g if ($sys->windows);

    $cmd=$env . " $cmd" if ($env);
    $cmd=$sys->{env} . " $cmd" if ($sys->{env});

    if ($sys->{islocal}) {
        $cmd.=" 2>&1" unless ($cmd=~/2>/);
    } elsif ($sys->{wmi}) {
        return Padv::Win::remote_win_cmd_sys($sys,$cmd);
    } else {
        $sshpath=Cfg::opt('sshpath');
        return 0 unless ($sys->{rsh});
        if ($sys->{rsh} eq 'ssh' && $sshpath) {
            $ssh = "$sshpath $sys->{sys}";
        } else {
            $rsh_options='';
            if ($sys->{rsh} eq 'ssh') {
                $rsh_options='-x -o NumberOfPasswordPrompts=0 -o StrictHostKeyChecking=no';
                $keyfile=Cfg::opt('keyfile'); # if system is in systemlist check?
                if ($keyfile) {
                    $rsh_options.=" -i $keyfile";
                }
            }
            $localpadv=Obj::padv($edr->localpadv);
            $rsh_bin=$localpadv->{cmd}{$sys->{rsh}};
            $ssh = "$rsh_bin $rsh_options $sys->{sys}";
        }

        $background=1 if ($cmd =~ /&\s*$/m);
        $multicmds=1 if ($cmd =~ /[\|;]/m);

        $cmd =~ s/\\/\\\\/g;
        $cmd =~ s/\`/\\\`/mg;
        $cmd =~ s/\"/\\\"/mg;
        $cmd =~ s/\$/\\\$/mg;

        # do not use "$cmd 2>&1", "LANG=C $cmd" and "$cmd; echo $?"
        # when the remote root shell is not determined, 
        # since they do not work well with csh/tcsh. 
        if ($sys->{shell}) {
            # if the remote shell is determined, then append redirection,
            $cmd.=" 2>&1" if ($cmd!~/2>/);

            $lang='LC_ALL=C LANG=C';
            $lang='LC_ALL=C; LANG=C; export LC_ALL LANG;' if ($multicmds);
            $cmd="$lang $cmd";

            # if using ssh or the command is for background running, need not use _CMD_EXIT_
            # if using rsh and not background command, then need use _CMD_EXIT_
            if ($sys->{rsh} ne 'ssh' && !$background) {
                $cmd="$cmd; echo _CMD_EXIT_=\\\$?";
            }
        }

        if ($sys->{shell} && $sys->{shell}=~/csh/m) {
            # if the remote root shell is tcsh/csh, then use padv specific sh.
            $shell=$sys->padv->{cmd}{sh} if ($sys->{padv});
            $shell||='/bin/sh';
           
            $cmd = "echo \"$cmd\" | $ssh $shell 2>&1";
        } else {
            # the remote root shell is bash/ksh/sh
            # or in initial stage when the remote root shell is not determined.
            # Notes: make sure the initial commands be csh/sh redirection compatible.
            $cmd = "$ssh \"$cmd\" 2>&1";
        }
    }

    # set $edr->{cmdfile} for main thread to track timeouts in threaded mode
    $edr->{cmdfile}||=EDR::cmdfile(Obj::tid()) if (!$sys->{notimeout});
    EDRu::writefile($cmd,$edr->{cmdfile}) if ($edr->{cmdfile});
    Msg::log("cmd $cmd");
    $begin=time();

    # To solve the ssh hang issue, temporarily lower the CPU priority before run the SSH commands
    # So that less chances to switch out the local ssh processes, and thus less chances to hang
    # For more ssh hang informations, check: http://www.snailbook.com/faq/background-jobs.auto.html
    $orig_cpu_priority=$sys->lower_cpu_priority();
    $counter = 0;
    while (1) {
        $rtn_out=$sys->run_cmd_with_backtick($cmd);
        $rtn_err=$edr->{cmdstderr};
        $exitcode=$edr->{cmdexit};
        $local_exitcode=$edr->{local_cmdexit};
        last if ($sys->{islocal} || $sys->{rsh} eq 'ssh');
        if ($local_exitcode &&
            (!$rtn_out ||
             $rtn_out=~/rcmd: socket: Cannot assign requested address/m ||
             $rtn_out=~/rcmd_af: primary connection shut down/m ||
             $rtn_out=~/poll: protocol failure in circuit setup/m
            )) {
            $duration=time()-$begin;
            last if ($duration > 75);
            # exceed upper limit of system concurrent TCP connections used by rsh (512/60s, ports lower than 1024)
            # wait 15 seconds for any TCP connection from time_wait to close
            # we detect this condition with the stdout: rcmd: socket: Cannot assign requested address
            # but sometime rsh just fail silently with exit_code 1 and not stdout, we consider this as well
            Msg::log("Local system exceed the upper limit of concurrent TCP connections used by rsh, sleep 15 seconds and retry $counter");
            sleep 15;
            $counter++;
            last if ($counter > 5);
            # give up if more than 75 seconds
        } else {
            last;
        }
    }
    unlink($edr->{cmdfile}) if ($edr->{cmdfile});
    $sys->restore_cpu_priority($orig_cpu_priority);

    $duration=time() - $begin;

    Msg::log("cmd stdout=\n$rtn_out") if ($rtn_out && ($cmd!~/tar -xvf/m));
    Msg::log("cmd stderr=\n$rtn_err") if ($rtn_err);
    Msg::log("cmd exit=$exitcode (duration: $duration seconds)");

    if (!$comtest && !$sys->{islocal} && (($local_exitcode == 255 && $sys->{rsh} eq 'ssh') || ($local_exitcode == 1 && $sys->{rsh} ne 'ssh'))) {
        $sys->cmd('echo');
        if ($edr->{cmdexit}) {
            # skip doing cleanup on $sys since $sys cannot be connected
            $sys->{cleanup} = 0;
            $msg = Msg::new("$edr->{script} cannot continue because remote system $sys->{sys} cannot be connected\nPlease check network connection and system state of $sys->{sys} and then try again\n", 20, 1184, "$edr->{script}", "$sys->{sys}", "$sys->{sys}");
            print "\n\n";
            $msg->die();
        } else {
            # restore original cmd stdout/stderr/exitcode if $sys is still alive
            $edr->{cmdexit} = $exitcode;
            $edr->{cmdstdout} = $rtn_out;
            $edr->{cmdstderr} = $rtn_err;
            $edr->{local_cmdexit} = $local_exitcode;
        }
    }
    return $rtn_out;
}

sub lower_cpu_priority {
    my $sys=shift;
    my ($localsys,$orig_cpu_priority);

    # do not setpriority if not runing with rsh or ssh
    return if (!$sys->{rsh});
    return if ($ENV{CPI_NO_PRIORITY});

    # Still see some strange hang on linux when running BAIT.
    # so only setpriority if localsys is HPUX to check whether any improvements for HPUX.
    $localsys=EDR::get('localsys');
    return unless ($localsys->hpux());

    eval {
        $orig_cpu_priority=getpriority(0, 0);
        setpriority(0, 0, $orig_cpu_priority-20);
    };

    return $orig_cpu_priority;
}

sub restore_cpu_priority {
    my ($sys,$cpu_priority)=@_;
    if ($sys->{rsh} && (defined $cpu_priority)) {
        eval{setpriority(0, 0, $cpu_priority)};
    }
    return;
}

sub run_cmd_with_backtick {
    my ($sys,$cmd)=@_;
    my ($echo_cmd_exit,$edr);
    my ($exitcode,$rtn_out,$rtn_err,$local_exitcode);

    $exitcode=undef;
    $echo_cmd_exit=1 if ($cmd=~/_CMD_EXIT_/m);

    $rtn_out=`$cmd`;
    $local_exitcode = $?;
    if ($echo_cmd_exit && $rtn_out=~/^(.*)_CMD_EXIT_=\s*(-?\d+)/mxs) {
        $rtn_out="$1";
        $exitcode="$2";
    }

    $edr=Obj::edr();

    # Get STDOUT ouput
    chomp($rtn_out);
    $rtn_out=$sys->remove_ssh_banner($rtn_out);
    $edr->{cmdstdout}=$rtn_out;

    # Get STDERR ouput
    $edr->{cmdstderr}="";

    # Get the exit code for the command
    if (! defined $exitcode) {
        $exitcode=$local_exitcode;
        $exitcode >>= 8 if ($exitcode > 0);
    }
    $exitcode -= 256 if ($exitcode > 127);
    $edr->{cmdexit}=$exitcode || 0;
    $local_exitcode >>= 8 if ($local_exitcode > 255);
    $edr->{local_cmdexit}=$local_exitcode;

    return $rtn_out;
}

# Remove SSH banner
sub remove_ssh_banner {
    my ($sys,$rtn)= @_;
    return "$rtn" unless ($sys->{banner});
    return '' if ($rtn eq $sys->{banner});
    $rtn=~s/\Q$sys->{banner}\E\n?//xg;
    return $rtn;
}

sub cmd_bg {
    my ($sys,$cmd)= @_;
    return $sys->cmd("nohup $cmd </dev/null >/dev/null 2>&1 &");
}

sub cmd_script {
    my ($sys,$script,$filename,$shell) = @_;
    my ($file,$edr,$rtn,$tmpdir,$cmd);

    $filename||='script_' . EDRu::randomstr(5);
    $edr=Obj::edr();
    $tmpdir=$edr->{tmpdir};
    $file="$tmpdir/$filename.$sys->{sys}";
    EDRu::writefile($script, $file);
    $edr->localsys->copy_to_sys($sys,$file,$tmpdir) unless ($sys->{islocal});
    if ($shell && $shell eq 'sh') {
        $cmd="_cmd_sh $file";
    } else {
        $sys->cmd("_cmd_chmod +x $file");
        $cmd="$file";
    }
    $rtn=$sys->cmd("$cmd");

    # For e1789748 and e2377076:
    # Sometimes, the script run into 'text busy' on local machine on AIX and Linux, re-run it again.
    if ($sys->{islocal} && ($sys->aix() || $sys->linux()) && ($rtn=~/(text busy|Text file busy)/m)) {
        Msg::log('Run into text busy, re-run again');
        sleep 1;
        $rtn=$sys->cmd("$cmd");
    }
    return $rtn;
}

sub set_value {
    my $sys=shift;
    Obj::set_value($sys->{pool}, @_);
    Thr::setq($sys->{pool}, @_) if (EDR::thread_support());
    return;
}

sub push_fatal_stop_proc {
    my ($sys,$proc)=@_;
    $sys->set_value('fatalstopfailprocs', 'push', $proc->{proc}) if ($proc->{fatal});
    return;
}

sub push_stop_checks_error {
    my ($sys,$msg)=@_;
    $sys->push_error($msg);
#print "push_stop_checks_error($sys->{sys},\n$msg->{msg})\n";
    Msg::log("stop_checks error on $sys->{sys}:\n$msg->{msg})");
    $sys->set_value('stop_checks', 1);
    return;
}

sub push_error {
    my ($sys,@msgs)=@_;

    my $umi_msg=Msg::umi_msg('CPI', 'ERROR', @msgs);
    return unless ($umi_msg);

    Msg::log($umi_msg);
    $sys->set_value('errors', 'push', $umi_msg);
    return;
}

sub push_warning {
    my ($sys,@msgs)=@_;

    my $umi_msg=Msg::umi_msg('CPI', 'WARNING', @msgs);
    return unless ($umi_msg);

    Msg::log($umi_msg);
    $sys->set_value('warnings', 'push', $umi_msg);
    return;
}

sub push_note {
    my ($sys,@msgs)=@_;

    my $umi_msg=Msg::umi_msg('CPI', 'NOTE', @msgs);
    return unless ($umi_msg);

    Msg::log($umi_msg);
    $sys->set_value('notes', 'push', $umi_msg);
    return;
}

sub print {
    my $sys=shift;
    print $sys->{sys}.":\n".EDRu::hash2def($sys, 'sys')."\n";
    return;
}

sub chown_root {
    my ($sys,$file) = @_;
    return $sys->padv->chown_root_sys($sys,$file);
}

sub get_root_home {
    my $sys = shift;
    return $sys->padv->get_root_home_sys($sys);
}

# get the netmask currently configured on the base IP of a NIC
sub defaultnetmask {
    my ($sys,$ip,$nic) = @_;
    my ($nm,$n,$hnm,@f,$ipv,$h);
    $ipv = '4' if (EDRu::ip_is_ipv4($ip));
    $ipv = '6' if (EDRu::ip_is_ipv6($ip));
    $hnm=$sys->padv->netmask_sys($sys, $nic, $ipv) if ($nic);
    if ($hnm) {
        if (($ipv==4) && ($sys->{ipv4})) {
            @f=split(/\./m,$hnm);
            return "$hnm" if ($#f==3);
                $hnm=~s/^0x//m;
            for my $n (0..3) {
                $h = hex(substr($hnm,$n*2,2));
                $nm .= $h;
                $nm .= '.' if ($n<3);
            }
        } elsif (($ipv==6) && ($sys->{ipv6})) {
            # When coming here, the prefix length is available,
            # figure out the netmask by the prefix
            $nm = $hnm;
            # When the requirement for IPv6 netmask isn't prefix,
            # uncomment the 3 lines below
            #$nm = ip_get_mask($nm, 6);
            #$nm = ip_bintoip($nm,6);
            #$nm = ip_compress_address($nm, 6);
        } else {
            $nm = '';
        }
    } else {
        # No mask/prefix detected by netmask_sys,
        # we need to figure out one by ourselves
        if (($ipv==4) && ($sys->{ipv4})) {
            $ip =~ s/\..*$//m;
            $nm = ($ip<128) ? '255.0.0.0' :
                  ($ip<192) ? '255.255.0.0' : '255.255.255.0';
        } elsif (($ipv==6) && ($sys->{ipv6})) {
            $nm = '';
            # Hardcode the netmask when $ip is not LINK-LOCAL or UNSPECIFIED
            # or LOOPBACK for it can work correctly with SITE-LOCAL and GLOBAL
            # IPv6 address.
            my $type = EDRu::iptype_ipv6($ip);
            # uncomment line below to return IPv6-address-formated netmask
            #$nm = "ffff:ffff:ffff:ffff::" if ($type =~ m/SITE-LOCAL|GLOBAL/i);
            $nm = '64' if ($type =~ m/SITE-LOCAL|GLOBAL/i);
        } else {
            $nm = '';
        }
    }
    return "$nm";
}

sub pkgs_patches {
    my $sys = shift;
    return $sys->padv->pkgs_patches_sys($sys);
}

sub install_pkgs {
    my ($sys,$archive,$pkgs) = @_;
    return $sys->padv->pkgs_install_sys($sys,$archive,$pkgs);
}

sub uninstall_pkgs {
    my ($sys,$pkgs) = @_;
    return $sys->padv->pkgs_uninstall_sys($sys,$pkgs);
}

sub install_patches {
    my ($sys,$archive,$patches) = @_;
    return $sys->padv->patches_install_sys($sys,$archive,$patches);
}

sub uninstall_patches {
    my ($sys,$patches) = @_;
    return $sys->padv->patches_uninstall_sys($sys,$patches);
}

sub proc_pids {
    my ($sys,$proc) = @_;
    my (@pids,$padv,$rsh,$ssh,$procs);
    $padv=$sys->padv;
    $procs=$padv->procs_sys($sys,$proc);
    for my $proc (split(/\n/,$procs)) {
        if ($sys->{islocal}) {
            $rsh=$padv->{cmd}{rsh};
            $ssh=$padv->{cmd}{ssh};
            next if ($rsh && $ssh && $proc=~/$rsh|$ssh/m);
        }
        if ($proc=~/^\s*\S+\s*(\d+)/mx) {
            push(@pids, $1);
        }
    }
    return \@pids;
}

# kill a list of processes
# return a list of unkilled pids if kill failed
sub kill_pids {
    my ($sys, @pids) = @_;
    return $sys->padv->kill_pids_sys($sys,@pids);
}

# verify an entered NIC is present on the system
sub is_nic {
    my ($sys,$nic) = @_;
    return $sys->padv->is_nic_sys($sys, $nic);
}

# handle file and directory specific actions for Windows and Unix
sub filename_is_absolute {
    my ($sys,$file)=@_;

    return unless ($file);
    return $sys->padv->filename_is_absolute($file);
}

sub exists {
    my ($sys,$file)=@_;

    return unless ($file);
    return $sys->padv->exists_sys($sys,$file);
}

sub is_dir {
    my ($sys,$dir)=@_;
    my ($rtn);

    return unless ($dir);

    if ($sys->{islocal}) {
        $rtn = (-d $dir) ? 1 : 0;
    } else {
        $rtn=$sys->padv->is_dir_sys($sys,$dir);
    }
    return $rtn;
}

sub is_file {
    my ($sys,$file)=@_;
    my ($rtn);

    return unless ($file);

    if ($sys->{islocal}) {
        $rtn = (-f $file) ? 1 : 0;
    } else {
        $rtn=$sys->padv->is_file_sys($sys,$file);
    }
    return $rtn;
}

sub is_symlink {
    my ($sys,$symlink)=@_;
    my ($rtn);

    return unless ($symlink);

    if ($sys->{islocal}) {
        $rtn = (-l $symlink) ? 1 : 0;
    } else {
        $rtn=$sys->padv->is_symlink_sys($sys,$symlink);
    }
    return $rtn;
}

sub is_executable {
    my ($sys,$file)=@_;
    my ($rtn);

    return unless ($file);

    if ($sys->{islocal}) {
        $rtn = (-X $file) ? 1 : 0;
    } else {
        $rtn=$sys->padv->is_executable_sys($sys,$file);
    }
    return $rtn;
}

sub catfile {
    my ($sys,$file)=@_;
    return $sys->padv->catfile_sys($sys,$file);
}

sub grepfile {
    my ($sys,$word,$file)=@_;
    return $sys->padv->grepfile_sys($sys,$word,$file);
}

sub lsfile {
    my ($sys,$file)=@_;
    return $sys->padv->lsfile_sys($sys,$file);
}

sub readfile {
    my ($sys,$file,$logflag)=@_;
    my ($rtn);

    return unless ($file && $sys->is_file($file));

    if ($sys->{islocal}) {
        $rtn = EDRu::readfile($file);
    } else {
        $rtn = $sys->padv->readfile_sys($sys,$file);
    }
    Msg::log("File contents of file '$file':\n$rtn") if ($logflag);
    return $rtn;
}

sub writefile {
    my ($sys,$msg,$file)=@_;
    my ($rtn);

    return unless ($msg && $file);

    if ($sys->{islocal}) {
        $rtn = EDRu::writefile($msg,$file);
    } else {
        $rtn = $sys->padv->writefile_sys($sys,$msg,$file);
    }
    return $rtn;
}

sub appendfile {
    my ($sys,$msg,$file)=@_;
    my ($rtn);

    return unless ($msg && $file);
    if ($sys->{islocal}) {
        $rtn = EDRu::appendfile($msg,$file);
    } else {
        $rtn = $sys->padv->appendfile_sys($sys,$msg,$file);
    }
    return $rtn;
}

sub createfile {
    my ($sys,$file,$size)=@_;

    return unless ($file);
    return $sys->padv->createfile_sys($sys,$file,$size);
}

sub copyfile {
    my ($sys,$file_src,$file_dest)=@_;

    return 0 unless ($file_src && $file_dest);
    return $sys->padv->copyfile_sys($sys,$file_src,$file_dest);
}

sub copy_to_sys {
    my ($sys_src,$sys_dest,$file_src,$file_dest,$ddoe)=@_;
    my ($filename,$stat,$rtn,$tmpdir);

    EDR::assert((defined $file_src), 'file_src arg should be defined');

    $file_dest||=$file_src;

    # keep the file stat same as before, if the target file is not in the EDR::tmpdir
    $tmpdir=EDR::tmpdir();
    $filename = $file_dest;
    unless ($file_dest =~ /$tmpdir/) {
        if ($sys_dest->is_dir($file_dest)) {
            $filename = "$file_dest/" . EDRu::basename($file_src);
        }
        $stat=$sys_dest->filestat($filename);
    }

    $rtn = EDRu::copy($sys_src,$file_src,$sys_dest,$file_dest,$ddoe);
    # when $sys_dest->{cleanup} is set, we'll not be able to change filestat because ssh/rsh comm is unconfigured
    $sys_dest->change_filestat($filename,$stat) if ($stat && $sys_dest->{cleanup});
    return $rtn;
}

sub movefile {
    my ($sys,$file_src,$file_dest)=@_;

    return unless ($file_src && $file_dest);
    return $sys->padv->movefile_sys($sys,$file_src,$file_dest);
}

sub filestat {
    my ($sys,$file,$filestat)=@_;

    return unless ($file);
    return $sys->padv->filestat_sys($sys,$file,$filestat);
}

sub change_filestat {
    my ($sys,$file,$filestat)=@_;

    return unless ($file);
    return $sys->padv->change_filestat_sys($sys,$file,$filestat);
}

sub filesize {
    my ($sys,$file)=@_;

    return unless ($file);
    return $sys->padv->filesize_sys($sys,$file);
}

sub chmod {
    my ($sys,$file,$mode)=@_;

    return unless ($file && $mode);
    return $sys->padv->chmod_sys($sys,$file,$mode);
}

sub mkdir {
    my ($sys,$dir)=@_;

    return unless ($dir);
    return $sys->padv->mkdir_sys($sys,$dir);
}

sub rm {
    my ($sys,$fileordir)=@_;

    return unless ($fileordir);
    return $sys->padv->rm_sys($sys,$fileordir);
}

sub timesync {
    my ($sys,$ntpserver)=@_;

    return unless ($ntpserver);
    return $sys->padv->timesync_sys($sys,$ntpserver);
}

sub configure_static_ip {
    my ($sys, $nic, $ip, $mask)=@_;
    return $sys->padv->configure_static_ip_sys($sys, $nic, $ip, $mask);
}

1;
